@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,7 +36,7 @@
 
 #include "client/client_priv.h"
 #include "compression.h"
-#include "m_ctype.h"
+#include "m_string.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
@@ -46,17 +46,22 @@
 #include "my_macros.h"
 #include "my_thread.h" /* because of signal()	*/
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/int2str.h"
+#include "mysql/strings/m_ctype.h"
+#include "nulls.h"
 #include "print_version.h"
 #include "sql_common.h"
+#include "str2int.h"
+#include "strxmov.h"
 #include "typelib.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-#define MAX_MYSQL_VAR 612
+#define MAX_MYSQL_VAR 8192
 #define SHUTDOWN_DEF_TIMEOUT 3600 /* Wait for shutdown */
 #define MAX_TRUNC_LENGTH 3
 
 const char *host = nullptr;
-char *user = nullptr, *opt_password = nullptr;
+char *user = nullptr;
 const char *default_charset = MYSQL_AUTODETECT_CHARSET_NAME;
 char truncated_var_names[MAX_MYSQL_VAR][MAX_TRUNC_LENGTH];
 char ex_var_names[MAX_MYSQL_VAR][FN_REFLEN];
@@ -64,10 +69,10 @@ ulonglong last_values[MAX_MYSQL_VAR];
 static int interval = 0;
 static bool option_force = false, interrupted = false, new_line = false,
             opt_compress = false, opt_relative = false, opt_verbose = false,
-            opt_vertical = false, tty_password = false, opt_nobeep;
+            opt_vertical = false, opt_nobeep;
 static bool debug_info_flag = false, debug_check_flag = false;
 static uint tcp_port = 0, option_wait = 0, option_silent = 0, nr_iterations;
-static uint opt_count_iterations = 0, my_end_arg;
+static uint opt_count_iterations = 0, my_end_arg = 0;
 static char *opt_bind_addr = nullptr;
 static ulong opt_connect_timeout, opt_shutdown_timeout;
 static char *unix_port = nullptr;
@@ -78,7 +83,7 @@ static bool opt_show_warnings = false;
 static uint opt_zstd_compress_level = default_zstd_compression_level;
 static char *opt_compress_algorithm = nullptr;
 #if defined(_WIN32)
-static char *shared_memory_base_name = 0;
+static char *shared_memory_base_name = nullptr;
 #endif
 static uint opt_protocol = 0;
 static myf error_flags; /* flags to pass to my_printf_error, like ME_BELL */
@@ -96,6 +101,7 @@ static uint ex_var_count, max_var_length, max_val_length;
 #include "sslopt-vars.h"
 
 #include "caching_sha2_passwordopt-vars.h"
+#include "multi_factor_passwordopt-vars.h"
 
 static void usage(void);
 extern "C" bool get_one_option(int optid, const struct my_option *opt,
@@ -145,9 +151,11 @@ enum commands {
   ADMIN_EXTENDED_STATUS,
   ADMIN_FLUSH_STATUS,
   ADMIN_FLUSH_PRIVILEGES,
+  ADMIN_START_REPLICA,
+  ADMIN_STOP_REPLICA,
+  ADMIN_FLUSH_THREADS,
   ADMIN_START_SLAVE,
-  ADMIN_STOP_SLAVE,
-  ADMIN_FLUSH_THREADS
+  ADMIN_STOP_SLAVE
 };
 static const char *command_names[] = {"create",
                                       "drop",
@@ -168,9 +176,11 @@ static const char *command_names[] = {"create",
                                       "extended-status",
                                       "flush-status",
                                       "flush-privileges",
+                                      "start-replica",
+                                      "stop-replica",
+                                      "flush-threads",
                                       "start-slave",
                                       "stop-slave",
-                                      "flush-threads",
                                       NullS};
 
 static TYPELIB command_typelib = {array_elements(command_names) - 1, "commands",
@@ -185,14 +195,14 @@ static struct my_option my_long_options[] = {
      &nr_iterations, &nr_iterations, nullptr, GET_UINT, REQUIRED_ARG, 0, 0, 0,
      nullptr, 0, nullptr},
 #ifdef NDEBUG
-    {"debug", '#', "This is a non-debug version. Catch this and exit.", 0, 0, 0,
-     GET_DISABLED, OPT_ARG, 0, 0, 0, 0, 0, 0},
+    {"debug", '#', "This is a non-debug version. Catch this and exit.", nullptr,
+     nullptr, nullptr, GET_DISABLED, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"debug-check", OPT_DEBUG_CHECK,
-     "This is a non-debug version. Catch this and exit.", 0, 0, 0, GET_DISABLED,
-     NO_ARG, 0, 0, 0, 0, 0, 0},
+     "This is a non-debug version. Catch this and exit.", nullptr, nullptr,
+     nullptr, GET_DISABLED, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"debug-info", OPT_DEBUG_INFO,
-     "This is a non-debug version. Catch this and exit.", 0, 0, 0, GET_DISABLED,
-     NO_ARG, 0, 0, 0, 0, 0, 0},
+     "This is a non-debug version. Catch this and exit.", nullptr, nullptr,
+     nullptr, GET_DISABLED, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
 #else
     {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", nullptr,
      nullptr, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
@@ -224,14 +234,10 @@ static struct my_option my_long_options[] = {
      REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"no-beep", 'b', "Turn off beep on error.", &opt_nobeep, &opt_nobeep,
      nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"password", 'p',
-     "Password to use when connecting to server. If password is not given it's "
-     "asked from the tty.",
-     nullptr, nullptr, nullptr, GET_PASSWORD, OPT_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+#include "multi_factor_passwordopt-longopts.h"
 #ifdef _WIN32
-    {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
-     NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"pipe", 'W', "Use named pipes to connect to server.", nullptr, nullptr,
+     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
 #endif
     {"port", 'P',
      "Port number to use for connection or 0 for default to, in "
@@ -253,8 +259,8 @@ static struct my_option my_long_options[] = {
 #if defined(_WIN32)
     {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
      "Base name of shared memory.", &shared_memory_base_name,
-     &shared_memory_base_name, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
-     0},
+     &shared_memory_base_name, nullptr, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0,
+     nullptr, 0, nullptr},
 #endif
     {"silent", 's', "Silently exit if one can't connect to server.", nullptr,
      nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
@@ -317,8 +323,7 @@ static struct my_option my_long_options[] = {
 
 static const char *load_default_groups[] = {"mysqladmin", "client", nullptr};
 
-bool get_one_option(int optid,
-                    const struct my_option *opt MY_ATTRIBUTE((unused)),
+bool get_one_option(int optid, const struct my_option *opt [[maybe_unused]],
                     char *argument) {
   int error = 0;
 
@@ -326,24 +331,7 @@ bool get_one_option(int optid,
     case 'c':
       opt_count_iterations = 1;
       break;
-    case 'p':
-      if (argument == disabled_my_option) {
-        // Don't require password
-        static char empty_password[] = {'\0'};
-        assert(empty_password[0] ==
-               '\0');  // Check that it has not been overwritten
-        argument = empty_password;
-      }
-      if (argument) {
-        char *start = argument;
-        my_free(opt_password);
-        opt_password = my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
-        while (*argument) *argument++ = 'x'; /* Destroy argument */
-        if (*start) start[1] = 0;            /* Cut length of argument */
-        tty_password = false;
-      } else
-        tty_password = true;
-      break;
+      PARSE_COMMAND_LINE_PASSWORD_OPTION;
     case 's':
       option_silent++;
       break;
@@ -381,6 +369,9 @@ bool get_one_option(int optid,
     case OPT_ENABLE_CLEARTEXT_PLUGIN:
       using_opt_enable_cleartext_plugin = true;
       break;
+    case 'C':
+      CLIENT_WARN_DEPRECATED("--compress", "--compression-algorithms");
+      break;
   }
   if (error) {
     usage();
@@ -397,16 +388,16 @@ int main(int argc, char *argv[]) {
   char **commands, **temp_argv;
 
   MY_INIT(argv[0]);
-  mysql_init(&mysql);
   my_getopt_use_args_separator = true;
   MEM_ROOT alloc{PSI_NOT_INSTRUMENTED, 512};
-  if (load_defaults("my", load_default_groups, &argc, &argv, &alloc))
+  if (load_defaults("my", load_default_groups, &argc, &argv, &alloc)) {
+    my_end(my_end_arg);
     return EXIT_FAILURE;
+  }
   my_getopt_use_args_separator = false;
 
   if ((ho_error =
            handle_options(&argc, &argv, my_long_options, get_one_option))) {
-    mysql_close(&mysql);
     my_end(my_end_arg);
     return ho_error;
   }
@@ -416,6 +407,7 @@ int main(int argc, char *argv[]) {
 
   if (argc == 0) {
     usage();
+    my_end(my_end_arg);
     return EXIT_FAILURE;
   }
 
@@ -423,11 +415,11 @@ int main(int argc, char *argv[]) {
   temp_argc = argc;
 
   commands = temp_argv;
-  if (tty_password) opt_password = get_tty_password(NullS);
 
   (void)signal(SIGINT, endprog);  /* Here if abort */
   (void)signal(SIGTERM, endprog); /* Here if abort */
 
+  mysql_init(&mysql);
   if (opt_bind_addr) mysql_options(&mysql, MYSQL_OPT_BIND, opt_bind_addr);
   if (opt_compress) mysql_options(&mysql, MYSQL_OPT_COMPRESS, NullS);
   if (opt_connect_timeout) {
@@ -436,6 +428,8 @@ int main(int argc, char *argv[]) {
   }
   if (SSL_SET_OPTIONS(&mysql)) {
     fprintf(stderr, "%s", SSL_SET_OPTIONS_ERROR);
+    mysql_close(&mysql);
+    my_end(my_end_arg);
     return EXIT_FAILURE;
   }
   if (opt_protocol)
@@ -475,13 +469,15 @@ int main(int argc, char *argv[]) {
 
   set_server_public_key(&mysql);
   set_get_server_public_key_option(&mysql);
+
+  set_password_options(&mysql);
   if (sql_connect(&mysql, option_wait)) {
     /*
       We couldn't get an initial connection and will definitely exit.
       The following just determines the exit-code we'll give.
     */
 
-    unsigned int err = mysql_errno(&mysql);
+    const unsigned int err = mysql_errno(&mysql);
     if (err >= CR_MIN_ERROR && err <= CR_MAX_ERROR)
       error = 1;
     else {
@@ -496,11 +492,11 @@ int main(int argc, char *argv[]) {
     }
   } else {
     /*
-      --count=0 aborts right here. Otherwise iff --sleep=t ("interval")
+      --count=0 aborts right here. Otherwise if --sleep=t ("interval")
       is given a t!=0, we get an endless loop, or n iterations if --count=n
       was given an n!=0. If --sleep wasn't given, we get one iteration.
 
-      To wit, --wait loops the connection-attempts, while --sleep loops
+      To wait, --wait loops the connection-attempts, while --sleep loops
       the command execution (endlessly if no --count is given).
     */
 
@@ -544,7 +540,7 @@ int main(int argc, char *argv[]) {
             */
           } else {
             /*
-              connexion broke, and we have no order to re-establish it. fail.
+              connection broke, and we have no order to re-establish it. fail.
             */
             if (!option_force) error = 1;
             break;
@@ -559,7 +555,7 @@ int main(int argc, char *argv[]) {
   }            /* got connection */
 
   mysql_close(&mysql);
-  my_free(opt_password);
+  free_passwords();
   my_free(user);
 #if defined(_WIN32)
   my_free(shared_memory_base_name);
@@ -574,7 +570,7 @@ int main(int argc, char *argv[]) {
   return error ? EXIT_FAILURE : EXIT_SUCCESS;
 }
 
-void endprog(int signal_number MY_ATTRIBUTE((unused))) { interrupted = true; }
+void endprog(int signal_number [[maybe_unused]]) { interrupted = true; }
 
 /**
    @brief connect to server, optionally waiting for same to come up
@@ -592,8 +588,12 @@ static bool sql_connect(MYSQL *mysql, uint wait) {
   bool info = false;
 
   for (;;) {
-    if (mysql_real_connect(mysql, host, user, opt_password, NullS, tcp_port,
+    if (mysql_real_connect(mysql, host, user, nullptr, NullS, tcp_port,
                            unix_port, CLIENT_REMEMBER_OPTIONS)) {
+      if (ssl_client_check_post_connect_ssl_setup(
+              mysql, [](const char *err) { fprintf(stderr, "%s\n", err); }))
+        return true;
+
       mysql->reconnect = true;
       if (info) {
         fputs("\n", stderr);
@@ -675,8 +675,6 @@ static int execute_commands(MYSQL *mysql, int argc, char **argv) {
     If this behaviour is ever changed, Docs should be notified.
   */
 
-  struct rand_struct rand_st;
-
   for (; argc > 0; argv++, argc--) {
     int option;
     bool log_warnings = true;
@@ -753,7 +751,7 @@ static int execute_commands(MYSQL *mysql, int argc, char **argv) {
       case ADMIN_REFRESH:
         if (mysql_refresh(mysql, (uint) ~(REFRESH_GRANT | REFRESH_STATUS |
                                           REFRESH_READ_LOCK | REFRESH_REPLICA |
-                                          REFRESH_MASTER))) {
+                                          REFRESH_SOURCE))) {
           my_printf_error(0, "refresh failed; error: '%s'", error_flags,
                           mysql_error(mysql));
           return -1;
@@ -881,8 +879,15 @@ static int execute_commands(MYSQL *mysql, int argc, char **argv) {
           return -1;
         }
 
-        assert(mysql_num_rows(res) < MAX_MYSQL_VAR);
-
+        if (mysql_num_rows(res) >= MAX_MYSQL_VAR) {
+          my_printf_error(0,
+                          "Too many rows returned: '%llu'. "
+                          "Expecting no more than '%d' rows",
+                          error_flags, (unsigned long long)mysql_num_rows(res),
+                          MAX_MYSQL_VAR);
+          mysql_free_result(res);
+          return -1;
+        }
         if (!opt_vertical)
           print_header(res);
         else {
@@ -988,15 +993,10 @@ static int execute_commands(MYSQL *mysql, int argc, char **argv) {
       }
       case ADMIN_PASSWORD: {
         char buff[128];
-        time_t start_time;
         char *typed_password = nullptr, *verified = nullptr, *tmp = nullptr;
         bool log_off = true, err = false;
         size_t password_len;
 
-        /* Do initialization the same way as we do in mysqld */
-        start_time = time((time_t *)nullptr);
-        randominit(&rand_st, (ulong)start_time, (ulong)start_time / 2);
-
         if (argc < 1) {
           my_printf_error(0, "Too few arguments to change password",
                           error_flags);
@@ -1024,7 +1024,7 @@ static int execute_commands(MYSQL *mysql, int argc, char **argv) {
 
         if (typed_password[0]) {
 #ifdef _WIN32
-          size_t pw_len = strlen(typed_password);
+          const size_t pw_len = strlen(typed_password);
           if (pw_len > 1 && typed_password[0] == '\'' &&
               typed_password[pw_len - 1] == '\'')
             printf(
@@ -1113,20 +1113,27 @@ static int execute_commands(MYSQL *mysql, int argc, char **argv) {
       }
 
       case ADMIN_START_SLAVE:
-        if (mysql_query(mysql, "START SLAVE")) {
-          my_printf_error(0, "Error starting slave: %s", error_flags,
+        CLIENT_WARN_DEPRECATED("start-slave", "start-replica");
+        [[fallthrough]];
+      case ADMIN_START_REPLICA:
+        if (mysql_query(mysql, "START REPLICA")) {
+          my_printf_error(0, "Error starting replication: %s", error_flags,
                           mysql_error(mysql));
           return -1;
         } else
-          puts("Slave started");
+          puts("Replication started");
         break;
+
       case ADMIN_STOP_SLAVE:
-        if (mysql_query(mysql, "STOP SLAVE")) {
-          my_printf_error(0, "Error stopping slave: %s", error_flags,
+        CLIENT_WARN_DEPRECATED("stop-slave", "stop-replica");
+        [[fallthrough]];
+      case ADMIN_STOP_REPLICA:
+        if (mysql_query(mysql, "STOP REPLICA")) {
+          my_printf_error(0, "Error stopping replication: %s", error_flags,
                           mysql_error(mysql));
           return -1;
         } else
-          puts("Slave stopped");
+          puts("Replication stopped");
         break;
 
       case ADMIN_PING:
@@ -1227,8 +1234,10 @@ static void usage(void) {
   refresh		Flush all tables and close and open logfiles\n\
   shutdown		Take server down\n\
   status		Gives a short status message from the server\n\
-  start-slave		Start slave\n\
-  stop-slave		Stop slave\n\
+  start-replica		Start replication\n\
+  start-slave		Deprecated: use start-replica instead\n\
+  stop-replica		Stop replication\n\
+  stop-slave		Deprecated: use stop-replica instead\n\
   variables             Prints variables available\n\
   version		Get version info from server");
 }
@@ -1314,7 +1323,7 @@ static void print_top(MYSQL_RES *result) {
 
 /* 3.rd argument, uint row, is not in use. Don't remove! */
 static void print_row(MYSQL_RES *result, MYSQL_ROW cur,
-                      uint row MY_ATTRIBUTE((unused))) {
+                      uint row [[maybe_unused]]) {
   uint i, length;
   MYSQL_FIELD *field;
 
@@ -1344,7 +1353,7 @@ static void print_relative_row(MYSQL_RES *result, MYSQL_ROW cur, uint row) {
   last_values[row] = tmp;
 }
 
-static void print_relative_row_vert(MYSQL_RES *result MY_ATTRIBUTE((unused)),
+static void print_relative_row_vert(MYSQL_RES *result [[maybe_unused]],
                                     MYSQL_ROW cur, uint row) {
   uint length;
   ulonglong tmp;
