@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,7 +34,7 @@
 
 #include "caching_sha2_passwordopt-vars.h"
 #include "client/client_priv.h"
-#include "m_ctype.h"
+#include "compression.h"
 #include "m_string.h"
 #include "my_alloc.h"
 #include "my_dbug.h"
@@ -43,27 +43,36 @@
 #include "my_macros.h"
 #include "my_sys.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h"
+#include "nulls.h"
 #include "print_version.h"
 #include "sslopt-vars.h"
+#include "strxnmov.h"
 #include "typelib.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-static char *host = 0, *opt_password = 0, *user = 0;
-static bool opt_show_keys = 0, opt_compress = 0, opt_count = 0, opt_status = 0;
-static bool tty_password = 0, opt_table_type = 0;
-static bool debug_info_flag = 0, debug_check_flag = 0;
+static char *host = nullptr, *user = nullptr;
+static bool opt_show_keys = false, opt_compress = false, opt_count = false,
+            opt_status = false;
+static bool opt_table_type = false;
+static bool debug_info_flag = false, debug_check_flag = false;
 static uint my_end_arg = 0;
 static uint opt_verbose = 0;
-static char *default_charset = (char *)MYSQL_AUTODETECT_CHARSET_NAME;
-static char *opt_plugin_dir = 0, *opt_default_auth = 0;
+static const char *default_charset = MYSQL_AUTODETECT_CHARSET_NAME;
+static char *opt_plugin_dir = nullptr, *opt_default_auth = nullptr;
 static uint opt_enable_cleartext_plugin = 0;
-static bool using_opt_enable_cleartext_plugin = 0;
+static bool using_opt_enable_cleartext_plugin = false;
+
+static uint opt_zstd_compress_level = default_zstd_compression_level;
+static char *opt_compress_algorithm = nullptr;
+
+#include "multi_factor_passwordopt-vars.h"
 
 #if defined(_WIN32)
-static char *shared_memory_base_name = 0;
+static char *shared_memory_base_name = nullptr;
 #endif
 static uint opt_protocol = 0;
-static char *opt_bind_addr = NULL;
+static char *opt_bind_addr = nullptr;
 
 static void get_options(int *argc, char ***argv);
 static uint opt_mysql_port = 0;
@@ -79,12 +88,12 @@ static void print_res_header(MYSQL_RES *result);
 static void print_res_top(MYSQL_RES *result);
 static void print_res_row(MYSQL_RES *result, MYSQL_ROW cur);
 
-static const char *load_default_groups[] = {"mysqlshow", "client", 0};
-static char *opt_mysql_unix_port = 0;
+static const char *load_default_groups[] = {"mysqlshow", "client", nullptr};
+static char *opt_mysql_unix_port = nullptr;
 
 int main(int argc, char **argv) {
   int error;
-  bool first_argument_uses_wildcards = 0;
+  bool first_argument_uses_wildcards = false;
   char *wild;
   MYSQL mysql;
   MY_INIT(argv[0]);
@@ -96,22 +105,22 @@ int main(int argc, char **argv) {
 
   get_options(&argc, &argv);
 
-  wild = 0;
+  wild = nullptr;
   if (argc) {
     char *pos = argv[argc - 1], *to;
     for (to = pos; *pos; pos++, to++) {
       switch (*pos) {
         case '*':
           *pos = '%';
-          first_argument_uses_wildcards = 1;
+          first_argument_uses_wildcards = true;
           break;
         case '?':
           *pos = '_';
-          first_argument_uses_wildcards = 1;
+          first_argument_uses_wildcards = true;
           break;
         case '%':
         case '_':
-          first_argument_uses_wildcards = 1;
+          first_argument_uses_wildcards = true;
           break;
         case '\\':
           pos++;
@@ -133,7 +142,10 @@ int main(int argc, char **argv) {
   }
   mysql_init(&mysql);
   if (opt_compress) mysql_options(&mysql, MYSQL_OPT_COMPRESS, NullS);
-  SSL_SET_OPTIONS(&mysql);
+  if (SSL_SET_OPTIONS(&mysql)) {
+    fprintf(stderr, "%s", SSL_SET_OPTIONS_ERROR);
+    exit(1);
+  }
   if (opt_protocol)
     mysql_options(&mysql, MYSQL_OPT_PROTOCOL, (char *)&opt_protocol);
   if (opt_bind_addr) mysql_options(&mysql, MYSQL_OPT_BIND, opt_bind_addr);
@@ -144,6 +156,13 @@ int main(int argc, char **argv) {
 #endif
   mysql_options(&mysql, MYSQL_SET_CHARSET_NAME, default_charset);
 
+  if (opt_compress_algorithm)
+    mysql_options(&mysql, MYSQL_OPT_COMPRESSION_ALGORITHMS,
+                  opt_compress_algorithm);
+
+  mysql_options(&mysql, MYSQL_OPT_ZSTD_COMPRESSION_LEVEL,
+                &opt_zstd_compress_level);
+
   if (opt_plugin_dir && *opt_plugin_dir)
     mysql_options(&mysql, MYSQL_PLUGIN_DIR, opt_plugin_dir);
 
@@ -154,18 +173,27 @@ int main(int argc, char **argv) {
     mysql_options(&mysql, MYSQL_ENABLE_CLEARTEXT_PLUGIN,
                   (char *)&opt_enable_cleartext_plugin);
 
-  mysql_options(&mysql, MYSQL_OPT_CONNECT_ATTR_RESET, 0);
+  mysql_options(&mysql, MYSQL_OPT_CONNECT_ATTR_RESET, nullptr);
   mysql_options4(&mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "program_name",
                  "mysqlshow");
   set_server_public_key(&mysql);
   set_get_server_public_key_option(&mysql);
-  if (!(mysql_real_connect(&mysql, host, user, opt_password,
+  set_password_options(&mysql);
+  if (!(mysql_real_connect(&mysql, host, user, nullptr,
                            (first_argument_uses_wildcards) ? "" : argv[0],
                            opt_mysql_port, opt_mysql_unix_port, 0))) {
     fprintf(stderr, "%s: %s\n", my_progname, mysql_error(&mysql));
     exit(1);
   }
-  mysql.reconnect = 1;
+  if (ssl_client_check_post_connect_ssl_setup(
+          &mysql, [](const char *err) { fprintf(stderr, "%s\n", err); })) {
+    mysql_close(&mysql);
+    free_passwords();
+    mysql_server_end();
+    my_end(my_end_arg);
+    exit(1);
+  }
+  mysql.reconnect = true;
 
   switch (argc) {
     case 0:
@@ -185,7 +213,7 @@ int main(int argc, char **argv) {
       break;
   }
   mysql_close(&mysql); /* Close & free connection */
-  my_free(opt_password);
+  free_passwords();
 #if defined(_WIN32)
   my_free(shared_memory_base_name);
 #endif
@@ -196,47 +224,51 @@ int main(int argc, char **argv) {
 
 static struct my_option my_long_options[] = {
     {"bind-address", 0, "IP address to bind to.", (uchar **)&opt_bind_addr,
-     (uchar **)&opt_bind_addr, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+     (uchar **)&opt_bind_addr, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr,
+     0, nullptr},
     {"character-sets-dir", 'c', "Directory for character set files.",
-     &charsets_dir, &charsets_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+     &charsets_dir, &charsets_dir, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0,
+     nullptr, 0, nullptr},
     {"default-character-set", OPT_DEFAULT_CHARSET,
-     "Set the default character set.", &default_charset, &default_charset, 0,
-     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+     "Set the default character set.", &default_charset, &default_charset,
+     nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"count", OPT_COUNT,
      "Show number of rows per table (may be slow for non-MyISAM tables).",
-     &opt_count, &opt_count, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+     &opt_count, &opt_count, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
+     nullptr},
     {"compress", 'C', "Use compression in server/client protocol.",
-     &opt_compress, &opt_compress, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-    {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", 0, 0, 0,
-     GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
+     &opt_compress, &opt_compress, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr,
+     0, nullptr},
+    {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", nullptr,
+     nullptr, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"debug-check", OPT_DEBUG_CHECK,
      "Check memory and open file usage at exit.", &debug_check_flag,
-     &debug_check_flag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+     &debug_check_flag, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
+     nullptr},
     {"debug-info", OPT_DEBUG_INFO, "Print some debug info at exit.",
-     &debug_info_flag, &debug_info_flag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+     &debug_info_flag, &debug_info_flag, nullptr, GET_BOOL, NO_ARG, 0, 0, 0,
+     nullptr, 0, nullptr},
     {"default_auth", OPT_DEFAULT_AUTH,
      "Default authentication client-side plugin to use.", &opt_default_auth,
-     &opt_default_auth, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+     &opt_default_auth, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0,
+     nullptr},
     {"enable_cleartext_plugin", OPT_ENABLE_CLEARTEXT_PLUGIN,
      "Enable/disable the clear text authentication plugin.",
-     &opt_enable_cleartext_plugin, &opt_enable_cleartext_plugin, 0, GET_BOOL,
-     OPT_ARG, 0, 0, 0, 0, 0, 0},
-    {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
-     0, 0, 0, 0, 0},
-    {"host", 'h', "Connect to host.", &host, &host, 0, GET_STR, REQUIRED_ARG, 0,
-     0, 0, 0, 0, 0},
+     &opt_enable_cleartext_plugin, &opt_enable_cleartext_plugin, nullptr,
+     GET_BOOL, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"help", '?', "Display this help and exit.", nullptr, nullptr, nullptr,
+     GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"host", 'h', "Connect to host.", &host, &host, nullptr, GET_STR,
+     REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"status", 'i', "Shows a lot of extra information about each table.",
-     &opt_status, &opt_status, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-    {"keys", 'k', "Show keys for table.", &opt_show_keys, &opt_show_keys, 0,
-     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-    {"password", 'p',
-     "Password to use when connecting to server. If password is not given, "
-     "it's "
-     "solicited on the tty.",
-     0, 0, 0, GET_PASSWORD, OPT_ARG, 0, 0, 0, 0, 0, 0},
+     &opt_status, &opt_status, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
+     nullptr},
+    {"keys", 'k', "Show keys for table.", &opt_show_keys, &opt_show_keys,
+     nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+#include "multi_factor_passwordopt-longopts.h"
     {"plugin_dir", OPT_PLUGIN_DIR, "Directory for client-side plugins.",
-     &opt_plugin_dir, &opt_plugin_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0,
-     0},
+     &opt_plugin_dir, &opt_plugin_dir, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0,
+     nullptr, 0, nullptr},
     {"port", 'P',
      "Port number to use for connection or 0 for default to, in "
      "order of preference, my.cnf, $MYSQL_TCP_PORT, "
@@ -244,38 +276,51 @@ static struct my_option my_long_options[] = {
      "/etc/services, "
 #endif
      "built-in default (" STRINGIFY_ARG(MYSQL_PORT) ").",
-     &opt_mysql_port, &opt_mysql_port, 0, GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0,
-     0},
+     &opt_mysql_port, &opt_mysql_port, nullptr, GET_UINT, REQUIRED_ARG, 0, 0, 0,
+     nullptr, 0, nullptr},
 #ifdef _WIN32
-    {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
-     NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"pipe", 'W', "Use named pipes to connect to server.", nullptr, nullptr,
+     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
 #endif
     {"protocol", OPT_MYSQL_PROTOCOL,
-     "The protocol to use for connection (tcp, socket, pipe, memory).", 0, 0, 0,
-     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+     "The protocol to use for connection (tcp, socket, pipe, memory).", nullptr,
+     nullptr, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
 #if defined(_WIN32)
     {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
      "Base name of shared memory.", &shared_memory_base_name,
-     &shared_memory_base_name, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
-     0},
+     &shared_memory_base_name, nullptr, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0,
+     nullptr, 0, nullptr},
 #endif
     {"show-table-type", 't', "Show table type column.", &opt_table_type,
-     &opt_table_type, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+     &opt_table_type, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"socket", 'S', "The socket file to use for connection.",
-     &opt_mysql_unix_port, &opt_mysql_unix_port, 0, GET_STR, REQUIRED_ARG, 0, 0,
-     0, 0, 0, 0},
+     &opt_mysql_unix_port, &opt_mysql_unix_port, nullptr, GET_STR, REQUIRED_ARG,
+     0, 0, 0, nullptr, 0, nullptr},
 #include "caching_sha2_passwordopt-longopts.h"
 #include "sslopt-longopts.h"
 
-    {"user", 'u', "User for login if not current user.", &user, &user, 0,
-     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+    {"user", 'u', "User for login if not current user.", &user, &user, nullptr,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"verbose", 'v',
      "More verbose output; you can use this multiple times to get even more "
      "verbose output.",
-     0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-    {"version", 'V', "Output version information and exit.", 0, 0, 0,
-     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-    {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
+     nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0,
+     nullptr},
+    {"version", 'V', "Output version information and exit.", nullptr, nullptr,
+     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"compression-algorithms", 0,
+     "Use compression algorithm in server/client protocol. Valid values "
+     "are any combination of 'zstd','zlib','uncompressed'.",
+     &opt_compress_algorithm, &opt_compress_algorithm, nullptr, GET_STR,
+     REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"zstd-compression-level", 0,
+     "Use this compression level in the client/server protocol, in case "
+     "--compression-algorithms=zstd. Valid range is between 1 and 22, "
+     "inclusive. Default is 3.",
+     &opt_zstd_compress_level, &opt_zstd_compress_level, nullptr, GET_UINT,
+     REQUIRED_ARG, 3, 1, 22, nullptr, 0, nullptr},
+    {nullptr, 0, nullptr, nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0,
+     0, nullptr, 0, nullptr}};
 
 static void usage(void) {
   print_version();
@@ -304,19 +349,7 @@ static bool get_one_option(int optid, const struct my_option *opt,
     case 'v':
       opt_verbose++;
       break;
-    case 'p':
-      if (argument == disabled_my_option)
-        argument = (char *)""; /* Don't require password */
-      if (argument) {
-        char *start = argument;
-        my_free(opt_password);
-        opt_password = my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
-        while (*argument) *argument++ = 'x'; /* Destroy argument */
-        if (*start) start[1] = 0;            /* Cut length of argument */
-        tty_password = 0;
-      } else
-        tty_password = 1;
-      break;
+      PARSE_COMMAND_LINE_PASSWORD_OPTION;
     case 'W':
 #ifdef _WIN32
       opt_protocol = MYSQL_PROTOCOL_PIPE;
@@ -331,7 +364,7 @@ static bool get_one_option(int optid, const struct my_option *opt,
       break;
     case '#':
       DBUG_PUSH(argument ? argument : "d:t:o");
-      debug_check_flag = 1;
+      debug_check_flag = true;
       break;
 #include "sslopt-case.h"
 
@@ -342,8 +375,11 @@ static bool get_one_option(int optid, const struct my_option *opt,
     case 'I': /* Info */
       usage();
       exit(0);
+    case 'C':
+      CLIENT_WARN_DEPRECATED("--compress", "--compression-algorithms");
+      break;
   }
-  return 0;
+  return false;
 }
 }  // extern "C"
 
@@ -353,7 +389,6 @@ static void get_options(int *argc, char ***argv) {
   if ((ho_error = handle_options(argc, argv, my_long_options, get_one_option)))
     exit(ho_error);
 
-  if (tty_password) opt_password = get_tty_password(NullS);
   if (opt_count) {
     /*
       We need to set verbose to 2 as we need to change the output to include
@@ -375,7 +410,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
   char query[NAME_LEN + 100];
   MYSQL_FIELD *field;
   MYSQL_RES *result;
-  MYSQL_ROW row = NULL, rrow;
+  MYSQL_ROW row = nullptr, rrow;
 
   if (!(result = mysql_list_dbs(mysql, wild))) {
     fprintf(stderr, "%s: Cannot list databases: %s\n", my_progname,
@@ -394,9 +429,9 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
     if (!my_strcasecmp(&my_charset_latin1, row[0], wild)) {
       mysql_free_result(result);
       if (opt_status)
-        return list_table_status(mysql, wild, NULL);
+        return list_table_status(mysql, wild, nullptr);
       else
-        return list_tables(mysql, wild, NULL);
+        return list_tables(mysql, wild, nullptr);
     }
   }
 
@@ -420,7 +455,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
 
     if (opt_verbose) {
       if (!(mysql_select_db(mysql, row[0]))) {
-        MYSQL_RES *tresult = mysql_list_tables(mysql, (char *)NULL);
+        MYSQL_RES *tresult = mysql_list_tables(mysql, (char *)nullptr);
         if (mysql_affected_rows(mysql) > 0) {
           sprintf(tables, "%6lu", (ulong)mysql_affected_rows(mysql));
           rowcount = 0;
@@ -434,7 +469,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
                 MYSQL_RES *rresult;
                 if ((rresult = mysql_store_result(mysql))) {
                   rrow = mysql_fetch_row(rresult);
-                  rowcount += (ulong)my_strtoull(rrow[0], (char **)0, 10);
+                  rowcount += (ulong)my_strtoull(rrow[0], (char **)nullptr, 10);
                   mysql_free_result(rresult);
                 }
               }
@@ -459,7 +494,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
     else
       print_row(row[0], length, tables, 6, rows, 12, NullS);
 
-    row = NULL;
+    row = nullptr;
   }
 
   print_trailer(length, (opt_verbose > 0 ? 6 : 0), (opt_verbose > 1 ? 12 : 0),
@@ -533,7 +568,7 @@ static int list_tables(MYSQL *mysql, const char *db, const char *table) {
     counter++;
     if (opt_verbose > 0) {
       if (!(mysql_select_db(mysql, db))) {
-        MYSQL_RES *rresult = mysql_list_fields(mysql, row[0], NULL);
+        MYSQL_RES *rresult = mysql_list_fields(mysql, row[0], nullptr);
         ulong rowcount = 0L;
         if (!rresult) {
           my_stpcpy(fields, "N/A");
@@ -548,7 +583,8 @@ static int list_tables(MYSQL *mysql, const char *db, const char *table) {
             if (!(mysql_query(mysql, query))) {
               if ((rresult = mysql_store_result(mysql))) {
                 rrow = mysql_fetch_row(rresult);
-                rowcount += (unsigned long)my_strtoull(rrow[0], (char **)0, 10);
+                rowcount +=
+                    (unsigned long)my_strtoull(rrow[0], (char **)nullptr, 10);
                 mysql_free_result(rresult);
               }
               sprintf(rows, "%10lu", rowcount);
@@ -648,7 +684,7 @@ static int list_fields(MYSQL *mysql, const char *db, const char *table,
       return 1;
     }
     row = mysql_fetch_row(result);
-    rows = (ulong)my_strtoull(row[0], (char **)0, 10);
+    rows = (ulong)my_strtoull(row[0], (char **)nullptr, 10);
     mysql_free_result(result);
   }
 
