@@ -70,6 +70,7 @@
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_file_bits.h"
 #include "mysql/components/services/bits/psi_memory_bits.h"
+#include "mysql/components/services/bits/psi_metric_bits.h"
 #include "mysql/components/services/bits/psi_stage_bits.h"
 #include "sql/stream_cipher.h"
 #include "string_with_len.h"
@@ -86,6 +87,7 @@ struct PSI_file_bootstrap;
 struct PSI_idle_bootstrap;
 struct PSI_mdl_bootstrap;
 struct PSI_memory_bootstrap;
+struct PSI_metric_bootstrap;
 struct PSI_mutex_bootstrap;
 struct PSI_rwlock_bootstrap;
 struct PSI_socket_bootstrap;
@@ -497,6 +499,7 @@ inline bool my_b_inited(const IO_CACHE *info) {
 constexpr int my_b_EOF = INT_MIN;
 
 inline int my_b_read(IO_CACHE *info, uchar *buffer, size_t count) {
+  assert(info->type != WRITE_CACHE);
   if ((size_t)(info->read_end - info->read_pos) >= count) {
     memcpy(buffer, info->read_pos, count);
     info->read_pos += count;
@@ -506,6 +509,7 @@ inline int my_b_read(IO_CACHE *info, uchar *buffer, size_t count) {
 }
 
 inline int my_b_write(IO_CACHE *info, const uchar *buffer, size_t count) {
+  assert(info->type != READ_CACHE);
   if ((size_t)(info->write_end - info->write_pos) >= count) {
     memcpy(info->write_pos, buffer, count);
     info->write_pos += count;
@@ -971,6 +975,8 @@ extern void set_psi_mdl_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_memory_bootstrap *psi_memory_hook;
 extern void set_psi_memory_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_mutex_bootstrap *psi_mutex_hook;
+extern void set_psi_metric_service(void *psi);
+extern MYSQL_PLUGIN_IMPORT PSI_metric_bootstrap *psi_metric_hook;
 extern void set_psi_mutex_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_rwlock_bootstrap *psi_rwlock_hook;
 extern void set_psi_rwlock_service(void *psi);
