@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -33,13 +32,13 @@
 #include <sys/types.h>
 #include <algorithm>
 
-#include "m_ctype.h"
 #include "my_base.h"
 #include "my_byteorder.h"
 #include "my_compare.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "myisampack.h"
+#include "mysql/strings/m_ctype.h"
 
 #define CMP_NUM(a, b) (((a) < (b)) ? -1 : ((a) == (b)) ? 0 : 1)
 
@@ -55,7 +54,7 @@ int ha_compare_text(const CHARSET_INFO *charset_info, const uchar *a,
 
 static int compare_bin(const uchar *a, uint a_length, const uchar *b,
                        uint b_length, bool part_key, bool skip_end_space) {
-  uint length = std::min(a_length, b_length);
+  const uint length = std::min(a_length, b_length);
   const uchar *end = a + length;
   int flag;
 
@@ -142,7 +141,7 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
   *diff_pos = 0;
   for (; (int)key_length > 0; key_length = next_key_length, keyseg++) {
     const uchar *end;
-    uint piks = !(keyseg->flag & HA_NO_SORT);
+    const uint piks = !(keyseg->flag & HA_NO_SORT);
     (*diff_pos)++;
     diff_pos[1] = (uint)(b - orig_b);
 
@@ -177,8 +176,8 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
       case HA_KEYTYPE_TEXT: /* Ascii; Key is converted */
         if (keyseg->flag & HA_SPACE_PACK) {
           uint pack_length;
-          int a_length = get_key_length(&a);
-          int b_length = get_key_pack_length(&b, &pack_length);
+          const int a_length = get_key_length(&a);
+          const int b_length = get_key_pack_length(&b, &pack_length);
           next_key_length = key_length - b_length - pack_length;
 
           if (piks &&
@@ -190,7 +189,8 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
           b += b_length;
           break;
         } else {
-          uint length = (uint)(end - a), a_length = length, b_length = length;
+          const uint length = (uint)(end - a), a_length = length,
+                     b_length = length;
           if (piks &&
               (flag = ha_compare_text(
                    keyseg->charset, a, a_length, b, b_length,
@@ -204,8 +204,8 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
       case HA_KEYTYPE_BIT:
         if (keyseg->flag & HA_SPACE_PACK) {
           uint pack_length;
-          int a_length = get_key_length(&a);
-          int b_length = get_key_pack_length(&b, &pack_length);
+          const int a_length = get_key_length(&a);
+          const int b_length = get_key_pack_length(&b, &pack_length);
           next_key_length = key_length - b_length - pack_length;
 
           if (piks && (flag = compare_bin(a, a_length, b, b_length,
@@ -217,7 +217,7 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
           b += b_length;
           break;
         } else {
-          uint length = keyseg->length;
+          const uint length = keyseg->length;
           if (piks && (flag = compare_bin(a, length, b, length,
                                           (bool)((nextflag & SEARCH_PREFIX) &&
                                                  next_key_length <= 0),
@@ -230,8 +230,8 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
       case HA_KEYTYPE_VARTEXT1:
       case HA_KEYTYPE_VARTEXT2: {
         uint pack_length;
-        int a_length = get_key_length(&a);
-        int b_length = get_key_pack_length(&b, &pack_length);
+        const int a_length = get_key_length(&a);
+        const int b_length = get_key_pack_length(&b, &pack_length);
         next_key_length = key_length - b_length - pack_length;
 
         if (piks &&
@@ -246,8 +246,8 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
       case HA_KEYTYPE_VARBINARY1:
       case HA_KEYTYPE_VARBINARY2: {
         uint pack_length;
-        int a_length = get_key_length(&a);
-        int b_length = get_key_pack_length(&b, &pack_length);
+        const int a_length = get_key_length(&a);
+        const int b_length = get_key_pack_length(&b, &pack_length);
         next_key_length = key_length - b_length - pack_length;
 
         if (piks && (flag = compare_bin(a, a_length, b, b_length,
@@ -259,8 +259,8 @@ int ha_key_cmp(const HA_KEYSEG *keyseg, const uchar *a, const uchar *b,
         b += b_length;
       } break;
       case HA_KEYTYPE_INT8: {
-        int i_1 = static_cast<signed char>(*a);
-        int i_2 = static_cast<signed char>(*b);
+        const int i_1 = static_cast<signed char>(*a);
+        const int i_2 = static_cast<signed char>(*b);
         if (piks && (flag = CMP_NUM(i_1, i_2)))
           return ((keyseg->flag & HA_REVERSE_SORT) ? -flag : flag);
         a = end;
