@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   Without limiting anything contained in the foregoing, this file,
   which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -45,6 +44,7 @@
 #define HAVE_PSI_TRANSACTION_INTERFACE
 #define HAVE_PSI_SOCKET_INTERFACE
 #define HAVE_PSI_MEMORY_INTERFACE
+#define HAVE_PSI_METRICS_INTERFACE
 #define HAVE_PSI_ERROR_INTERFACE
 #define HAVE_PSI_IDLE_INTERFACE
 #define HAVE_PSI_METADATA_INTERFACE
@@ -73,6 +73,7 @@
 #include "mysql/psi/psi_idle.h"
 #include "mysql/psi/psi_mdl.h"
 #include "mysql/psi/psi_memory.h"
+#include "mysql/psi/psi_metric.h"
 #include "mysql/psi/psi_mutex.h"
 #include "mysql/psi/psi_rwlock.h"
 #include "mysql/psi/psi_socket.h"
@@ -572,8 +573,6 @@ static void set_metadata_lock_status_noop(PSI_metadata_lock *,
 static void set_metadata_lock_duration_noop(PSI_metadata_lock *,
                                             opaque_mdl_duration) {}
 
-static void set_metadata_lock_type_noop(PSI_metadata_lock *, opaque_mdl_type) {}
-
 static void destroy_metadata_lock_noop(PSI_metadata_lock *) {}
 
 static PSI_metadata_locker *start_metadata_wait_noop(
@@ -585,9 +584,8 @@ static void end_metadata_wait_noop(PSI_metadata_locker *, int) {}
 
 static PSI_mdl_service_t psi_mdl_noop = {
     create_metadata_lock_noop,       set_metadata_lock_status_noop,
-    set_metadata_lock_duration_noop, set_metadata_lock_type_noop,
-    destroy_metadata_lock_noop,      start_metadata_wait_noop,
-    end_metadata_wait_noop};
+    set_metadata_lock_duration_noop, destroy_metadata_lock_noop,
+    start_metadata_wait_noop,        end_metadata_wait_noop};
 
 struct PSI_mdl_bootstrap *psi_mdl_hook = nullptr;
 PSI_mdl_service_t *psi_mdl_service = &psi_mdl_noop;
@@ -1021,3 +1019,29 @@ void set_psi_tls_channel_service(void *psi) {
 }
 
 // ===========================================================================
+
+static void register_meters_noop(PSI_meter_info_v1 *, size_t) {}
+
+static void unregister_meters_noop(PSI_meter_info_v1 *, size_t) {}
+
+static void register_change_notification_noop(meter_registration_changes_v1_t) {
+}
+
+static void unregister_change_notification_noop(
+    meter_registration_changes_v1_t) {}
+
+static void send_change_notification_noop(const char *, MeterNotifyType) {}
+
+static PSI_metric_service_t psi_metric_noop = {
+    register_meters_noop, unregister_meters_noop,
+    register_change_notification_noop, unregister_change_notification_noop,
+    send_change_notification_noop};
+
+struct PSI_metric_bootstrap *psi_metric_hook = nullptr;
+PSI_metric_service_t *psi_metric_service = &psi_metric_noop;
+
+void set_psi_metric_service(void *psi) {
+  psi_metric_service = (PSI_metric_service_t *)psi;
+}
+
+// ===========================================================================
