@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -605,8 +604,8 @@ void Client::pfs_end_state(uint32_t err_num, const char *err_mesg) {
   mysql_mutex_lock(&s_table_mutex);
   assert(s_num_clones == 1);
 
-  bool provisioning = (get_data_dir() == nullptr);
-  bool failed = (err_num != 0);
+  const bool provisioning = (get_data_dir() == nullptr);
+  const bool failed = (err_num != 0);
 
   /* In case provisioning is successful, clone operation is still
   in progress and will continue after restart. */
@@ -661,7 +660,7 @@ void Client::uninit_pfs() {
 
 uint32_t Client::limit_buffer(uint32_t buffer_size) {
   /* Limit total buffer size to 128 M */
-  uint32_t max_buffer_size = 128 * 1024 * 1024;
+  const uint32_t max_buffer_size = 128 * 1024 * 1024;
   auto num_tasks = get_max_concurrency();
 
   auto limit = max_buffer_size / num_tasks;
@@ -679,7 +678,7 @@ uint32_t Client::limit_workers(uint32_t num_workers) {
     /* Zero is also valid result for the limit. Workers are over and above
     the master task. So, anything less than 64M would mean no workers to
     spawn immediately. */
-    uint32_t limit = clone_max_network_bandwidth / 64;
+    const uint32_t limit = clone_max_network_bandwidth / 64;
     if (num_workers > limit) {
       num_workers = limit;
     }
@@ -691,7 +690,7 @@ uint32_t Client::limit_workers(uint32_t num_workers) {
     /* Zero is also valid result for the limit. Workers are over and above
     the master task. So, anything less than 64M would mean no workers to
     spawn immediately. */
-    uint32_t limit = clone_max_io_bandwidth / 64;
+    const uint32_t limit = clone_max_io_bandwidth / 64;
     if (num_workers > limit) {
       num_workers = limit;
     }
@@ -830,7 +829,7 @@ int Client::clone() {
     }
 
     /* If clone is successful, clear any error happened during exit. */
-    bool clear_err = (err == 0);
+    const bool clear_err = (err == 0);
     mysql_service_clone_protocol->mysql_clone_disconnect(get_thd(), m_conn,
                                                          abort_net, clear_err);
 
@@ -892,13 +891,6 @@ int Client::connect_remote(bool is_restart, bool use_aux) {
   mysql_clone_ssl_context ssl_context;
 
   ssl_context.m_enable_compression = clone_enable_compression;
-
-  if (ssl_context.m_enable_compression) {
-    ssl_context.m_compression_algorithm =
-        clone_compression_lib_names[clone_compression_algorithm];
-    ssl_context.m_compression_level = clone_zstd_compression_level;
-  }
-
   ssl_context.m_server_extn =
       ssl_context.m_enable_compression ? &m_conn_server_extn : nullptr;
   ssl_context.m_ssl_mode = m_share->m_ssl_mode;
@@ -1154,7 +1146,7 @@ int Client::extract_string(const uchar *&packet, size_t &length,
     }
   }
   /* purecov: begin deadcode */
-  int err = ER_CLONE_PROTOCOL;
+  const int err = ER_CLONE_PROTOCOL;
   my_error(err, MYF(0), "Wrong Clone RPC response length for parameters");
   return (err);
   /* purecov: end */
@@ -1432,7 +1424,7 @@ int Client::receive_response(Command_RPC com, bool use_aux) {
   auto &info = get_thread_info();
 
   /* Skip setting returned locators for restart */
-  bool skip_apply = (com == COM_REINIT);
+  const bool skip_apply = (com == COM_REINIT);
 
   /* For graceful exit we wait for remote to send
   the end of command message */
@@ -1772,7 +1764,7 @@ int Client::set_error(const uchar *buffer, size_t length) {
   buffer += 4;
   length -= 4;
 
-  int err = ER_CLONE_DONOR;
+  const int err = ER_CLONE_DONOR;
 
   if (is_master()) {
     char err_buf[MYSYS_ERRMSG_SIZE];
