@@ -69,10 +69,6 @@ uint clone_max_io_bandwidth;
 /** Clone system variable: If network compression is enabled */
 bool clone_enable_compression;
 
-ulong clone_compression_algorithm;
-
-uint clone_zstd_compression_level;
-
 /** Clone system variable: valid list of donor addresses. */
 static char *clone_valid_donor_list;
 
@@ -223,7 +219,7 @@ static bool scan_donor_list(const std::string &donor_list,
         entry_len = comma_pos - begin_pos;
       }
 
-      std::string entry = donor_list.substr(begin_pos, entry_len);
+      const std::string entry = donor_list.substr(begin_pos, entry_len);
       auto colon_pos = entry.find(":");
 
       /* Bad entry if no separator is found or found in beginning. */
@@ -333,7 +329,7 @@ static int check_donor_addr_format(MYSQL_THD thd, SYS_VAR *var [[maybe_unused]],
     /* purecov: end */
   }
 
-  std::string addrs(addrs_cstring);
+  const std::string addrs(addrs_cstring);
 
   Donor_Callback callback = [](std::string, uint32_t) { return (false); };
 
@@ -650,28 +646,6 @@ static MYSQL_SYSVAR_BOOL(enable_compression, clone_enable_compression,
                          "If compression is done at network", nullptr, nullptr,
                          false); /* Disable compression by default */
 
-enum enum_clone_compression_algorithm { ZLIB = 0, ZSTD };
-const char *clone_compression_lib_names[] = {COMPRESSION_ALGORITHM_ZLIB,
-                                             COMPRESSION_ALGORITHM_ZSTD, NullS};
-
-static TYPELIB clone_compression_algorithms_typelib = {
-    array_elements(clone_compression_lib_names) - 1,
-    "clone_compression_algorithms_typelib", clone_compression_lib_names,
-    nullptr};
-
-static MYSQL_SYSVAR_ENUM(compression_algorithm, clone_compression_algorithm,
-                         PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_MEMALLOC,
-                         "compression algorithm used in clone", nullptr,
-                         nullptr, enum_clone_compression_algorithm::ZSTD,
-                         &clone_compression_algorithms_typelib);
-
-static MYSQL_SYSVAR_UINT(zstd_compression_level, clone_zstd_compression_level,
-                         PLUGIN_VAR_NOCMDARG, "zstd compression level", nullptr,
-                         nullptr, 3, /* Default */
-                         1,          /* Minimum */
-                         10,         /* Maximum */
-                         1);
-
 /** List of valid donor addresses allowed to clone from. */
 static MYSQL_SYSVAR_STR(valid_donor_list, clone_valid_donor_list,
                         PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_MEMALLOC,
@@ -752,8 +726,6 @@ static SYS_VAR *clone_system_variables[] = {
     MYSQL_SYSVAR(donor_timeout_after_network_failure),
     MYSQL_SYSVAR(delay_after_data_drop),
     MYSQL_SYSVAR(exclude_plugins_list),
-    MYSQL_SYSVAR(compression_algorithm),
-    MYSQL_SYSVAR(zstd_compression_level),
     nullptr};
 
 /** Declare clone plugin */
