@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -93,8 +92,11 @@ class Pipeline_stats_member_message : public Plugin_gcs_message {
     // Length of the payload item: 1 byte
     PIT_TRANSACTION_GTIDS_PRESENT = 12,
 
+    // Length of the payload item: 8 bytes
+    PIT_SENT_TIMESTAMP = 13,
+
     // No valid type codes can appear after this one.
-    PIT_MAX = 13
+    PIT_MAX = 14
   };
 
   /**
@@ -235,6 +237,18 @@ class Pipeline_stats_member_message : public Plugin_gcs_message {
   */
   Flow_control_mode get_flow_control_mode();
 
+  /**
+    Return the time at which the message contained in the buffer was sent.
+    @see Metrics_handler::get_current_time()
+
+    @param[in] buffer            the buffer to decode from.
+    @param[in] length            the buffer length
+
+    @return the time on which the message was sent.
+  */
+  static uint64_t get_sent_timestamp(const unsigned char *buffer,
+                                     size_t length);
+
  protected:
   /**
     Encodes the message contents for transmission.
