@@ -32,7 +32,6 @@
 #include <string>
 #include <string_view>
 #include <thread>
-#include <type_traits>
 #include <vector>
 
 #include "hexify.h"
@@ -152,16 +151,17 @@ using SocketTimestampOld =
 
 #ifdef SO_TIMESTAMP
 // linux + freebsd
-using SocketTimestamp =
-    SocketTimestampBase<SOL_SOCKET, SO_TIMESTAMP,
+using SocketTimestamp = SocketTimestampBase<SOL_SOCKET, SO_TIMESTAMP,
 #ifdef SO_TIMESTAMP_NEW
-                        std::conditional_t<SO_TIMESTAMP == SO_TIMESTAMP_NEW,
-                                           SocketTimestampNew::value_type,
-                                           SocketTimestampOld::value_type>
+#if SO_TIMESTAMP == SO_TIMESTAMP_NEW
+                                            SocketTimestampNew::value_type
 #else
-                        timeval
+                                            SocketTimestampOld::value_type
 #endif
-                        >;
+#else
+                                            timeval
+#endif
+                                            >;
 #endif
 
 template <int Lvl, int Type, class V>
@@ -205,16 +205,18 @@ using SocketTimestampNanosecondOld =
 #endif
 
 #ifdef SO_TIMESTAMPNS
-using SocketTimestampNanosecond = SocketTimestampNanosecondBase<
-    SOL_SOCKET, SO_TIMESTAMPNS,
+using SocketTimestampNanosecond =
+    SocketTimestampNanosecondBase<SOL_SOCKET, SO_TIMESTAMPNS,
 #ifdef SO_TIMESTAMPNS_NEW
-    std::conditional_t<SO_TIMESTAMPNS == SO_TIMESTAMPNS_NEW,
-                       SocketTimestampNanosecondNew::value_type,
-                       SocketTimestampNanosecondOld::value_type>
+#if SO_TIMESTAMPNS == SO_TIMESTAMPNS_NEW
+                                  SocketTimestampNanosecondNew::value_type
 #else
-    timespec
+                                  SocketTimestampNanosecondOld::value_type
 #endif
-    >;
+#else
+                                  timespec
+#endif
+                                  >;
 #endif
 
 template <int Lvl, int Type, class V>
@@ -290,17 +292,20 @@ using SocketTimestampingOld =
 #endif
 
 #ifdef SO_TIMESTAMPING
-using SocketTimestamping = SocketTimestampingBase<
-    SOL_SOCKET, SO_TIMESTAMPING,
+using SocketTimestamping =
+    SocketTimestampingBase<SOL_SOCKET, SO_TIMESTAMPING,
 #ifdef SO_TIMESTAMPING_NEW
-    std::conditional_t<SO_TIMESTAMPING == SO_TIMESTAMPING_NEW,
-                       SocketTimestampingNew::value_type,
-                       SocketTimestampingOld::value_type>
+#if SO_TIMESTAMPING == SO_TIMESTAMPING_NEW
+                           SocketTimestampingNew::value_type
 #else
-    std::conditional_t<is_type_complete_v<struct scm_timestamping>,
-                       scm_timestamping, fallback::scm_timestamping>
+                           SocketTimestampingOld::value_type
 #endif
-    >;
+#else
+                           std::conditional_t<
+                               is_type_complete_v<struct scm_timestamping>,
+                               scm_timestamping, fallback::scm_timestamping>
+#endif
+                           >;
 #endif
 
 // if they are defined, check they have the expected value.
