@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2022, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2022, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -92,6 +91,10 @@ class SharedServer {
 
   void setup_mysqld_accounts();
 
+  void install_plugins();
+
+  static void install_plugins(MysqlClient &cli);
+
   void flush_privileges();
 
   void flush_privileges(MysqlClient &cli);
@@ -99,12 +102,41 @@ class SharedServer {
   // get all connections, but ignore internal connections and this
   // connection.
   static stdx::expected<std::vector<uint64_t>, MysqlError> user_connection_ids(
-      MysqlClient &cli);
+      MysqlClient &cli) {
+    return user_connection_ids(cli, default_usernames());
+  }
+
+  static stdx::expected<std::vector<uint64_t>, MysqlError> user_connection_ids(
+      MysqlClient &cli, const std::vector<std::string> &usernames);
+
+  static std::vector<std::string> default_usernames() {
+    return {
+        admin_account().username,
+        caching_sha2_empty_password_account().username,
+        caching_sha2_password_account().username,
+        native_empty_password_account().username,
+        native_password_account().username,
+        sha256_empty_password_account().username,
+        sha256_password_account().username,
+        sha256_short_password_account().username,
+    };
+  }
 
   // close all connections.
-  void close_all_connections();
+  stdx::expected<void, MysqlError> close_all_connections() {
+    return close_all_connections(default_usernames());
+  }
 
-  void close_all_connections(MysqlClient &cli);
+  stdx::expected<void, MysqlError> close_all_connections(
+      const std::vector<std::string> &usernames);
+
+  static stdx::expected<void, MysqlError> close_all_connections(
+      MysqlClient &cli) {
+    return close_all_connections(cli, default_usernames());
+  }
+
+  static stdx::expected<void, MysqlError> close_all_connections(
+      MysqlClient &cli, const std::vector<std::string> &usernames);
 
   // set some session-vars back to defaults.
   void reset_to_defaults();
