@@ -1,16 +1,15 @@
--- Copyright (c) 2003, 2025, Oracle and/or its affiliates.
+-- Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License, version 2.0,
 -- as published by the Free Software Foundation.
 --
--- This program is designed to work with certain software (including
+-- This program is also distributed with certain software (including
 -- but not limited to OpenSSL) that is licensed under separate terms,
 -- as designated in a particular file or component or in included license
 -- documentation.  The authors of MySQL hereby grant you an additional
 -- permission to link the program and your derivative works with the
--- separately licensed software that they have either included with
--- the program or referenced in the documentation.
+-- separately licensed software that they have included with MySQL.
 --
 -- This program is distributed in the hope that it will be useful,
 -- but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -757,10 +756,14 @@ INSERT INTO global_grants SELECT user, host, 'SESSION_VARIABLES_ADMIN', IF(grant
 FROM mysql.user WHERE super_priv = 'Y' AND @hadSessionVariablesAdminPriv = 0;
 COMMIT;
 
--- Add the privilege SET_USER_ID for every user who has the privilege SUPER
--- provided that there isn't a user who already has the privilige SET_USER_ID.
-SET @hadSetUserIdPriv = (SELECT COUNT(*) FROM global_grants WHERE priv = 'SET_USER_ID');
-INSERT INTO global_grants SELECT user, host, 'SET_USER_ID', IF(grant_priv = 'Y', 'Y', 'N')
+-- Add the privileges SET_ANY_DEFINER and ALLOW_NONEXISTENT_DEFINER for every user who has the privilege SUPER
+-- provided that there isn't a user who already has and of the privileges SET_USER_ID, SET_ANY_DEFINER
+--   or ALLOW_NONEXISTENT_DEFINER.
+SET @hadSetUserIdPriv = (SELECT COUNT(*) FROM global_grants
+  WHERE priv IN ('SET_USER_ID','SET_ANY_DEFINER','ALLOW_NONEXISTENT_DEFINER'));
+INSERT INTO global_grants SELECT user, host, 'SET_ANY_DEFINER', IF(grant_priv = 'Y', 'Y', 'N')
+FROM mysql.user WHERE super_priv = 'Y' AND @hadSetUserIdPriv = 0;
+INSERT INTO global_grants SELECT user, host, 'ALLOW_NONEXISTENT_DEFINER', IF(grant_priv = 'Y', 'Y', 'N')
 FROM mysql.user WHERE super_priv = 'Y' AND @hadSetUserIdPriv = 0;
 COMMIT;
 
@@ -1708,4 +1711,16 @@ ALTER TABLE mysql.columns_priv DROP PRIMARY KEY,
 ALTER TABLE mysql.procs_priv DROP PRIMARY KEY,
                              ADD PRIMARY KEY (`Host`,`User`,`Db`,`Routine_name`,`Routine_type`);
 
+-- Add the privilege SET_ANY_DEFINER for every user who has privilege SET_USER_ID privilege
+SET @hadSetAnyDefinerPriv = (SELECT COUNT(*) FROM global_grants WHERE priv = 'SET_ANY_DEFINER');
+INSERT INTO global_grants SELECT user, host, 'SET_ANY_DEFINER',
+IF (WITH_GRANT_OPTION = 'Y', 'Y', 'N') FROM global_grants WHERE priv = 'SET_USER_ID' AND @hadSetAnyDefinerPriv = 0;
+COMMIT;
+
+-- Add the privilege ALLOW_NONEXISTENT_DEFINER for every user who has privilege SET_USER_ID privilege
+SET @hadAllowNonexistentDefinerPriv = (SELECT COUNT(*) FROM global_grants WHERE priv = 'ALLOW_NONEXISTENT_DEFINER');
+INSERT INTO global_grants SELECT user, host, 'ALLOW_NONEXISTENT_DEFINER',
+IF (WITH_GRANT_OPTION = 'Y', 'Y', 'N') FROM global_grants WHERE priv = 'SET_USER_ID' AND @hadAllowNonexistentDefinerPriv = 0;
+COMMIT;
+
 SET @@session.sql_mode = @old_sql_mode;
