@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -81,6 +80,7 @@ using std::min;
 */
 
 #ifdef MYSQL_SERVER
+
 /*
   The following variables/functions should really not be declared
   extern, but as it's hard to include sql_class.h here, we have to
@@ -91,13 +91,6 @@ extern void thd_increment_bytes_received(size_t length);
 
 /* Additional instrumentation hooks for the server */
 #include "mysql_com_server.h"
-
-/* Refresh cached values in the THD for the server. */
-static void server_store_cached_values(const NET *net) {
-  // Refresh only if this net is the net of the current_thd
-  if (current_thd != nullptr && current_thd->get_net() == net)
-    current_thd->store_cached_properties(THD::cached_properties::RW_STATUS);
-}
 #endif
 
 static bool net_write_buff(NET *, const uchar *, size_t);
@@ -173,7 +166,6 @@ bool my_net_init(NET *net, Vio *vio) {
   net->last_errno = 0;
 #ifdef MYSQL_SERVER
   net->extension = nullptr;
-  server_store_cached_values(net);
 #else
   NET_EXTENSION *ext = net_extension_init();
   ext->net_async_context->cur_pos = net->buff + net->where_b;
@@ -250,7 +242,7 @@ bool net_realloc(NET *net, size_t length) {
 #ifdef MYSQL_SERVER
   net->buff = net->write_pos = buff;
 #else
-  size_t cur_pos_offset = NET_ASYNC_DATA(net)->cur_pos - net->buff;
+  const size_t cur_pos_offset = NET_ASYNC_DATA(net)->cur_pos - net->buff;
   net->buff = net->write_pos = buff;
   NET_ASYNC_DATA(net)->cur_pos = net->buff + cur_pos_offset;
 #endif
@@ -545,11 +537,11 @@ static int begin_packet_write_state(NET *net, uchar command,
   }
   NET_ASYNC *net_async = NET_ASYNC_DATA(net);
   size_t total_len = packet_len + prefix_len;
-  bool include_command = (command < COM_END);
+  const bool include_command = (command < COM_END);
   if (include_command) {
     ++total_len;
   }
-  size_t packet_count = 1 + total_len / MAX_PACKET_LENGTH;
+  const size_t packet_count = 1 + total_len / MAX_PACKET_LENGTH;
   reset_packet_write_state(net);
 
   struct io_vec *vec;
@@ -670,7 +662,7 @@ static int begin_packet_write_state(NET *net, uchar command,
       Final iovec, the payload itself. Send however many bytes from
       packet we have left, and advance our packet pointer.
     */
-    size_t remaining_bytes = packet_size - bytes_queued;
+    const size_t remaining_bytes = packet_size - bytes_queued;
     (*vec).iov_base = const_cast<uchar *>(packet);
     (*vec).iov_len = remaining_bytes;
 
@@ -766,7 +758,7 @@ static net_async_status net_write_vector_nonblocking(NET *net, ssize_t *res) {
       }
       return NET_ASYNC_COMPLETE;
     }
-    size_t bytes_written = static_cast<size_t>(*res);
+    const size_t bytes_written = static_cast<size_t>(*res);
     vec->iov_len -= bytes_written;
     vec->iov_base = (char *)vec->iov_base + bytes_written;
 
@@ -910,9 +902,9 @@ bool net_write_command(NET *net, uchar command, const uchar *header,
   }
   int3store(buff, static_cast<uint>(length));
   buff[3] = (uchar)net->pkt_nr++;
-  bool rc = net_write_buff(net, buff, header_size) ||
-            (head_len && net_write_buff(net, header, head_len)) ||
-            net_write_buff(net, packet, len) || net_flush(net);
+  const bool rc = net_write_buff(net, buff, header_size) ||
+                  (head_len && net_write_buff(net, header, head_len)) ||
+                  net_write_buff(net, packet, len) || net_flush(net);
   return rc;
 }
 
@@ -998,7 +990,7 @@ static bool net_write_raw_loop(NET *net, const uchar *buf, size_t count) {
   unsigned int retry_count = 0;
 
   while (count) {
-    size_t sentcnt = vio_write(net->vio, buf, count);
+    const size_t sentcnt = vio_write(net->vio, buf, count);
 
     /* VIO_SOCKET_ERROR (-1) indicates an error. */
     if (sentcnt == VIO_SOCKET_ERROR) {
@@ -1303,9 +1295,6 @@ bool net_write_packet(NET *net, const uchar *packet, size_t length) {
     return true;
 
   net->reading_or_writing = 2;
-#ifdef MYSQL_SERVER
-  server_store_cached_values(net);
-#endif
 
   const bool do_compress = net->compress;
   if (do_compress) {
@@ -1314,9 +1303,6 @@ bool net_write_packet(NET *net, const uchar *packet, size_t length) {
       net->last_errno = ER_OUT_OF_RESOURCES;
       /* In the server, allocation failure raises a error. */
       net->reading_or_writing = 0;
-#ifdef MYSQL_SERVER
-      server_store_cached_values(net);
-#endif
       return true;
     }
   }
@@ -1330,9 +1316,6 @@ bool net_write_packet(NET *net, const uchar *packet, size_t length) {
   if (do_compress) my_free(const_cast<uchar *>(packet));
 
   net->reading_or_writing = 0;
-#ifdef MYSQL_SERVER
-  server_store_cached_values(net);
-#endif
 
   /* Socket can't be used any more */
   if (net->error == NET_ERROR_SOCKET_NOT_READABLE) {
@@ -1371,7 +1354,7 @@ static bool net_read_raw_loop(NET *net, size_t count) {
   time_t start_time = 0;
   if (timeout_on_full_packet) start_time = time(&start_time);
   while (count) {
-    size_t recvcnt = vio_read(net->vio, buf, count);
+    const size_t recvcnt = vio_read(net->vio, buf, count);
 
     /* VIO_SOCKET_ERROR (-1) indicates an error. */
     if (recvcnt == VIO_SOCKET_ERROR) {
@@ -1708,9 +1691,6 @@ static net_async_status net_read_packet_nonblocking(NET *net, ulong *ret) {
     case NET_ASYNC_PACKET_READ_IDLE:
       net_async->async_packet_read_state = NET_ASYNC_PACKET_READ_HEADER;
       net->reading_or_writing = 0;
-#ifdef MYSQL_SERVER
-      server_store_cached_values(net);
-#endif
       [[fallthrough]];
     case NET_ASYNC_PACKET_READ_HEADER:
       /*
@@ -1787,9 +1767,6 @@ end:
 
   net->read_pos[*ret] = 0;
   net->reading_or_writing = 0;
-#ifdef MYSQL_SERVER
-  server_store_cached_values(net);
-#endif
   if (net->compress) {
     mysql_compress_context *mysql_compress_ctx = compress_context(net);
     if (my_uncompress(mysql_compress_ctx, net->buff + net->where_b,
@@ -1810,9 +1787,6 @@ end:
 error:
   *ret = packet_error;
   net->reading_or_writing = 0;
-#ifdef MYSQL_SERVER
-  server_store_cached_values(net);
-#endif
   return NET_ASYNC_COMPLETE;
 }
 
@@ -1869,9 +1843,9 @@ begin:
                       "net->where_b : %lu, multi_byte_packet : %u ",
                       buf_length, net->remain_in_buf, first_packet_offset,
                       start_of_packet, net->where_b, multi_byte_packet));
-  size_t remain_in_buf = buf_length - start_of_packet;
+  const size_t remain_in_buf = buf_length - start_of_packet;
   if (remain_in_buf >= NET_HEADER_SIZE) {
-    size_t read_length = uint3korr(net->buff + start_of_packet);
+    const size_t read_length = uint3korr(net->buff + start_of_packet);
     DBUG_PRINT("info", ("read_length : %zu", read_length));
     if (!read_length) {
       start_of_packet += NET_HEADER_SIZE; /* End of multi-byte packet */
@@ -1948,8 +1922,8 @@ static ulong net_read_update_offsets(NET *net, size_t start_of_packet,
   net->read_pos = net->buff + first_packet_offset + NET_HEADER_SIZE;
   net->buf_length = buf_length;
   net->remain_in_buf = (ulong)(buf_length - start_of_packet);
-  ulong len = ((ulong)(start_of_packet - first_packet_offset) -
-               NET_HEADER_SIZE - multi_byte_packet);
+  const ulong len = ((ulong)(start_of_packet - first_packet_offset) -
+                     NET_HEADER_SIZE - multi_byte_packet);
   if (net->remain_in_buf) {
     /*
       If multi byte packet is non-zero then there is a zero length
@@ -1997,50 +1971,51 @@ static ulong net_read_update_offsets(NET *net, size_t start_of_packet,
 static net_async_status net_read_compressed_nonblocking(NET *net,
                                                         ulong *len_ptr) {
   DBUG_TRACE;
-  NET_ASYNC *net_async = NET_ASYNC_DATA(net);
   assert(net->compress);
   ulong &len = *len_ptr;
-  if (net_async->mp_state.mp_status != NET_ASYNC_NOT_READY)
-    net_read_init_offsets(net, net_async->mp_state.mp_start_of_packet,
-                          net_async->mp_state.mp_first_packet_offset,
-                          net_async->mp_state.mp_multi_byte_packet,
-                          net_async->mp_state.mp_buf_length);
+
+  /* Maintain the local states to read the multipacket asynchronously */
+  static size_t start_of_packet;
+  static size_t first_packet_offset;
+  static size_t buf_length;
+  static uint multi_byte_packet = 0;
+  static net_async_status status = NET_ASYNC_COMPLETE;
+
+  if (status != NET_ASYNC_NOT_READY)
+    net_read_init_offsets(net, start_of_packet, first_packet_offset,
+                          multi_byte_packet, buf_length);
 
   for (;;) {
     /*  Read the current packet in net->buff */
-    if (net_read_process_buffer(net, net_async->mp_state.mp_start_of_packet,
-                                net_async->mp_state.mp_buf_length,
-                                net_async->mp_state.mp_multi_byte_packet,
-                                net_async->mp_state.mp_first_packet_offset))
+    if (net_read_process_buffer(net, start_of_packet, buf_length,
+                                multi_byte_packet, first_packet_offset))
       break;
 
     /*
       Read the mysql packet from vio, uncompress it and make it accessible
       through net->buff.
     */
-    net_async->mp_state.mp_status = net_read_packet_nonblocking(net, &len);
-    if (net_async->mp_state.mp_status == NET_ASYNC_NOT_READY) {
-      net->save_char = net->buff[net_async->mp_state.mp_first_packet_offset];
-      net->buf_length = net_async->mp_state.mp_buf_length;
-      return net_async->mp_state.mp_status;
+    status = net_read_packet_nonblocking(net, &len);
+    if (status == NET_ASYNC_NOT_READY) {
+      net->save_char = net->buff[first_packet_offset];
+      net->buf_length = buf_length;
+      return status;
     }
 
     if (len == packet_error) {
-      net_async->mp_state.mp_status = NET_ASYNC_COMPLETE;
-      return net_async->mp_state.mp_status;
+      status = NET_ASYNC_COMPLETE;
+      return status;
     }
-    net_async->mp_state.mp_buf_length += len;
+    buf_length += len;
   }
   /*
     Once the packets are read in the net->buff, adjust the tracking offsets to
     the appropriate values.
   */
-  len = net_read_update_offsets(net, net_async->mp_state.mp_start_of_packet,
-                                net_async->mp_state.mp_first_packet_offset,
-                                net_async->mp_state.mp_buf_length,
-                                net_async->mp_state.mp_multi_byte_packet);
-  net_async->mp_state.mp_status = NET_ASYNC_COMPLETE;
-  return net_async->mp_state.mp_status;
+  len = net_read_update_offsets(net, start_of_packet, first_packet_offset,
+                                buf_length, multi_byte_packet);
+  status = NET_ASYNC_COMPLETE;
+  return status;
 }
 
 /**
@@ -2054,31 +2029,33 @@ static net_async_status net_read_compressed_nonblocking(NET *net,
 static net_async_status net_read_uncompressed_nonblocking(NET *net,
                                                           ulong *len_ptr) {
   DBUG_TRACE;
-  NET_ASYNC *net_async = NET_ASYNC_DATA(net);
   assert(!net->compress);
   ulong &len = *len_ptr;
 
+  // Maintain the local states
+  static net_async_status status = NET_ASYNC_COMPLETE;
+  static ulong save_pos;
+  static ulong total_length;
+
   // Initialize the states
-  if (net_async->mp_state.mp_status == NET_ASYNC_COMPLETE) {
-    net_async->mp_state.mp_save_pos = net->where_b;
-    net_async->mp_state.mp_total_length = 0;
+  if (status == NET_ASYNC_COMPLETE) {
+    save_pos = net->where_b;
+    total_length = 0;
   }
 
-  net_async->mp_state.mp_status = net_read_packet_nonblocking(net, &len);
-  net_async->mp_state.mp_total_length += len;
+  status = net_read_packet_nonblocking(net, &len);
+  total_length += len;
   net->where_b += len;
 
-  if (len == MAX_PACKET_LENGTH)
-    net_async->mp_state.mp_status = NET_ASYNC_NOT_READY;
-  if (net_async->mp_state.mp_status == NET_ASYNC_NOT_READY)
-    return net_async->mp_state.mp_status;
+  if (len == MAX_PACKET_LENGTH) status = NET_ASYNC_NOT_READY;
+  if (status == NET_ASYNC_NOT_READY) return status;
 
   // Update the offsets
-  net->where_b = net_async->mp_state.mp_save_pos;
-  len = net_async->mp_state.mp_total_length;
+  net->where_b = save_pos;
+  len = total_length;
   net->read_pos = net->buff + net->where_b;
-  net_async->mp_state.mp_status = NET_ASYNC_COMPLETE;
-  return net_async->mp_state.mp_status;
+  status = NET_ASYNC_COMPLETE;
+  return status;
 }
 
 /**
@@ -2098,9 +2075,6 @@ static size_t net_read_packet(NET *net, size_t *complen) {
   *complen = 0;
 
   net->reading_or_writing = 1;
-#ifdef MYSQL_SERVER
-  server_store_cached_values(net);
-#endif
 
   /*
     We should reset compress_packet_nr even before reading the header because
@@ -2158,18 +2132,12 @@ end:
     net->error = NET_ERROR_SOCKET_UNUSABLE;
   DBUG_DUMP("net read", net->buff + net->where_b, pkt_len);
   net->reading_or_writing = 0;
-#ifdef MYSQL_SERVER
-  server_store_cached_values(net);
-#endif
   return pkt_len;
 
 error:
   if (net->error == NET_ERROR_SOCKET_NOT_WRITABLE)
     net->error = NET_ERROR_SOCKET_UNUSABLE;
   net->reading_or_writing = 0;
-#ifdef MYSQL_SERVER
-  server_store_cached_values(net);
-#endif
   return packet_error;
 }
 
@@ -2234,7 +2202,7 @@ static void net_read_uncompressed_packet(NET *net, size_t &len) {
   len = net_read_packet(net, &complen);
   if (len == MAX_PACKET_LENGTH) {
     /* First packet of a multi-packet.  Concatenate the packets */
-    ulong save_pos = net->where_b;
+    const ulong save_pos = net->where_b;
     size_t total_length = 0;
     do {
       net->where_b += len;
