@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -55,19 +54,19 @@ Abstract_restrictions::~Abstract_restrictions() = default;
 /**
   DB Restrictions constructor
 */
-DB_restrictions::DB_restrictions() : Abstract_restrictions() {}
+DB_restrictions::DB_restrictions()
+    : Abstract_restrictions(), m_restrictions() {}
 
 /**
   Copy constructor for DB Restrictions
 
   @param [in] other Source DB restrictions
 */
-DB_restrictions::DB_restrictions(const DB_restrictions &other) {
-  copy_restrictions(other);
-}
+DB_restrictions::DB_restrictions(const DB_restrictions &other)
+    : m_restrictions(other.m_restrictions) {}
 
 /** Destructor */
-DB_restrictions::~DB_restrictions() { clear(); }
+DB_restrictions::~DB_restrictions() { m_restrictions.clear(); }
 
 /**
   Assignment operator
@@ -76,8 +75,7 @@ DB_restrictions::~DB_restrictions() { clear(); }
 */
 DB_restrictions &DB_restrictions::operator=(const DB_restrictions &other) {
   if (this != &other) {
-    clear();
-    copy_restrictions(other);
+    m_restrictions = other.m_restrictions;
   }
   return *this;
 }
@@ -89,8 +87,8 @@ DB_restrictions &DB_restrictions::operator=(const DB_restrictions &other) {
 */
 DB_restrictions &DB_restrictions::operator=(DB_restrictions &&restrictions) {
   if (this != &restrictions) {
-    clear();
-    add(restrictions);
+    this->clear();
+    this->add(restrictions);
   }
   return *this;
 }
@@ -104,9 +102,7 @@ DB_restrictions &DB_restrictions::operator=(DB_restrictions &&restrictions) {
 
 */
 bool DB_restrictions::operator==(const DB_restrictions &restrictions) const {
-  if (is_empty() && restrictions.is_empty()) return true;
-  if (is_empty() || restrictions.is_empty()) return false;
-  return (*m_restrictions == *restrictions.m_restrictions);
+  return (m_restrictions == restrictions.m_restrictions);
 }
 
 /**
@@ -116,14 +112,13 @@ bool DB_restrictions::operator==(const DB_restrictions &restrictions) const {
   @param [in] revoke_privs Privileges to be restricted
 */
 void DB_restrictions::add(const std::string &db_name,
-                          const Access_bitmask revoke_privs) {
-  auto restrictions = create_restrictions_if_needed();
-  if (restrictions->find(db_name) != restrictions->end()) {
+                          const ulong revoke_privs) {
+  if (m_restrictions.find(db_name) != m_restrictions.end()) {
     /* Partial revokes already exists on this DB so update them */
-    (*restrictions)[db_name] |= (revoke_privs);
+    m_restrictions[db_name] |= (revoke_privs);
   } else {
     /* Partial revokes do not exist on this DB so add them */
-    restrictions->emplace(db_name, revoke_privs);
+    m_restrictions.emplace(db_name, revoke_privs);
   }
 }
 
@@ -133,10 +128,7 @@ void DB_restrictions::add(const std::string &db_name,
   @param [in] restrictions List of <database, privileges>
 */
 void DB_restrictions::add(const DB_restrictions &restrictions) {
-  if (!restrictions.m_restrictions) {
-    return;
-  }
-  for (auto &itr : *(restrictions.m_restrictions))
+  for (auto &itr : restrictions.m_restrictions)
     add(itr.first.c_str(), itr.second);
 }
 
@@ -171,7 +163,7 @@ bool DB_restrictions::add(const Json_object &json_object) {
         return true;
       const Json_string *db_string = nullptr;
       const Json_array *privs_array = nullptr;
-      Access_bitmask priv_mask;
+      ulong priv_mask;
 
       if (db_dom) {
         db_string = down_cast<const Json_string *>(db_dom);
@@ -201,12 +193,10 @@ bool DB_restrictions::add(const Json_object &json_object) {
   @param [in] revoke_privs List of privileges to remove
 */
 void DB_restrictions::remove(const std::string &db_name,
-                             const Access_bitmask revoke_privs) {
-  if (m_restrictions != nullptr) {
-    auto rest_itr = m_restrictions->find(db_name);
-    if (rest_itr != m_restrictions->end()) {
-      remove(revoke_privs, rest_itr->second);
-    }
+                             const ulong revoke_privs) {
+  auto rest_itr = m_restrictions.find(db_name);
+  if (rest_itr != m_restrictions.end()) {
+    remove(revoke_privs, rest_itr->second);
   }
 }
 
@@ -219,16 +209,14 @@ void DB_restrictions::remove(const std::string &db_name,
 
   @param [in] revoke_privs Privileges to be removed
 */
-void DB_restrictions::remove(const Access_bitmask revoke_privs) {
-  if (m_restrictions != nullptr) {
-    for (auto rest_itr = m_restrictions->begin();
-         rest_itr != m_restrictions->end();) {
-      remove(revoke_privs, rest_itr->second);
-      if (rest_itr->second == 0)
-        rest_itr = m_restrictions->erase(rest_itr);
-      else
-        ++rest_itr;
-    }
+void DB_restrictions::remove(const ulong revoke_privs) {
+  for (auto rest_itr = m_restrictions.begin();
+       rest_itr != m_restrictions.end();) {
+    remove(revoke_privs, rest_itr->second);
+    if (rest_itr->second == 0)
+      rest_itr = m_restrictions.erase(rest_itr);
+    else
+      ++rest_itr;
   }
 }
 
@@ -238,9 +226,9 @@ void DB_restrictions::remove(const Access_bitmask revoke_privs) {
   @param [in]  remove_restrictions Restriction to be removed
   @param [out] restriction_mask    Resultant value to be returned
 */
-void DB_restrictions::remove(const Access_bitmask remove_restrictions,
-                             Access_bitmask &restriction_mask) const noexcept {
-  const Access_bitmask mask = restriction_mask ^ remove_restrictions;
+void DB_restrictions::remove(const ulong remove_restrictions,
+                             ulong &restriction_mask) const noexcept {
+  const ulong mask = restriction_mask ^ remove_restrictions;
   restriction_mask = restriction_mask & mask;
 }
 
@@ -254,37 +242,31 @@ void DB_restrictions::remove(const Access_bitmask remove_restrictions,
     @retval true  Entry found.
     @retval false Entry not found. Do not rely on access.
 */
-bool DB_restrictions::find(const std::string &db_name,
-                           Access_bitmask &access) const {
-  if (!is_empty()) {
-    const auto &itr = m_restrictions->find(db_name);
-    if (itr != m_restrictions->end()) {
-      access = itr->second;
-      return true;
-    }
+bool DB_restrictions::find(const std::string &db_name, ulong &access) const {
+  const auto &itr = m_restrictions.find(db_name);
+  if (itr != m_restrictions.end()) {
+    access = itr->second;
+    return true;
   }
   return false;
 }
 
-/**
-  Status function to check if restriction list is empty.
-  m_restrictions may not be initialized yet, that also indicates that the
-  restrictions are empty.
-*/
-bool DB_restrictions::is_empty() const {
-  return (m_restrictions != nullptr ? m_restrictions->empty() : true);
-}
+/** Status function to check if restriction list is empty */
+bool DB_restrictions::is_empty() const { return m_restrictions.empty(); }
+
+/** Status function to check if restriction list is non-empty */
+bool DB_restrictions::is_not_empty() const { return !is_empty(); }
 
 /** Status function to get number of entries in restriction list */
-size_t DB_restrictions::size() const {
-  if (is_empty()) return 0;
-  return m_restrictions->size();
-}
+size_t DB_restrictions::size() const { return m_restrictions.size(); }
 
 /** Clear restriction list */
 void DB_restrictions::clear() {
-  delete m_restrictions;
-  m_restrictions = nullptr;
+  /*
+    we use swap (with temporary object) trick here to force the container to
+    return the memory
+  */
+  db_revocations().swap(m_restrictions);
 }
 
 /**
@@ -293,13 +275,12 @@ void DB_restrictions::clear() {
   This is used while storing restriction list in ACL table.
 */
 void DB_restrictions::get_as_json(Json_array &restrictions_array) const {
-  assert(m_restrictions != nullptr);
-  for (auto &revocations_itr : *m_restrictions) {
+  for (auto &revocations_itr : m_restrictions) {
     Json_array privileges;
     Json_object revocations_obj;
     Json_string db_name(revocations_itr.first.c_str());
     revocations_obj.add_clone(consts::Database, &db_name);
-    Access_bitmask revokes_mask = revocations_itr.second;
+    ulong revokes_mask = revocations_itr.second;
     while (revokes_mask != 0) {
       Json_string priv_str(get_one_priv(revokes_mask));
       privileges.append_clone(&priv_str);
@@ -320,16 +301,16 @@ void DB_restrictions::get_as_json(Json_array &restrictions_array) const {
     @retval true  Otherwise
 */
 bool DB_restrictions::has_more_restrictions(const DB_restrictions &other,
-                                            Access_bitmask access) const {
+                                            ulong access) const {
   if (other.size() == 0) return false;
   access = access & DB_ACLS;
   db_revocations other_revocations = other.get();
 
   for (const auto &entry : other_revocations) {
     /* We are only interested in privileges represented by access */
-    Access_bitmask other_mask = entry.second & access;
+    ulong other_mask = entry.second & access;
     if (other_mask) {
-      Access_bitmask self_restrictions;
+      ulong self_restrictions;
       /*
         If there exists a restriction that other list has but *this does not,
         it means, other list has at least one more restriction than *this.
@@ -374,8 +355,7 @@ bool DB_restrictions::has_more_restrictions(const DB_restrictions &other,
 */
 std::unique_ptr<Restrictions_aggregator>
 Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
-                                        const char *db,
-                                        const Access_bitmask rights,
+                                        const char *db, const ulong rights,
                                         bool is_grant_revoke_all_on_db) {
   std::unique_ptr<Restrictions_aggregator> aggregator = nullptr;
   /*
@@ -384,27 +364,26 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
   */
   if (mysqld_partial_revokes() == false || acl_user == nullptr)
     return aggregator;
-  enum_sql_command command = thd->lex->sql_command;
+  const enum_sql_command command = thd->lex->sql_command;
   const Security_context *security_context = thd->security_context();
   /* Fetch grantor Auth_id */
   const Auth_id grantor = fetch_grantor(security_context);
   /* Fetch grantee Auth_id */
   const Auth_id grantee = fetch_grantee(acl_user);
   /* Fetch access information of grantor */
-  Access_bitmask grantor_global_access;
+  ulong grantor_global_access;
   Restrictions grantor_restrictions;
   fetch_grantor_access(security_context, db, grantor_global_access,
                        grantor_restrictions);
   /* Fetch access information of grantee */
-  Access_bitmask grantee_global_access;
+  ulong grantee_global_access;
   Restrictions grantee_restrictions;
   fetch_grantee_access(acl_user, grantee_global_access, grantee_restrictions);
   if (db) {
     /* Fetch DB privileges of grantor */
-    Access_bitmask grantor_db_access = fetch_grantor_db_access(thd, db);
+    ulong grantor_db_access = fetch_grantor_db_access(thd, db);
     /* Fetch DB privileges of grantee */
-    Access_bitmask grantee_db_access =
-        fetch_grantee_db_access(thd, acl_user, db);
+    ulong grantee_db_access = fetch_grantee_db_access(thd, acl_user, db);
     switch (command) {
       case SQLCOM_GRANT:
         aggregator.reset(new DB_restrictions_aggregator_db_grant(
@@ -455,11 +434,10 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
 
 std::unique_ptr<Restrictions_aggregator>
 Restrictions_aggregator_factory::create(
-    const Auth_id &grantor, const Auth_id &grantee,
-    const Access_bitmask grantor_access, const Access_bitmask grantee_access,
-    const DB_restrictions &grantor_db_restrictions,
-    const DB_restrictions &grantee_db_restrictions,
-    const Access_bitmask required_access, Db_access_map *db_map) {
+    const Auth_id &grantor, const Auth_id &grantee, const ulong grantor_access,
+    const ulong grantee_access, const DB_restrictions &grantor_db_restrictions,
+    const DB_restrictions &grantee_db_restrictions, const ulong required_access,
+    Db_access_map *db_map) {
   std::unique_ptr<Restrictions_aggregator> aggregator = nullptr;
   /* Create aggregator only if partial_revokes system variable is ON */
   if (mysqld_partial_revokes() == false) return aggregator;
@@ -483,7 +461,7 @@ Auth_id Restrictions_aggregator_factory::fetch_grantor(
   // Fetch the grantor auth_id from security context on master
   grantor_user = sctx->priv_user();
   grantor_host = sctx->priv_host();
-  Auth_id grantor(grantor_user, grantor_host);
+  const Auth_id grantor(grantor_user, grantor_host);
   return grantor;
 }
 
@@ -497,10 +475,11 @@ Auth_id Restrictions_aggregator_factory::fetch_grantor(
 Auth_id Restrictions_aggregator_factory::fetch_grantee(
     const ACL_USER *acl_user) {
   // Fetch the grantee auth_id
-  std::string grantee_user(acl_user->user != nullptr ? acl_user->user : "");
-  std::string grantee_host(
+  const std::string grantee_user(acl_user->user != nullptr ? acl_user->user
+                                                           : "");
+  const std::string grantee_host(
       acl_user->host.get_host() != nullptr ? acl_user->host.get_host() : "");
-  Auth_id grantee(grantee_user, grantee_host);
+  const Auth_id grantee(grantee_user, grantee_host);
   return grantee;
 }
 
@@ -512,8 +491,8 @@ Auth_id Restrictions_aggregator_factory::fetch_grantee(
 
   @returns privilege access to the grantor on the specified database
 */
-Access_bitmask Restrictions_aggregator_factory::fetch_grantor_db_access(
-    THD *thd, const char *db) {
+ulong Restrictions_aggregator_factory::fetch_grantor_db_access(THD *thd,
+                                                               const char *db) {
   LEX_CSTRING db_str;
   db_str.str = db;
   db_str.length = sizeof(db);
@@ -529,7 +508,7 @@ Access_bitmask Restrictions_aggregator_factory::fetch_grantor_db_access(
 
   @returns privilege access to the grantee on the specified database
 */
-Access_bitmask Restrictions_aggregator_factory::fetch_grantee_db_access(
+ulong Restrictions_aggregator_factory::fetch_grantee_db_access(
     THD *thd, const ACL_USER *acl_user, const char *db) {
   return acl_get(thd, acl_user->host.get_host(), acl_user->host.get_host(),
                  (acl_user->user ? acl_user->user : ""), db, false);
@@ -544,7 +523,7 @@ Access_bitmask Restrictions_aggregator_factory::fetch_grantee_db_access(
   @param  [out] restrictions  fetch grantor's restrictions
 */
 void Restrictions_aggregator_factory::fetch_grantor_access(
-    const Security_context *sctx, const char *db, Access_bitmask &global_access,
+    const Security_context *sctx, const char *db, ulong &global_access,
     Restrictions &restrictions) {
   /* Fetch global privileges of current user */
   global_access = sctx->master_access(db ? db : "");
@@ -553,8 +532,7 @@ void Restrictions_aggregator_factory::fetch_grantor_access(
 }
 
 void Restrictions_aggregator_factory::fetch_grantee_access(
-    const ACL_USER *grantee, Access_bitmask &global_access,
-    Restrictions &restrictions) {
+    const ACL_USER *grantee, ulong &global_access, Restrictions &restrictions) {
   assert(assert_acl_cache_read_lock(current_thd));
   global_access = grantee->access;
   restrictions = acl_restrictions->find_restrictions(grantee);
@@ -572,9 +550,8 @@ void Restrictions_aggregator_factory::fetch_grantee_access(
 */
 Restrictions_aggregator::Restrictions_aggregator(
     const Auth_id &grantor, const Auth_id grantee,
-    const Access_bitmask grantor_global_access,
-    const Access_bitmask grantee_global_access,
-    const Access_bitmask requested_access)
+    const ulong grantor_global_access, const ulong grantee_global_access,
+    const ulong requested_access)
     : m_grantor(grantor),
       m_grantee(grantee),
       m_grantor_global_access(grantor_global_access),
@@ -607,11 +584,10 @@ Restrictions_aggregator::~Restrictions_aggregator() = default;
 */
 DB_restrictions_aggregator::DB_restrictions_aggregator(
     const Auth_id &grantor, const Auth_id grantee,
-    const Access_bitmask grantor_global_access,
-    const Access_bitmask grantee_global_access,
+    const ulong grantor_global_access, const ulong grantee_global_access,
     const DB_restrictions &grantor_db_restrictions,
     const DB_restrictions &grantee_db_restrictions,
-    const Access_bitmask requested_access, const Security_context *sctx)
+    const ulong requested_access, const Security_context *sctx)
     : Restrictions_aggregator(grantor, grantee, grantor_global_access,
                               grantee_global_access, requested_access),
       m_grantor_rl(grantor_db_restrictions),
@@ -658,7 +634,7 @@ bool DB_restrictions_aggregator::generate(Abstract_restrictions &restrictions) {
                   nothing needs to be filtered
 */
 bool DB_restrictions_aggregator::find_if_require_next_level_operation(
-    Access_bitmask &rights) const {
+    ulong &rights) const {
   if (m_privs_not_processed) {
     rights = m_privs_not_processed;
     return true;
@@ -692,12 +668,11 @@ bool DB_restrictions_aggregator::find_if_require_next_level_operation(
     @retval true  Collision detected. Error raised.
 */
 bool DB_restrictions_aggregator::check_db_access_and_restrictions_collision(
-    const Access_bitmask grantee_db_access,
-    const Access_bitmask grantee_restrictions,
+    const ulong grantee_db_access, const ulong grantee_restrictions,
     const std::string &db_name) noexcept {
   if (grantee_db_access & grantee_restrictions) {
     // find out the least significant bit(lsb)
-    const Access_bitmask lsb = grantee_db_access & ~(grantee_db_access - 1);
+    const ulong lsb = grantee_db_access & ~(grantee_db_access - 1);
     // find out the position of the lsb
     size_t index = static_cast<size_t>(std::log2(lsb));
     my_error(ER_PARTIAL_REVOKE_AND_DB_GRANT_BOTH_EXISTS, MYF(0),
@@ -718,8 +693,7 @@ bool DB_restrictions_aggregator::check_db_access_and_restrictions_collision(
   @param [in] restrictions_mask Confirmed restrictions
 */
 void DB_restrictions_aggregator::set_if_db_level_operation(
-    const Access_bitmask requested_access,
-    const Access_bitmask restrictions_mask) noexcept {
+    const ulong requested_access, const ulong restrictions_mask) noexcept {
   /*
     DB level privilegss are filtered as following -
     1. If restriction_mask is 0 then there is nothing to filter from the
@@ -758,7 +732,7 @@ void DB_restrictions_aggregator::set_if_db_level_operation(
 */
 void DB_restrictions_aggregator::aggregate_restrictions(
     SQL_OP sql_op, const Db_access_map *db_map, DB_restrictions &restrictions) {
-  if (!m_grantor_rl.is_empty()) {
+  if (m_grantor_rl.is_not_empty()) {
     if (test_all_bits(m_grantee_global_access, m_requested_access) &&
         m_grantee_rl.is_empty()) {
       /*
@@ -768,10 +742,10 @@ void DB_restrictions_aggregator::aggregate_restrictions(
       */
       restrictions = m_grantee_rl;
     } else {
-      Access_bitmask restrictions_mask;
+      ulong restrictions_mask;
       DB_restrictions grantee_rl = m_grantee_rl;
       for (const auto &grantor_rl_itr : m_grantor_rl()) {
-        Access_bitmask grantee_revokes;
+        ulong grantee_revokes;
         if (grantee_rl.find(grantor_rl_itr.first, grantee_revokes)) {
           /*
             If grantor and grantee both have restriction list then aggregration
@@ -783,11 +757,10 @@ void DB_restrictions_aggregator::aggregate_restrictions(
                as global privileges. Remaining restrictions after negation
                by Grantor and Grantee's global privileges.
           */
-          Access_bitmask rm1 = ((grantor_rl_itr.second & m_requested_access) &
-                                ~m_grantee_global_access);
-          Access_bitmask rm2 =
-              (grantor_rl_itr.second & m_grantee_global_access &
-               grantee_revokes & m_grantor_global_access);
+          ulong rm1 = ((grantor_rl_itr.second & m_requested_access) &
+                       ~m_grantee_global_access);
+          ulong rm2 = (grantor_rl_itr.second & m_grantee_global_access &
+                       grantee_revokes & m_grantor_global_access);
           restrictions_mask = rm1 | rm2;
 
           /*
@@ -798,7 +771,7 @@ void DB_restrictions_aggregator::aggregate_restrictions(
             the GRANT statement has only SELECT on db, then
             retain INSERT on db.
           */
-          Access_bitmask remove_mask = grantee_revokes & m_requested_access;
+          ulong remove_mask = grantee_revokes & m_requested_access;
           grantee_rl.remove(grantor_rl_itr.first, remove_mask);
           /* Remove the schemas without any restrictions remaining */
           grantee_rl.remove(0);
@@ -809,7 +782,7 @@ void DB_restrictions_aggregator::aggregate_restrictions(
           */
           restrictions_mask = grantor_rl_itr.second & m_requested_access;
 
-          Access_bitmask grantee_db_access = 0;
+          ulong grantee_db_access = 0;
           if (sql_op == SQL_OP::GLOBAL_GRANT) {
             grantee_db_access = get_grantee_db_access(grantor_rl_itr.first);
           } else if (sql_op == SQL_OP::SET_ROLE && db_map != nullptr) {
@@ -839,7 +812,7 @@ void DB_restrictions_aggregator::aggregate_restrictions(
        (2) Restrictions in grantee's schemas which are not relevant with
            the requested access. Grantee must retain these restrictions.
       */
-      if (!grantee_rl.is_empty()) {
+      if (grantee_rl.is_not_empty()) {
         for (auto &grantee_rl_itr : grantee_rl()) {
           grantee_rl_itr.second &=
               ~(m_grantor_global_access & m_requested_access);
@@ -849,7 +822,7 @@ void DB_restrictions_aggregator::aggregate_restrictions(
         restrictions.add(grantee_rl);
       }
     }
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     restrictions = m_grantee_rl;
     for (auto &grantee_rl_itr : restrictions()) {
       restrictions.remove(grantee_rl_itr.first.c_str(), m_requested_access);
@@ -874,18 +847,12 @@ void DB_restrictions_aggregator::aggregate_restrictions(
                         access.
   @returns DB level access.
 */
-Access_bitmask DB_restrictions_aggregator::get_grantee_db_access(
+ulong DB_restrictions_aggregator::get_grantee_db_access(
     const std::string &db_name) const {
-  ulong db_access;
-  if (m_sctx && m_sctx->get_num_active_roles() > 0) {
-    LEX_CSTRING db = {db_name.c_str(), db_name.length()};
-    db_access = m_sctx->db_acl(db, false);
-  } else {
-    db_access = acl_get(current_thd, m_grantee.host().c_str(),
-                        m_sctx ? m_sctx->ip().str : m_grantee.host().c_str(),
-                        m_grantee.user().c_str(), db_name.c_str(), false);
-  }
-  return db_access;
+  return Security_context::check_db_level_access(
+      current_thd, m_sctx, m_grantee.host().c_str(),
+      m_sctx ? m_sctx->ip().str : m_grantee.host().c_str(),
+      m_grantee.user().c_str(), db_name.c_str(), db_name.length());
 }
 
 /**
@@ -899,9 +866,9 @@ Access_bitmask DB_restrictions_aggregator::get_grantee_db_access(
   @param [out] access    Access on the specified DB.
 */
 void DB_restrictions_aggregator::get_grantee_db_access(
-    const std::string &db_name, Access_bitmask &access) const {
+    const std::string &db_name, ulong &access) const {
   if (m_sctx && m_sctx->get_num_active_roles() > 0) {
-    LEX_CSTRING db = {db_name.c_str(), db_name.length()};
+    const LEX_CSTRING db = {db_name.c_str(), db_name.length()};
     access = m_sctx->db_acl(db, false);
   }
 }
@@ -920,11 +887,10 @@ void DB_restrictions_aggregator::get_grantee_db_access(
 */
 DB_restrictions_aggregator_set_role::DB_restrictions_aggregator_set_role(
     const Auth_id &grantor, const Auth_id grantee,
-    const Access_bitmask grantor_global_access,
-    const Access_bitmask grantee_global_access,
+    const ulong grantor_global_access, const ulong grantee_global_access,
     const DB_restrictions &grantor_db_restrictions,
     const DB_restrictions &grantee_db_restrictions,
-    const Access_bitmask requested_access, Db_access_map *db_map)
+    const ulong requested_access, Db_access_map *db_map)
     : DB_restrictions_aggregator(grantor, grantee, grantor_global_access,
                                  grantee_global_access, grantor_db_restrictions,
                                  grantee_db_restrictions, requested_access,
@@ -972,14 +938,14 @@ void DB_restrictions_aggregator_set_role::aggregate(
     DB_restrictions &db_restrictions) {
   assert(m_status == Status::Validated);
 
-  if (!m_grantee_rl.is_empty()) {
+  if (m_grantee_rl.is_not_empty()) {
     /*
       At this point, we already have aggregated DB privileges and grantee's
       restrictions. Therefore, negate the restrictions with the DB privileges.
       In other words remove grantee's restrictions and corresponding grantor's
       privilege.
     */
-    Access_bitmask restrictions, privileges;
+    ulong restrictions, privileges;
     for (auto it = m_db_map->begin(); it != m_db_map->end();) {
       privileges = it->second;
       if (m_grantee_rl.find(it->first, restrictions)) {
@@ -1017,11 +983,10 @@ void DB_restrictions_aggregator_set_role::aggregate(
 DB_restrictions_aggregator_global_grant::
     DB_restrictions_aggregator_global_grant(
         const Auth_id &grantor, const Auth_id grantee,
-        const Access_bitmask grantor_global_access,
-        const Access_bitmask grantee_global_access,
+        const ulong grantor_global_access, const ulong grantee_global_access,
         const DB_restrictions &grantor_db_restrictions,
         const DB_restrictions &grantee_db_restrictions,
-        const Access_bitmask requested_access, const Security_context *sctx)
+        const ulong requested_access, const Security_context *sctx)
     : DB_restrictions_aggregator(grantor, grantee, grantor_global_access,
                                  grantee_global_access, grantor_db_restrictions,
                                  grantee_db_restrictions, requested_access,
@@ -1040,20 +1005,18 @@ DB_restrictions_aggregator_global_grant::
 */
 Restrictions_aggregator::Status
 DB_restrictions_aggregator_global_grant::validate() {
-  if (!m_grantee_rl.is_empty()) {
-    for (auto &grantee_rl_itr : m_grantee_rl()) {
-      Access_bitmask grantee_db_access =
-          get_grantee_db_access(grantee_rl_itr.first.c_str());
-      /*
-        Remove the restrictions from grantor's restrictions for which
-        grantee already has DB level privileges.
-      */
-      if (check_db_access_and_restrictions_collision(
-              grantee_db_access, grantee_rl_itr.second,
-              grantee_rl_itr.first.c_str())) {
-        return (m_status = Status::Error);
-        break;
-      }
+  for (auto &grantee_rl_itr : m_grantee_rl()) {
+    ulong grantee_db_access =
+        get_grantee_db_access(grantee_rl_itr.first.c_str());
+    /*
+      Remove the restrictions from grantor's restrictions for which
+      grantee already has DB level privileges.
+    */
+    if (check_db_access_and_restrictions_collision(
+            grantee_db_access, grantee_rl_itr.second,
+            grantee_rl_itr.first.c_str())) {
+      return (m_status = Status::Error);
+      break;
     }
   }
   /*
@@ -1102,11 +1065,10 @@ void DB_restrictions_aggregator_global_grant::aggregate(
 DB_restrictions_aggregator_global_revoke::
     DB_restrictions_aggregator_global_revoke(
         const Auth_id &grantor, const Auth_id grantee,
-        const Access_bitmask grantor_global_access,
-        const Access_bitmask grantee_global_access,
+        const ulong grantor_global_access, const ulong grantee_global_access,
         const DB_restrictions &grantor_db_restrictions,
         const DB_restrictions &grantee_db_restrictions,
-        const Access_bitmask requested_access, const Security_context *sctx)
+        const ulong requested_access, const Security_context *sctx)
     : DB_restrictions_aggregator(grantor, grantee, grantor_global_access,
                                  grantee_global_access, grantor_db_restrictions,
                                  grantee_db_restrictions, requested_access,
@@ -1132,9 +1094,9 @@ DB_restrictions_aggregator_global_revoke::validate() {
       revoke from grantee.
     */
     return (m_status = Status::No_op);
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     return validate_if_grantee_rl_not_empty();
-  } else if (!m_grantor_rl.is_empty()) {
+  } else if (m_grantor_rl.is_not_empty()) {
     for (auto &itr : m_grantor_rl.get()) {
       /*
         Grantor cannot revoke the privileges from grantee which are in
@@ -1184,7 +1146,7 @@ void DB_restrictions_aggregator_global_revoke::aggregate(
 Restrictions_aggregator::Status
 DB_restrictions_aggregator_global_revoke::validate_if_grantee_rl_not_empty() {
   for (auto &grantee_rl_itr : m_grantee_rl.get()) {
-    Access_bitmask grantor_revokes;
+    ulong grantor_revokes;
     if (m_grantor_rl.find(grantee_rl_itr.first.c_str(), grantor_revokes)) {
       /*
         Grantor cannot revoke the requested privileges from grantee which are
@@ -1197,7 +1159,7 @@ DB_restrictions_aggregator_global_revoke::validate_if_grantee_rl_not_empty() {
         break;
       }
     }
-    Access_bitmask grantee_db_access =
+    ulong grantee_db_access =
         get_grantee_db_access(grantee_rl_itr.first.c_str());
     /*
       Check if grantee has same DB level privilege as well as
@@ -1228,11 +1190,10 @@ DB_restrictions_aggregator_global_revoke::validate_if_grantee_rl_not_empty() {
 DB_restrictions_aggregator_global_revoke_all::
     DB_restrictions_aggregator_global_revoke_all(
         const Auth_id &grantor, const Auth_id grantee,
-        const Access_bitmask grantor_global_access,
-        const Access_bitmask grantee_global_access,
+        const ulong grantor_global_access, const ulong grantee_global_access,
         const DB_restrictions &grantor_db_restrictions,
         const DB_restrictions &grantee_db_restrictions,
-        const Access_bitmask requested_access, const Security_context *sctx)
+        const ulong requested_access, const Security_context *sctx)
     : DB_restrictions_aggregator_global_revoke(
           grantor, grantee, grantor_global_access, grantee_global_access,
           grantor_db_restrictions, grantee_db_restrictions, requested_access,
@@ -1243,9 +1204,9 @@ Restrictions_aggregator::Status
 DB_restrictions_aggregator_global_revoke_all::validate() {
   if (m_grantor_rl == m_grantee_rl) {
     return (m_status = Status::No_op);
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     return validate_if_grantee_rl_not_empty();
-  } else if (!m_grantor_rl.is_empty()) {
+  } else if (m_grantor_rl.is_not_empty()) {
     for (auto &itr : m_grantor_rl.get()) {
       /*
         Grantor cannot revoke the privileges from grantee which are in
@@ -1292,14 +1253,12 @@ void DB_restrictions_aggregator_global_revoke_all::aggregate(
 */
 DB_restrictions_aggregator_db_grant::DB_restrictions_aggregator_db_grant(
     const Auth_id &grantor, const Auth_id grantee,
-    const Access_bitmask grantor_global_access,
-    const Access_bitmask grantee_global_access,
-    const Access_bitmask grantor_db_access,
-    const Access_bitmask grantee_db_access,
+    const ulong grantor_global_access, const ulong grantee_global_access,
+    const ulong grantor_db_access, const ulong grantee_db_access,
     const DB_restrictions &grantor_db_restrictions,
     const DB_restrictions &grantee_db_restrictions,
-    const Access_bitmask requested_access, bool is_grant_all,
-    const std::string &db_name, const Security_context *sctx)
+    const ulong requested_access, bool is_grant_all, const std::string &db_name,
+    const Security_context *sctx)
     : DB_restrictions_aggregator(grantor, grantee, grantor_global_access,
                                  grantee_global_access, grantor_db_restrictions,
                                  grantee_db_restrictions, requested_access,
@@ -1312,9 +1271,9 @@ DB_restrictions_aggregator_db_grant::DB_restrictions_aggregator_db_grant(
 /** Validation function for database level grant statement. */
 Restrictions_aggregator::Status
 DB_restrictions_aggregator_db_grant::validate() {
-  Access_bitmask grantee_part_revokes = 0;
+  ulong grantee_part_revokes = 0;
   if (m_grantee_rl.find(m_db_name.c_str(), grantee_part_revokes)) {
-    Access_bitmask grantee_db_access = m_grantee_db_access;
+    ulong grantee_db_access = m_grantee_db_access;
     get_grantee_db_access(m_db_name, grantee_db_access);
     /*
       Same DB level privilege and partial revoke cannot co-exist.
@@ -1358,7 +1317,7 @@ void DB_restrictions_aggregator_db_grant::aggregate(
   */
   assert(m_status == Status::Validated);
   restrictions = m_grantee_rl;
-  Access_bitmask grantee_db_revokes = 0;
+  ulong grantee_db_revokes = 0;
 
   for (auto &entry : restrictions()) {
     if (m_db_name.compare(entry.first) == 0) {
@@ -1391,13 +1350,11 @@ void DB_restrictions_aggregator_db_grant::aggregate(
 */
 DB_restrictions_aggregator_db_revoke::DB_restrictions_aggregator_db_revoke(
     const Auth_id &grantor, const Auth_id grantee,
-    const Access_bitmask grantor_global_access,
-    const Access_bitmask grantee_global_access,
-    const Access_bitmask grantor_db_access,
-    const Access_bitmask grantee_db_access,
+    const ulong grantor_global_access, const ulong grantee_global_access,
+    const ulong grantor_db_access, const ulong grantee_db_access,
     const DB_restrictions &grantor_db_restrictions,
     const DB_restrictions &grantee_db_restrictions,
-    const Access_bitmask requested_access, bool is_revoke_all,
+    const ulong requested_access, bool is_revoke_all,
     const std::string &db_name, const Security_context *sctx)
     : DB_restrictions_aggregator(grantor, grantee, grantor_global_access,
                                  grantee_global_access, grantor_db_restrictions,
@@ -1411,9 +1368,9 @@ DB_restrictions_aggregator_db_revoke::DB_restrictions_aggregator_db_revoke(
 /**  Validation function for database level revoke statement. */
 Restrictions_aggregator::Status
 DB_restrictions_aggregator_db_revoke::validate() {
-  Access_bitmask grantee_part_revokes = 0;
+  ulong grantee_part_revokes = 0;
   if (m_grantee_rl.find(m_db_name.c_str(), grantee_part_revokes)) {
-    Access_bitmask grantee_db_access = m_grantee_db_access;
+    ulong grantee_db_access = m_grantee_db_access;
     get_grantee_db_access(m_db_name, grantee_db_access);
 
     /*
@@ -1458,7 +1415,7 @@ void DB_restrictions_aggregator_db_revoke::aggregate(
       Filter out the access for which grantee does not have DB level access but
       the Global level access.
     */
-    const Access_bitmask revoke_mask =
+    const ulong revoke_mask =
         (m_grantee_global_access &
          (m_requested_access & (m_requested_access ^ m_grantee_db_access)));
     if (revoke_mask)  // Create restrictions only if there is a restriction mask
@@ -1504,7 +1461,7 @@ Restrictions &Restrictions::operator=(Restrictions &&restrictions) {
 
 /* DB restrictions comparator */
 bool Restrictions::has_more_db_restrictions(const Restrictions &other,
-                                            Access_bitmask access) {
+                                            ulong access) {
   return m_db_restrictions.has_more_restrictions(other.db(), access);
 }
 
