@@ -36,6 +36,9 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 extern bool opt_source_verify_checksum;
 
+using Format_description_event = mysql::binlog::event::Format_description_event;
+using Log_event_type = mysql::binlog::event::Log_event_type;
+
 /// @brief This class holds the context of the iterator.
 ///
 /// The context of the iterator contains runtime data, such as the current
@@ -245,7 +248,7 @@ class Binlog_iterator_ctx {
     DBUG_TRACE;
     auto event_type = static_cast<Log_event_type>(buffer[EVENT_TYPE_OFFSET]);
     switch (event_type) {
-      case binary_log::GTID_LOG_EVENT: {
+      case mysql::binlog::event::GTID_LOG_EVENT: {
         Gtid_log_event gtid_ev(reinterpret_cast<const char *>(buffer),
                                &m_current_fde);
         if (!gtid_ev.is_valid())
@@ -332,7 +335,6 @@ class Binlog_iterator_ctx {
     auto saved_pos{reader.position()};
 
     unsigned char buffer[LOG_EVENT_HEADER_LEN];
-    memset(buffer, 0, LOG_EVENT_HEADER_LEN);
 
     while (true) {
       // save the position so that we can seek to it later on
@@ -359,21 +361,20 @@ class Binlog_iterator_ctx {
         // have we read a gtid? if so, we need to check if we skip the
         // transaction
         switch (static_cast<Log_event_type>(buffer[EVENT_TYPE_OFFSET])) {
-          case binary_log::GTID_LOG_EVENT: {
+          case mysql::binlog::event::GTID_LOG_EVENT: {
             // seek to the beginning of the log event and deserialize the GTID
             // event
             unsigned int bytes_read{0};
-            unsigned char
-                gtid_buffer_stack[binary_log::Gtid_event::MAX_EVENT_LENGTH];
-            memset(gtid_buffer_stack, 0,
-                   binary_log::Gtid_event::MAX_EVENT_LENGTH);
+            unsigned char gtid_buffer_stack
+                [mysql::binlog::event::Gtid_event::MAX_EVENT_LENGTH];
             unsigned char *gtid_buffer = gtid_buffer_stack;
 
             // swap buffer in the context
             auto *saved_buffer = get_buffer();
             auto saved_buffer_capacity = get_buffer_capacity();
             set_buffer(gtid_buffer);
-            set_buffer_capacity(binary_log::Gtid_event::MAX_EVENT_LENGTH);
+            set_buffer_capacity(
+                mysql::binlog::event::Gtid_event::MAX_EVENT_LENGTH);
             auto swap_and_free_buffer_guard{create_scope_guard([&]() {
               set_buffer(saved_buffer);
               set_buffer_capacity(saved_buffer_capacity);
@@ -459,7 +460,7 @@ static Previous_gtids_log_event *find_previous_gtids_event(
     auto *ev = binlog_file_reader.read_event_object();
     if (ev == nullptr) return nullptr;
 
-    if (ev->get_type_code() == binary_log::PREVIOUS_GTIDS_LOG_EVENT) {
+    if (ev->get_type_code() == mysql::binlog::event::PREVIOUS_GTIDS_LOG_EVENT) {
       return dynamic_cast<Previous_gtids_log_event *>(ev);
     }
 
@@ -635,7 +636,6 @@ DEFINE_METHOD(Binlog_iterator_service_get_status, FileStorage::get,
   auto &ctx = *it->m_ctx;
   auto &reader = ctx.get_reader();
   *bytes_read = 0;
-  memset(buffer, 0, buffer_capacity);
 
   // update the file cursor (and get the event size, which we can disregard
   // here)
@@ -673,7 +673,7 @@ DEFINE_METHOD(Binlog_iterator_service_get_status, FileStorage::get,
   // save FORMAT_DESCRIPTION event
   *bytes_read = bytes_read_by_lowlevel_function;
   switch (static_cast<Log_event_type>(buffer[EVENT_TYPE_OFFSET])) {
-    case binary_log::FORMAT_DESCRIPTION_EVENT: {
+    case mysql::binlog::event::FORMAT_DESCRIPTION_EVENT: {
       // If we are processing a format description event we save it. This is
       // probably a bit pedantic, since we read binary logs generated by
       // this server. Therefore, instantiating a format description event
@@ -681,7 +681,7 @@ DEFINE_METHOD(Binlog_iterator_service_get_status, FileStorage::get,
       // to the fact that some users may edit the index file and force
       // binary logs into the server from different versions, we play it
       // safe here.
-      binary_log::Format_description_event fde(
+      mysql::binlog::event::Format_description_event fde(
           reinterpret_cast<const char *>(buffer), &ctx.get_fde());
       ctx.set_fde(fde);
       break;
@@ -709,7 +709,8 @@ DEFINE_BOOL_METHOD(FileStorage::get_next_entry_size,
 
   // events cannot be larger than 1GB (MAX_MAX_ALLOWED_PACKET)
   // if this limitation is ever lifted, this needs to be removed
-  if (event_size > static_cast<uint64_t>(binary_log::max_log_event_size))
+  if (event_size >
+      static_cast<uint64_t>(mysql::binlog::event::max_log_event_size))
     return 1;
   *size = event_size;
 
