@@ -22,7 +22,6 @@
 
 #ifndef BINLOG_READER_INCLUDED
 #define BINLOG_READER_INCLUDED
-#include "sql/binlog_crypt_data.h"
 #include "sql/binlog_istream.h"
 #include "sql/log_event.h"
 
@@ -41,8 +40,8 @@
 */
 Binlog_read_error::Error_type binlog_event_deserialize(
     const unsigned char *event_data, unsigned int event_data_len,
-    const Format_description_event *fde, bool verify_checksum,
-    Log_event **event);
+    const mysql::binlog::event::Format_description_event *fde,
+    bool verify_checksum, Log_event **event);
 
 class Default_binlog_event_allocator {
  public:
@@ -79,9 +78,10 @@ class Binlog_event_data_istream {
      @retval true Error
   */
   template <class ALLOCATOR>
-  bool read_event_data(unsigned char **data, unsigned int *length,
-                       ALLOCATOR *allocator, bool verify_checksum,
-                       enum_binlog_checksum_alg checksum_alg) {
+  bool read_event_data(
+      unsigned char **data, unsigned int *length, ALLOCATOR *allocator,
+      bool verify_checksum,
+      mysql::binlog::event::enum_binlog_checksum_alg checksum_alg) {
     DBUG_TRACE;
     if (read_event_header() || check_event_header()) return true;
 
@@ -98,11 +98,6 @@ class Binlog_event_data_istream {
     return false;
   }
 
-  bool start_decryption(binary_log::Start_encryption_event *see);
-  void reset_crypto() noexcept { crypto_data.disable(); }
-
-  Binlog_crypt_data crypto_data;
-
  protected:
   unsigned char m_header[LOG_EVENT_MINIMAL_HEADER_LEN];
   /**
@@ -153,7 +148,6 @@ class Binlog_event_data_istream {
      streams. So Binlog_read_error pointer is defined here. It should be
      initialized in constructor by caller.
   */
-
   Binlog_read_error *m_error;
 
  private:
@@ -161,21 +155,6 @@ class Binlog_event_data_istream {
   unsigned int m_max_event_size;
   unsigned int m_event_length = 0;
 
-  class Decryption_buffer final {
-   public:
-    ~Decryption_buffer();
-    bool set_size(size_t size);
-    uchar *data();
-
-   private:
-    bool resize(size_t new_size);
-
-    uchar *m_buffer = nullptr;
-    size_t m_size = 0;
-    uint m_number_of_events_with_half_the_size = 0;
-  };
-  Decryption_buffer m_decryption_buffer;
-
   /**
      Fill the event data into the given buffer and verify checksum if
      'verify_checksum' is true.
@@ -187,8 +166,9 @@ class Binlog_event_data_istream {
      @retval false Success
      @retval true Error
   */
-  bool fill_event_data(unsigned char *event_data, bool verify_checksum,
-                       enum_binlog_checksum_alg checksum_alg);
+  bool fill_event_data(
+      unsigned char *event_data, bool verify_checksum,
+      mysql::binlog::event::enum_binlog_checksum_alg checksum_alg);
 };
 
 /**
@@ -217,8 +197,9 @@ class Binlog_event_object_istream {
      @retval nullptr Error
   */
   template <class ALLOCATOR>
-  Log_event *read_event_object(const Format_description_event &fde,
-                               bool verify_checksum, ALLOCATOR *allocator) {
+  Log_event *read_event_object(
+      const mysql::binlog::event::Format_description_event &fde,
+      bool verify_checksum, ALLOCATOR *allocator) {
     DBUG_TRACE;
     unsigned char *data = nullptr;
     unsigned int length = 0;
@@ -290,7 +271,8 @@ class IBasic_binlog_file_reader {
   ///
   /// The event is a member by this reader, so the caller must not use
   /// the returned reference after this reader has been deleted.
-  virtual const Format_description_event &format_description_event() const = 0;
+  virtual const mysql::binlog::event::Format_description_event &
+  format_description_event() const = 0;
 };
 
 /**
@@ -347,8 +329,6 @@ class Basic_binlog_file_reader : public IBasic_binlog_file_reader {
     DBUG_TRACE;
     if (m_ifile.open(file_name)) return true;
 
-    m_data_istream.reset_crypto();
-
     Format_description_log_event *fd = read_fdle(offset);
 
     if (!fd) return has_fatal_error();
@@ -368,7 +348,8 @@ class Basic_binlog_file_reader : public IBasic_binlog_file_reader {
   */
   void close() {
     m_ifile.close();
-    m_fde = Format_description_event(BINLOG_VERSION, server_version);
+    m_fde = mysql::binlog::event::Format_description_event(BINLOG_VERSION,
+                                                           server_version);
   }
 
   bool is_open() const { return m_ifile.is_open(); }
@@ -391,17 +372,10 @@ class Basic_binlog_file_reader : public IBasic_binlog_file_reader {
     m_event_start_pos = position();
     Log_event *ev = m_object_istream.read_event_object(m_fde, m_verify_checksum,
                                                        &m_allocator);
-    if (ev && ev->get_type_code() == binary_log::FORMAT_DESCRIPTION_EVENT) {
-      Format_description_log_event *const new_fde =
-          down_cast<Format_description_log_event *>(ev);
-      m_fde = *new_fde;
-    } else if (ev &&
-               ev->get_type_code() == binary_log::START_5_7_ENCRYPTION_EVENT &&
-               m_data_istream.start_decryption(
-                   down_cast<Start_encryption_log_event *>(ev))) {
-      delete ev;
-      ev = nullptr;
-    }
+    if (ev != nullptr &&
+        ev->get_type_code() == mysql::binlog::event::FORMAT_DESCRIPTION_EVENT)
+      m_fde =
+          dynamic_cast<mysql::binlog::event::Format_description_event &>(*ev);
     return ev;
   }
 
@@ -422,10 +396,12 @@ class Basic_binlog_file_reader : public IBasic_binlog_file_reader {
   Event_object_istream *event_object_istream() { return &m_object_istream; }
   ALLOCATOR *allocator() { return &m_allocator; }
 
-  void set_format_description_event(const Format_description_event &fde) {
+  void set_format_description_event(
+      const mysql::binlog::event::Format_description_event &fde) {
     m_fde = fde;
   }
-  const Format_description_event &format_description_event() const override {
+  const mysql::binlog::event::Format_description_event &
+  format_description_event() const override {
     return m_fde;
   }
   my_off_t event_start_pos() { return m_event_start_pos; }
@@ -435,10 +411,6 @@ class Basic_binlog_file_reader : public IBasic_binlog_file_reader {
    */
   void reset_error() { m_error.set_type(Binlog_read_error::SUCCESS); }
 
-  bool start_decryption(binary_log::Start_encryption_event *see) {
-    return m_data_istream.start_decryption(see);
-  }
-
  private:
   Binlog_read_error m_error;
 
@@ -447,7 +419,7 @@ class Basic_binlog_file_reader : public IBasic_binlog_file_reader {
   Event_object_istream m_object_istream;
   ALLOCATOR m_allocator;
 
-  Format_description_event m_fde;
+  mysql::binlog::event::Format_description_event m_fde;
   bool m_verify_checksum = false;
   my_off_t m_event_start_pos = 0;
 
@@ -478,28 +450,16 @@ class Basic_binlog_file_reader : public IBasic_binlog_file_reader {
           m_fde, m_verify_checksum, &allocator);
 
       if (ev == nullptr) break;
-      if (ev->get_type_code() == binary_log::FORMAT_DESCRIPTION_EVENT) {
+      if (ev->get_type_code() ==
+          mysql::binlog::event::FORMAT_DESCRIPTION_EVENT) {
         delete fdle;
-        Format_description_log_event *new_fdev =
-            down_cast<Format_description_log_event *>(ev);
-        fdle = new_fdev;
+        fdle = dynamic_cast<Format_description_log_event *>(ev);
         m_fde = *fdle;
-        assert(m_fde.footer()->checksum_alg ==
-                   binary_log::BINLOG_CHECKSUM_ALG_OFF ||
-               m_fde.footer()->checksum_alg ==
-                   binary_log::BINLOG_CHECKSUM_ALG_CRC32);
-      } else if (ev->get_type_code() ==
-                     binary_log::START_5_7_ENCRYPTION_EVENT &&
-                 m_data_istream.start_decryption(
-                     down_cast<Start_encryption_log_event *>(ev))) {
-        delete ev;
-        ev = nullptr;
-        break;
       } else {
-        binary_log::Log_event_type type = ev->get_type_code();
+        mysql::binlog::event::Log_event_type type = ev->get_type_code();
         delete ev;
-        if (type != binary_log::PREVIOUS_GTIDS_LOG_EVENT &&
-            type != binary_log::ROTATE_EVENT)
+        if (type != mysql::binlog::event::PREVIOUS_GTIDS_LOG_EVENT &&
+            type != mysql::binlog::event::ROTATE_EVENT)
           break;
       }
     }
