@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -236,9 +235,13 @@ class Connection_handler_manager {
   static void wait_till_no_connection();
 
   /**
-    Performs required initializations after the daemonization of the mysqld
-    process.
+    Return number of connections in thread-safe way.
   */
-  void post_daemonize_init();
+  static uint get_connection_count() {
+    mysql_mutex_lock(&LOCK_connection_count);
+    const uint res = connection_count;
+    mysql_mutex_unlock(&LOCK_connection_count);
+    return res;
+  }
 };
 #endif  // CONNECTION_HANDLER_MANAGER_INCLUDED.
