@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <fcntl.h>
 
+#include <limits.h>
 #ifndef _WIN32
 #include <netdb.h>
 #endif
@@ -48,18 +49,20 @@
 #endif
 #include <algorithm>
 #include <atomic>
+#include <memory>  // std::unique_ptr
 #include <new>
 #include <utility>
 
 #include "m_string.h"
 #include "my_dbug.h"
 #include "my_io.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_thread.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_thread.h"
 #include "mysqld_error.h"
+#include "sql-common/net_ns.h"
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/conn_handler/channel_info.h"               // Channel_info
 #include "sql/conn_handler/init_net_server_extension.h"  // init_net_server_extension
@@ -85,8 +88,11 @@ extern "C" {
 
 using std::max;
 
+// Test accept this many times
+static constexpr const uint MAX_ACCEPT_RETRY{10};
+
 /** Number of connection errors when selecting on the listening port */
-static std::atomic<ulong> connection_errors_select{0};
+static std::atomic<ulong> connection_errors_query_block{0};
 
 /** Number of connection errors when accepting sockets in the listening port. */
 static std::atomic<ulong> connection_errors_accept{0};
@@ -94,7 +100,28 @@ static std::atomic<ulong> connection_errors_accept{0};
 /** Number of connection errors from TCP wrappers. */
 static std::atomic<ulong> connection_errors_tcpwrap{0};
 
-ulong get_connection_errors_select() { return connection_errors_select.load(); }
+namespace {
+struct FreeAddrInfoDeleter {
+  void operator()(addrinfo *ai) {
+    if (ai != nullptr) {
+      freeaddrinfo(ai);
+    }
+  }
+};
+
+using AddrInfoPtr = std::unique_ptr<addrinfo, FreeAddrInfoDeleter>;
+AddrInfoPtr GetAddrInfoPtr(const char *node, const char *service,
+                           const addrinfo *hints) {
+  addrinfo *p = nullptr;
+  int err = getaddrinfo(node, service, hints, &p);
+  AddrInfoPtr nrv{p};
+  return (err == 0 ? std::move(nrv) : nullptr);
+}
+}  // namespace
+
+ulong get_connection_errors_query_block() {
+  return connection_errors_query_block.load();
+}
 
 ulong get_connection_errors_accept() { return connection_errors_accept.load(); }
 
@@ -119,12 +146,14 @@ class Channel_info_local_socket : public Channel_info {
   MYSQL_SOCKET m_connect_sock;
 
  protected:
-  virtual Vio *create_and_init_vio() const {
+  Vio *create_and_init_vio() const override {
     Vio *vio =
         mysql_socket_vio_new(m_connect_sock, VIO_TYPE_SOCKET, VIO_LOCALHOST);
 #ifdef USE_PPOLL_IN_VIO
     if (vio != nullptr) {
-      vio->thread_id = my_thread_self();
+      // Unset thread_id, to ensure that all shutdowns explicitly set the
+      // current real_id from the THD.
+      vio->thread_id.reset();
       vio->signal_mask = mysqld_signal_mask;
     }
 #endif
@@ -140,18 +169,18 @@ class Channel_info_local_socket : public Channel_info {
   Channel_info_local_socket(MYSQL_SOCKET connect_socket)
       : m_connect_sock(connect_socket) {}
 
-  virtual THD *create_thd() {
+  THD *create_thd() override {
     THD *thd = Channel_info::create_thd();
 
-    if (thd != NULL) {
+    if (thd != nullptr) {
       init_net_server_extension(thd);
       thd->security_context()->set_host_ptr(my_localhost, strlen(my_localhost));
     }
     return thd;
   }
 
-  virtual void send_error_and_close_channel(uint errorcode, int error,
-                                            bool senderror) {
+  void send_error_and_close_channel(uint errorcode, int error,
+                                    bool senderror) override {
     Channel_info::send_error_and_close_channel(errorcode, error, senderror);
 
     mysql_socket_shutdown(m_connect_sock, SHUT_RDWR);
@@ -175,16 +204,29 @@ class Channel_info_tcpip_socket : public Channel_info {
     ordinary connection.
   */
   bool m_is_admin_conn;
+#ifdef HAVE_SETNS
+  /*
+    Network namespace associated with the socket.
+  */
+  std::string m_network_namespace;
+#endif
 
  protected:
-  virtual Vio *create_and_init_vio() const {
+  Vio *create_and_init_vio() const override {
     Vio *vio = mysql_socket_vio_new(m_connect_sock, VIO_TYPE_TCPIP, 0);
 #ifdef USE_PPOLL_IN_VIO
     if (vio != nullptr) {
-      vio->thread_id = my_thread_self();
+      vio->thread_id.reset();
       vio->signal_mask = mysqld_signal_mask;
     }
 #endif
+
+#ifdef HAVE_SETNS
+    strncpy(vio->network_namespace, m_network_namespace.c_str(),
+            sizeof(vio->network_namespace) - 1);
+    vio->network_namespace[sizeof(vio->network_namespace) - 1] = '\0';
+#endif
+
     return vio;
   }
 
@@ -199,25 +241,36 @@ class Channel_info_tcpip_socket : public Channel_info {
   Channel_info_tcpip_socket(MYSQL_SOCKET connect_socket, bool is_admin_conn)
       : m_connect_sock(connect_socket), m_is_admin_conn(is_admin_conn) {}
 
-  virtual THD *create_thd() {
+  THD *create_thd() override {
     THD *thd = Channel_info::create_thd();
 
-    if (thd != NULL) {
+    if (thd != nullptr) {
       thd->set_admin_connection(m_is_admin_conn);
       init_net_server_extension(thd);
     }
     return thd;
   }
 
-  virtual void send_error_and_close_channel(uint errorcode, int error,
-                                            bool senderror) {
+  void send_error_and_close_channel(uint errorcode, int error,
+                                    bool senderror) override {
     Channel_info::send_error_and_close_channel(errorcode, error, senderror);
 
     mysql_socket_shutdown(m_connect_sock, SHUT_RDWR);
     mysql_socket_close(m_connect_sock);
   }
 
-  virtual bool is_admin_connection() const { return m_is_admin_conn; }
+  bool is_admin_connection() const override { return m_is_admin_conn; }
+
+#ifdef HAVE_SETNS
+  /**
+    Set a network namespace for channel.
+
+    @param network_namespace  Network namespace associated with a channel.
+  */
+  void set_network_namespace(const std::string &network_namespace) {
+    m_network_namespace = network_namespace;
+  }
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -246,14 +299,15 @@ const char *ipv6_all_addresses = "::";
   the get_listener_socket that setup a TCP listener socket to listen.
 */
 class TCP_socket {
-  std::string m_bind_addr_str;  // IP address as string.
-  uint m_tcp_port;              // TCP port to bind to
+  std::string m_bind_addr_str;      // IP address as string.
+  std::string m_network_namespace;  // Network namespace if specified
+  uint m_tcp_port;                  // TCP port to bind to
   uint m_backlog;       // Backlog length for queue of pending connections.
   uint m_port_timeout;  // Port timeout
 
   MYSQL_SOCKET create_socket(const struct addrinfo *addrinfo_list,
                              int addr_family, struct addrinfo **use_addrinfo) {
-    for (const struct addrinfo *cur_ai = addrinfo_list; cur_ai != NULL;
+    for (const struct addrinfo *cur_ai = addrinfo_list; cur_ai != nullptr;
          cur_ai = cur_ai->ai_next) {
       if (cur_ai->ai_family != addr_family) continue;
 
@@ -263,7 +317,7 @@ class TCP_socket {
 
       char ip_addr[INET6_ADDRSTRLEN];
 
-      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), NULL, 0,
+      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), nullptr, 0,
                           NI_NUMERICHOST)) {
         ip_addr[0] = 0;
       }
@@ -275,7 +329,7 @@ class TCP_socket {
       } else {
         LogErr(INFORMATION_LEVEL, ER_CONN_TCP_CREATED, (const char *)ip_addr);
 
-        *use_addrinfo = (struct addrinfo *)cur_ai;
+        *use_addrinfo = const_cast<addrinfo *>(cur_ai);
         return sock;
       }
     }
@@ -290,13 +344,15 @@ class TCP_socket {
     events.
 
     @param  bind_addr_str  ip address as string value.
+    @param  network_namespace_str  network namespace as string value
     @param  tcp_port  tcp port number.
     @param  backlog backlog specifying length of pending connection queue.
     @param  port_timeout port timeout value
   */
-  TCP_socket(std::string bind_addr_str, uint tcp_port, uint backlog,
-             uint port_timeout)
+  TCP_socket(std::string bind_addr_str, std::string network_namespace_str,
+             uint tcp_port, uint backlog, uint port_timeout)
       : m_bind_addr_str(bind_addr_str),
+        m_network_namespace(network_namespace_str),
         m_tcp_port(tcp_port),
         m_backlog(backlog),
         m_port_timeout(port_timeout) {}
@@ -307,8 +363,7 @@ class TCP_socket {
     @retval   valid socket if successful else MYSQL_INVALID_SOCKET on failure.
   */
   MYSQL_SOCKET get_listener_socket() {
-    struct addrinfo *ai;
-    const char *bind_address_str = NULL;
+    const char *bind_address_str = nullptr;
 
     LogErr(INFORMATION_LEVEL, ER_CONN_TCP_ADDRESS, m_bind_addr_str.c_str(),
            m_tcp_port);
@@ -324,6 +379,19 @@ class TCP_socket {
     char port_buf[NI_MAXSERV];
     snprintf(port_buf, NI_MAXSERV, "%d", m_tcp_port);
 
+    if (!m_network_namespace.empty()) {
+#ifdef HAVE_SETNS
+      if (set_network_namespace(m_network_namespace))
+        return MYSQL_INVALID_SOCKET;
+#else
+      LogErr(ERROR_LEVEL, ER_NETWORK_NAMESPACES_NOT_SUPPORTED);
+      return MYSQL_INVALID_SOCKET;
+#endif
+    }
+
+    // Create a RAII guard for addrinfo struct.
+    AddrInfoPtr ai_ptr{nullptr};
+
     if (native_strcasecmp(m_bind_addr_str.c_str(), MY_BIND_ALL_ADDRESSES) ==
         0) {
       /*
@@ -334,18 +402,20 @@ class TCP_socket {
       */
 
       bool ipv6_available = false;
-      if (!getaddrinfo(ipv6_all_addresses, port_buf, &hints, &ai)) {
+      ai_ptr = GetAddrInfoPtr(ipv6_all_addresses, port_buf, &hints);
+      if (ai_ptr) {
         /*
           IPv6 might be available (the system might be able to resolve an IPv6
           address, but not be able to create an IPv6-socket). Try to create a
           dummy IPv6-socket. Do not instrument that socket by P_S.
         */
 
-        MYSQL_SOCKET s = mysql_socket_socket(0, AF_INET6, SOCK_STREAM, 0);
+        const MYSQL_SOCKET s = mysql_socket_socket(0, AF_INET6, SOCK_STREAM, 0);
         ipv6_available = mysql_socket_getfd(s) != INVALID_SOCKET;
         if (ipv6_available) mysql_socket_close(s);
       }
-      if (ipv6_available) {
+      if (ipv6_available &&
+          DBUG_EVALUATE_IF("sim_ipv6_unavailable", false, true)) {
         LogErr(INFORMATION_LEVEL, ER_CONN_TCP_IPV6_AVAILABLE);
 
         // Address info (ai) for IPv6 address is already set.
@@ -355,47 +425,55 @@ class TCP_socket {
         LogErr(INFORMATION_LEVEL, ER_CONN_TCP_IPV6_UNAVAILABLE);
 
         // Retrieve address info (ai) for IPv4 address.
-
-        if (getaddrinfo(ipv4_all_addresses, port_buf, &hints, &ai)) {
+        ai_ptr = GetAddrInfoPtr(ipv4_all_addresses, port_buf, &hints);
+        if (!ai_ptr) {
 #ifdef _WIN32
           Socket_error_message_buf msg_buff;
-          FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, socket_errno,
+          FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, socket_errno,
                         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-                        (LPTSTR)msg_buff, sizeof(msg_buff), NULL);
+                        (LPTSTR)msg_buff, sizeof(msg_buff), nullptr);
           LogErr(ERROR_LEVEL, ER_CONN_TCP_ERROR_WITH_STRERROR, msg_buff);
 #else
           LogErr(ERROR_LEVEL, ER_CONN_TCP_ERROR_WITH_STRERROR, strerror(errno));
 #endif
           LogErr(ERROR_LEVEL, ER_CONN_TCP_CANT_RESOLVE_HOSTNAME);
+#ifdef HAVE_SETNS
+          if (!m_network_namespace.empty())
+            (void)restore_original_network_namespace();
+#endif
           return MYSQL_INVALID_SOCKET;
-        }
-
+        }  // !ai_ptr
         bind_address_str = ipv4_all_addresses;
       }
     } else {
-      if (getaddrinfo(m_bind_addr_str.c_str(), port_buf, &hints, &ai)) {
+      ai_ptr = GetAddrInfoPtr(m_bind_addr_str.c_str(), port_buf, &hints);
+      if (!ai_ptr) {
 #ifdef _WIN32
         Socket_error_message_buf msg_buff;
-        FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, socket_errno,
+        FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, socket_errno,
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-                      (LPTSTR)msg_buff, sizeof(msg_buff), NULL);
+                      (LPTSTR)msg_buff, sizeof(msg_buff), nullptr);
         LogErr(ERROR_LEVEL, ER_CONN_TCP_ERROR_WITH_STRERROR, msg_buff);
 #else
         LogErr(ERROR_LEVEL, ER_CONN_TCP_ERROR_WITH_STRERROR, strerror(errno));
 #endif
         LogErr(ERROR_LEVEL, ER_CONN_TCP_CANT_RESOLVE_HOSTNAME);
-        return MYSQL_INVALID_SOCKET;
-      }
+#ifdef HAVE_SETNS
+        if (!m_network_namespace.empty())
+          (void)restore_original_network_namespace();
+#endif
 
+        return MYSQL_INVALID_SOCKET;
+      }  // !ai_ptr
       bind_address_str = m_bind_addr_str.c_str();
     }
 
     // Log all the IP-addresses
-    for (struct addrinfo *cur_ai = ai; cur_ai != NULL;
+    for (struct addrinfo *cur_ai = ai_ptr.get(); cur_ai != nullptr;
          cur_ai = cur_ai->ai_next) {
       char ip_addr[INET6_ADDRSTRLEN];
 
-      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), NULL, 0,
+      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), nullptr, 0,
                           NI_NUMERICHOST)) {
         LogErr(ERROR_LEVEL, ER_CONN_TCP_IP_NOT_LOGGED);
         continue;
@@ -415,18 +493,24 @@ class TCP_socket {
     */
 
     struct addrinfo *a = nullptr;
-    MYSQL_SOCKET listener_socket = create_socket(ai, AF_INET, &a);
+
+    MYSQL_SOCKET listener_socket = create_socket(ai_ptr.get(), AF_INET, &a);
 
     if (mysql_socket_getfd(listener_socket) == INVALID_SOCKET)
-      listener_socket = create_socket(ai, AF_INET6, &a);
+      listener_socket = create_socket(ai_ptr.get(), AF_INET6, &a);
+
+#ifdef HAVE_SETNS
+    if (!m_network_namespace.empty() && restore_original_network_namespace())
+      return MYSQL_INVALID_SOCKET;
+#endif
 
     // Report user-error if we failed to create a socket.
     if (mysql_socket_getfd(listener_socket) == INVALID_SOCKET) {
 #ifdef _WIN32
       Socket_error_message_buf msg_buff;
-      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, socket_errno,
+      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, socket_errno,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)msg_buff,
-                    sizeof(msg_buff), NULL);
+                    sizeof(msg_buff), nullptr);
       LogErr(ERROR_LEVEL, ER_CONN_TCP_ERROR_WITH_STRERROR, msg_buff);
 #else
       LogErr(ERROR_LEVEL, ER_CONN_TCP_ERROR_WITH_STRERROR, strerror(errno));
@@ -485,14 +569,14 @@ class TCP_socket {
       this_wait = retry * retry / 3 + 1;
       sleep(this_wait);
     }
-    freeaddrinfo(ai);
+
     if (ret < 0) {
       DBUG_PRINT("error", ("Got error: %d from bind", socket_errno));
 #ifdef _WIN32
       Socket_error_message_buf msg_buff;
-      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, socket_errno,
+      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, socket_errno,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)msg_buff,
-                    sizeof(msg_buff), NULL);
+                    sizeof(msg_buff), nullptr);
       LogErr(ERROR_LEVEL, ER_CONN_TCP_BIND_FAIL, msg_buff);
 #else
       LogErr(ERROR_LEVEL, ER_CONN_TCP_BIND_FAIL, strerror(socket_errno));
@@ -505,9 +589,9 @@ class TCP_socket {
     if (mysql_socket_listen(listener_socket, static_cast<int>(m_backlog)) < 0) {
 #ifdef _WIN32
       Socket_error_message_buf msg_buff;
-      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, socket_errno,
+      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, socket_errno,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)msg_buff,
-                    sizeof(msg_buff), NULL);
+                    sizeof(msg_buff), nullptr);
       LogErr(ERROR_LEVEL, ER_CONN_TCP_START_FAIL, msg_buff);
 #else
       LogErr(ERROR_LEVEL, ER_CONN_TCP_START_FAIL, strerror(errno));
@@ -537,12 +621,12 @@ class TCP_socket {
 */
 class Unix_socket {
   std::string m_unix_sockname;  // pathname for socket to bind to.
-  uint m_backlog;  // backlog specifying lenght of pending queue connection.
+  uint m_backlog;  // backlog specifying length of pending queue connection.
   /**
     Create a lockfile which contains the pid of the mysqld instance started
     and pathname as name of unix socket pathname appended with .lock
 
-    @retval   false if lockfile creation is successful else true if lockfile
+    @retval   False if lockfile creation is successful else true if lockfile
               file could not be created.
 
   */
@@ -577,7 +661,9 @@ class Unix_socket {
       return MYSQL_INVALID_SOCKET;
     }
 
-    if (create_lockfile()) {
+    if (create_lockfile() ||
+        DBUG_EVALUATE_IF("simulate_create_socket_lockfile_error", true,
+                         false)) {
       LogErr(ERROR_LEVEL, ER_CONN_UNIX_LOCK_FILE_FAIL);
       return MYSQL_INVALID_SOCKET;
     }
@@ -750,8 +836,8 @@ bool Unix_socket::create_lockfile() {
 ///////////////////////////////////////////////////////////////////////////
 
 Mysqld_socket_listener::Mysqld_socket_listener(
-    const std::list<std::string> &bind_addresses, uint tcp_port,
-    const char *admin_bind_addr, uint admin_tcp_port,
+    const std::list<Bind_address_info> &bind_addresses, uint tcp_port,
+    const Bind_address_info &admin_bind_addr, uint admin_tcp_port,
     bool use_separate_thread_for_admin, uint backlog, uint port_timeout,
     std::string unix_sockname)
     : m_bind_addresses(bind_addresses),
@@ -799,15 +885,16 @@ void Mysqld_socket_listener::add_socket_to_listener(
 }
 
 void Mysqld_socket_listener::setup_connection_events(
-    const socket_map_t &socket_map) {
+    const socket_vector_t &socket_vector) {
 #ifdef HAVE_POLL
-  const socket_map_t::size_type total_number_of_addresses_to_bind =
-      socket_map.size();
+  const socket_vector_t::size_type total_number_of_addresses_to_bind =
+      socket_vector.size();
   m_poll_info.m_fds.reserve(total_number_of_addresses_to_bind);
   m_poll_info.m_pfs_fds.reserve(total_number_of_addresses_to_bind);
 #endif
 
-  for (const auto &element : socket_map) add_socket_to_listener(element.first);
+  for (const auto &socket_element : socket_vector)
+    add_socket_to_listener(socket_element.m_socket);
 }
 
 /**
@@ -842,9 +929,9 @@ static bool accept_connection(MYSQL_SOCKET listen_sock,
     if ((connection_errors_accept++ & 255) == 0) {  // This can happen often
 #ifdef _WIN32
       Socket_error_message_buf msg_buff;
-      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, socket_errno,
+      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, socket_errno,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)msg_buff,
-                    sizeof(msg_buff), NULL);
+                    sizeof(msg_buff), nullptr);
       LogErr(ERROR_LEVEL, ER_CONN_SOCKET_ACCEPT_FAILED, msg_buff);
 #else
       LogErr(ERROR_LEVEL, ER_CONN_SOCKET_ACCEPT_FAILED, strerror(errno));
@@ -872,7 +959,7 @@ bool check_connection_refused_by_tcp_wrapper(MYSQL_SOCKET connect_sock) {
   struct request_info req;
   signal(SIGCHLD, SIG_DFL);
   request_init(&req, RQ_DAEMON, libwrap_name, RQ_FILE,
-               mysql_socket_getfd(connect_sock), NULL);
+               mysql_socket_getfd(connect_sock), nullptr);
   fromhost(&req);
 
   if (!hosts_access(&req)) {
@@ -965,30 +1052,37 @@ static inline void wait_for_admin_thread_started() {
   Listen to admin interface and accept incoming connection on it.
   This function is run in a separate thread.
 
-  @param admin_socket  pointer to a socket for listening to admin interface
-
   @return operation result. false on success, true on error
 */
-static bool handle_admin_socket(MYSQL_SOCKET *admin_socket) {
+static bool handle_admin_socket(
+    /// pointer to a socket for listening to admin interface
+    MYSQL_SOCKET admin_socket
+#ifdef HAVE_SETNS
+    ,
+    /// network namespace associated with the socket
+    const std::string &network_namespace_for_listening_socket
+#endif
+) {
   mark_admin_thread_started();
 
 #ifdef HAVE_POLL
   static const int NUMBER_OF_POLLED_FDS = 1;
   struct pollfd fds[NUMBER_OF_POLLED_FDS] = {
-      {mysql_socket_getfd(*admin_socket), POLLIN, 0}};
+      {mysql_socket_getfd(admin_socket), POLLIN, 0}};
 #else
   fd_set client_fds;
   FD_ZERO(&client_fds);
-  FD_SET(mysql_socket_getfd(*admin_socket), &client_fds);
-  int max_used_connection = mysql_socket_getfd(*admin_socket);
+  FD_SET(mysql_socket_getfd(admin_socket), &client_fds);
+  const int max_used_connection = mysql_socket_getfd(admin_socket);
 #endif
 
   while (!connection_events_loop_aborted()) {
 #ifdef HAVE_POLL
-    int retval = poll(fds, NUMBER_OF_POLLED_FDS, -1);
+    const int retval = poll(fds, NUMBER_OF_POLLED_FDS, -1);
 #else
     fd_set read_fds = client_fds;
-    int retval = select(max_used_connection, &read_fds, 0, 0, 0);
+    const int retval =
+        select(max_used_connection, &read_fds, nullptr, nullptr, nullptr);
 #endif
 
     if (retval < 0 && socket_errno != SOCKET_EINTR) {
@@ -997,7 +1091,7 @@ static bool handle_admin_socket(MYSQL_SOCKET *admin_socket) {
         There is not much details to report about the client,
         increment the server global status variable.
       */
-      ++connection_errors_select;
+      ++connection_errors_query_block;
       if (!select_errors++ && !connection_events_loop_aborted())
         LogErr(ERROR_LEVEL, ER_CONN_SOCKET_SELECT_FAILED, socket_errno);
     }
@@ -1006,14 +1100,36 @@ static bool handle_admin_socket(MYSQL_SOCKET *admin_socket) {
 
     if (connection_events_loop_aborted()) return false;
 
+#ifdef HAVE_SETNS
+    /*
+      If a network namespace is specified for a listening socket then set this
+      network namespace as active before call to accept().
+      It is not clear from manuals whether a socket returned by a call to
+      accept() borrows a network namespace from a server socket used for
+      accepting a new connection. For that reason, assign a network namespace
+      explicitly before calling accept().
+    */
+    if (!network_namespace_for_listening_socket.empty() &&
+        set_network_namespace(network_namespace_for_listening_socket))
+      return true;
+#endif
+
     MYSQL_SOCKET connect_sock;
-    if (accept_connection(*admin_socket, &connect_sock)) continue;
+    const bool accept_retval = accept_connection(admin_socket, &connect_sock);
+
+#ifdef HAVE_SETNS
+    if (!network_namespace_for_listening_socket.empty() &&
+        restore_original_network_namespace())
+      return true;
+#endif
+
+    if (accept_retval) continue;
 
 #ifdef HAVE_LIBWRAP
     if (check_connection_refused_by_tcp_wrapper(connect_sock)) return true;
 #endif  // HAVE_LIBWRAP
 
-    Channel_info *channel_info =
+    Channel_info_tcpip_socket *channel_info =
         new (std::nothrow) Channel_info_tcpip_socket(connect_sock, true);
     if (channel_info == nullptr) {
       (void)mysql_socket_shutdown(connect_sock, SHUT_RDWR);
@@ -1022,6 +1138,12 @@ static bool handle_admin_socket(MYSQL_SOCKET *admin_socket) {
       return true;
     }
 
+#ifdef HAVE_SETNS
+    if (!network_namespace_for_listening_socket.empty())
+      channel_info->set_network_namespace(
+          network_namespace_for_listening_socket);
+#endif
+
     Connection_handler_manager *mgr =
         Connection_handler_manager::get_instance();
 
@@ -1031,6 +1153,8 @@ static bool handle_admin_socket(MYSQL_SOCKET *admin_socket) {
   return false;
 }
 
+using admin_thread_arg_t = std::pair<MYSQL_SOCKET, std::string>;
+
 /**
   Initialize thread's internal structures, run thread loop,
   deinitialize thread's internal structure on thread exit.
@@ -1040,11 +1164,18 @@ static bool handle_admin_socket(MYSQL_SOCKET *admin_socket) {
 extern "C" void *admin_socket_thread(void *arg) {
   my_thread_init();
 
-  MYSQL_SOCKET *admin_socket = (MYSQL_SOCKET *)arg;
-  (void)handle_admin_socket(admin_socket);
+  std::unique_ptr<admin_thread_arg_t> arg_for_admin_socket_thread(
+      (admin_thread_arg_t *)arg);
+
+  (void)handle_admin_socket(arg_for_admin_socket_thread->first
+#ifdef HAVE_SETNS
+                            ,
+                            arg_for_admin_socket_thread->second
+#endif
+  );
 
   my_thread_end();
-  my_thread_exit(0);
+  my_thread_exit(nullptr);
 
   return nullptr;
 }
@@ -1076,15 +1207,23 @@ static inline void initialize_thread_context() {
   Spawn a thread for handling incoming connections request on admin interface.
 
   @param admin_socket  A socket to listen corresponding admin interface.
-
+  @param network_namespace Network namespace to use for communicating
+                           via admin socket
   @return Operation result. false on success, true on failure.
 */
-static inline bool spawn_admin_thread(MYSQL_SOCKET *admin_socket) {
+static inline bool spawn_admin_thread(MYSQL_SOCKET admin_socket,
+                                      const std::string &network_namespace) {
   initialize_thread_context();
 
-  int ret = mysql_thread_create(
+  admin_thread_arg_t *arg_for_admin_socket_thread =
+      new (std::nothrow) admin_thread_arg_t(admin_socket, network_namespace);
+
+  if (arg_for_admin_socket_thread == nullptr) return true;
+
+  const int ret = mysql_thread_create(
       key_thread_handle_con_admin_sockets, &admin_socket_thread_id,
-      &admin_socket_thread_attrib, admin_socket_thread, (void *)admin_socket);
+      &admin_socket_thread_attrib, admin_socket_thread,
+      (void *)arg_for_admin_socket_thread);
 
   (void)my_thread_attr_destroy(&admin_socket_thread_attrib);
 
@@ -1098,6 +1237,16 @@ static inline bool spawn_admin_thread(MYSQL_SOCKET *admin_socket) {
   return false;
 }
 
+bool Mysqld_socket_listener::check_and_spawn_admin_connection_handler_thread()
+    const {
+  if (m_use_separate_thread_for_admin) {
+    if (spawn_admin_thread(m_admin_interface_listen_socket,
+                           m_admin_bind_address.network_namespace))
+      return true;
+  }
+  return false;
+}
+
 bool Mysqld_socket_listener::setup_listener() {
   /*
     It's matter to add a socket for admin connection listener firstly,
@@ -1105,31 +1254,35 @@ bool Mysqld_socket_listener::setup_listener() {
     It is done in order to check availability of new incoming connection
     on admin interface with higher priority than on other interfaces..
   */
-  if (m_admin_bind_address) {
-    TCP_socket tcp_socket(m_admin_bind_address, m_admin_tcp_port, m_backlog,
-                          m_port_timeout);
+  if (!m_admin_bind_address.address.empty()) {
+    TCP_socket tcp_socket(m_admin_bind_address.address,
+                          m_admin_bind_address.network_namespace,
+                          m_admin_tcp_port, m_backlog, m_port_timeout);
 
     MYSQL_SOCKET mysql_socket = tcp_socket.get_listener_socket();
     if (mysql_socket.fd == INVALID_SOCKET) return true;
 
     m_admin_interface_listen_socket = mysql_socket;
 
-    if (m_use_separate_thread_for_admin) {
-      if (spawn_admin_thread(&m_admin_interface_listen_socket)) return true;
-    } else
-      m_socket_map.insert(std::pair<MYSQL_SOCKET, bool>(mysql_socket, false));
+    if (!m_use_separate_thread_for_admin) {
+      m_socket_vector.emplace_back(mysql_socket, Socket_type::TCP_SOCKET,
+                                   &m_admin_bind_address.network_namespace,
+                                   Socket_interface_type::ADMIN_INTERFACE);
+    }
   }
 
   // Setup tcp socket listener
   if (m_tcp_port) {
-    for (const auto &bind_address : m_bind_addresses) {
-      TCP_socket tcp_socket(bind_address, m_tcp_port, m_backlog,
-                            m_port_timeout);
+    for (const auto &bind_address_info : m_bind_addresses) {
+      TCP_socket tcp_socket(bind_address_info.address,
+                            bind_address_info.network_namespace, m_tcp_port,
+                            m_backlog, m_port_timeout);
 
       MYSQL_SOCKET mysql_socket = tcp_socket.get_listener_socket();
       if (mysql_socket.fd == INVALID_SOCKET) return true;
-
-      m_socket_map.insert(std::pair<MYSQL_SOCKET, bool>(mysql_socket, false));
+      m_socket_vector.emplace_back(mysql_socket, Socket_type::TCP_SOCKET,
+                                   &bind_address_info.network_namespace,
+                                   Socket_interface_type::DEFAULT_INTERFACE);
     }
   }
 #if defined(HAVE_SYS_UN_H)
@@ -1139,19 +1292,18 @@ bool Mysqld_socket_listener::setup_listener() {
 
     MYSQL_SOCKET mysql_socket = unix_socket.get_listener_socket();
     if (mysql_socket.fd == INVALID_SOCKET) return true;
-
-    m_socket_map.insert(std::pair<MYSQL_SOCKET, bool>(mysql_socket, true));
+    Listen_socket s(mysql_socket, Socket_type::UNIX_SOCKET);
+    m_socket_vector.push_back(s);
     m_unlink_sockname = true;
   }
 #endif /* HAVE_SYS_UN_H */
 
-  setup_connection_events(m_socket_map);
+  setup_connection_events(m_socket_vector);
 
   return false;
 }
 
-MYSQL_SOCKET Mysqld_socket_listener::get_ready_socket(
-    bool *is_unix_socket, bool *is_admin_socket) const {
+const Listen_socket *Mysqld_socket_listener::get_listen_socket() const {
 /*
   In case admin interface was set up, then first check whether an admin socket
   ready to accept a new connection. Doing this way provides higher priority
@@ -1159,57 +1311,47 @@ MYSQL_SOCKET Mysqld_socket_listener::get_ready_socket(
 */
 #ifdef HAVE_POLL
   uint start_index = 0;
-
-  if (m_admin_bind_address && !m_use_separate_thread_for_admin) {
+  if (!m_admin_bind_address.address.empty() &&
+      !m_use_separate_thread_for_admin) {
     if (m_poll_info.m_fds[0].revents & POLLIN) {
-      *is_unix_socket = false;
-      *is_admin_socket = true;
-      return m_admin_interface_listen_socket;
+      return &m_socket_vector[0];
     } else
       start_index = 1;
   }
 
-  *is_admin_socket = false;
-  for (uint i = start_index; i < m_socket_map.size(); ++i) {
+  for (uint i = start_index; i < m_socket_vector.size(); ++i) {
     if (m_poll_info.m_fds[i].revents & POLLIN) {
-      MYSQL_SOCKET listen_sock = m_poll_info.m_pfs_fds[i];
-      *is_unix_socket = m_socket_map.at(listen_sock);
-
-      return listen_sock;
+      return &m_socket_vector[i];
     }
   }
 
 #else  // HAVE_POLL
-  if (m_admin_bind_address && !m_use_separate_thread_for_admin &&
+  if (!m_admin_bind_address.address.empty() &&
+      !m_use_separate_thread_for_admin &&
       FD_ISSET(mysql_socket_getfd(m_admin_interface_listen_socket),
-               &m_select_info.m_read_fds)) {
-    *is_unix_socket = false;
-    *is_admin_socket = true;
-    return m_admin_interface_listen_socket;
+               const_cast<fd_set *>(&m_select_info.m_read_fds))) {
+    return &m_socket_vector[0];
   }
 
-  *is_admin_socket = false;
-  for (socket_map_const_iterator_t sock_map_const_iter = m_socket_map.cbegin();
-       sock_map_const_iter != m_socket_map.cend(); ++sock_map_const_iter) {
-    if (FD_ISSET(mysql_socket_getfd(sock_map_const_iter->first),
-                 &m_select_info.m_read_fds)) {
-      *is_unix_socket = sock_map_const_iter->second;
-      return sock_map_const_iter->first;
+  for (const auto &socket_element : m_socket_vector) {
+    if (FD_ISSET(mysql_socket_getfd(socket_element.m_socket),
+                 const_cast<fd_set *>(&m_select_info.m_read_fds))) {
+      return &socket_element;
     }
   }
 
 #endif  // HAVE_POLL
-
-  return MYSQL_INVALID_SOCKET;
+  return nullptr;
+  ;
 }
 
 Channel_info *Mysqld_socket_listener::listen_for_connection_event() {
 #ifdef HAVE_POLL
-  int retval = poll(&m_poll_info.m_fds[0], m_socket_map.size(), -1);
+  int retval = poll(&m_poll_info.m_fds[0], m_socket_vector.size(), -1);
 #else
   m_select_info.m_read_fds = m_select_info.m_client_fds;
   int retval = select((int)m_select_info.m_max_used_connection,
-                      &m_select_info.m_read_fds, 0, 0, 0);
+                      &m_select_info.m_read_fds, nullptr, nullptr, nullptr);
 #endif
 
   if (retval < 0 && socket_errno != SOCKET_EINTR) {
@@ -1218,53 +1360,89 @@ Channel_info *Mysqld_socket_listener::listen_for_connection_event() {
       There is not much details to report about the client,
       increment the server global status variable.
     */
-    ++connection_errors_select;
+    ++connection_errors_query_block;
     if (!select_errors++ && !connection_events_loop_aborted())
       LogErr(ERROR_LEVEL, ER_CONN_SOCKET_SELECT_FAILED, socket_errno);
   }
 
-  if (retval < 0 || connection_events_loop_aborted()) return NULL;
+  if (retval < 0 || connection_events_loop_aborted()) return nullptr;
 
   /* Is this a new connection request ? */
-  bool is_unix_socket = false, is_admin_sock;
-  MYSQL_SOCKET listen_sock = get_ready_socket(&is_unix_socket, &is_admin_sock);
+  const Listen_socket *listen_socket = get_listen_socket();
   /*
     When poll/select returns control flow then at least one ready server socket
     must exist. Check that get_ready_socket() returns a valid socket.
   */
-  DBUG_ASSERT(mysql_socket_getfd(listen_sock) != INVALID_SOCKET);
-
+  assert(listen_socket != nullptr);
   MYSQL_SOCKET connect_sock;
-  if (accept_connection(listen_sock, &connect_sock)) return nullptr;
+#ifdef HAVE_SETNS
+  /*
+    If a network namespace is specified for a listening socket then set this
+    network namespace as active before call to accept().
+    It is not clear from manuals whether a socket returned by a call to
+    accept() borrows a network namespace from a server socket used for
+    accepting a new connection. For that reason, assign a network namespace
+    explicitly before calling accept().
+  */
+  std::string network_namespace_for_listening_socket;
+  if (listen_socket->m_socket_type == Socket_type::TCP_SOCKET) {
+    network_namespace_for_listening_socket =
+        (listen_socket->m_network_namespace != nullptr
+             ? *listen_socket->m_network_namespace
+             : std::string(""));
+    if (!network_namespace_for_listening_socket.empty() &&
+        set_network_namespace(network_namespace_for_listening_socket))
+      return nullptr;
+  }
+#endif
+  if (accept_connection(listen_socket->m_socket, &connect_sock)) {
+#ifdef HAVE_SETNS
+    if (!network_namespace_for_listening_socket.empty())
+      (void)restore_original_network_namespace();
+#endif
+    return nullptr;
+  }
+
+#ifdef HAVE_SETNS
+  if (!network_namespace_for_listening_socket.empty() &&
+      restore_original_network_namespace())
+    return nullptr;
+#endif
 
 #ifdef HAVE_LIBWRAP
-  if (!is_unix_socket &&
+  if ((listen_socket->m_socket_type == Socket_type::TCP_SOCKET) &&
       check_connection_refused_by_tcp_wrapper(connect_sock)) {
     return nullptr;
   }
 #endif  // HAVE_LIBWRAP
 
-  Channel_info *channel_info = NULL;
-  if (is_unix_socket)
+  Channel_info *channel_info = nullptr;
+  if (listen_socket->m_socket_type == Socket_type::UNIX_SOCKET)
     channel_info = new (std::nothrow) Channel_info_local_socket(connect_sock);
   else
-    channel_info = new (std::nothrow)
-        Channel_info_tcpip_socket(connect_sock, is_admin_sock);
-  if (channel_info == NULL) {
+    channel_info = new (std::nothrow) Channel_info_tcpip_socket(
+        connect_sock, (listen_socket->m_socket_interface ==
+                       Socket_interface_type::ADMIN_INTERFACE));
+  if (channel_info == nullptr) {
     (void)mysql_socket_shutdown(connect_sock, SHUT_RDWR);
     (void)mysql_socket_close(connect_sock);
     connection_errors_internal++;
-    return NULL;
+    return nullptr;
   }
 
+#ifdef HAVE_SETNS
+  if (listen_socket->m_socket_type == Socket_type::TCP_SOCKET &&
+      !network_namespace_for_listening_socket.empty())
+    static_cast<Channel_info_tcpip_socket *>(channel_info)
+        ->set_network_namespace(network_namespace_for_listening_socket);
+#endif
   return channel_info;
 }
 
 void Mysqld_socket_listener::close_listener() {
-  for (socket_map_const_iterator_t sock_map_const_iter = m_socket_map.cbegin();
-       sock_map_const_iter != m_socket_map.cend(); ++sock_map_const_iter) {
-    (void)mysql_socket_shutdown(sock_map_const_iter->first, SHUT_RDWR);
-    (void)mysql_socket_close(sock_map_const_iter->first);
+  for (const auto &socket_element : m_socket_vector) {
+    (void)mysql_socket_shutdown(socket_element.m_socket, SHUT_RDWR);
+    (void)mysql_socket_close(socket_element.m_socket);
   }
 
   /*
@@ -1274,28 +1452,36 @@ void Mysqld_socket_listener::close_listener() {
     referenced by the data member m_admin_interface_listen_socket.
   */
   if (m_use_separate_thread_for_admin) {
+    if (admin_socket_thread_id != my_thread_handle{}) {
 #ifdef _WIN32
-    /*
-      For Windows, first close the socket referenced by the data member
-      m_admin_interface_listen_socket. It results in return from select()
-      API call running from a separate thread.
-    */
-    (void)mysql_socket_close(m_admin_interface_listen_socket);
-    my_thread_join(&admin_socket_thread_id, nullptr);
+      /*
+        For Windows, first close the socket referenced by the data member
+        m_admin_interface_listen_socket. It results in return from select()
+        API call running from a separate thread.
+      */
+      (void)mysql_socket_close(m_admin_interface_listen_socket);
+      m_admin_interface_listen_socket.fd = INVALID_SOCKET;
+      my_thread_join(&admin_socket_thread_id, nullptr);
 #else
-    // First, finish listening thread.
-    pthread_kill(admin_socket_thread_id.thread, SIGUSR1);
-    my_thread_join(&admin_socket_thread_id, nullptr);
+      // First, finish listening thread.
+      pthread_kill(admin_socket_thread_id.thread, SIGALRM);
+      my_thread_join(&admin_socket_thread_id, nullptr);
+
+#endif
+      mysql_mutex_destroy(&LOCK_start_admin_thread);
+      mysql_cond_destroy(&COND_start_admin_thread);
+    }  // if (admin_socket_thread_id != my_thread_handle{})
     /*
       After a thread listening on admin interface finished, it is safe
-      to close listening socket.
+      to close listening socket. But socket is opened before thread is
+      spawned, so we may need to close the socket even if
+      admin_socket_thread_id has not been assigned.
     */
-    (void)mysql_socket_close(m_admin_interface_listen_socket);
-#endif
-
-    mysql_mutex_destroy(&LOCK_start_admin_thread);
-    mysql_cond_destroy(&COND_start_admin_thread);
-  }
+    if (m_admin_interface_listen_socket.fd != INVALID_SOCKET) {
+      (void)mysql_socket_close(m_admin_interface_listen_socket);
+      m_admin_interface_listen_socket.fd = INVALID_SOCKET;
+    }
+  }  // use_separate_thread_for_admin
 
 #if defined(HAVE_SYS_UN_H)
   if (m_unix_sockname != "" && m_unlink_sockname) {
@@ -1305,5 +1491,10 @@ void Mysqld_socket_listener::close_listener() {
   }
 #endif
 
-  m_socket_map.clear();
+#ifdef HAVE_SETNS
+  release_network_namespace_resources();
+#endif
+
+  m_socket_vector.clear();
+  m_bind_addresses.clear();
 }
