@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +34,7 @@
 #include <optional>
 
 #include "my_dbug.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"      // MAX_BLOB_WIDTH
 #include "sql/item_func.h"  // agg_arg_charsets_for_comparison()
 #include "sql/sql_class.h"  // THD
@@ -158,8 +158,17 @@ bool Item_func_regexp::set_pattern() {
 bool Item_func_regexp_instr::fix_fields(THD *thd, Item **arguments) {
   if (Item_func_regexp::fix_fields(thd, arguments)) return true;
 
-  if (return_option().has_value() && return_option() != 0 &&
-      return_option() != 1) {
+  const int the_index = retopt_arg_pos();
+  if (the_index != -1 && arg_count >= static_cast<uint>(the_index) + 1) {
+    if (!args[the_index]->const_item()) {
+      return false;
+    }
+  }
+
+  std::optional<int> retopt = return_option();
+  if (current_thd->is_error()) return true;
+
+  if (retopt.has_value() && retopt != 0 && retopt != 1) {
     my_error(ER_WRONG_ARGUMENTS, MYF(0),
              "regexp_instr: return_option must be 1 or 0.");
     return true;
@@ -183,6 +192,12 @@ longlong Item_func_regexp_instr::val_int() {
   std::optional<int> occ = occurrence();
   std::optional<int> retopt = return_option();
 
+  if (retopt.has_value() && retopt != 0 && retopt != 1) {
+    my_error(ER_WRONG_ARGUMENTS, MYF(0),
+             "regexp_instr: return_option must be 1 or 0.");
+    return error_int();
+  }
+
   if (set_pattern() || !pos.has_value() || !occ.has_value() ||
       !retopt.has_value()) {
     return error_int();
@@ -320,9 +335,9 @@ Item_func_icu_version::Item_func_icu_version(const POS &pos)
                               strlen(icu_version_string()), system_charset_info,
                               DERIVATION_SYSCONST) {}
 
-bool Item_func_icu_version::itemize(Parse_context *pc, Item **res) {
+bool Item_func_icu_version::do_itemize(Parse_context *pc, Item **res) {
   if (skip_itemize(res)) return false;
-  if (super::itemize(pc, res)) return true;
+  if (super::do_itemize(pc, res)) return true;
   pc->thd->lex->set_stmt_unsafe(LEX::BINLOG_STMT_UNSAFE_SYSTEM_FUNCTION);
   return false;
 }
