@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -74,17 +74,16 @@ Item_row::Item_row(Item *head, const mem_root_deque<Item *> &tail)
   }
 }
 
-bool Item_row::itemize(Parse_context *pc, Item **res) {
+bool Item_row::do_itemize(Parse_context *pc, Item **res) {
   if (skip_itemize(res)) return false;
-  if (super::itemize(pc, res)) return true;
+  if (super::do_itemize(pc, res)) return true;
   for (uint i = 0; i < arg_count; i++) {
     if (items[i]->itemize(pc, &items[i])) return true;
   }
   return false;
 }
 
-void Item_row::illegal_method_call(
-    const char *method MY_ATTRIBUTE((unused))) const {
+void Item_row::illegal_method_call(const char *method [[maybe_unused]]) const {
   DBUG_TRACE;
   DBUG_PRINT("error", ("!!! %s method was called for row item", method));
   assert(0);
@@ -92,7 +91,7 @@ void Item_row::illegal_method_call(
 }
 
 bool Item_row::fix_fields(THD *thd, Item **) {
-  assert(fixed == 0);
+  assert(!fixed);
   null_value = false;
   set_nullable(false);
   bool types_assigned = true;
@@ -116,7 +115,7 @@ bool Item_row::fix_fields(THD *thd, Item **) {
     // item->is_null() may have raised an error.
     if (thd->is_error()) return true;
 
-    set_nullable(is_nullable() | item->is_nullable());
+    set_nullable(is_nullable() || item->is_nullable());
     add_accum_properties(item);
   }
   if (types_assigned) set_data_type(MYSQL_TYPE_NULL);
