@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -613,7 +612,7 @@ FullTextSearchIterator::FullTextSearchIterator(THD *thd, TABLE *table,
   // hand, maintains alternative plans with and without index scans throughout
   // the planning, and doesn't determine whether it should use the indexed or
   // non-indexed plan until the full query plan has been constructed.
-  if (thd->lex->using_hypergraph_optimizer()) {
+  if (thd->lex->using_hypergraph_optimizer) {
     // Should not already be enabled.
     assert(!ft_func->score_from_index_scan);
     // Should operate on the main object.
@@ -923,7 +922,7 @@ bool TableValueConstructorIterator::Init() {
 }
 
 int TableValueConstructorIterator::Read() {
-  if (m_row_it == m_row_value_list.end()) return -1;
+  if (*m_examined_rows == m_row_value_list.size()) return -1;
 
   // If the TVC has a single row, we don't create Item_values_column reference
   // objects during resolving. We will instead use the single row directly from
@@ -941,9 +940,9 @@ int TableValueConstructorIterator::Read() {
       // Item_values_column object cannot be const.
       ref->set_value(const_cast<Item *>(value));
     }
+    ++m_row_it;
   }
 
-  ++m_row_it;
   ++*m_examined_rows;
   return 0;
 }
