@@ -35,7 +35,6 @@ enum options_mysqld {
   OPT_BINLOG_FORMAT,
   OPT_BINLOG_IGNORE_DB,
   OPT_BINLOG_MAX_FLUSH_QUEUE_TIME,
-  OPT_EXPIRE_LOGS_DAYS,
   OPT_BINLOG_EXPIRE_LOGS_SECONDS,
   OPT_BIN_LOG,
   OPT_BOOTSTRAP,
@@ -128,13 +127,14 @@ enum options_mysqld {
   OPT_USE_ADMIN_SSL,
   OPT_TRANSACTION_WRITE_SET_EXTRACTION,
   OPT_SHOW_SLAVE_AUTH_INFO_DEPRECATED,
-  OPT_DISCONNECT_SLAVE_EVENT_COUNT,
-  OPT_ABORT_SLAVE_EVENT_COUNT,
   OPT_REPLICA_PARALLEL_TYPE,
   OPT_SSL_SESSION_CACHE_TIMEOUT,
   OPT_REPLICA_PARALLEL_WORKERS,
   OPT_OLD_STYLE_USER_LIMITS,
-  OPT_SYNC_RELAY_LOG_INFO
+  OPT_SYNC_RELAY_LOG_INFO,
+  OPT_BINLOG_TRANSACTION_DEPENDENCY_TRACKING,
+  OPT_CHARACTER_SET_CLIENT_HANDSHAKE,
+  OPT_OLD_OPTION
 };
 
 #endif  // OPTIONS_MYSQLD_INCLUDED
