@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,13 +32,13 @@
 
 #include "field_types.h"
 #include "lex_string.h"
-#include "m_ctype.h"
 #include "my_alloc.h"
 #include "my_base.h"
 #include "my_compiler.h"
 
 #include "my_inttypes.h"
 #include "my_sys.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/derror.h"
@@ -84,32 +83,10 @@ struct Column_parse_context : public Parse_context {
 */
 class PT_column_attr_base : public Parse_tree_node_tmpl<Column_parse_context> {
  protected:
-  PT_column_attr_base() = default;
+  explicit PT_column_attr_base(const POS &pos)
+      : Parse_tree_node_tmpl<Column_parse_context>(pos) {}
 
  public:
-  enum Attr_type {
-    AT_NONE,
-    AT_COLLATE_COLUMN_ATTR,
-    AT_DEFAULT_COLUMN_ATTR,
-    AT_GENERATED_DEFAULT_VAL_COLUMN_ATTR,
-    AT_SECONDARY_COLUMN_ATTR,
-    AT_NOT_NULL_COLUMN_ATTR,
-    AT_NULL_COLUMN_ATTR,
-    AT_UNIQUE_KEY_COLUMN_ATTR,
-    AT_CLUSTERING_KEY_COLUMN_ATTR,
-    AT_PRIMARY_KEY_COLUMN_ATTR,
-    AT_CHECK_CONSTRAINT_COLUMN_ATTR,
-    AT_CONSTRAINT_ENFORCEMENT_ATTR,
-    AT_COMMENT_COLUMN_ATTR,
-    AT_ON_UPDATE_COLUMN_ATTR,
-    AT_AUTO_INCREMENT_COLUMN_ATTR,
-    AT_SERIAL_DEFAULT_VALUE_COLUMN_ATTR,
-    AT_COLUMN_FORMAT_COLUMN_ATTR,
-    AT_STORAGE_MEDIA_COLUMN_ATTR,
-    AT_SRID_COLUMN_ATTR,
-    AT_COLUMN_VISIBILITY_ATTR
-  };
-
   typedef decltype(Alter_info::flags) alter_info_flags_t;
 
   virtual void apply_type_flags(ulong *) const {}
@@ -129,7 +106,6 @@ class PT_column_attr_base : public Parse_tree_node_tmpl<Column_parse_context> {
       Sql_check_constraint_spec_list *check_const_list [[maybe_unused]]) {
     return false;
   }
-  virtual Attr_type attr_type() const { return AT_NONE; }
 
   /**
     Check for the [NOT] ENFORCED characteristic.
@@ -171,12 +147,11 @@ class PT_column_attr_base : public Parse_tree_node_tmpl<Column_parse_context> {
 */
 class PT_null_column_attr : public PT_column_attr_base {
  public:
+  explicit PT_null_column_attr(const POS &pos) : PT_column_attr_base(pos) {}
   void apply_type_flags(ulong *type_flags) const override {
     *type_flags &= ~NOT_NULL_FLAG;
     *type_flags |= EXPLICIT_NULL_FLAG;
   }
-
-  enum Attr_type attr_type() const override { return AT_NULL_COLUMN_ATTR; }
 };
 
 /**
@@ -185,11 +160,11 @@ class PT_null_column_attr : public PT_column_attr_base {
   @ingroup ptn_column_attrs
 */
 class PT_not_null_column_attr : public PT_column_attr_base {
+ public:
+  explicit PT_not_null_column_attr(const POS &pos) : PT_column_attr_base(pos) {}
   void apply_type_flags(ulong *type_flags) const override {
     *type_flags |= NOT_NULL_FLAG;
   }
-
-  enum Attr_type attr_type() const override { return AT_NOT_NULL_COLUMN_ATTR; }
 };
 
 /**
@@ -199,11 +174,11 @@ class PT_not_null_column_attr : public PT_column_attr_base {
 */
 class PT_secondary_column_attr : public PT_column_attr_base {
  public:
+  explicit PT_secondary_column_attr(const POS &pos)
+      : PT_column_attr_base(pos) {}
   void apply_type_flags(unsigned long *type_flags) const override {
     *type_flags |= NOT_SECONDARY_FLAG;
   }
-
-  enum Attr_type attr_type() const override { return AT_SECONDARY_COLUMN_ATTR; }
 };
 
 /**
@@ -211,39 +186,22 @@ class PT_secondary_column_attr : public PT_column_attr_base {
 
   @ingroup ptn_column_attrs
 */
-class PT_unique_key_column_attr : public PT_column_attr_base {
+class PT_unique_combo_clustering_key_column_attr : public PT_column_attr_base {
  public:
-  void apply_type_flags(ulong *type_flags) const override {
-    *type_flags |= UNIQUE_FLAG;
+  explicit PT_unique_combo_clustering_key_column_attr(const POS &pos, enum keytype key_type)
+      : PT_column_attr_base(pos), m_key_type(key_type) {}
+
+  void apply_type_flags(ulong *type_flags) const noexcept override {
+    if (m_key_type & KEYTYPE_UNIQUE) *type_flags |= UNIQUE_FLAG;
+    if (m_key_type & KEYTYPE_CLUSTERING) *type_flags |= CLUSTERING_FLAG;
   }
 
   void apply_alter_info_flags(ulonglong *flags) const override {
     *flags |= Alter_info::ALTER_ADD_INDEX;
   }
 
-  enum Attr_type attr_type() const override {
-    return AT_UNIQUE_KEY_COLUMN_ATTR;
-  }
-};
-
-/**
-  Node for the @SQL{CLUSTERING [KEY]} column attribute
-
-  @ingroup ptn_column_attrs
-*/
-class PT_clustering_key_column_attr : public PT_column_attr_base {
- public:
-  void apply_type_flags(ulong *type_flags) const override {
-    *type_flags |= CLUSTERING_FLAG;
-  }
-
-  void apply_alter_info_flags(ulonglong *flags) const override {
-    *flags |= Alter_info::ALTER_ADD_INDEX;
-  }
-
-  enum Attr_type attr_type() const override {
-    return AT_CLUSTERING_KEY_COLUMN_ATTR;
-  }
+ private:
+  const enum keytype m_key_type;
 };
 
 /**
@@ -253,6 +211,9 @@ class PT_clustering_key_column_attr : public PT_column_attr_base {
 */
 class PT_primary_key_column_attr : public PT_column_attr_base {
  public:
+  explicit PT_primary_key_column_attr(const POS &pos)
+      : PT_column_attr_base(pos) {}
+
   void apply_type_flags(ulong *type_flags) const override {
     *type_flags |= PRI_KEY_FLAG | NOT_NULL_FLAG;
   }
@@ -260,10 +221,6 @@ class PT_primary_key_column_attr : public PT_column_attr_base {
   void apply_alter_info_flags(ulonglong *flags) const override {
     *flags |= Alter_info::ALTER_ADD_INDEX;
   }
-
-  enum Attr_type attr_type() const override {
-    return AT_PRIMARY_KEY_COLUMN_ATTR;
-  }
 };
 
 /**
@@ -276,7 +233,9 @@ class PT_check_constraint_column_attr : public PT_column_attr_base {
   Sql_check_constraint_spec col_cc_spec;
 
  public:
-  explicit PT_check_constraint_column_attr(LEX_STRING &name, Item *expr) {
+  explicit PT_check_constraint_column_attr(const POS &pos, LEX_STRING &name,
+                                           Item *expr)
+      : super(pos) {
     col_cc_spec.name = name;
     col_cc_spec.check_expr = expr;
   }
@@ -296,14 +255,10 @@ class PT_check_constraint_column_attr : public PT_column_attr_base {
     return (check_const_list->push_back(&col_cc_spec));
   }
 
-  bool contextualize(Column_parse_context *pc) override {
-    return (super::contextualize(pc) ||
+  bool do_contextualize(Column_parse_context *pc) override {
+    return (super::do_contextualize(pc) ||
             col_cc_spec.check_expr->itemize(pc, &col_cc_spec.check_expr));
   }
-
-  enum Attr_type attr_type() const override {
-    return AT_CHECK_CONSTRAINT_COLUMN_ATTR;
-  }
 };
 
 /**
@@ -313,17 +268,13 @@ class PT_check_constraint_column_attr : public PT_column_attr_base {
 */
 class PT_constraint_enforcement_attr : public PT_column_attr_base {
  public:
-  explicit PT_constraint_enforcement_attr(bool enforced)
-      : m_enforced(enforced) {}
+  explicit PT_constraint_enforcement_attr(const POS &pos, bool enforced)
+      : PT_column_attr_base(pos), m_enforced(enforced) {}
 
   bool has_constraint_enforcement() const override { return true; }
 
   bool is_constraint_enforced() const override { return m_enforced; }
 
-  enum Attr_type attr_type() const override {
-    return AT_CONSTRAINT_ENFORCEMENT_ATTR;
-  }
-
  private:
   const bool m_enforced;
 };
@@ -337,12 +288,10 @@ class PT_comment_column_attr : public PT_column_attr_base {
   const LEX_CSTRING comment;
 
  public:
-  explicit PT_comment_column_attr(const LEX_CSTRING &comment)
-      : comment(comment) {}
+  explicit PT_comment_column_attr(const POS &pos, const LEX_CSTRING &comment)
+      : PT_column_attr_base(pos), comment(comment) {}
 
   void apply_comment(LEX_CSTRING *to) const override { *to = comment; }
-
-  enum Attr_type attr_type() const override { return AT_COMMENT_COLUMN_ATTR; }
 };
 
 /**
@@ -353,7 +302,7 @@ class PT_comment_column_attr : public PT_column_attr_base {
 class PT_collate_column_attr : public PT_column_attr_base {
  public:
   explicit PT_collate_column_attr(const POS &pos, const CHARSET_INFO *collation)
-      : m_pos(pos), m_collation(collation) {
+      : PT_column_attr_base(pos), m_collation(collation) {
     assert(m_collation != nullptr);
   }
 
@@ -367,10 +316,7 @@ class PT_collate_column_attr : public PT_column_attr_base {
     return merge_charset_and_collation(*to, m_collation, to);
   }
 
-  enum Attr_type attr_type() const override { return AT_COLLATE_COLUMN_ATTR; }
-
  private:
-  const POS m_pos;
   const CHARSET_INFO *const m_collation;
 };
 
@@ -387,20 +333,20 @@ class PT_default_column_attr : public PT_column_attr_base {
   Item *item;
 
  public:
-  explicit PT_default_column_attr(Item *item) : item(item) {}
+  explicit PT_default_column_attr(const POS &pos, Item *item)
+      : super(pos), item(item) {}
   void apply_default_value(Item **value) const override { *value = item; }
-  bool contextualize(Column_parse_context *pc) override {
+
+  bool do_contextualize(Column_parse_context *pc) override {
     if (pc->is_generated) {
       my_error(ER_WRONG_USAGE, MYF(0), "DEFAULT", "generated column");
       return true;
     }
-    return super::contextualize(pc) || item->itemize(pc, &item);
+    return super::do_contextualize(pc) || item->itemize(pc, &item);
   }
   void apply_type_flags(ulong *type_flags) const override {
     if (item->type() == Item::NULL_ITEM) *type_flags |= EXPLICIT_NULL_FLAG;
   }
-
-  enum Attr_type attr_type() const override { return AT_DEFAULT_COLUMN_ATTR; }
 };
 
 /**
@@ -412,24 +358,23 @@ class PT_on_update_column_attr : public PT_column_attr_base {
   typedef PT_column_attr_base super;
 
   const uint8 precision;
-  Item *item;
+  Item *item = nullptr;
 
  public:
-  explicit PT_on_update_column_attr(uint8 precision) : precision(precision) {}
+  explicit PT_on_update_column_attr(const POS &pos, uint8 precision)
+      : super(pos), precision(precision) {}
   void apply_on_update_value(Item **value) const override { *value = item; }
 
-  bool contextualize(Column_parse_context *pc) override {
+  bool do_contextualize(Column_parse_context *pc) override {
     if (pc->is_generated) {
       my_error(ER_WRONG_USAGE, MYF(0), "ON UPDATE", "generated column");
       return true;
     }
-    if (super::contextualize(pc)) return true;
+    if (super::do_contextualize(pc)) return true;
 
     item = new (pc->thd->mem_root) Item_func_now_local(precision);
     return item == nullptr;
   }
-
-  enum Attr_type attr_type() const override { return AT_ON_UPDATE_COLUMN_ATTR; }
 };
 
 /**
@@ -441,19 +386,18 @@ class PT_auto_increment_column_attr : public PT_column_attr_base {
   typedef PT_column_attr_base super;
 
  public:
+  explicit PT_auto_increment_column_attr(const POS &pos)
+      : PT_column_attr_base(pos) {}
+
   void apply_type_flags(ulong *type_flags) const override {
     *type_flags |= AUTO_INCREMENT_FLAG | NOT_NULL_FLAG;
   }
-  bool contextualize(Column_parse_context *pc) override {
+  bool do_contextualize(Column_parse_context *pc) override {
     if (pc->is_generated) {
       my_error(ER_WRONG_USAGE, MYF(0), "AUTO_INCREMENT", "generated column");
       return true;
     }
-    return super::contextualize(pc);
-  }
-
-  enum Attr_type attr_type() const override {
-    return AT_AUTO_INCREMENT_COLUMN_ATTR;
+    return super::do_contextualize(pc);
   }
 };
 
@@ -466,22 +410,21 @@ class PT_serial_default_value_column_attr : public PT_column_attr_base {
   typedef PT_column_attr_base super;
 
  public:
+  explicit PT_serial_default_value_column_attr(const POS &pos) : super(pos) {}
+
   void apply_type_flags(ulong *type_flags) const override {
     *type_flags |= AUTO_INCREMENT_FLAG | NOT_NULL_FLAG | UNIQUE_FLAG;
   }
   void apply_alter_info_flags(ulonglong *flags) const override {
     *flags |= Alter_info::ALTER_ADD_INDEX;
   }
-  bool contextualize(Column_parse_context *pc) override {
+  bool do_contextualize(Column_parse_context *pc) override {
     if (pc->is_generated) {
       my_error(ER_WRONG_USAGE, MYF(0), "SERIAL DEFAULT VALUE",
                "generated column");
       return true;
     }
-    return super::contextualize(pc);
-  }
-  enum Attr_type attr_type() const override {
-    return AT_SERIAL_DEFAULT_VALUE_COLUMN_ATTR;
+    return super::do_contextualize(pc);
   }
 };
 
@@ -496,19 +439,16 @@ class PT_column_format_column_attr : public PT_column_attr_base {
   column_format_type format;
 
  public:
-  explicit PT_column_format_column_attr(
+  explicit PT_column_format_column_attr(const POS &pos,
       column_format_type format, const LEX_CSTRING &zip_dict_name) noexcept
-      : format(format), m_zip_dict_name(zip_dict_name) {}
+      : super(pos), format(format), m_zip_dict_name(zip_dict_name) {}
 
   void apply_type_flags(ulong *type_flags) const override {
     *type_flags &= ~(FIELD_FLAGS_COLUMN_FORMAT_MASK);
     *type_flags |= format << FIELD_FLAGS_COLUMN_FORMAT;
   }
-  bool contextualize(Column_parse_context *pc) override {
-    return super::contextualize(pc);
-  }
-  enum Attr_type attr_type() const override {
-    return AT_COLUMN_FORMAT_COLUMN_ATTR;
+  bool do_contextualize(Column_parse_context *pc) override {
+    return super::do_contextualize(pc);
   }
   void apply_zip_dict(LEX_CSTRING *to) const noexcept override {
     *to = m_zip_dict_name;
@@ -529,22 +469,19 @@ class PT_storage_media_column_attr : public PT_column_attr_base {
   ha_storage_media media;
 
  public:
-  explicit PT_storage_media_column_attr(ha_storage_media media)
-      : media(media) {}
+  explicit PT_storage_media_column_attr(const POS &pos, ha_storage_media media)
+      : super(pos), media(media) {}
 
   void apply_type_flags(ulong *type_flags) const override {
     *type_flags &= ~(FIELD_FLAGS_STORAGE_MEDIA_MASK);
     *type_flags |= media << FIELD_FLAGS_STORAGE_MEDIA;
   }
-  bool contextualize(Column_parse_context *pc) override {
+  bool do_contextualize(Column_parse_context *pc) override {
     if (pc->is_generated) {
       my_error(ER_WRONG_USAGE, MYF(0), "STORAGE", "generated column");
       return true;
     }
-    return super::contextualize(pc);
-  }
-  enum Attr_type attr_type() const override {
-    return AT_STORAGE_MEDIA_COLUMN_ATTR;
+    return super::do_contextualize(pc);
   }
 };
 
@@ -555,13 +492,12 @@ class PT_srid_column_attr : public PT_column_attr_base {
   gis::srid_t m_srid;
 
  public:
-  explicit PT_srid_column_attr(gis::srid_t srid) : m_srid(srid) {}
+  explicit PT_srid_column_attr(const POS &pos, gis::srid_t srid)
+      : super(pos), m_srid(srid) {}
 
   void apply_srid_modifier(std::optional<gis::srid_t> *srid) const override {
     *srid = m_srid;
   }
-
-  enum Attr_type attr_type() const override { return AT_SRID_COLUMN_ATTR; }
 };
 
 /// Node for the generated default value, column attribute
@@ -569,7 +505,8 @@ class PT_generated_default_val_column_attr : public PT_column_attr_base {
   typedef PT_column_attr_base super;
 
  public:
-  PT_generated_default_val_column_attr(Item *expr) {
+  explicit PT_generated_default_val_column_attr(const POS &pos, Item *expr)
+      : super(pos) {
     m_default_value_expression.expr_item = expr;
     m_default_value_expression.set_field_stored(true);
   }
@@ -579,7 +516,7 @@ class PT_generated_default_val_column_attr : public PT_column_attr_base {
     *default_value_expression = &m_default_value_expression;
   }
 
-  bool contextualize(Column_parse_context *pc) override {
+  bool do_contextualize(Column_parse_context *pc) override {
     // GC and default value expressions are mutually exclusive and thus only
     // one is allowed to be present on the same column definition.
     if (pc->is_generated) {
@@ -587,15 +524,11 @@ class PT_generated_default_val_column_attr : public PT_column_attr_base {
       return true;
     }
     Parse_context expr_pc(pc->thd, pc->select);
-    return super::contextualize(pc) ||
+    return super::do_contextualize(pc) ||
            m_default_value_expression.expr_item->itemize(
                &expr_pc, &m_default_value_expression.expr_item);
   }
 
-  enum Attr_type attr_type() const override {
-    return AT_GENERATED_DEFAULT_VAL_COLUMN_ATTR;
-  }
-
  private:
   Value_generator m_default_value_expression;
 };
@@ -606,18 +539,16 @@ class PT_generated_default_val_column_attr : public PT_column_attr_base {
   @ingroup ptn_column_attrs
 */
 class PT_column_visibility_attr : public PT_column_attr_base {
+  typedef PT_column_attr_base super;
+
  public:
-  explicit PT_column_visibility_attr(bool is_visible)
-      : m_is_visible(is_visible) {}
+  explicit PT_column_visibility_attr(const POS &pos, bool is_visible)
+      : super(pos), m_is_visible(is_visible) {}
   void apply_type_flags(unsigned long *type_flags) const override {
     *type_flags &= ~FIELD_IS_INVISIBLE;
     if (!m_is_visible) *type_flags |= FIELD_IS_INVISIBLE;
   }
 
-  enum Attr_type attr_type() const override {
-    return AT_COLUMN_VISIBILITY_ATTR;
-  }
-
  private:
   const bool m_is_visible;
 };
@@ -634,7 +565,8 @@ class PT_type : public Parse_tree_node {
   const enum_field_types type;
 
  protected:
-  explicit PT_type(enum_field_types type) : type(type) {}
+  explicit PT_type(const POS &pos, enum_field_types type)
+      : Parse_tree_node(pos), type(type) {}
 
  public:
   virtual ulong get_type_flags() const { return 0; }
@@ -664,9 +596,9 @@ class PT_numeric_type : public PT_type {
   using Parent_type = std::remove_const<decltype(PT_type::type)>::type;
 
  public:
-  PT_numeric_type(THD *thd, Numeric_type type_arg, const char *length,
-                  const char *dec, ulong options)
-      : PT_type(static_cast<Parent_type>(type_arg)),
+  PT_numeric_type(const POS &pos, THD *thd, Numeric_type type_arg,
+                  const char *length, const char *dec, ulong options)
+      : PT_type(pos, static_cast<Parent_type>(type_arg)),
         length(length),
         dec(dec),
         options(options) {
@@ -683,9 +615,9 @@ class PT_numeric_type : public PT_type {
                    ER_THD(thd, ER_WARN_DEPRECATED_FLOAT_UNSIGNED));
     }
   }
-  PT_numeric_type(THD *thd, Int_type type_arg, const char *length,
-                  ulong options)
-      : PT_type(static_cast<enum_field_types>(type_arg)),
+  PT_numeric_type(const POS &pos, THD *thd, Int_type type_arg,
+                  const char *length, ulong options)
+      : PT_type(pos, static_cast<enum_field_types>(type_arg)),
         length(length),
         dec(nullptr),
         options(options) {
@@ -714,9 +646,10 @@ class PT_bit_type : public PT_type {
   const char *length;
 
  public:
-  PT_bit_type() : PT_type(MYSQL_TYPE_BIT), length("1") {}
-  explicit PT_bit_type(const char *length)
-      : PT_type(MYSQL_TYPE_BIT), length(length) {}
+  explicit PT_bit_type(const POS &pos)
+      : PT_type(pos, MYSQL_TYPE_BIT), length("1") {}
+  explicit PT_bit_type(const POS &pos, const char *length)
+      : PT_type(pos, MYSQL_TYPE_BIT), length(length) {}
 
   const char *get_length() const override { return length; }
 };
@@ -728,7 +661,7 @@ class PT_bit_type : public PT_type {
 */
 class PT_boolean_type : public PT_type {
  public:
-  PT_boolean_type() : PT_type(MYSQL_TYPE_TINY) {}
+  explicit PT_boolean_type(const POS &pos) : PT_type(pos, MYSQL_TYPE_TINY) {}
   const char *get_length() const override { return "1"; }
 };
 
@@ -746,17 +679,17 @@ class PT_char_type : public PT_type {
   using Parent_type = std::remove_const<decltype(PT_type::type)>::type;
 
  public:
-  PT_char_type(Char_type char_type, const char *length,
+  PT_char_type(const POS &pos, Char_type char_type, const char *length,
                const CHARSET_INFO *charset, bool force_binary = false)
-      : PT_type(static_cast<Parent_type>(char_type)),
+      : PT_type(pos, static_cast<Parent_type>(char_type)),
         length(length),
         charset(charset),
         force_binary(force_binary) {
     assert(charset == nullptr || !force_binary);
   }
-  PT_char_type(Char_type char_type, const CHARSET_INFO *charset,
+  PT_char_type(const POS &pos, Char_type char_type, const CHARSET_INFO *charset,
                bool force_binary = false)
-      : PT_char_type(char_type, "1", charset, force_binary) {}
+      : PT_char_type(pos, char_type, "1", charset, force_binary) {}
   ulong get_type_flags() const override {
     return force_binary ? BINCMP_FLAG : 0;
   }
@@ -786,16 +719,16 @@ class PT_blob_type : public PT_type {
   using Parent_type = std::remove_const<decltype(PT_type::type)>::type;
 
  public:
-  PT_blob_type(Blob_type blob_type, const CHARSET_INFO *charset,
+  PT_blob_type(const POS &pos, Blob_type blob_type, const CHARSET_INFO *charset,
                bool force_binary = false)
-      : PT_type(static_cast<Parent_type>(blob_type)),
+      : PT_type(pos, static_cast<Parent_type>(blob_type)),
         length(nullptr),
         charset(charset),
         force_binary(force_binary) {
     assert(charset == nullptr || !force_binary);
   }
-  explicit PT_blob_type(const char *length)
-      : PT_type(MYSQL_TYPE_BLOB),
+  explicit PT_blob_type(const POS &pos, const char *length)
+      : PT_type(pos, MYSQL_TYPE_BLOB),
         length(length),
         charset(&my_charset_bin),
         force_binary(false) {}
@@ -814,7 +747,7 @@ class PT_blob_type : public PT_type {
 */
 class PT_year_type : public PT_type {
  public:
-  PT_year_type() : PT_type(MYSQL_TYPE_YEAR) {}
+  explicit PT_year_type(const POS &pos) : PT_type(pos, MYSQL_TYPE_YEAR) {}
 };
 
 /**
@@ -824,7 +757,7 @@ class PT_year_type : public PT_type {
 */
 class PT_date_type : public PT_type {
  public:
-  PT_date_type() : PT_type(MYSQL_TYPE_DATE) {}
+  explicit PT_date_type(const POS &pos) : PT_type(pos, MYSQL_TYPE_DATE) {}
 };
 
 enum class Time_type : ulong {
@@ -843,8 +776,8 @@ class PT_time_type : public PT_type {
   using Parent_type = std::remove_const<decltype(PT_type::type)>::type;
 
  public:
-  PT_time_type(Time_type time_type, const char *dec)
-      : PT_type(static_cast<Parent_type>(time_type)), dec(dec) {}
+  PT_time_type(const POS &pos, Time_type time_type, const char *dec)
+      : PT_type(pos, static_cast<Parent_type>(time_type)), dec(dec) {}
 
   const char *get_dec() const override { return dec; }
 };
@@ -861,14 +794,14 @@ class PT_timestamp_type : public PT_type {
   ulong type_flags;
 
  public:
-  explicit PT_timestamp_type(const char *dec)
-      : super(MYSQL_TYPE_TIMESTAMP2), dec(dec), type_flags(0) {}
+  explicit PT_timestamp_type(const POS &pos, const char *dec)
+      : super(pos, MYSQL_TYPE_TIMESTAMP2), dec(dec), type_flags(0) {}
 
   const char *get_dec() const override { return dec; }
   ulong get_type_flags() const override { return type_flags; }
 
-  bool contextualize(Parse_context *pc) override {
-    if (super::contextualize(pc)) return true;
+  bool do_contextualize(Parse_context *pc) override {
+    if (super::do_contextualize(pc)) return true;
     /*
       TIMESTAMP fields are NOT NULL by default, unless the variable
       explicit_defaults_for_timestamp is true.
@@ -899,8 +832,8 @@ class PT_spacial_type : public PT_type {
   Field::geometry_type geo_type;
 
  public:
-  explicit PT_spacial_type(Field::geometry_type geo_type)
-      : PT_type(MYSQL_TYPE_GEOMETRY), geo_type(geo_type) {}
+  explicit PT_spacial_type(const POS &pos, Field::geometry_type geo_type)
+      : PT_type(pos, MYSQL_TYPE_GEOMETRY), geo_type(geo_type) {}
 
   const CHARSET_INFO *get_charset() const override { return &my_charset_bin; }
   uint get_uint_geom_type() const override { return geo_type; }
@@ -918,9 +851,9 @@ class PT_enum_type_tmpl : public PT_type {
   using Parent_type = std::remove_const<decltype(PT_type::type)>::type;
 
  public:
-  PT_enum_type_tmpl(List<String> *interval_list, const CHARSET_INFO *charset,
-                    bool force_binary)
-      : PT_type(static_cast<Parent_type>(enum_type)),
+  PT_enum_type_tmpl(const POS &pos, List<String> *interval_list,
+                    const CHARSET_INFO *charset, bool force_binary)
+      : PT_type(pos, static_cast<Parent_type>(enum_type)),
         interval_list(interval_list),
         charset(charset),
         force_binary(force_binary) {
@@ -950,7 +883,7 @@ typedef PT_enum_type_tmpl<Enum_type::SET> PT_set_type;
 
 class PT_serial_type : public PT_type {
  public:
-  PT_serial_type() : PT_type(MYSQL_TYPE_LONGLONG) {}
+  explicit PT_serial_type(const POS &pos) : PT_type(pos, MYSQL_TYPE_LONGLONG) {}
 
   ulong get_type_flags() const override {
     return AUTO_INCREMENT_FLAG | NOT_NULL_FLAG | UNSIGNED_FLAG | UNIQUE_FLAG;
@@ -965,7 +898,7 @@ class PT_serial_type : public PT_type {
 */
 class PT_json_type : public PT_type {
  public:
-  PT_json_type() : PT_type(MYSQL_TYPE_JSON) {}
+  explicit PT_json_type(const POS &pos) : PT_type(pos, MYSQL_TYPE_JSON) {}
   const CHARSET_INFO *get_charset() const override { return &my_charset_bin; }
 };
 
@@ -979,24 +912,24 @@ class PT_field_def_base : public Parse_tree_node {
   typedef decltype(Alter_info::flags) alter_info_flags_t;
 
  public:
-  enum_field_types type;
-  ulong type_flags;
-  const char *length;
-  const char *dec;
-  const CHARSET_INFO *charset;
-  bool has_explicit_collation;
-  uint uint_geom_type;
-  List<String> *interval_list;
-  alter_info_flags_t alter_info_flags;
-  LEX_CSTRING comment;
-  Item *default_value;
-  Item *on_update_value;
-  Value_generator *gcol_info;
+  enum_field_types type = MYSQL_TYPE_INVALID;
+  ulong type_flags = 0;
+  const char *length = nullptr;
+  const char *dec = nullptr;
+  const CHARSET_INFO *charset = nullptr;
+  bool has_explicit_collation = false;
+  uint uint_geom_type = 0;
+  List<String> *interval_list = nullptr;
+  alter_info_flags_t alter_info_flags = 0;
+  LEX_CSTRING comment = EMPTY_CSTR;
+  Item *default_value = nullptr;
+  Item *on_update_value = nullptr;
+  Value_generator *gcol_info = nullptr;
   /// Holds the expression to generate default values
-  Value_generator *default_val_info;
-  std::optional<gis::srid_t> m_srid;
+  Value_generator *default_val_info = nullptr;
+  std::optional<gis::srid_t> m_srid{};
   // List of column check constraint's specification.
-  Sql_check_constraint_spec_list *check_const_spec_list{nullptr};
+  Sql_check_constraint_spec_list *check_const_spec_list = nullptr;
   /**
     Compression dictionary name (in column definition)
     CREATE TABLE t1(
@@ -1011,19 +944,13 @@ class PT_field_def_base : public Parse_tree_node {
  protected:
   PT_type *type_node;
 
-  explicit PT_field_def_base(PT_type *type_node)
-      : has_explicit_collation(false),
-        alter_info_flags(0),
-        comment(EMPTY_CSTR),
-        default_value(nullptr),
-        on_update_value(nullptr),
-        gcol_info(nullptr),
-        default_val_info(nullptr),
-        type_node(type_node) {}
+  explicit PT_field_def_base(const POS &pos, PT_type *type_node)
+      : super(pos), type_node(type_node) {}
 
  public:
-  bool contextualize(Parse_context *pc) override {
-    if (super::contextualize(pc) || type_node->contextualize(pc)) return true;
+  bool do_contextualize(Parse_context *pc) override {
+    if (super::do_contextualize(pc) || type_node->contextualize(pc))
+      return true;
 
     type = type_node->type;
     type_flags = type_node->get_type_flags();
@@ -1038,48 +965,11 @@ class PT_field_def_base : public Parse_tree_node {
     return false;
   }
 
- private:
-  bool is_overridden(Mem_root_array<PT_column_attr_base *> *attrs,
-                     PT_column_attr_base *cand) {
-    // skip de-duplicating these types. NONE is used for (secondary) engine
-    // attributes, cf. make_column_engine_attribute and
-    // make_column_secondary_engine_attribute
-    switch (cand->attr_type()) {
-      case PT_column_attr_base::AT_NONE:  // engine attributes
-      case PT_column_attr_base::AT_CHECK_CONSTRAINT_COLUMN_ATTR:  // additive
-      case PT_column_attr_base::AT_COLLATE_COLUMN_ATTR:  // avoid test breaks
-        return false;
-      default:
-        break;
-    }
-
-    for (auto attr : *attrs) {
-      if (attr->attr_type() == cand->attr_type()) return true;
-    }
-    return false;
-  }
-
  protected:
+  template <class T>
   bool contextualize_attrs(Column_parse_context *pc,
-                           Mem_root_array<PT_column_attr_base *> *attrs) {
+                           Mem_root_array<T *> *attrs) {
     if (attrs != nullptr) {
-      if (attrs->size() > 1) {
-        // Keep only last instance of most attributes that override any
-        // preceding ones. This avoids contextualizing attributes that will
-        // later be ignored anyway.
-        Mem_root_array<PT_column_attr_base *> *unique_attrs = new (pc->mem_root)
-            Mem_root_array<PT_column_attr_base *>(pc->mem_root);
-        unique_attrs->reserve(attrs->size());
-        for (long i = static_cast<long>(attrs->size()) - 1; i >= 0; i--) {
-          if (is_overridden(unique_attrs, (*attrs)[i])) continue;
-          unique_attrs->push_back((*attrs)[i]);  // reversed order..
-        }
-        attrs->clear();
-        // Reverse order again to get back original order. Needed for attributes
-        // which are opposed, e.g. NOT NULL vs NULL.
-        for (auto attr : *unique_attrs) attrs->push_front(attr);
-      }
-
       for (auto attr : *attrs) {
         if (attr->contextualize(pc)) return true;
         attr->apply_type_flags(&type_flags);
@@ -1110,13 +1000,13 @@ class PT_field_def : public PT_field_def_base {
   Mem_root_array<PT_column_attr_base *> *opt_attrs;
 
  public:
-  PT_field_def(PT_type *type_node_arg,
+  PT_field_def(const POS &pos, PT_type *type_node_arg,
                Mem_root_array<PT_column_attr_base *> *opt_attrs)
-      : super(type_node_arg), opt_attrs(opt_attrs) {}
+      : super(pos, type_node_arg), opt_attrs(opt_attrs) {}
 
-  bool contextualize(Parse_context *pc_arg) override {
+  bool do_contextualize(Parse_context *pc_arg) override {
     Column_parse_context pc(pc_arg->thd, pc_arg->select, false);
-    if (super::contextualize(&pc) || contextualize_attrs(&pc, opt_attrs))
+    if (super::do_contextualize(&pc) || contextualize_attrs(&pc, opt_attrs))
       return true;
 
     move_cf_appliers(pc_arg, &pc);
@@ -1137,17 +1027,17 @@ class PT_generated_field_def : public PT_field_def_base {
   Mem_root_array<PT_column_attr_base *> *opt_attrs;
 
  public:
-  PT_generated_field_def(PT_type *type_node_arg, Item *expr,
+  PT_generated_field_def(const POS &pos, PT_type *type_node_arg, Item *expr,
                          Virtual_or_stored virtual_or_stored,
                          Mem_root_array<PT_column_attr_base *> *opt_attrs)
-      : super(type_node_arg),
+      : super(pos, type_node_arg),
         virtual_or_stored(virtual_or_stored),
         expr(expr),
         opt_attrs(opt_attrs) {}
 
-  bool contextualize(Parse_context *pc_arg) override {
+  bool do_contextualize(Parse_context *pc_arg) override {
     Column_parse_context pc(pc_arg->thd, pc_arg->select, true);
-    if (super::contextualize(&pc) || contextualize_attrs(&pc, opt_attrs) ||
+    if (super::do_contextualize(&pc) || contextualize_attrs(&pc, opt_attrs) ||
         expr->itemize(&pc, &expr))
       return true;
 
