@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2005, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,14 +32,11 @@
 #include <utility>
 
 #include "lex_string.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "map_helpers.h"
 #include "my_bitmap.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_psi_config.h"
 #include "my_sqlcommand.h"
@@ -50,11 +46,14 @@
 #include "mysql/components/services/bits/psi_memory_bits.h"
 #include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
+#include "nulls.h"
 #include "sql/derror.h"
 #include "sql/discrete_interval.h"
 #include "sql/field.h"
@@ -75,6 +74,8 @@
 #include "sql/table.h"  // TABLE_SHARE
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
+#include "string_with_len.h"
+#include "strxmov.h"
 #include "template_utils.h"
 #include "thr_mutex.h"
 
@@ -453,11 +454,11 @@ int Partition_helper::ph_write_row(uchar *buf) {
   uint32 part_id;
   int error;
   longlong func_value;
-  bool have_auto_increment =
+  const bool have_auto_increment =
       m_table->next_number_field && buf == m_table->record[0];
   THD *thd = get_thd();
-  sql_mode_t saved_sql_mode = thd->variables.sql_mode;
-  bool saved_autoinc_field_has_expl_non_null_value =
+  const sql_mode_t saved_sql_mode = thd->variables.sql_mode;
+  const bool saved_autoinc_field_has_expl_non_null_value =
       m_table->autoinc_field_has_explicit_non_null_value;
 #ifndef NDEBUG
   my_bitmap_map *old_map;
@@ -759,7 +760,7 @@ void Partition_helper::get_auto_increment_first_field(
     this first one, we must exclusively lock the generator until the statement
     is done.
   */
-  int binlog_format = thd_binlog_format(thd);
+  const int binlog_format = thd_binlog_format(thd);
   if (!m_auto_increment_safe_stmt_log_lock &&
       thd->lex->sql_command != SQLCOM_INSERT &&
       binlog_format != BINLOG_FORMAT_UNSPEC &&
@@ -783,7 +784,7 @@ void Partition_helper::get_auto_increment_first_field(
 
 void Partition_helper::set_auto_increment_if_higher() {
   Field_num *field = static_cast<Field_num *>(m_table->found_next_number_field);
-  ulonglong nr =
+  const ulonglong nr =
       (field->is_unsigned() || field->val_int() > 0) ? field->val_int() : 0;
   lock_auto_increment();
   if (!m_part_share->auto_inc_initialized) {
@@ -803,7 +804,8 @@ void Partition_helper::ph_release_auto_increment() {
   if (m_table->s->next_number_keypart) {
     release_auto_increment_all_parts();
   } else if (m_handler->next_insert_id) {
-    ulonglong max_reserved = m_handler->auto_inc_interval_for_cur_row.maximum();
+    const ulonglong max_reserved =
+        m_handler->auto_inc_interval_for_cur_row.maximum();
     lock_auto_increment();
     m_part_share->release_auto_inc_if_possible(
         get_thd(), m_table->s, m_handler->next_insert_id, max_reserved);
@@ -835,8 +837,8 @@ void Partition_helper::ph_release_auto_increment() {
 uint32 Partition_helper::ph_calculate_key_hash_value(Field **field_array) {
   ulong nr1 = 1;
   ulong nr2 = 4;
-  bool use_51_hash = (*field_array)->table->part_info->key_algorithm ==
-                     enum_key_algorithm::KEY_ALGORITHM_51;
+  const bool use_51_hash = (*field_array)->table->part_info->key_algorithm ==
+                           enum_key_algorithm::KEY_ALGORITHM_51;
 
   do {
     Field *field = *field_array;
@@ -860,7 +862,7 @@ uint32 Partition_helper::ph_calculate_key_hash_value(Field **field_array) {
             continue;
           }
           /* Force this to my_hash_sort_bin, which was used in 5.1! */
-          uint len = field->pack_length();
+          const uint len = field->pack_length();
           uint64 tmp1 = nr1;
           uint64 tmp2 = nr2;
 
@@ -891,7 +893,7 @@ uint32 Partition_helper::ph_calculate_key_hash_value(Field **field_array) {
             continue;
           }
           /* Force this to my_hash_sort_bin, which was used in 5.1! */
-          uint len = field->pack_length();
+          const uint len = field->pack_length();
           uint64 tmp1 = nr1;
           uint64 tmp2 = nr2;
 
@@ -1004,7 +1006,7 @@ bool Partition_helper::print_partition_error(int error) {
 
 void Partition_helper::prepare_change_partitions() {
   List_iterator<partition_element> part_it(m_part_info->partitions);
-  uint num_subparts =
+  const uint num_subparts =
       m_part_info->is_sub_partitioned() ? m_part_info->num_subparts : 1;
   uint temp_partitions = m_part_info->temp_partitions.elements;
   bool first = true;
@@ -1126,7 +1128,7 @@ error:
 int Partition_helper::check_misplaced_rows(uint read_part_id, bool repair) {
   int result = 0;
   THD *thd = get_thd();
-  bool ignore = thd->lex->is_ignore();
+  const bool ignore = thd->lex->is_ignore();
   uint32 correct_part_id;
   longlong func_value;
   ha_rows num_misplaced_rows = 0;
@@ -1272,7 +1274,7 @@ int Partition_helper::check_misplaced_rows(uint read_part_id, bool repair) {
     }
   }
 
-  int tmp_result = rnd_end_in_part(read_part_id, true);
+  const int tmp_result = rnd_end_in_part(read_part_id, true);
   return result ? result : tmp_result;
 }
 
@@ -1961,8 +1963,8 @@ int Partition_helper::ph_index_first(uchar *buf) {
 int Partition_helper::ph_index_last(uchar *buf) {
   DBUG_TRACE;
 
-  int error = HA_ERR_END_OF_FILE;
-  uint part_id = m_part_info->get_first_used_partition();
+  const int error = HA_ERR_END_OF_FILE;
+  const uint part_id = m_part_info->get_first_used_partition();
   if (part_id == MY_BIT_NONE) {
     /* No partition to scan. */
     return error;
@@ -2187,8 +2189,8 @@ int Partition_helper::ph_read_range_first(const key_range *start_key,
                                           const key_range *end_key,
                                           bool eq_range_arg, bool sorted) {
   int error = HA_ERR_END_OF_FILE;
-  bool have_start_key = (start_key != nullptr);
-  uint part_id = m_part_info->get_first_used_partition();
+  const bool have_start_key = (start_key != nullptr);
+  const uint part_id = m_part_info->get_first_used_partition();
   DBUG_TRACE;
 
   if (part_id == MY_BIT_NONE) {
@@ -2285,7 +2287,7 @@ int Partition_helper::partition_scan_set_up(uchar *buf, bool idx_read_flag) {
       Verify this, also bitmap must have at least one bit set otherwise
       the result from this table is the empty set.
     */
-    uint start_part = m_part_info->get_first_used_partition();
+    const uint start_part = m_part_info->get_first_used_partition();
     if (start_part == MY_BIT_NONE) {
       DBUG_PRINT("info", ("scan with no partition to scan"));
       return HA_ERR_END_OF_FILE;
@@ -2693,7 +2695,7 @@ int Partition_helper::handle_ordered_index_scan_key_not_found() {
 
 int Partition_helper::handle_ordered_next(uchar *buf, bool is_next_same) {
   int error;
-  uint part_id = m_top_entry;
+  const uint part_id = m_top_entry;
   uchar *rec_buf = m_queue->empty() ? nullptr : m_queue->top() + m_rec_offset;
   uchar *read_buf;
   DBUG_TRACE;
@@ -2807,7 +2809,7 @@ int Partition_helper::handle_ordered_next(uchar *buf, bool is_next_same) {
 
 int Partition_helper::handle_ordered_prev(uchar *buf) {
   int error;
-  uint part_id = m_top_entry;
+  const uint part_id = m_top_entry;
   uchar *rec_buf = m_queue->empty() ? nullptr : m_queue->top() + m_rec_offset;
   uchar *read_buf;
   DBUG_TRACE;
