@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -98,6 +97,10 @@ extern PSI_memory_key key_memory_xa_recovered_transactions;
 extern PSI_memory_key key_memory_Row_data_memory_memory;
 extern PSI_memory_key key_memory_Rpl_info_file_buffer;
 extern PSI_memory_key key_memory_Rpl_info_table;
+extern PSI_memory_key key_memory_rpl_thd_context;
+extern PSI_memory_key key_memory_applier;
+extern PSI_memory_key key_memory_recovery;
+extern PSI_memory_key key_memory_show_binlog_events;
 extern PSI_memory_key key_memory_REPLICA_INFO;
 extern PSI_memory_key key_memory_ST_SCHEMA_TABLE;
 extern PSI_memory_key key_memory_Slave_applier_json_diff_vector;
@@ -133,7 +136,7 @@ extern PSI_memory_key key_memory_global_system_variables;
 extern PSI_memory_key key_memory_errmsgs_handler;
 extern PSI_memory_key key_memory_handlerton_objects;
 extern PSI_memory_key key_memory_hash_index_key_buffer;
-extern PSI_memory_key key_memory_hash_join;
+extern PSI_memory_key key_memory_hash_op;
 extern PSI_memory_key key_memory_help;
 extern PSI_memory_key key_memory_histograms;
 extern PSI_memory_key key_memory_host_cache_hostname;
