@@ -21,7 +21,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "sql/rpl_binlog_sender.h"
-#include "libbinlogevents/include/codecs/factory.h"
+#include "mysql/binlog/event/codecs/factory.h"
 
 #include <stdio.h>
 #include <algorithm>
@@ -31,7 +31,6 @@
 #include <utility>
 
 #include "lex_string.h"
-#include "libbinlogevents/include/binlog_event.h"  // binary_log::max_log_event_size
 #include "map_helpers.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
@@ -40,6 +39,7 @@
 #include "my_sys.h"
 #include "my_thread.h"
 #include "mysql.h"
+#include "mysql/binlog/event/binlog_event.h"  // mysql::binlog::event::max_log_event_size
 #include "mysql/components/services/bits/psi_stage_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/my_loglevel.h"
@@ -71,7 +71,7 @@
 #ifndef NDEBUG
 static uint binlog_dump_count = 0;
 #endif
-using binary_log::checksum_crc32;
+using mysql::binlog::event::checksum_crc32;
 
 const uint32 Binlog_sender::PACKET_MIN_SIZE = 4096;
 const uint32 Binlog_sender::PACKET_MAX_SIZE = UINT_MAX32;
@@ -79,6 +79,11 @@ const ushort Binlog_sender::PACKET_SHRINK_COUNTER_THRESHOLD = 100;
 const float Binlog_sender::PACKET_GROW_FACTOR = 2.0;
 const float Binlog_sender::PACKET_SHRINK_FACTOR = 0.5;
 
+using mysql::binlog::event::Binary_log_event;
+using mysql::binlog::event::enum_binlog_checksum_alg;
+using mysql::binlog::event::Log_event_footer;
+using mysql::binlog::event::Log_event_type;
+
 /**
   @class Observe_transmission_guard
 
@@ -107,27 +112,30 @@ class Observe_transmission_guard {
     @param prev_event_type The type of the event processed just before the
                            current one
   */
-  Observe_transmission_guard(bool &flag, binary_log::Log_event_type event_type,
-                             const char *event_ptr,
-                             binary_log::enum_binlog_checksum_alg checksum_alg,
-                             binary_log::Log_event_type prev_event_type)
+  Observe_transmission_guard(
+      bool &flag, mysql::binlog::event::Log_event_type event_type,
+      const char *event_ptr,
+      mysql::binlog::event::enum_binlog_checksum_alg checksum_alg,
+      mysql::binlog::event::Log_event_type prev_event_type)
       : m_saved(flag), m_to_set(flag) {
     if (opt_replication_sender_observe_commit_only) {
       switch (event_type) {
-        case binary_log::TRANSACTION_PAYLOAD_EVENT:
-        case binary_log::XID_EVENT:
-        case binary_log::XA_PREPARE_LOG_EVENT: {
+        case mysql::binlog::event::TRANSACTION_PAYLOAD_EVENT:
+        case mysql::binlog::event::XID_EVENT:
+        case mysql::binlog::event::XA_PREPARE_LOG_EVENT: {
           m_to_set = true;
           break;
         }
-        case binary_log::QUERY_EVENT: {
+        case mysql::binlog::event::QUERY_EVENT: {
           bool first_event_after_gtid =
-              prev_event_type == binary_log::ANONYMOUS_GTID_LOG_EVENT ||
-              prev_event_type == binary_log::GTID_LOG_EVENT;
+              prev_event_type ==
+                  mysql::binlog::event::ANONYMOUS_GTID_LOG_EVENT ||
+              prev_event_type == mysql::binlog::event::GTID_LOG_EVENT;
 
           Format_description_log_event fd_ev;
           fd_ev.common_footer->checksum_alg = checksum_alg;
-          Query_log_event ev(event_ptr, &fd_ev, binary_log::QUERY_EVENT);
+          Query_log_event ev(event_ptr, &fd_ev,
+                             mysql::binlog::event::QUERY_EVENT);
           if (first_event_after_gtid)
             m_to_set = (strcmp("BEGIN", ev.query) != 0);
           else
@@ -174,7 +182,7 @@ class Sender_context_guard {
                       of the next event processing round.
   */
   Sender_context_guard(Binlog_sender &target,
-                       binary_log::Log_event_type event_type)
+                       mysql::binlog::event::Log_event_type event_type)
       : m_target(target), m_event_type(event_type) {}
 
   /**
@@ -189,7 +197,7 @@ class Sender_context_guard {
   /** The object to be guarded */
   Binlog_sender &m_target;
   /** The currently being processed event type */
-  binary_log::Log_event_type m_event_type;
+  mysql::binlog::event::Log_event_type m_event_type;
 };
 
 /**
@@ -252,7 +260,7 @@ Binlog_sender::Binlog_sender(THD *thd, const char *start_file,
       m_flag(flag),
       m_observe_transmission(false),
       m_transmit_started(false),
-      m_prev_event_type(binary_log::UNKNOWN_EVENT) {}
+      m_prev_event_type(mysql::binlog::event::UNKNOWN_EVENT) {}
 
 void Binlog_sender::init() {
   DBUG_TRACE;
@@ -345,7 +353,7 @@ void Binlog_sender::init() {
   m_wait_new_events =
       !((thd->server_id == 0) || ((m_flag & BINLOG_DUMP_NON_BLOCK) != 0));
   /* Binary event can be vary large. So set it to max allowed packet. */
-  thd->variables.max_allowed_packet = binary_log::max_log_event_size;
+  thd->variables.max_allowed_packet = mysql::binlog::event::max_log_event_size;
 
 #ifndef NDEBUG
   if (opt_sporadic_binlog_dump_fail && (binlog_dump_count++ % 2))
@@ -584,9 +592,10 @@ int Binlog_sender::send_events(File_reader &reader, my_off_t end_pos) {
 
       /*
         It is reading events before end_pos of active binlog file. In theory,
-        it should never return nullptr. But RESET MASTER doesn't check if there
-        is any dump thread working. So it is possible that the active binlog
-        file is reopened and truncated to 0 after RESET MASTER.
+        it should never return nullptr. But RESET BINARY LOGS AND GTIDS doesn't
+        check if there is any dump thread working. So it is possible that the
+        active binlog file is reopened and truncated to 0 after
+        RESET BINARY LOGS AND GTIDS.
       */
       set_fatal_error(log_read_error_msg(Binlog_read_error::SYSTEM_IO));
       return 1;
@@ -596,7 +605,7 @@ int Binlog_sender::send_events(File_reader &reader, my_off_t end_pos) {
     if (unlikely(check_event_type(event_type, log_file, log_pos))) return 1;
 
     DBUG_EXECUTE_IF("dump_thread_wait_before_send_xid", {
-      if (event_type == binary_log::XID_EVENT) {
+      if (event_type == mysql::binlog::event::XID_EVENT) {
         thd->get_protocol()->flush();
         const char act[] =
             "now "
@@ -671,7 +680,7 @@ int Binlog_sender::send_events(File_reader &reader, my_off_t end_pos) {
       if (unlikely(send_packet())) return 1;
 
       DBUG_EXECUTE_IF("dump_thread_wait_after_send_write_rows", {
-        if (event_type == binary_log::WRITE_ROWS_EVENT) {
+        if (event_type == mysql::binlog::event::WRITE_ROWS_EVENT) {
           thd->get_protocol()->flush();
           static constexpr char act[] =
               "now "
@@ -699,7 +708,7 @@ int Binlog_sender::send_events(File_reader &reader, my_off_t end_pos) {
 
 bool Binlog_sender::check_event_type(Log_event_type type, const char *log_file,
                                      my_off_t log_pos) {
-  if (type == binary_log::ANONYMOUS_GTID_LOG_EVENT) {
+  if (type == mysql::binlog::event::ANONYMOUS_GTID_LOG_EVENT) {
     /*
       Normally, there will not be any anonymous events when
       auto_position is enabled, since both the master and the slave
@@ -735,7 +744,7 @@ bool Binlog_sender::check_event_type(Log_event_type type, const char *log_file,
       set_fatal_error(buf);
       return true;
     }
-  } else if (type == binary_log::GTID_LOG_EVENT) {
+  } else if (type == mysql::binlog::event::GTID_LOG_EVENT) {
     /*
       Normally, there will not be any GTID events when master has
       GTID_MODE=OFF, since GTID events are not generated when
@@ -761,7 +770,7 @@ inline bool Binlog_sender::skip_event(const uchar *event_ptr,
 
   uint8 event_type = (Log_event_type)event_ptr[LOG_EVENT_OFFSET];
   switch (event_type) {
-    case binary_log::GTID_LOG_EVENT: {
+    case mysql::binlog::event::GTID_LOG_EVENT: {
       Format_description_log_event fd_ev;
       fd_ev.common_footer->checksum_alg = m_event_checksum_alg;
       Gtid_log_event gtid_ev(reinterpret_cast<const char *>(event_ptr), &fd_ev);
@@ -770,7 +779,7 @@ inline bool Binlog_sender::skip_event(const uchar *event_ptr,
       gtid.gno = gtid_ev.get_gno();
       return m_exclude_gtid->contains_gtid(gtid);
     }
-    case binary_log::ROTATE_EVENT:
+    case mysql::binlog::event::ROTATE_EVENT:
       return false;
   }
   return in_exclude_group;
@@ -1011,7 +1020,7 @@ extern TYPELIB binlog_checksum_typelib;
 void Binlog_sender::init_checksum_alg() {
   DBUG_TRACE;
 
-  m_slave_checksum_alg = binary_log::BINLOG_CHECKSUM_ALG_UNDEF;
+  m_slave_checksum_alg = mysql::binlog::event::BINLOG_CHECKSUM_ALG_UNDEF;
 
   /* Protects m_thd->user_vars. */
   mysql_mutex_lock(&m_thd->LOCK_thd_data);
@@ -1023,7 +1032,8 @@ void Binlog_sender::init_checksum_alg() {
   if (uv && uv->ptr()) {
     m_slave_checksum_alg = static_cast<enum_binlog_checksum_alg>(
         find_type(uv->ptr(), &binlog_checksum_typelib, 1) - 1);
-    assert(m_slave_checksum_alg < binary_log::BINLOG_CHECKSUM_ALG_ENUM_END);
+    assert(m_slave_checksum_alg <
+           mysql::binlog::event::BINLOG_CHECKSUM_ALG_ENUM_END);
   }
 
   mysql_mutex_unlock(&m_thd->LOCK_thd_data);
@@ -1058,7 +1068,7 @@ int Binlog_sender::fake_rotate_event(const char *next_log_file,
     real and fake Rotate events (if necessary)
   */
   int4store(header, 0);
-  header[EVENT_TYPE_OFFSET] = binary_log::ROTATE_EVENT;
+  header[EVENT_TYPE_OFFSET] = mysql::binlog::event::ROTATE_EVENT;
   int4store(header + SERVER_ID_OFFSET, server_id);
   int4store(header + EVENT_LEN_OFFSET, static_cast<uint32>(event_len));
   int4store(header + LOG_POS_OFFSET, 0);
@@ -1120,7 +1130,8 @@ int Binlog_sender::send_format_description_event(File_reader &reader,
        Log_event::get_type_str((Log_event_type)event_ptr[EVENT_TYPE_OFFSET])));
 
   if (event_ptr == nullptr ||
-      event_ptr[EVENT_TYPE_OFFSET] != binary_log::FORMAT_DESCRIPTION_EVENT) {
+      event_ptr[EVENT_TYPE_OFFSET] !=
+          mysql::binlog::event::FORMAT_DESCRIPTION_EVENT) {
     set_fatal_error("Could not find format_description_event in binlog file");
     return 1;
   }
@@ -1140,11 +1151,13 @@ int Binlog_sender::send_format_description_event(File_reader &reader,
   m_event_checksum_alg =
       Log_event_footer::get_checksum_alg((const char *)event_ptr, event_len);
 
-  assert(m_event_checksum_alg < binary_log::BINLOG_CHECKSUM_ALG_ENUM_END ||
-         m_event_checksum_alg == binary_log::BINLOG_CHECKSUM_ALG_UNDEF);
+  assert(m_event_checksum_alg <
+             mysql::binlog::event::BINLOG_CHECKSUM_ALG_ENUM_END ||
+         m_event_checksum_alg ==
+             mysql::binlog::event::BINLOG_CHECKSUM_ALG_UNDEF);
 
   /* Slave does not support checksum, but binary events include checksum */
-  if (m_slave_checksum_alg == binary_log::BINLOG_CHECKSUM_ALG_UNDEF &&
+  if (m_slave_checksum_alg == mysql::binlog::event::BINLOG_CHECKSUM_ALG_UNDEF &&
       event_checksum_on()) {
     set_fatal_error(
         "Replica can not handle replication events with the "
@@ -1189,56 +1202,7 @@ int Binlog_sender::send_format_description_event(File_reader &reader,
   if (event_checksum_on() && event_updated)
     calc_event_checksum(event_ptr, event_len);
 
-  if (send_packet()) return 1;
-
-  // Let's check if next event is Start encryption event
-  // If we go outside the file read_event will also return an error
-  const auto binlog_pos_after_fdle = reader.position();
-  if (read_event(reader, &event_ptr, &event_len, true)) {
-    reader.seek(binlog_pos_after_fdle);
-    set_last_pos(binlog_pos_after_fdle);
-    return 0;
-  }
-
-  binlog_read_error = binlog_event_deserialize(
-      event_ptr, event_len, &reader.format_description_event(), false, &ev);
-
-  if (binlog_read_error.has_error()) {
-    set_fatal_error(binlog_read_error.get_str());
-    return 1;
-  }
-
-  if (ev && ev->get_type_code() == binary_log::START_5_7_ENCRYPTION_EVENT) {
-    Start_encryption_log_event *sele =
-        down_cast<Start_encryption_log_event *>(ev);
-
-    if (!sele->is_valid()) {
-      set_fatal_error("Start encryption log event is invalid");
-      return 1;
-    }
-
-    if (reader.start_decryption(sele)) {
-      set_fatal_error("Could not decrypt binlog: encryption key error");
-      return 1;
-    }
-
-    if (start_pos <= BIN_LOG_HEADER_SIZE) {
-      const auto log_pos = reader.position();
-      // We have read start encryption event from master binlog, but we have
-      // not sent it to slave. We need to inform slave that master position
-      // has advanced.
-      if (unlikely(send_heartbeat_event(log_pos))) return 1;
-    }
-  } else {
-    reader.seek(binlog_pos_after_fdle);
-    set_last_pos(binlog_pos_after_fdle);
-  }
-
-  if (ev) {
-    delete ev;
-  }
-
-  return 0;
+  return send_packet();
 }
 
 int Binlog_sender::has_previous_gtid_log_event(File_reader &reader,
@@ -1253,7 +1217,8 @@ int Binlog_sender::has_previous_gtid_log_event(File_reader &reader,
     return 1;
   }
 
-  *found = (event[EVENT_TYPE_OFFSET] == binary_log::PREVIOUS_GTIDS_LOG_EVENT);
+  *found = (event[EVENT_TYPE_OFFSET] ==
+            mysql::binlog::event::PREVIOUS_GTIDS_LOG_EVENT);
   return 0;
 }
 
@@ -1278,8 +1243,7 @@ const char *Binlog_sender::log_read_error_msg(
 }
 
 inline int Binlog_sender::read_event(File_reader &reader, uchar **event_ptr,
-                                     uint32 *event_len,
-                                     bool readahead [[maybe_unused]]) {
+                                     uint32 *event_len) {
   DBUG_TRACE;
 
   if (reset_transmit_packet(0, 0)) return 1;
@@ -1317,7 +1281,7 @@ inline int Binlog_sender::read_event(File_reader &reader, uchar **event_ptr,
   DBUG_PRINT("info", ("Read event %s", Log_event::get_type_str(Log_event_type(
                                            (*event_ptr)[EVENT_TYPE_OFFSET]))));
 #ifndef NDEBUG
-  if (!readahead && check_event_count()) return 1;
+  if (check_event_count()) return 1;
 #endif
   return 0;
 }
@@ -1339,7 +1303,7 @@ int Binlog_sender::send_heartbeat_event_v1(my_off_t log_pos) {
 
   /* Timestamp field */
   int4store(header, 0);
-  header[EVENT_TYPE_OFFSET] = binary_log::HEARTBEAT_LOG_EVENT;
+  header[EVENT_TYPE_OFFSET] = mysql::binlog::event::HEARTBEAT_LOG_EVENT;
   int4store(header + SERVER_ID_OFFSET, server_id);
   int4store(header + EVENT_LEN_OFFSET, event_len);
   int4store(header + LOG_POS_OFFSET, static_cast<uint32>(log_pos));
@@ -1352,12 +1316,12 @@ int Binlog_sender::send_heartbeat_event_v2(my_off_t log_pos) {
   DBUG_TRACE;
   DBUG_EXECUTE_IF("heartbeat_event_with_position_greater_than_4_gb",
                   { assert(log_pos > 4294967296); };);
-  auto codec = binary_log::codecs::Factory::build_codec(
-      binary_log::HEARTBEAT_LOG_EVENT_V2);
+  auto codec = mysql::binlog::event::codecs::Factory::build_codec(
+      mysql::binlog::event::HEARTBEAT_LOG_EVENT_V2);
   const char *filename = m_linfo.log_file_name;
   const char *p = filename + dirname_length(filename);
   const std::string log_filename{p, strlen(p)};
-  binary_log::Heartbeat_event_v2 hb{};
+  mysql::binlog::event::Heartbeat_event_v2 hb{};
   const size_t binlog_checksum_size =
       (event_checksum_on() ? BINLOG_CHECKSUM_LEN : 0);
   const size_t max_event_len =
@@ -1381,7 +1345,7 @@ int Binlog_sender::send_heartbeat_event_v2(my_off_t log_pos) {
   // craft the header by hand
   /* Timestamp field */
   int4store(header, 0);
-  header[EVENT_TYPE_OFFSET] = binary_log::HEARTBEAT_LOG_EVENT_V2;
+  header[EVENT_TYPE_OFFSET] = mysql::binlog::event::HEARTBEAT_LOG_EVENT_V2;
   int4store(header + SERVER_ID_OFFSET, server_id);
   int4store(header + EVENT_LEN_OFFSET, event_len);
   int4store(header + LOG_POS_OFFSET, static_cast<uint32>(log_pos));
