@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,7 +32,6 @@
 #include <map>
 #include <utility>
 
-#include "m_ctype.h"
 #include "m_string.h"
 #include "mf_wcomp.h"  // wild_one, wild_many
 #include "my_dbug.h"
@@ -42,6 +40,7 @@
 #include "my_sys.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/auth/auth_acls.h"
@@ -60,6 +59,7 @@
 #include "sql/table.h"  // Table_ref
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
+#include "string_with_len.h"
 #include "template_utils.h"  // my_free_container_pointers
 
 extern PSI_memory_key key_memory_array_buffer;
