@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,19 +30,20 @@
 
 #include "field_types.h"
 #include "lex_string.h"
-#include "m_string.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
 
 #include "my_inttypes.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
-#include "mysql/components/services/psi_statement_bits.h"
+#include "mysql/components/services/bits/psi_statement_bits.h"
 #include "sql/sql_class.h"  // Query_arena
+#include "sql/sql_const.h"
 #include "sql/sql_error.h"
 #include "sql/sql_lex.h"
 #include "sql/sql_list.h"
 #include "sql_string.h"
+#include "string_with_len.h"
 
 class Item;
 class Item_case_expr;
@@ -52,7 +53,7 @@ class sp_handler;
 class sp_head;
 class sp_pcontext;
 class sp_variable;
-struct TABLE_LIST;
+class Table_ref;
 
 ///////////////////////////////////////////////////////////////////////////
 // This file contains SP-instruction classes.
@@ -66,7 +67,7 @@ class sp_printable {
  public:
   virtual void print(const THD *thd, String *str) = 0;
 
-  virtual ~sp_printable() {}
+  virtual ~sp_printable() = default;
 };
 
 /**
@@ -91,7 +92,7 @@ class sp_branch_instr {
   */
   virtual void backpatch(uint dest) = 0;
 
-  virtual ~sp_branch_instr() {}
+  virtual ~sp_branch_instr() = default;
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -158,8 +159,7 @@ class sp_instr : public sp_printable {
     index to the next instruction. Jump instruction will add their
     destination to the leads list.
   */
-  virtual uint opt_mark(sp_head *,
-                        List<sp_instr> *leads MY_ATTRIBUTE((unused))) {
+  virtual uint opt_mark(sp_head *, List<sp_instr> *leads [[maybe_unused]]) {
     m_marked = true;
     return get_ip() + 1;
   }
@@ -170,8 +170,7 @@ class sp_instr : public sp_printable {
     used to prevent the mark sweep from looping for ever. Return the
     end destination.
   */
-  virtual uint opt_shortcut_jump(sp_head *,
-                                 sp_instr *start MY_ATTRIBUTE((unused))) {
+  virtual uint opt_shortcut_jump(sp_head *, sp_instr *start [[maybe_unused]]) {
     return get_ip();
   }
 
@@ -181,8 +180,7 @@ class sp_instr : public sp_printable {
     must also take care of their destination pointers. Forward jumps get
     pushed to the backpatch list 'ibp'.
   */
-  virtual void opt_move(uint dst,
-                        List<sp_branch_instr> *ibp MY_ATTRIBUTE((unused))) {
+  virtual void opt_move(uint dst, List<sp_branch_instr> *ibp [[maybe_unused]]) {
     m_ip = dst;
   }
 
@@ -296,6 +294,8 @@ class sp_lex_instr : public sp_instr {
   */
   bool reset_lex_and_exec_core(THD *thd, uint *nextp, bool open_tables);
 
+  bool execute_expression(THD *thd, uint *nextp);
+
   /**
     (Re-)parse the query corresponding to this instruction and return a new
     LEX-object.
@@ -378,6 +378,10 @@ class sp_lex_instr : public sp_instr {
   */
   virtual void get_query(String *sql_query) const;
 
+  /**
+    Some expressions may be re-parsed as SELECT statements, but need to be
+    adjusted to another SQL command. This function facilitates that change.
+  */
   virtual void adjust_sql_command(LEX *) {}
 
   /**
@@ -402,7 +406,7 @@ class sp_lex_instr : public sp_instr {
 
     @return Error flag.
   */
-  virtual bool on_after_expr_parsing(THD *thd MY_ATTRIBUTE((unused))) {
+  virtual bool on_after_expr_parsing(THD *thd [[maybe_unused]]) {
     return false;
   }
 
@@ -423,7 +427,7 @@ class sp_lex_instr : public sp_instr {
     mem-root is freed when a reparse is triggered or the stored
     routine is dropped.
   */
-  MEM_ROOT m_lex_mem_root;
+  MEM_ROOT m_lex_mem_root{PSI_NOT_INSTRUMENTED, MEM_ROOT_BLOCK_SIZE};
 
   /**
     Indicates whether this sp_lex_instr instance is responsible for
@@ -448,13 +452,13 @@ class sp_lex_instr : public sp_instr {
     List of additional tables this statement needs to lock when it
     enters/leaves prelocked mode on its own.
   */
-  TABLE_LIST *m_prelocking_tables;
+  Table_ref *m_prelocking_tables;
 
   /**
     The value m_lex->query_tables_own_last should be set to this when the
     statement enters/leaves prelocked mode on its own.
   */
-  TABLE_LIST **m_lex_query_tables_own_last;
+  Table_ref **m_lex_query_tables_own_last;
 
   /**
     List of all the Item_trigger_field's of instruction.
@@ -564,6 +568,7 @@ class sp_instr_set : public sp_lex_instr {
   LEX_CSTRING get_expr_query() const override { return m_value_query; }
 
   void adjust_sql_command(LEX *lex) override {
+    assert(lex->sql_command == SQLCOM_SELECT);
     lex->sql_command = SQLCOM_SET_OPTION;
   }
 
@@ -851,6 +856,11 @@ class sp_lex_branch_instr : public sp_lex_instr, public sp_branch_instr {
     m_dest = dest;
   }
 
+  void adjust_sql_command(LEX *lex) override {
+    assert(lex->sql_command == SQLCOM_SELECT);
+    lex->sql_command = SQLCOM_END;
+  }
+
  protected:
   /// Where we will go.
   uint m_dest;
@@ -948,7 +958,7 @@ class sp_instr_set_case_expr : public sp_lex_branch_instr {
   /////////////////////////////////////////////////////////////////////////
 
   /*
-    NOTE: set_destination() and backpatch() are overriden here just because the
+    NOTE: set_destination() and backpatch() are overridden here just because the
     m_dest attribute is not used by this class, so there is no need to do
     anything about it.
 
