@@ -1,16 +1,15 @@
-/* Copyright (c) 2022, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2022, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -480,7 +479,7 @@ static bool is_generated_invisible_primary_key(const KEY *key) {
   @retval   KEY*     KEY instance of a generated primary key.
   @retval   nullptr  If table does not have a generated invisible primary key.
 */
-static const KEY *find_generated_invisible_primary_key(TABLE *table) {
+static const KEY *find_generated_invisible_primary_key(const TABLE *table) {
   assert(table != nullptr && table->s != nullptr);
   // GIPK is not supported for the partitioned table for now.
   if (table->part_info != nullptr) return nullptr;
@@ -493,6 +492,6 @@ static const KEY *find_generated_invisible_primary_key(TABLE *table) {
   return nullptr;
 }
 
-bool table_has_generated_invisible_primary_key(TABLE *table) {
+bool table_has_generated_invisible_primary_key(const TABLE *table) {
   return (find_generated_invisible_primary_key(table) != nullptr);
 }
