@@ -448,10 +448,6 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
 
   DEBUG_SYNC(thd, "swap_partition_before_exchange");
 
-  if (dd::sdi::drop_all_for_table(thd, swap_table_def) ||
-      dd::sdi::drop_all_for_table(thd, part_table_def)) {
-    return true;
-  }
   const int ha_error = part_handler->exchange_partition(
       swap_part_id, part_table_def, swap_table_def);
 
@@ -479,6 +475,9 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
     (void)thd->locked_tables_list.reopen_tables(thd);
     return true;
   } else {
+    bool drop_sdi_res = dd::sdi::drop_all_for_table(thd, swap_table_def) ||
+                        dd::sdi::drop_all_for_table(thd, part_table_def);
+
     if (part_table->file->ht->flags & HTON_SUPPORTS_ATOMIC_DDL) {
       handlerton *hton = part_table->file->ht;
 
@@ -513,7 +512,7 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
       std::unique_ptr<THD, decltype(rollback_post_ddl_reopen_lambda)>
           rollback_post_ddl_reopen_guard(thd, rollback_post_ddl_reopen_lambda);
 
-      if (thd->dd_client()->update(part_table_def) ||
+      if (drop_sdi_res || thd->dd_client()->update(part_table_def) ||
           thd->dd_client()->update(swap_table_def) ||
           write_bin_log(thd, true, thd->query().str, thd->query().length,
                         true)) {
@@ -531,7 +530,8 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
       close_all_tables_for_name(thd, part_table->s, false, nullptr);
       (void)thd->locked_tables_list.reopen_tables(thd);
 
-      if (write_bin_log(thd, true, thd->query().str, thd->query().length))
+      if (drop_sdi_res ||
+          write_bin_log(thd, true, thd->query().str, thd->query().length))
         return true;
     }
   }
