@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -38,9 +38,12 @@
 #include "mysqld_error.h"
 #include "sql/dd/cache/dictionary_client.h"  // dd::cache::Dictionary_client
 #include "sql/dd/dd_table.h"                 // dd::table_storage_engine
+#include "sql/dd/properties.h"               // dd::Properties
 #include "sql/dd/types/abstract_table.h"     // dd::Abstract_table
 #include "sql/dd/types/table.h"              // dd::Table
 #include "sql/dd_sql_view.h"                 // View_metadata_updater
+#include "sql/debug_sync.h"                  // DEBUG_SYNC
+#include "sql/derror.h"                      // ER_THD
 #include "sql/handler.h"
 #include "sql/log.h"          // query_logger
 #include "sql/mysqld.h"       // lower_case_table_names
@@ -63,23 +66,23 @@ class Schema;
 
 typedef std::set<handlerton *> post_ddl_htons_t;
 
-static TABLE_LIST *rename_tables(
-    THD *thd, TABLE_LIST *table_list, bool *int_commit_done,
+static Table_ref *rename_tables(
+    THD *thd, Table_ref *table_list, bool *int_commit_done,
     post_ddl_htons_t *post_ddl_htons,
     Foreign_key_parents_invalidator *fk_invalidator);
 
-static TABLE_LIST *reverse_table_list(TABLE_LIST *table_list);
+static Table_ref *reverse_table_list(Table_ref *table_list);
 
 namespace {
 struct table_list_hash {
-  size_t operator()(const TABLE_LIST *table) const {
+  size_t operator()(const Table_ref *table) const {
     return static_cast<size_t>(murmur3_32(table->mdl_request.key.ptr(),
                                           table->mdl_request.key.length(), 0));
   }
 };
 
 struct table_list_equal {
-  bool operator()(const TABLE_LIST *a, const TABLE_LIST *b) const {
+  bool operator()(const Table_ref *a, const Table_ref *b) const {
     return a->mdl_request.key.is_equal(&b->mdl_request.key);
   }
 };
@@ -87,7 +90,7 @@ struct table_list_equal {
 
 /**
   Check if connection owns SNRW metadata lock on table or view.
-  Report apropriate error if not.
+  Report appropriate error if not.
 
   @note Unlike find_table_for_mdl_upgrade() this call can handle views.
 */
@@ -115,7 +118,7 @@ static bool check_if_owns_upgradable_mdl(THD *thd, const char *db,
 */
 
 static void find_and_set_explicit_duration_for_schema_mdl(
-    THD *thd, TABLE_LIST *table,
+    THD *thd, Table_ref *table,
     Prealloced_array<MDL_request *, 1> *schema_reqs) {
   auto same_db = [table](const MDL_request *mdl_request) {
     return table->db_length == mdl_request->key.db_name_length() &&
@@ -150,9 +153,9 @@ static void find_and_set_explicit_duration_for_schema_mdl(
   @return True - on failure, false - on success.
 */
 
-bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
-  TABLE_LIST *ren_table = 0;
-  DBUG_ENTER("mysql_rename_tables");
+bool mysql_rename_tables(THD *thd, Table_ref *table_list) {
+  Table_ref *ren_table = nullptr;
+  DBUG_TRACE;
 
   mysql_ha_rm_tables(thd, table_list);
 
@@ -166,7 +169,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
   if (query_logger.is_log_table_enabled(QUERY_LOG_GENERAL) ||
       query_logger.is_log_table_enabled(QUERY_LOG_SLOW)) {
     int to_table;
-    const char *rename_log_table[2] = {NULL, NULL};
+    const char *rename_log_table[2] = {nullptr, nullptr};
 
     /*
       Rules for rename of a log table:
@@ -192,7 +195,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
         log_table_rename--;
         if (rename_log_table[log_table_rename]) {
           if (to_table)
-            rename_log_table[log_table_rename] = NULL;
+            rename_log_table[log_table_rename] = nullptr;
           else {
             /*
               Two renames of "log_table TO" w/o rename "TO log_table" in
@@ -200,7 +203,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
             */
             my_error(ER_CANT_RENAME_LOG_TABLE, MYF(0), ren_table->table_name,
                      ren_table->table_name);
-            DBUG_RETURN(true);
+            return true;
           }
         } else {
           if (to_table) {
@@ -210,7 +213,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
             */
             my_error(ER_CANT_RENAME_LOG_TABLE, MYF(0), ren_table->table_name,
                      ren_table->table_name);
-            DBUG_RETURN(true);
+            return true;
           } else {
             /* save the name of the log table to report an error */
             rename_log_table[log_table_rename] = ren_table->table_name;
@@ -225,7 +228,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
       else
         my_error(ER_CANT_RENAME_LOG_TABLE, MYF(0), rename_log_table[1],
                  rename_log_table[1]);
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
@@ -258,10 +261,10 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
       RENAMES TABLES under LOCK TABLES hard to use in 3rd-party online
       ALTER TABLE tools.
     */
-    malloc_unordered_set<TABLE_LIST *, table_list_hash, table_list_equal>
+    malloc_unordered_set<Table_ref *, table_list_hash, table_list_equal>
         new_names(PSI_INSTRUMENT_ME);
 
-    TABLE_LIST *new_table;
+    Table_ref *new_table;
     for (ren_table = table_list; ren_table; ren_table = new_table->next_local) {
       new_table = ren_table->next_local;
 
@@ -269,7 +272,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
       if (new_name_it == new_names.end()) {
         if (check_if_owns_upgradable_mdl(thd, ren_table->db,
                                          ren_table->table_name))
-          DBUG_RETURN(true);
+          return true;
       } else {
         new_names.erase(new_name_it);
       }
@@ -284,21 +287,20 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
     */
   }
 
-  if (lock_table_names(thd, table_list, 0, thd->variables.lock_wait_timeout, 0,
-                       &schema_reqs) ||
+  if (lock_table_names(thd, table_list, nullptr,
+                       thd->variables.lock_wait_timeout, 0, &schema_reqs) ||
       lock_trigger_names(thd, table_list))
-    DBUG_RETURN(true);
+    return true;
 
   const dd::Table *table_def = nullptr;
-  TABLE_LIST *table;
-  for (table = table_list; table && table->next_local;
+  for (Table_ref *table = table_list; table && table->next_local;
        table = table->next_local) {
     if (thd->dd_client()->acquire(table->db, table->table_name, &table_def)) {
       return true;
     }
     if (table_def && table_def->hidden() == dd::Abstract_table::HT_HIDDEN_SE) {
       my_error(ER_NO_SUCH_TABLE, MYF(0), table->db, table->table_name);
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
@@ -326,7 +328,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
   if ((ren_table = rename_tables(thd, table_list, &int_commit_done,
                                  &post_ddl_htons, &fk_invalidator))) {
     /* Rename didn't succeed;  rename back the tables in reverse order */
-    TABLE_LIST *table;
+    Table_ref *table;
 
     if (int_commit_done) {
       /* Reverse the table list */
@@ -370,8 +372,8 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
 
     for (ren_table = table_list; ren_table;
          ren_table = ren_table->next_local->next_local) {
-      TABLE_LIST *new_table = ren_table->next_local;
-      DBUG_ASSERT(new_table);
+      Table_ref *new_table = ren_table->next_local;
+      assert(new_table);
 
       uncommitted_tables.add_table(ren_table);
       uncommitted_tables.add_table(new_table);
@@ -424,9 +426,9 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
         on them are acquired at LOCK TABLES time and are unaffected by
         RENAME TABLES.
       */
-      malloc_unordered_set<TABLE_LIST *, table_list_hash, table_list_equal>
+      malloc_unordered_set<Table_ref *, table_list_hash, table_list_equal>
           to_release(PSI_INSTRUMENT_ME), to_keep(PSI_INSTRUMENT_ME);
-      TABLE_LIST *new_table;
+      Table_ref *new_table;
       for (ren_table = table_list; ren_table;
            ren_table = new_table->next_local) {
         new_table = ren_table->next_local;
@@ -441,12 +443,12 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
 
       error = thd->locked_tables_list.reopen_tables(thd);
 
-      for (TABLE_LIST *t : to_release) {
+      for (Table_ref *t : to_release) {
         // Also releases locks with EXPLICIT duration for the same name.
         thd->mdl_context.release_all_locks_for_name(t->mdl_request.ticket);
       }
 
-      for (TABLE_LIST *t : to_keep) {
+      for (Table_ref *t : to_keep) {
         thd->mdl_context.set_lock_duration(t->mdl_request.ticket, MDL_EXPLICIT);
         t->mdl_request.ticket->downgrade_lock(MDL_SHARED_NO_READ_WRITE);
         find_and_set_explicit_duration_for_schema_mdl(thd, t, &schema_reqs);
@@ -471,7 +473,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
         Error happened and we failed to revert all changes. We simply close
         all tables involved.
       */
-      thd->locked_tables_list.unlink_all_closed_tables(thd, NULL, 0);
+      thd->locked_tables_list.unlink_all_closed_tables(thd, nullptr, 0);
       /*
         We need to keep metadata locks on both old and new table names
         to avoid breaking foreign key invariants for LOCK TABLES.
@@ -481,15 +483,15 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
         Prune list of duplicates first as setting explicit duration for the
         same MDL ticket twice is disallowed.
       */
-      malloc_unordered_set<TABLE_LIST *, table_list_hash, table_list_equal>
+      malloc_unordered_set<Table_ref *, table_list_hash, table_list_equal>
           to_keep(PSI_INSTRUMENT_ME);
-      TABLE_LIST *new_table;
+      Table_ref *new_table;
       for (ren_table = table_list; ren_table;
            ren_table = new_table->next_local) {
         new_table = ren_table->next_local;
         to_keep.insert(new_table);
       }
-      for (TABLE_LIST *t : to_keep) {
+      for (Table_ref *t : to_keep) {
         thd->mdl_context.set_lock_duration(t->mdl_request.ticket, MDL_EXPLICIT);
         t->mdl_request.ticket->downgrade_lock(MDL_SHARED_NO_READ_WRITE);
         find_and_set_explicit_duration_for_schema_mdl(thd, t, &schema_reqs);
@@ -499,7 +501,7 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
 
   if (!error) my_ok(thd);
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 /*
@@ -512,11 +514,11 @@ bool mysql_rename_tables(THD *thd, TABLE_LIST *table_list) {
   RETURN
     pointer to new (reversed) list
 */
-static TABLE_LIST *reverse_table_list(TABLE_LIST *table_list) {
-  TABLE_LIST *prev = 0;
+static Table_ref *reverse_table_list(Table_ref *table_list) {
+  Table_ref *prev = nullptr;
 
   while (table_list) {
-    TABLE_LIST *next = table_list->next_local;
+    Table_ref *next = table_list->next_local;
     table_list->next_local = prev;
     prev = table_list;
     table_list = next;
@@ -547,7 +549,7 @@ static TABLE_LIST *reverse_table_list(TABLE_LIST *table_list) {
   @return False on success, True if rename failed.
 */
 
-static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
+static bool do_rename(THD *thd, Table_ref *ren_table, const char *new_db,
                       const char *new_table_name, const char *new_table_alias,
                       bool *int_commit_done,
                       std::set<handlerton *> *post_ddl_htons,
@@ -555,13 +557,13 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
   const char *new_alias = new_table_name;
   const char *old_alias = ren_table->table_name;
 
-  DBUG_ENTER("do_rename");
+  DBUG_TRACE;
 
   if (lower_case_table_names == 2) {
     old_alias = ren_table->alias;
     new_alias = new_table_alias;
   }
-  DBUG_ASSERT(new_alias);
+  assert(new_alias);
 
   // Fail if the target table already exists
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
@@ -574,47 +576,47 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
       thd->dd_client()->acquire(new_db, new_alias, &to_table) ||
       thd->dd_client()->acquire_for_modification(
           ren_table->db, ren_table->table_name, &from_at))
-    DBUG_RETURN(true);
+    return true;
 
   if (to_table != nullptr) {
     my_error(ER_TABLE_EXISTS_ERROR, MYF(0), new_alias);
-    DBUG_RETURN(true);
+    return true;
   }
 
   if (from_schema == nullptr) {
     my_error(ER_BAD_DB_ERROR, MYF(0), ren_table->db);
-    DBUG_RETURN(true);
+    return true;
   }
 
   if (to_schema == nullptr) {
     my_error(ER_BAD_DB_ERROR, MYF(0), new_db);
-    DBUG_RETURN(true);
+    return true;
   }
 
   if (from_at == nullptr) {
     my_error(ER_NO_SUCH_TABLE, MYF(0), ren_table->db, old_alias);
-    DBUG_RETURN(true);
+    return true;
   }
 
   // So here we know the source table exists and the target table does
   // not exist. Next is to act based on the table type.
   switch (from_at->type()) {
     case dd::enum_table_type::BASE_TABLE: {
-      handlerton *hton = NULL;
+      handlerton *hton = nullptr;
       dd::Table *from_table = dynamic_cast<dd::Table *>(from_at);
       // If the engine is not found, my_error() has already been called
-      if (dd::table_storage_engine(thd, from_table, &hton)) DBUG_RETURN(true);
+      if (dd::table_storage_engine(thd, from_table, &hton)) return true;
 
       if ((hton->flags & HTON_SUPPORTS_ATOMIC_DDL) && (hton->post_ddl))
         post_ddl_htons->insert(hton);
 
       if (check_table_triggers_are_not_in_the_same_schema(ren_table->db,
                                                           *from_table, new_db))
-        DBUG_RETURN(true);
+        return true;
 
       // The below code assumes that only SE capable of atomic DDL support FK.
-      DBUG_ASSERT(!(hton->flags & HTON_SUPPORTS_FOREIGN_KEYS) ||
-                  (hton->flags & HTON_SUPPORTS_ATOMIC_DDL));
+      assert(!(hton->flags & HTON_SUPPORTS_FOREIGN_KEYS) ||
+             (hton->flags & HTON_SUPPORTS_ATOMIC_DDL));
 
       /*
         If we are performing rename with intermediate commits then
@@ -623,7 +625,43 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
         acquire locks on parent and child tables relies on this
         invariant.
       */
-      DBUG_ASSERT(!(*int_commit_done) || fk_invalidator->is_empty());
+      assert(!(*int_commit_done) || fk_invalidator->is_empty());
+
+      // Find if table uses general tablespace and is it encrypted.
+      bool is_general_tablespace = false;
+      bool is_table_encrypted = false;
+      dd::Encrypt_result new_er =
+          dd::is_tablespace_encrypted(thd, *from_table, &is_general_tablespace);
+      if (new_er.error) {
+        return true;
+      }
+      is_table_encrypted = new_er.value;
+      if (!is_general_tablespace &&
+          from_table->options().exists("encrypt_type")) {
+        dd::String_type et;
+        (void)from_table->options().get("encrypt_type", &et);
+        assert(et.empty() == false);
+        is_table_encrypted = is_encrypted(et);
+      }
+
+      /*
+        Check if we are allowed to move the table, if destination schema
+        is changed and its default encryption differs from tables
+        encryption type.
+      */
+      if (from_schema->id() != to_schema->id() &&
+          to_schema->default_encryption() != is_table_encrypted) {
+        if (opt_table_encryption_privilege_check) {
+          if (check_table_encryption_admin_access(thd)) {
+            my_error(ER_CANNOT_SET_TABLE_ENCRYPTION, MYF(0));
+            return true;
+          }
+        } else if (to_schema->default_encryption() && !is_table_encrypted) {
+          push_warning(thd, Sql_condition::SL_WARNING,
+                       WARN_UNENCRYPTED_TABLE_IN_ENCRYPTED_DB,
+                       ER_THD(thd, WARN_UNENCRYPTED_TABLE_IN_ENCRYPTED_DB));
+        }
+      }
 
       /*
         Obtain exclusive metadata lock on all tables being referenced by the
@@ -663,7 +701,7 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
                                   &orphans_mdl_requests)) {
             // See explanation for clearing foreign key invalidator below.
             fk_invalidator->clear();
-            DBUG_RETURN(true);
+            return true;
           }
 
           MDL_request_list::Iterator it(orphans_mdl_requests);
@@ -676,7 +714,7 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
                                              mdl_request->key.name())) {
               // See explanation for clearing foreign key invalidator below.
               fk_invalidator->clear();
-              DBUG_RETURN(true);
+              return true;
             }
           }
         }
@@ -695,10 +733,42 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
             happen. So it is safe to clear invalidator.
           */
           fk_invalidator->clear();
-          DBUG_RETURN(true);
+          return true;
         }
       }
 
+      if (lock_check_constraint_names_for_rename(
+              thd, ren_table->db, old_alias, from_table, new_db, new_alias)) {
+        /*
+          Preserve the invariant that FK invalidator is empty after each
+          step of non-atomic RENAME TABLE.
+        */
+        fk_invalidator->clear();
+        return true;
+      }
+
+      /*
+        Orphan non-self-referencing foreign keys may become non-orphan/adopted
+        self-referencing foreign keys. Check that table has compatible
+        referenced column and parent key for such foreign key. Also, update
+        DD.UNIQUE_CONSTRAINT_NAME.
+      */
+      if ((hton->flags & HTON_SUPPORTS_FOREIGN_KEYS) &&
+          adjust_adopted_self_ref_fk_for_simple_rename_table(
+              thd, ren_table->db, old_alias, new_db, new_alias, hton)) {
+        if (*int_commit_done) {
+          Implicit_substatement_state_guard substatement_guard(thd);
+          trans_rollback_stmt(thd);
+          trans_rollback(thd);
+          /*
+            Preserve the invariant that FK invalidator is empty after each
+            step of non-atomic RENAME TABLE.
+          */
+          fk_invalidator->clear();
+        }
+        return true;
+      }
+
       /*
         We commit changes to data-dictionary immediately after renaming
         table in storage engine if SE doesn't support atomic DDL or
@@ -726,7 +796,7 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
           the above changes we need to clean-up them before returning.
         */
         if (*int_commit_done && (hton->flags & HTON_SUPPORTS_ATOMIC_DDL)) {
-          Disable_gtid_state_update_guard disabler(thd);
+          Implicit_substatement_state_guard substatement_guard(thd);
           trans_rollback_stmt(thd);
           // Full rollback in case we have THD::transaction_rollback_request.
           trans_rollback(thd);
@@ -736,7 +806,7 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
           */
           fk_invalidator->clear();
         }
-        DBUG_RETURN(true);
+        return true;
       }
 
       /*
@@ -744,7 +814,7 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
         rename and changes to FK we need to do it now.
       */
       if (*int_commit_done && (hton->flags & HTON_SUPPORTS_ATOMIC_DDL)) {
-        Disable_gtid_state_update_guard disabler(thd);
+        Implicit_substatement_state_guard substatement_guard(thd);
 
         if (trans_commit_stmt(thd) || trans_commit(thd)) {
           /*
@@ -752,7 +822,7 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
             step of non-atomic RENAME TABLE.
           */
           fk_invalidator->clear();
-          DBUG_RETURN(true);
+          return true;
         }
       }
 
@@ -779,16 +849,17 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
 
       break;
     }
-    case dd::enum_table_type::SYSTEM_VIEW:  // Fall through
+    case dd::enum_table_type::SYSTEM_VIEW:
+      [[fallthrough]];
     case dd::enum_table_type::USER_VIEW: {
       // Changing the schema of a view is not allowed.
       if (strcmp(ren_table->db, new_db)) {
         my_error(ER_FORBID_SCHEMA_CHANGE, MYF(0), ren_table->db, new_db);
-        DBUG_RETURN(true);
+        return true;
       }
 
       /* Rename view in the data-dictionary. */
-      Disable_gtid_state_update_guard disabler(thd);
+      Implicit_substatement_state_guard substatement_guard(thd);
 
       // Set schema id and view name.
       from_at->set_name(new_alias);
@@ -800,25 +871,27 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
           // Full rollback in case we have THD::transaction_rollback_request.
           trans_rollback(thd);
         }
-        DBUG_RETURN(true);
+        return true;
       }
 
       if (*int_commit_done) {
-        if (trans_commit_stmt(thd) || trans_commit(thd)) DBUG_RETURN(true);
+        if (trans_commit_stmt(thd) || trans_commit(thd)) return true;
       }
 
       sp_cache_invalidate();
       break;
     }
     default:
-      DBUG_ASSERT(false); /* purecov: deadcode */
+      assert(false); /* purecov: deadcode */
   }
 
   // Now, we know that rename succeeded, and can log the schema access
   thd->add_to_binlog_accessed_dbs(ren_table->db);
   thd->add_to_binlog_accessed_dbs(new_db);
 
-  DBUG_RETURN(false);
+  DEBUG_SYNC(thd, "do_renames_before_return");
+
+  return false;
 }
 /*
   Rename all tables in list;
@@ -851,22 +924,22 @@ static bool do_rename(THD *thd, TABLE_LIST *ren_table, const char *new_db,
           goes wrong.
 */
 
-static TABLE_LIST *rename_tables(
-    THD *thd, TABLE_LIST *table_list, bool *int_commit_done,
-    post_ddl_htons_t *post_ddl_htons,
-    Foreign_key_parents_invalidator *fk_invalidator)
+static Table_ref *rename_tables(THD *thd, Table_ref *table_list,
+                                bool *int_commit_done,
+                                post_ddl_htons_t *post_ddl_htons,
+                                Foreign_key_parents_invalidator *fk_invalidator)
 
 {
-  TABLE_LIST *ren_table, *new_table;
+  Table_ref *ren_table, *new_table;
 
-  DBUG_ENTER("rename_tables");
+  DBUG_TRACE;
 
   for (ren_table = table_list; ren_table; ren_table = new_table->next_local) {
     new_table = ren_table->next_local;
     if (do_rename(thd, ren_table, new_table->db, new_table->table_name,
                   new_table->alias, int_commit_done, post_ddl_htons,
                   fk_invalidator))
-      DBUG_RETURN(ren_table);
+      return ren_table;
   }
-  DBUG_RETURN(0);
+  return nullptr;
 }
