@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,12 +28,12 @@
 #include <string>
 #include <utility>
 
-#include "m_ctype.h"
 #include "my_base.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
 #include "my_sys.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld.h"  // opt_table_encryption_privilege_check
 #include "mysqld_error.h"
@@ -77,9 +76,9 @@ bool validate_tspnamelen(const LEXSTR &name) {
     return true;
   }
 
-  if (name.length > NAME_LEN ||
-      my_numchars_mb(system_charset_info, name.str, name.str + name.length) >
-          NAME_CHAR_LEN) {
+  if (name.length > NAME_LEN || system_charset_info->cset->numchars(
+                                    system_charset_info, name.str,
+                                    name.str + name.length) > NAME_CHAR_LEN) {
     // Byte length exceeding NAME_LEN, and character length exceeding
     // NAME_CHAR_LEN not allowed
     my_error(ER_TOO_LONG_IDENT, MYF(0), name.str);
@@ -119,7 +118,7 @@ st_alter_tablespace::st_alter_tablespace(
 
 bool validate_tablespace_name_length(const char *tablespace_name) {
   assert(tablespace_name != nullptr);
-  LEX_CSTRING tspname = {tablespace_name, strlen(tablespace_name)};
+  const LEX_CSTRING tspname = {tablespace_name, strlen(tablespace_name)};
   return validate_tspnamelen(tspname);
 }
 
@@ -385,7 +384,7 @@ bool intermediate_commit_unless_atomic_ddl(THD *thd, handlerton *hton) {
     return false;
   }
   /* purecov: begin inspected */
-  Implicit_substatement_state_guard substatement_guard{thd};
+  const Implicit_substatement_state_guard substatement_guard{thd};
   return (trans_commit_stmt(thd) || trans_commit(thd));
   /* purecov: end */
 }
@@ -502,7 +501,7 @@ bool Sql_cmd_create_tablespace::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
+  const dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
 
   // Check if same tablespace already exists.
   auto tsn = dd::make_string_type(m_tablespace_name);
@@ -633,7 +632,7 @@ bool Sql_cmd_create_tablespace::execute(THD *thd) {
         return true;
       }
 
-      Implicit_substatement_state_guard substatement_guard{thd};
+      const Implicit_substatement_state_guard substatement_guard{thd};
       (void)trans_commit_stmt(thd);
       (void)trans_commit(thd);
       /* purecov: end */
@@ -672,7 +671,7 @@ bool Sql_cmd_drop_tablespace::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
+  const dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
 
   const dd::Tablespace *old_ts_def = nullptr;
   if (dc.acquire(m_tablespace_name.str, &old_ts_def)) {
@@ -716,7 +715,7 @@ bool Sql_cmd_drop_tablespace::execute(THD *thd) {
                               nullptr,
                               *m_options};
 
-  int ha_error =
+  const int ha_error =
       hton->alter_tablespace(hton, thd, &ts_info, old_ts_def, nullptr);
   if (map_errors(ha_error, "DROP TABLEPSPACE", &ts_info)) {
     if (ha_error == HA_ERR_TABLESPACE_MISSING && !ddl_is_atomic(hton)) {
@@ -732,7 +731,7 @@ bool Sql_cmd_drop_tablespace::execute(THD *thd) {
         return true;
       }
 
-      Implicit_substatement_state_guard substatement_guard{thd};
+      const Implicit_substatement_state_guard substatement_guard{thd};
       (void)trans_commit_stmt(thd);
       (void)trans_commit(thd);
       /* purecov: end */
@@ -783,7 +782,7 @@ static bool set_table_encryption_type(THD *thd, const dd::Tablespace &ts,
                                       Table_pair_list *tpl,
                                       const LEX_STRING &requested_encryption,
                                       MDL_request_list *table_mdl_reqs) {
-  bool is_request_to_encrypt = dd::is_encrypted(requested_encryption);
+  const bool is_request_to_encrypt = dd::is_encrypted(requested_encryption);
 
   // If the source tablespace encryption type is same as request type.
   dd::String_type source_tablespace_encryption;
@@ -892,7 +891,7 @@ static bool upgrade_lock_for_tables_in_tablespace(
   if (table_mdl_reqs->elements() == 0) return false;
 
   // Install KILL QUERY immunizer.
-  dd::DD_kill_immunizer m_kill_immunizer(thd);
+  const dd::DD_kill_immunizer m_kill_immunizer(thd);
 
   DEBUG_SYNC(thd, "upgrade_lock_for_tables_in_tablespace_kill_point");
 
@@ -934,7 +933,7 @@ bool Sql_cmd_alter_tablespace::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser(&dc);
+  const dd::cache::Dictionary_client::Auto_releaser releaser(&dc);
 
   auto tsmp = get_mod_pair<dd::Tablespace>(&dc, m_tablespace_name.str);
   if (tsmp.first == nullptr) {
@@ -1078,7 +1077,7 @@ bool Sql_cmd_alter_tablespace_add_datafile::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
+  const dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
 
   auto tsmp = get_mod_pair<dd::Tablespace>(&dc, m_tablespace_name.str);
   if (tsmp.first == nullptr) {
@@ -1169,7 +1168,7 @@ bool Sql_cmd_alter_tablespace_drop_datafile::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
+  const dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
 
   auto tsmp = get_mod_pair<dd::Tablespace>(&dc, m_tablespace_name.str);
   if (tsmp.first == nullptr) {
@@ -1255,7 +1254,7 @@ bool Sql_cmd_alter_tablespace_rename::execute(THD *thd) {
     return true;
   }
   dd::cache::Dictionary_client *dc = thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser(dc);
+  const dd::cache::Dictionary_client::Auto_releaser releaser(dc);
 
   dd::String_type old_name = dd::make_string_type(m_tablespace_name);
   dd::String_type new_name = dd::make_string_type(m_new_name);
@@ -1412,7 +1411,7 @@ bool Sql_cmd_create_undo_tablespace::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
+  const dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
 
   // Check if same tablespace already exists.
   auto tsn = dd::make_string_type(m_undo_tablespace_name);
@@ -1493,7 +1492,7 @@ bool Sql_cmd_create_undo_tablespace::execute(THD *thd) {
         return true;
       }
 
-      Implicit_substatement_state_guard substatement_guard{thd};
+      const Implicit_substatement_state_guard substatement_guard{thd};
       (void)trans_commit_stmt(thd);
       (void)trans_commit(thd);
     }
@@ -1557,7 +1556,7 @@ bool Sql_cmd_alter_undo_tablespace::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
+  const dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
 
   // Get the existing dd::Tablespace for this tablespace name.
   auto tsn = dd::make_string_type(m_undo_tablespace_name);
@@ -1591,7 +1590,7 @@ bool Sql_cmd_alter_undo_tablespace::execute(THD *thd) {
       hton->alter_tablespace(hton, thd, &ts_info, tsmp.first, tsmp.second);
   if (map_errors(ha_error, "ALTER UNDO TABLEPSPACE", &ts_info)) {
     if (!ddl_is_atomic(hton)) {
-      Implicit_substatement_state_guard substatement_guard{thd};
+      const Implicit_substatement_state_guard substatement_guard{thd};
       (void)trans_commit_stmt(thd);
       (void)trans_commit(thd);
     }
@@ -1654,7 +1653,7 @@ bool Sql_cmd_drop_undo_tablespace::execute(THD *thd) {
   }
 
   auto &dc = *thd->dd_client();
-  dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
+  const dd::cache::Dictionary_client::Auto_releaser releaser{&dc};
 
   // Get the existing dd::Tablespace for this tablespace name.
   auto tsn = dd::make_string_type(m_undo_tablespace_name);
@@ -1677,7 +1676,7 @@ bool Sql_cmd_drop_undo_tablespace::execute(THD *thd) {
                               nullptr,
                               *m_options};
 
-  int ha_error = hton->alter_tablespace(hton, thd, &ts_info, ts, nullptr);
+  const int ha_error = hton->alter_tablespace(hton, thd, &ts_info, ts, nullptr);
   if (map_errors(ha_error, "DROP UNDO TABLEPSPACE", &ts_info)) {
     if (!ddl_is_atomic(hton)) {
       /*
@@ -1691,7 +1690,7 @@ bool Sql_cmd_drop_undo_tablespace::execute(THD *thd) {
         return true;
       }
 
-      Implicit_substatement_state_guard substatement_guard{thd};
+      const Implicit_substatement_state_guard substatement_guard{thd};
       (void)trans_commit_stmt(thd);
       (void)trans_commit(thd);
     }
