@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.5.1.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 #ifndef YY_MY_SQL_PARSER_MNT_JENKINS_WORKSPACE_PS8_0_AUTOBUILD_RELEASE_TEST_PERCONA_SERVER_SQL_SQL_YACC_H_INCLUDED
 # define YY_MY_SQL_PARSER_MNT_JENKINS_WORKSPACE_PS8_0_AUTOBUILD_RELEASE_TEST_PERCONA_SERVER_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
@@ -850,6 +854,10 @@ extern int my_sql_parser_debug;
     GENERATE_SYM = 1203,
     DOLLAR_QUOTED_STRING_SYM = 1204,
     PARSE_TREE_SYM = 1205,
+    LOG_SYM = 1206,
+    GTIDS_SYM = 1207,
+    PARALLEL_SYM = 1208,
+    S3_SYM = 1209,
     CLIENT_STATS_SYM = 1301,
     CLUSTERING_SYM = 1302,
     COMPRESSION_DICTIONARY_SYM = 1303,
@@ -1664,6 +1672,10 @@ extern int my_sql_parser_debug;
 #define GENERATE_SYM 1203
 #define DOLLAR_QUOTED_STRING_SYM 1204
 #define PARSE_TREE_SYM 1205
+#define LOG_SYM 1206
+#define GTIDS_SYM 1207
+#define PARALLEL_SYM 1208
+#define S3_SYM 1209
 #define CLIENT_STATS_SYM 1301
 #define CLUSTERING_SYM 1302
 #define COMPRESSION_DICTIONARY_SYM 1303
