@@ -24,6 +24,7 @@
 #define SYSTEM_VARIABLES_INCLUDED
 
 #include <stddef.h>
+#include <stdint.h>
 #include <sys/types.h>
 
 #include "my_base.h"  // ha_rows
@@ -41,7 +42,7 @@
 class MY_LOCALE;
 class Time_zone;
 
-typedef ulonglong sql_mode_t;
+using sql_mode_t = uint64_t;
 struct LIST;
 
 // Values for binlog_format sysvar
@@ -135,19 +136,22 @@ enum enum_default_table_encryption {
 };
 
 /* Bits for different SQL modes modes (including ANSI mode) */
-#define MODE_REAL_AS_FLOAT 1
-#define MODE_PIPES_AS_CONCAT 2
-#define MODE_ANSI_QUOTES 4
-#define MODE_IGNORE_SPACE 8
-#define MODE_NOT_USED 16
-#define MODE_ONLY_FULL_GROUP_BY 32
-#define MODE_NO_UNSIGNED_SUBTRACTION 64
-#define MODE_NO_DIR_IN_CREATE 128
-#define MODE_ANSI 262144L
-#define MODE_NO_AUTO_VALUE_ON_ZERO (MODE_ANSI * 2)
-#define MODE_NO_BACKSLASH_ESCAPES (MODE_NO_AUTO_VALUE_ON_ZERO * 2)
-#define MODE_STRICT_TRANS_TABLES (MODE_NO_BACKSLASH_ESCAPES * 2)
-#define MODE_STRICT_ALL_TABLES (MODE_STRICT_TRANS_TABLES * 2)
+inline constexpr sql_mode_t MODE_REAL_AS_FLOAT = 1;
+inline constexpr sql_mode_t MODE_PIPES_AS_CONCAT = 2;
+inline constexpr sql_mode_t MODE_ANSI_QUOTES = 4;
+inline constexpr sql_mode_t MODE_IGNORE_SPACE = 8;
+inline constexpr sql_mode_t MODE_NOT_USED = 16;
+inline constexpr sql_mode_t MODE_ONLY_FULL_GROUP_BY = 32;
+inline constexpr sql_mode_t MODE_NO_UNSIGNED_SUBTRACTION = 64;
+inline constexpr sql_mode_t MODE_NO_DIR_IN_CREATE = 128;
+inline constexpr sql_mode_t MODE_ANSI = 0x40000;
+inline constexpr sql_mode_t MODE_NO_AUTO_VALUE_ON_ZERO = MODE_ANSI * 2;
+inline constexpr sql_mode_t MODE_NO_BACKSLASH_ESCAPES =
+    MODE_NO_AUTO_VALUE_ON_ZERO * 2;
+inline constexpr sql_mode_t MODE_STRICT_TRANS_TABLES =
+    MODE_NO_BACKSLASH_ESCAPES * 2;
+inline constexpr sql_mode_t MODE_STRICT_ALL_TABLES =
+    MODE_STRICT_TRANS_TABLES * 2;
 /*
  * NO_ZERO_DATE, NO_ZERO_IN_DATE and ERROR_FOR_DIVISION_BY_ZERO modes are
  * removed in 5.7 and their functionality is merged with STRICT MODE.
@@ -155,31 +159,34 @@ enum enum_default_table_encryption {
  * but they are not used. Setting these modes in 5.7 will give warning and
  * have no effect.
  */
-#define MODE_NO_ZERO_IN_DATE (MODE_STRICT_ALL_TABLES * 2)
-#define MODE_NO_ZERO_DATE (MODE_NO_ZERO_IN_DATE * 2)
-#define MODE_INVALID_DATES (MODE_NO_ZERO_DATE * 2)
-#define MODE_ERROR_FOR_DIVISION_BY_ZERO (MODE_INVALID_DATES * 2)
-#define MODE_TRADITIONAL (MODE_ERROR_FOR_DIVISION_BY_ZERO * 2)
-#define MODE_HIGH_NOT_PRECEDENCE (1ULL << 29)
-#define MODE_NO_ENGINE_SUBSTITUTION (MODE_HIGH_NOT_PRECEDENCE * 2)
-#define MODE_PAD_CHAR_TO_FULL_LENGTH (1ULL << 31)
+inline constexpr sql_mode_t MODE_NO_ZERO_IN_DATE = MODE_STRICT_ALL_TABLES * 2;
+inline constexpr sql_mode_t MODE_NO_ZERO_DATE = MODE_NO_ZERO_IN_DATE * 2;
+inline constexpr sql_mode_t MODE_INVALID_DATES = MODE_NO_ZERO_DATE * 2;
+inline constexpr sql_mode_t MODE_ERROR_FOR_DIVISION_BY_ZERO =
+    MODE_INVALID_DATES * 2;
+inline constexpr sql_mode_t MODE_TRADITIONAL =
+    MODE_ERROR_FOR_DIVISION_BY_ZERO * 2;
+inline constexpr sql_mode_t MODE_HIGH_NOT_PRECEDENCE = 1ULL << 29;
+inline constexpr sql_mode_t MODE_NO_ENGINE_SUBSTITUTION =
+    MODE_HIGH_NOT_PRECEDENCE * 2;
+inline constexpr sql_mode_t MODE_PAD_CHAR_TO_FULL_LENGTH = 1ULL << 31;
 /*
   If this mode is set the fractional seconds which cannot fit in given fsp will
   be truncated.
 */
-#define MODE_TIME_TRUNCATE_FRACTIONAL (1ULL << 32)
+inline constexpr sql_mode_t MODE_TIME_TRUNCATE_FRACTIONAL = 1ULL << 32;
 
-#define MODE_LAST (1ULL << 33)
+inline constexpr sql_mode_t MODE_LAST = 1ULL << 33;
 
-#define MODE_ALLOWED_MASK                                                      \
-  (MODE_REAL_AS_FLOAT | MODE_PIPES_AS_CONCAT | MODE_ANSI_QUOTES |              \
-   MODE_IGNORE_SPACE | MODE_NOT_USED | MODE_ONLY_FULL_GROUP_BY |               \
-   MODE_NO_UNSIGNED_SUBTRACTION | MODE_NO_DIR_IN_CREATE | MODE_ANSI |          \
-   MODE_NO_AUTO_VALUE_ON_ZERO | MODE_NO_BACKSLASH_ESCAPES |                    \
-   MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES | MODE_NO_ZERO_IN_DATE |  \
-   MODE_NO_ZERO_DATE | MODE_INVALID_DATES | MODE_ERROR_FOR_DIVISION_BY_ZERO |  \
-   MODE_TRADITIONAL | MODE_HIGH_NOT_PRECEDENCE | MODE_NO_ENGINE_SUBSTITUTION | \
-   MODE_PAD_CHAR_TO_FULL_LENGTH | MODE_TIME_TRUNCATE_FRACTIONAL)
+inline constexpr sql_mode_t MODE_ALLOWED_MASK =
+    (MODE_REAL_AS_FLOAT | MODE_PIPES_AS_CONCAT | MODE_ANSI_QUOTES |
+     MODE_IGNORE_SPACE | MODE_NOT_USED | MODE_ONLY_FULL_GROUP_BY |
+     MODE_NO_UNSIGNED_SUBTRACTION | MODE_NO_DIR_IN_CREATE | MODE_ANSI |
+     MODE_NO_AUTO_VALUE_ON_ZERO | MODE_NO_BACKSLASH_ESCAPES |
+     MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES | MODE_NO_ZERO_IN_DATE |
+     MODE_NO_ZERO_DATE | MODE_INVALID_DATES | MODE_ERROR_FOR_DIVISION_BY_ZERO |
+     MODE_TRADITIONAL | MODE_HIGH_NOT_PRECEDENCE | MODE_NO_ENGINE_SUBSTITUTION |
+     MODE_PAD_CHAR_TO_FULL_LENGTH | MODE_TIME_TRUNCATE_FRACTIONAL);
 
 /*
   We can safely ignore and reset these obsolete mode bits while replicating:
@@ -705,6 +712,12 @@ struct System_variables {
     @sa Sys_explain_format
    */
   Explain_format_type explain_format;
+
+  /// Max size of set operations hash buffer size.
+  ulonglong set_operations_buffer_size;
+#ifndef NDEBUG
+  char *debug_set_operations_secondary_overflow_at;
+#endif
 };
 
 /**
