@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,14 +24,18 @@
 #ifndef WINDOWS_INCLUDED
 #define WINDOWS_INCLUDED
 
-#include "my_dbug.h"
+#include <assert.h>
+#include <sys/types.h>
+#include <cstring>  // std::memcpy
+
 #include "my_inttypes.h"
 #include "sql/enum_query_type.h"
 #include "sql/handler.h"
-#include "sql/item.h"
 #include "sql/mem_root_array.h"
 #include "sql/sql_lex.h"
+#include "sql/sql_list.h"
 #include "sql/table.h"
+
 /*
   Some Window-related symbols must be known to sql_lex.h which is a frequently
   included header.
@@ -39,23 +43,44 @@
   Server, those symbols go into this header:
 */
 #include "sql/window_lex.h"
-#include "sql_string.h"
-
-#include <sys/types.h>
-#include <cstring>  // std::memcpy
-
-#include "sql/sql_error.h"
-#include "sql/sql_list.h"
 
+class Cached_item;
+class Item;
 class Item_func;
+class Item_string;
 class Item_sum;
 class PT_border;
 class PT_frame;
 class PT_order_list;
 class PT_window;
+class String;
 class THD;
 class Temp_table_param;
 
+/**
+  Position hints for the frame buffer are saved for these kind of row
+  accesses, cf. #Window::m_frame_buffer_positions.
+*/
+enum class Window_retrieve_cached_row_reason {
+  WONT_UPDATE_HINT = -1,  // special value when using restore_special_row
+  FIRST_IN_PARTITION = 0,
+  CURRENT = 1,
+  FIRST_IN_FRAME = 2,
+  LAST_IN_FRAME = 3,
+  LAST_IN_PEERSET = 4,
+  MISC_POSITIONS = 5  // NTH_VALUE, LEAD/LAG have dynamic indexes 5..N
+};
+
+/**
+  The number of windows is limited to avoid the stack blowing up
+  when constructing iterators recursively. There used to be no limit, but
+  it would be unsafe since QEP_shared::m_idx of tmp tables assigned for windows
+  would exceed the old range of its type plan_idx (int8). It
+  has now been widened, so the max number of windows could be increased
+  if need be, modulo other problems. We could also make it a system variable.
+*/
+constexpr int kMaxWindows = 127;
+
 /**
   Represents the (explicit) window of a SQL 2003 section 7.11 \<window clause\>,
   or the implicit (inlined) window of a window function call, or a reference to
@@ -83,11 +108,10 @@ class Window {
    *
    *------------------------------------------------------------------------*/
  protected:
-  SELECT_LEX *m_select;                 ///< The SELECT the window is on
+  Query_block *m_query_block;           ///< The SELECT the window is on
   PT_order_list *const m_partition_by;  ///< \<window partition clause\>
   PT_order_list *const m_order_by;      ///< \<window order clause\>
   ORDER *m_sorting_order;               ///< merged partition/order by
-  bool m_sort_redundant;                ///< Can use sort from previous w
   PT_frame *const m_frame;              ///< \<window frame clause\>
   Item_string *m_name;                  ///< \<window name\>
   /**
@@ -126,7 +150,7 @@ class Window {
     (At least) one window function needs the cardinality of the partition of
     the current row to evaluate the wf for the current row
   */
-  bool m_needs_card;
+  bool m_needs_partition_cardinality;
 
   /**
     The functions are optimizable with ROW unit. For example SUM is, MAX is
@@ -163,13 +187,6 @@ class Window {
   */
   bool m_opt_last_row;
 
-  /**
-    Can be true if first window after a join: we may need to restore the input
-    record after buffered window processing if EQRefIterator's caching logic
-    presumes the record hasn't been modified (when last qep_tab uses JT_EQ_REF).
-  */
-  bool m_needs_restore_input_row;
-
   /**
     The last window to be evaluated at execution time.
   */
@@ -180,7 +197,7 @@ class Window {
     int64 m_rowno;
     bool m_from_last;
     st_offset() : m_rowno(0), m_from_last(false) {}
-    /*
+    /**
       Used for sorting offsets in ascending order for faster traversal of
       frame buffer tmp file
     */
@@ -190,7 +207,7 @@ class Window {
   struct st_ll_offset {
     int64 m_rowno;  ///< negative values is LEAD
     st_ll_offset() : m_rowno(INT_MIN64 /* uninitialized marker*/) {}
-    /*
+    /**
       Used for sorting offsets in ascending order for faster traversal of
       frame buffer tmp file
     */
@@ -198,7 +215,8 @@ class Window {
   };
 
   struct st_nth {
-    Mem_root_array_YY<st_offset> m_offsets;  // sorted set of NTH_VALUE offsets
+    Mem_root_array_YY<st_offset>
+        m_offsets;  ///< sorted set of NTH_VALUE offsets
   };
 
   struct st_lead_lag {
@@ -215,10 +233,12 @@ class Window {
   st_lead_lag m_opt_lead_lag;
 
  protected:
-  const Window *m_ancestor;             ///< resolved from existing window name
-  List<Item_sum> m_functions;           ///< window functions based on 'this'
-  List<Cached_item> m_partition_items;  ///< items for the PARTITION BY columns
-  List<Cached_item> m_order_by_items;   ///< items for the ORDER BY exprs.
+  const Window *m_ancestor;    ///< resolved from existing window name
+  List<Item_sum> m_functions;  ///< window functions based on 'this'
+  Mem_root_array<Cached_item *>
+      m_partition_items;  ///< items for the PARTITION BY columns
+  Mem_root_array<Cached_item *>
+      m_order_by_items;  ///< items for the ORDER BY exprs.
 
   /*------------------------------------------------------------------------
    *
@@ -226,25 +246,11 @@ class Window {
    *
    *------------------------------------------------------------------------*/
  public:
-  /**
-    Position hints for the frame buffer are saved for these kind of row
-    accesses, cf. #m_frame_buffer_positions.
-  */
-  enum retrieve_cached_row_reason {
-    REA_WONT_UPDATE_HINT =
-        -1,  // special value when using restore_special_record
-    REA_FIRST_IN_PARTITION = 0,
-    REA_CURRENT = 1,
-    REA_FIRST_IN_FRAME = 2,
-    REA_LAST_IN_FRAME = 3,
-    REA_LAST_IN_PEERSET = 4,
-    REA_MISC_POSITIONS = 5  // NTH_VALUE, LEAD/LAG have dynamic indexes 5..N
-  };
-
   /**
     Cardinality of m_frame_buffer_positions if no NTH_VALUE, LEAD/LAG
   */
-  static constexpr int FRAME_BUFFER_POSITIONS_CARD = REA_MISC_POSITIONS;
+  static constexpr int FRAME_BUFFER_POSITIONS_CARD =
+      static_cast<int>(Window_retrieve_cached_row_reason::MISC_POSITIONS);
 
   /**
     Holds information about a position in the buffer frame as stored in a
@@ -262,7 +268,7 @@ class Window {
   };
 
   /**
-    Execution state: used iff m_needs_frame_buffering. Holds four pointers to
+    Execution state: used iff m_needs_frame_buffering. Holds pointers to
     positions in the file in m_frame_buffer. We use these saved positions to
     avoid having to position to the first row in the partition and then
     making many read calls to find the desired row. By repositioning to a
@@ -309,38 +315,50 @@ class Window {
   /**
     See #m_tmp_pos
   */
-  void save_pos(retrieve_cached_row_reason reason) {
-    m_tmp_pos.m_rowno = m_frame_buffer_positions[reason].m_rowno;
+  void save_pos(Window_retrieve_cached_row_reason reason) {
+    const int reason_index = static_cast<int>(reason);
+    m_tmp_pos.m_rowno = m_frame_buffer_positions[reason_index].m_rowno;
     std::memcpy(m_tmp_pos.m_position,
-                m_frame_buffer_positions[reason].m_position,
+                m_frame_buffer_positions[reason_index].m_position,
                 frame_buffer()->file->ref_length);
   }
 
   /**
     See #m_tmp_pos
   */
-  void restore_pos(retrieve_cached_row_reason reason) {
-    m_frame_buffer_positions[reason].m_rowno = m_tmp_pos.m_rowno;
-    std::memcpy(m_frame_buffer_positions[reason].m_position,
+  void restore_pos(Window_retrieve_cached_row_reason reason) {
+    const int reason_index = static_cast<int>(reason);
+    m_frame_buffer_positions[reason_index].m_rowno = m_tmp_pos.m_rowno;
+    std::memcpy(m_frame_buffer_positions[reason_index].m_position,
                 m_tmp_pos.m_position, frame_buffer()->file->ref_length);
   }
 
   /**
     Copy frame buffer position hint from one to another.
   */
-  void copy_pos(retrieve_cached_row_reason from_reason,
-                retrieve_cached_row_reason to_reason) {
-    m_frame_buffer_positions[to_reason].m_rowno =
-        m_frame_buffer_positions[from_reason].m_rowno;
+  void copy_pos(Window_retrieve_cached_row_reason from_reason,
+                Window_retrieve_cached_row_reason to_reason) {
+    const int from_index = static_cast<int>(from_reason);
+    const int to_index = static_cast<int>(to_reason);
+    m_frame_buffer_positions[to_index].m_rowno =
+        m_frame_buffer_positions[from_index].m_rowno;
 
-    std::memcpy(m_frame_buffer_positions[to_reason].m_position,
-                m_frame_buffer_positions[from_reason].m_position,
+    std::memcpy(m_frame_buffer_positions[to_index].m_position,
+                m_frame_buffer_positions[from_index].m_position,
                 frame_buffer()->file->ref_length);
   }
 
   /**
-     Keys for m_frame_buffer_cache and m_special_rows_cache, for special
-     rows.
+     Keys for m_special_rows_cache, for special
+     rows (see the comment on m_special_row_cache). Note that they are negative,
+     so that they will never collide with actual row numbers in the frame.
+     This allows us to treat them interchangeably with real row numbers
+     as function arguments; e.g., bring_back_frame_row() can restore either
+     a “normal” row from the frame, or one of the special rows, and does not
+     need to take in separate flags for the two.
+     TODO(Chaithra): We have only one special key. Do we need the enum?
+     Also m_special_rows_cache/cache_length/max_cache_length should be looked
+     into.
   */
   enum Special_keys {
     /**
@@ -350,12 +368,10 @@ class Window {
      the partition, and restore it later.
     */
     FBC_FIRST_IN_NEXT_PARTITION = -1,
-    /// The last row cached in the frame buffer; needed to resurrect input row
-    FBC_LAST_BUFFERED_ROW = -2,
     // Insert new values here.
     // And keep the ones below up to date.
     FBC_FIRST_KEY = FBC_FIRST_IN_NEXT_PARTITION,
-    FBC_LAST_KEY = FBC_LAST_BUFFERED_ROW,
+    FBC_LAST_KEY = FBC_FIRST_IN_NEXT_PARTITION,
   };
 
  protected:
@@ -366,9 +382,10 @@ class Window {
   Temp_table_param *m_frame_buffer_param;
 
   /**
-    Execution state: Holds the temporary output table (for next step) parameters
+    Holds whether this window should be “short-circuit”, ie., goes directly
+    to the query output instead of to a temporary table.
   */
-  Temp_table_param *m_outtable_param;
+  bool m_short_circuit = false;
 
   /**
     Execution state: used iff m_needs_frame_buffering. Holds the TABLE
@@ -405,7 +422,10 @@ class Window {
 
   /**
     Holds a fixed number of copies of special rows; each copy can use up to
-    #m_special_rows_cache_max_length bytes.
+    #m_special_rows_cache_max_length bytes. Special rows are those that are
+    not part of our frame, but that we need to store away nevertheless, because
+    they might be in the table's buffers, which we need for our own purposes
+    during window processing.
     cf. the Special_keys enumeration.
   */
   uchar *m_special_rows_cache;
@@ -429,7 +449,7 @@ class Window {
 
   /**
     Execution state: used iff m_needs_last_peer_in_frame. True if a row
-    leaving the frame is the last row in the peer set withing the frame.
+    leaving the frame is the last row in the peer set within the frame.
   */
   int64 m_is_last_row_in_peerset_within_frame;
 
@@ -458,20 +478,25 @@ class Window {
     Execution state: The number of the row being visited for its contribution
     to a window function, relative to the start of the partition. Note that
     this will often be different from the current row for which we are
-    processing the window function, reading it for output. That is given by
-    m_rowno_in_partition, q.v.
+    processing the window function, readying it for output. That is given by
+    \c m_rowno_in_partition, q.v. Contrast also with \c m_rowno_in_frame
+    which is frame relative.
   */
   int64 m_rowno_being_visited;
 
   /**
-    Execution state: the row number of the current row within a frame, cf.
-    m_is_last_row_in_frame, relative to start of the frame. 1-based.
+    Execution state: the row number of the row we are looking at for evaluating
+    its contribution to some window function(s).  It is relative to start of
+    the frame and 1-based. It is typically set after fetching a row from the
+    frame buffer using \c bring_back_frame_row and before calling
+    \c copy_funcs.  Cf. also \c m_is_last_row_in_frame.
   */
   int64 m_rowno_in_frame;
 
   /**
     Execution state: The row number of the current row being readied for
     output within the partition. 1-based.
+    In \c process_buffered_windowing_record this is known as \c current_row.
   */
   int64 m_rowno_in_partition;
 
@@ -489,18 +514,67 @@ class Window {
    *
    *------------------------------------------------------------------------*/
  public:
+  /*
+    For RANGE bounds, we need to be able to check if a given row is
+    before the lower bound, or after the upper bound, as we don't know
+    ahead of time how many rows to skip (unlike for ROW bounds).
+    Often, the lower and upper bounds are the same, as they are
+    inclusive.
+
+    We solve this by having an array of comparators of the type
+
+       value ⋛ <cache>(ref_value)
+
+    where ⋛ is a three-way comparator, and <cache>(ref_value) is an
+    Item_cache where we can store the value of the reference row
+    before we invoke the comparison. For instance, if we have a
+    row bound such as
+
+        ... OVER (ORDER BY a,b) FROM t1
+
+     we will have an array with the two comparators
+
+        t1.a ⋛ <cache>(a)
+        t1.b ⋛ <cache>(b)
+
+     and when we evaluate the frame bounds for a given row, we insert
+     the reference values in the caches and then do the comparison to
+     figure out if we're before, in or after the bound. As this is a
+     lexicographic comparison, we only need to evaluate the second
+     comparator if the first one returns equality.
+
+     The expressions on the right-hand side can be somewhat more
+     complicated; say that we have a window specification like
+
+        ... OVER (ORDER BY a RANGE BETWEEN 2 PRECEDING AND 3 FOLLOWING)
+
+     In this case, the lower bound and upper bound are different
+     (both arrays will always contain only a single element):
+
+        Lower: t1.a ⋛ <cache>(a) - 2
+        Upper: t1.a ⋛ <cache>(a) + 3
+
+     ie., there is an Item_func_plus or Item_func_minus with some
+     constant expression as the second argument.
+
+     The comparators for the lower bound is stored in [0], and for
+     the upper bound in [1].
+   */
+  Bounds_checked_array<Arg_comparator> m_comparators[2];
+
   /**
-    RANGE bound determination computation; first index is a value of the
-    enum_window_border_type enum; second index 0 for the start bound, 1 for
-    the end bound. Each item is Item_func_lt/gt.
-  */
-  Item_func *m_comparators[WBT_VALUE_FOLLOWING + 1][2];
+    The logical ordering index (into LogicalOrderings) needed by this window's
+    PARTITION BY and ORDER BY clauses together (if any; else, 0). Used only by
+    the hypergraph join optimizer.
+   */
+  int m_ordering_idx;
+
   /**
-    Each item has inverse operation of the corresponding
-    comparator in m_comparators. Determines if comparison
-    should continue with next field in order by list.
-  */
-  Item_func *m_inverse_comparators[WBT_VALUE_FOLLOWING + 1][2];
+    Used temporarily by the hypergraph join optimizer to mark which windows
+    are referred to by a given ordering (so that one doesn't try to satisfy
+    a window's ordering by an ordering referring to that window).
+   */
+  bool m_mark;
 
  protected:
   /**
@@ -531,6 +605,12 @@ class Window {
   */
   int64 m_last_rowno_in_range_frame;
 
+  /**
+    Execution state. Only used for ROWS frame optimized MIN/MAX.
+    The (1-based) number in the partition of first row in the current frame.
+  */
+  int64 m_first_rowno_in_rows_frame;
+
   /*------------------------------------------------------------------------
    *
    * Window function special behaviour toggles. These boolean flag influence
@@ -545,7 +625,7 @@ class Window {
    *     copy_<kind>_window_functions()  [see process_buffered_windowing_record]
    *     w.set_<toggle>(false)
    *
-   * to achive a special semantic, since we cannot pass down extra parameters.
+   * to achieve a special semantic, since we cannot pass down extra parameters.
    *
    *------------------------------------------------------------------------*/
 
@@ -588,11 +668,10 @@ class Window {
   */
   Window(Item_string *name, PT_order_list *part, PT_order_list *ord,
          PT_frame *frame, bool is_reference, Item_string *inherit)
-      : m_select(nullptr),
+      : m_query_block(nullptr),
         m_partition_by(part),
         m_order_by(ord),
         m_sorting_order(nullptr),
-        m_sort_redundant(false),
         m_frame(frame),
         m_name(name),
         m_def_pos(0),
@@ -601,18 +680,18 @@ class Window {
         m_needs_frame_buffering(false),
         m_needs_peerset(false),
         m_needs_last_peer_in_frame(false),
-        m_needs_card(false),
+        m_needs_partition_cardinality(false),
         m_row_optimizable(true),
         m_range_optimizable(true),
         m_static_aggregates(false),
         m_opt_first_row(false),
         m_opt_last_row(false),
-        m_needs_restore_input_row(false),
         m_last(false),
         m_ancestor(nullptr),
+        m_partition_items(*THR_MALLOC),
+        m_order_by_items(*THR_MALLOC),
         m_tmp_pos(nullptr, -1),
         m_frame_buffer_param(nullptr),
-        m_outtable_param(nullptr),
         m_frame_buffer(nullptr),
         m_frame_buffer_total_rows(0),
         m_frame_buffer_partition_offset(0),
@@ -694,7 +773,7 @@ class Window {
 
   /**
     Get the ORDER BY, if any. That is, the first we find along
-    the ancestor chain. Uniqueness checked in #setup_windows
+    the ancestor chain. Uniqueness checked in #setup_windows1
     SQL 2011 7.11 GR 1.b.i.5.A-C
   */
   const PT_order_list *effective_order_by() const {
@@ -732,7 +811,7 @@ class Window {
         p = w->m_partition_by;
       } else {
         /* See #setup_windows for checking */
-        DBUG_ASSERT(w->m_partition_by == nullptr);
+        assert(w->m_partition_by == nullptr);
       }
       w = w->m_ancestor;
     }
@@ -775,9 +854,15 @@ class Window {
 
   /**
     Check that the semantic requirements for window functions over this
-    window are fulfilled, and accumulate evaluation requirements
+    window are fulfilled, and accumulate evaluation requirements.
+    This is run at resolution.
   */
-  bool check_window_functions(THD *thd, SELECT_LEX *select);
+  bool check_window_functions1(THD *thd, Query_block *select);
+  /**
+    Like check_window_functions1() but contains checks which must wait until
+    the start of the execution phase.
+  */
+  bool check_window_functions2(THD *thd);
 
   /**
     For RANGE frames we need to do computations involving add/subtract and
@@ -849,8 +934,8 @@ class Window {
   */
   bool at_partition_border() const { return m_partition_border; }
 
-  void save_special_record(uint64 special_rowno, TABLE *t);
-  void restore_special_record(uint64 special_rowno, uchar *record);
+  void save_special_row(uint64 special_rowno, TABLE *t);
+  void restore_special_row(uint64 special_rowno, uchar *record);
 
   /**
     Resolve any named window to its definition
@@ -859,7 +944,7 @@ class Window {
   static bool resolve_reference(THD *thd, Item_sum *wf, PT_window **m_window);
 
   /**
-    Semantic checking of windows.
+    Semantic checking of windows. Run at resolution.
 
     * Process any window inheritance, that is a window, that in its
     specification refer to another named window.
@@ -890,47 +975,69 @@ class Window {
     @param select           The select for which we are doing windowing
     @param ref_item_array   The base ref items
     @param tables           The list of tables involved
-    @param fields           The list of selected fields
-    @param all_fields       The list of all fields, including hidden ones
+    @param fields           The list of all fields, including hidden ones
     @param windows          The list of windows defined for this select
 
     @return false if success, true if error
   */
-  static bool setup_windows(THD *thd, SELECT_LEX *select,
-                            Ref_item_array ref_item_array, TABLE_LIST *tables,
-                            List<Item> &fields, List<Item> &all_fields,
-                            List<Window> &windows);
+  static bool setup_windows1(THD *thd, Query_block *select,
+                             Ref_item_array ref_item_array, Table_ref *tables,
+                             mem_root_deque<Item *> *fields,
+                             List<Window> *windows);
+  /**
+    Like setup_windows1() but contains operations which must wait until
+    the start of the execution phase.
+
+    @param thd              The session's execution thread
+    @param windows          The list of windows defined for this select
+
+    @return false if success, true if error
+  */
+  static bool setup_windows2(THD *thd, List<Window> *windows);
 
   /**
-    Remove unused window definitions. Do this only after syntactic and
-    semantic checking for errors has been performed.
+    Check window definitions to remove unused windows. We do this
+    only after syntactic and semantic checking for errors has been performed.
+    Eliminate redundant sorts after unused windows are removed.
 
-    @param thd             The session's execution thread
     @param windows         The list of windows defined for this select
   */
-  static void remove_unused_windows(THD *thd, List<Window> &windows);
+  static void eliminate_unused_objects(List<Window> *windows);
 
   /**
     Resolve and set up the PARTITION BY or an ORDER BY list of a window.
 
     @param thd              The session's execution thread
-    @param ref_item_array
+    @param ref_item_array   The base ref items
     @param tables           The list of tables involved
-    @param fields           The list of selected fields
-    @param all_fields       The list of all fields, including hidden ones
+    @param fields           The list of all fields, including hidden ones
     @param o                A list of order by expressions
     @param partition_order  If true, o represent a windowing PARTITION BY,
            else it represents a windowing ORDER BY
     @returns false if success, true if error
   */
   bool resolve_window_ordering(THD *thd, Ref_item_array ref_item_array,
-                               TABLE_LIST *tables, List<Item> &fields,
-                               List<Item> &all_fields, ORDER *o,
+                               Table_ref *tables,
+                               mem_root_deque<Item *> *fields, ORDER *o,
                                bool partition_order);
   /**
     Return true if this window's name is not unique in windows
   */
-  bool check_unique_name(List<Window> &windows);
+  bool check_unique_name(const List<Window> &windows);
+
+  /**
+    Specify that this window is to be evaluated after a given
+    temporary table. This means that all expressions that have been
+    materialized (as given by items_to_copy) will be replaced with the
+    given temporary table fields. (If there are multiple materializations,
+    this function must be called for each of them in order.)
+
+    Only the hypergraph join optimizer uses this currently; the old
+    join optimizer instead uses Item_ref objects that point to the
+    base slice, which is then replaced at runtime depending on which
+    temporary table we are to evaluate from.
+  */
+  void apply_temp_table(THD *thd, const Func_ptr_array &items_to_copy);
 
   /**
     Set up cached items for an partition or an order by list
@@ -944,7 +1051,7 @@ class Window {
 
     @returns false if success, true if error
   */
-  bool setup_ordering_cached_items(THD *thd, SELECT_LEX *select,
+  bool setup_ordering_cached_items(THD *thd, Query_block *select,
                                    const PT_order_list *o,
                                    bool partition_order);
 
@@ -955,8 +1062,8 @@ class Window {
     @return true if we have such a clause, which means we need to sort the
             input table before evaluating the window functions, unless it has
             been made redundant by a previous windowing step, cf.
-            m_sort_redundant, or due to a single row result set, cf.
-            SELECT_LEX::is_implicitly_grouped().
+            reorder_and_eliminate_sorts, or due to a single row result set,
+            cf. Query_block::is_implicitly_grouped().
   */
   bool needs_sorting() const { return m_sorting_order != nullptr; }
 
@@ -984,7 +1091,9 @@ class Window {
     some window function(s) on this window,
     @returns true if that is the case, else false
   */
-  bool needs_card() const { return m_needs_card; }
+  bool needs_partition_cardinality() const {
+    return m_needs_partition_cardinality;
+  }
 
   /**
     Return true if the set of window functions are all ROW unit optimizable.
@@ -1020,14 +1129,9 @@ class Window {
   bool is_last() const { return m_last; }
 
   /**
-    See #m_needs_restore_input_row
+    An override used by the hypergraph join optimizer only.
   */
-  void set_needs_restore_input_row(bool b) { m_needs_restore_input_row = b; }
-
-  /**
-    See #m_needs_restore_input_row
-  */
-  bool needs_restore_input_row() const { return m_needs_restore_input_row; }
+  void set_is_last(bool last) { m_last = last; }
 
   /**
     See #m_opt_nth_row
@@ -1059,15 +1163,10 @@ class Window {
   */
   void set_frame_buffer(TABLE *tab) { m_frame_buffer = tab; }
 
-  /**
-   Getter for m_outtable_param, q.v.
-   */
-  Temp_table_param *outtable_param() const { return m_outtable_param; }
-
-  /**
-   Setter for m_outtable_param, q.v.
-   */
-  void set_outtable_param(Temp_table_param *p) { m_outtable_param = p; }
+  bool short_circuit() const { return m_short_circuit; }
+  void set_short_circuit(bool short_circuit) {
+    m_short_circuit = short_circuit;
+  }
 
   /**
     Getter for m_part_row_number, q.v., the current row number within the
@@ -1188,7 +1287,7 @@ class Window {
     See #m_is_last_row_in_frame
   */
   bool is_last_row_in_frame() const {
-    return m_is_last_row_in_frame || m_select->table_list.elements == 0;
+    return m_is_last_row_in_frame || m_query_block->m_table_list.elements == 0;
   }
 
   /**
@@ -1225,6 +1324,17 @@ class Window {
   */
   void set_rowno_in_partition(int64 rowno) { m_rowno_in_partition = rowno; }
 
+  /**
+    See #m_first_rowno_in_rows_frame
+  */
+  void set_first_rowno_in_rows_frame(int64 rowno) {
+    m_first_rowno_in_rows_frame = rowno;
+  }
+
+  int64 first_rowno_in_rows_frame() const {
+    return m_first_rowno_in_rows_frame;
+  }
+
   /**
     See #m_first_rowno_in_range_frame
   */
@@ -1282,9 +1392,14 @@ class Window {
   /**
     Free up any resource used to process the window functions of this window,
     e.g. temporary files and in-memory data structures. Called when done
-    with all window processing steps from SELECT_LEX::cleanup.
+    with all window processing steps from Query_block::cleanup.
   */
-  void cleanup(THD *thd);
+  void cleanup();
+
+  /**
+    Free structures that were set up during preparation of window functions
+  */
+  void destroy();
 
   /**
    Reset window state for a new partition.
@@ -1295,24 +1410,19 @@ class Window {
   void reset_partition_state() { reset_execution_state(RL_PARTITION); }
 
   /**
-    Reset execution state for next call to JOIN::exec, cf. JOIN::reset.
+    Reset execution state for next call to JOIN::exec, cf. JOIN::reset,
+    or using [Buffering]WindowingIterator::Init.
   */
   void reset_round() { reset_execution_state(RL_ROUND); }
 
-  /**
-    Reset resolution and execution state to prepare for next execution of a
-    prepared statement.
-  */
-  void reinit_before_use() { reset_execution_state(RL_FULL); }
-
   /**
     Reset execution state for LEAD/LAG for the current row in partition.
   */
   void reset_lead_lag();
 
-  enum Reset_level { RL_FULL, RL_ROUND, RL_PARTITION };
-
  private:
+  enum Reset_level { RL_ROUND, RL_PARTITION };
+
   /// Common function for all types of resetting
   void reset_execution_state(Reset_level level);
 
@@ -1322,65 +1432,9 @@ class Window {
   */
   void reset_all_wf_state();
 
-  /**
-    Collects evaluation requirements from a window function,
-    used by Item_sum::check_wf_semantics and its overrides.
-  */
-  struct Evaluation_requirements {
-    /**
-      Set to true if window function requires row buffering
-    */
-    bool needs_buffer;
-    /**
-      Set to true if we need peerset for evaluation (e.g. CUME_DIST)
-    */
-    bool needs_peerset;
-    /**
-      Set to true if we need last peer for evaluation within a frame
-      (e.g. JSON_OBJECTAGG)
-    */
-    bool needs_last_peer_in_frame;
-    /**
-      Set to true if we need FIRST_VALUE or optimized MIN/MAX
-    */
-    bool opt_first_row;
-    /**
-      Set to true if we need LAST_VALUE or optimized MIN/MAX
-    */
-    bool opt_last_row;
-    st_offset opt_nth_row;    ///< Used if we have NTH_VALUE
-    st_ll_offset opt_ll_row;  ///< Used if we have LEAD or LAG
-    /**
-      Set to true if we can compute a sliding window by a combination of
-      undoing the contribution of rows going out of the frame and adding the
-      contribution of rows coming into the frame.  For example, SUM and AVG
-      allows this, but MAX/MIN do not. Only applicable if the frame has ROW
-      bounds unit.
-    */
-    bool row_optimizable;
-    /**
-      Similar to row_optimizable but for RANGE frame bounds unit
-    */
-    bool range_optimizable;
+  const char *printable_name() const;
 
-    Evaluation_requirements()
-        : needs_buffer(false),
-          needs_peerset(false),
-          needs_last_peer_in_frame(false),
-          opt_first_row(false),
-          opt_last_row(false),
-          row_optimizable(true),
-          range_optimizable(true) {}
-  };
-
-  const char *printable_name() const {
-    if (m_name == nullptr) return "<unnamed window>";
-    // Since Item_string::val_str() ignores the argument, it is safe
-    // to use nullptr as argument.
-    return m_name->val_str(nullptr)->ptr();
-  }
-
-  void print(THD *thd, String *str, enum_query_type qt,
+  void print(const THD *thd, String *str, enum_query_type qt,
              bool expand_definition) const;
 
   bool has_windowing_steps() const;
@@ -1393,7 +1447,7 @@ class Window {
 
     @returns the aggregated sorting costs of the windowing
   */
-  static double compute_cost(double cost, List<Window> &windows);
+  static double compute_cost(double cost, const List<Window> &windows);
 
  private:
   /**
@@ -1401,8 +1455,9 @@ class Window {
     @param  before  True if 'before' is wanted; false if 'after' is.
   */
   bool before_or_after_frame(bool before);
-  void print_frame(String *str, enum_query_type qt) const;
-  void print_border(String *str, PT_border *b, enum_query_type qt) const;
+  void print_frame(const THD *thd, String *str, enum_query_type qt) const;
+  void print_border(const THD *thd, String *str, PT_border *b,
+                    enum_query_type qt) const;
 
   /**
     Reorder windows and eliminate redundant ordering. If a window has the
@@ -1412,22 +1467,22 @@ class Window {
     SQL standard when it comes to repeatability of non-deterministic (partially
     ordered) result sets for windowing inside a query, cf. #equal_sort.
     If more than two have the same ordering, the same applies, we only sort
-    before the first (sort equivalent) window.
+    before the first (sort equivalent) window. The hypergraph optimizer uses
+    the interesting order framework instead, eliding all sorts eliminated by
+    this function and possibly more.
+
+    Note that we do not merge windows that have the same ordering requirements,
+    so we may still get the extra materialization and multiple rounds of
+    buffering. Yet, we should get _correctness_, as long as materialization
+    preserves row ordering (which it does for all of our supported temporary
+    table types).
 
     If the result set is implicitly grouped, we also skip any sorting for
     windows.
 
     @param windows     list of windows
-    @param first_exec  if true, the as done a part of a first prepare, not a
-                       reprepare. On a reprepare the analysis part will be
-                       skipped, since the flag m_sort_redundant flag is stable
-                       across prepares.
-    @todo in WL#6570 we may set m_sorting_order only once, during preparation,
-    then Window::m_sort_redundant could be removed, as well as the first_exec
-    argument.
   */
-  static void reorder_and_eliminate_sorts(List<Window> &windows,
-                                          bool first_exec);
+  static void reorder_and_eliminate_sorts(List<Window> *windows);
 
   /**
     Return true of the physical[1] sort orderings for the two windows are the
@@ -1457,17 +1512,73 @@ class Window {
   bool check_constant_bound(THD *thd, PT_border *border);
 
   /**
-    Check that frame borders are sane, e.g. they are not negative .
+    Check that frame borders are sane; resolution phase.
 
     @param thd      Session thread
-    @param w        The window whose frame we are checking
-    @param f        The frame to check, if any
-    @param prepare  false at EXECUTE ps prepare time, else true
 
-   @returns true if error
+    @returns true if error
   */
-  static bool check_border_sanity(THD *thd, Window *w, const PT_frame *f,
-                                  bool prepare);
+  bool check_border_sanity1(THD *thd);
+  /**
+    Like check_border_sanity1() but contains checks which must wait until
+    the start of the execution phase.
+
+    @param thd      Session thread
+
+    @returns true if error
+  */
+  bool check_border_sanity2(THD *thd);
+};
+
+/**
+  Collects evaluation requirements from a window function,
+  used by Item_sum::check_wf_semantics and its overrides.
+*/
+struct Window_evaluation_requirements {
+  /**
+    Set to true if window function requires row buffering
+  */
+  bool needs_buffer;
+  /**
+    Set to true if we need peerset for evaluation (e.g. CUME_DIST)
+  */
+  bool needs_peerset;
+  /**
+    Set to true if we need last peer for evaluation within a frame
+    (e.g. JSON_OBJECTAGG)
+  */
+  bool needs_last_peer_in_frame;
+  /**
+    Set to true if we need FIRST_VALUE or optimized MIN/MAX
+  */
+  bool opt_first_row;
+  /**
+    Set to true if we need LAST_VALUE or optimized MIN/MAX
+  */
+  bool opt_last_row;
+  Window::st_offset opt_nth_row;    ///< Used if we have NTH_VALUE
+  Window::st_ll_offset opt_ll_row;  ///< Used if we have LEAD or LAG
+  /**
+    Set to true if we can compute a sliding window by a combination of
+    undoing the contribution of rows going out of the frame and adding the
+    contribution of rows coming into the frame.  For example, SUM and AVG
+    allows this, but MAX/MIN do not. Only applicable if the frame has ROW
+    bounds unit.
+  */
+  bool row_optimizable;
+  /**
+    Similar to row_optimizable but for RANGE frame bounds unit
+  */
+  bool range_optimizable;
+
+  Window_evaluation_requirements()
+      : needs_buffer(false),
+        needs_peerset(false),
+        needs_last_peer_in_frame(false),
+        opt_first_row(false),
+        opt_last_row(false),
+        row_optimizable(true),
+        range_optimizable(true) {}
 };
 
 #endif /* WINDOWS_INCLUDED */
