@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2017, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -31,26 +31,27 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "arch0log.h"
 #include "clone0clone.h"
+#include "log0buf.h"
+#include "log0chkp.h"
+#include "log0encryption.h"
+#include "log0files_governor.h"
+#include "log0write.h"
 #include "srv0start.h"
+#include "ut0mutex.h"
 
 /** Chunk size for archiving redo log */
 const uint ARCH_LOG_CHUNK_SIZE = 1024 * 1024;
 
-/** Get redo file, header and trailer size
-@param[out]	file_sz		redo file size
-@param[out]	header_sz	redo header size
-@param[out]	trailer_sz	redo trailer size */
-void Log_Arch_Client_Ctx::get_header_size(ib_uint64_t &file_sz, uint &header_sz,
-                                          uint &trailer_sz) {
-  file_sz = srv_log_file_size;
+os_offset_t Log_Arch_Client_Ctx::get_archived_file_size() const {
+  return m_group->get_file_size();
+}
+
+void Log_Arch_Client_Ctx::get_header_size(uint &header_sz,
+                                          uint &trailer_sz) const {
   header_sz = LOG_FILE_HDR_SIZE;
   trailer_sz = OS_FILE_LOG_BLOCK_SIZE;
 }
 
-/** Start redo log archiving
-@param[out]	header	redo header. Caller must allocate buffer.
-@param[in]	len	buffer length
-@return error code */
 int Log_Arch_Client_Ctx::start(byte *header, uint len) {
   ut_ad(len >= LOG_FILE_HDR_SIZE);
 
@@ -70,9 +71,9 @@ int Log_Arch_Client_Ctx::start(byte *header, uint len) {
 
 /** Stop redo log archiving. Exact trailer length is returned as out
 parameter which could be less than the redo block size.
-@param[out]	trailer	redo trailer. Caller must allocate buffer.
-@param[in,out]	len	trailer length
-@param[out]	offset	trailer block offset
+@param[out]     trailer redo trailer. Caller must allocate buffer.
+@param[in,out]  len     trailer length
+@param[out]     offset  trailer block offset
 @return error code */
 int Log_Arch_Client_Ctx::stop(byte *trailer, uint32_t &len, uint64_t &offset) {
   lsn_t start_lsn;
@@ -104,8 +105,8 @@ int Log_Arch_Client_Ctx::stop(byte *trailer, uint32_t &len, uint64_t &offset) {
 }
 
 /** Get archived data file details
-@param[in]	cbk_func	callback called for each file
-@param[in]	ctx		callback function context
+@param[in]      cbk_func        callback called for each file
+@param[in]      ctx             callback function context
 @return error code */
 int Log_Arch_Client_Ctx::get_files(Log_Arch_Cbk *cbk_func, void *ctx) {
   ut_ad(m_state == ARCH_CLIENT_STATE_STOPPED);
@@ -199,51 +200,52 @@ void Log_Arch_Client_Ctx::release() {
   m_state = ARCH_CLIENT_STATE_INIT;
 }
 
-/** Update checkpoint LSN and related information in redo
-log header block.
-@param[in,out]	header		redo log header buffer
-@param[in]	checkpoint_lsn	checkpoint LSN for recovery */
-void Arch_Log_Sys::update_header(byte *header, lsn_t checkpoint_lsn) {
-  lsn_t start_lsn;
-  lsn_t lsn_offset;
-  ib_uint64_t file_size;
+os_offset_t Arch_Log_Sys::get_recommended_file_size() const {
+  if (log_sys == nullptr) {
+    ut_d(ut_error);
+    /* This shouldn't be executed, but if there was a bug,
+    we would prefer to return some value instead of crash,
+    because the archiver must not crash the server. */
+    ut_o(return srv_redo_log_capacity_used / LOG_N_FILES);
+  }
+  return log_sys->m_capacity.next_file_size();
+}
 
-  start_lsn = m_current_group->get_begin_lsn();
-  file_size = m_current_group->get_file_size();
-
-  start_lsn = ut_uint64_align_down(start_lsn, OS_FILE_LOG_BLOCK_SIZE);
+void Arch_Log_Sys::update_header(byte *header, lsn_t file_start_lsn,
+                                 lsn_t checkpoint_lsn) {
+  ut_a(file_start_lsn % OS_FILE_LOG_BLOCK_SIZE == 0);
 
   /* Copy Header information. */
-  log_files_header_fill(header, start_lsn, LOG_HEADER_CREATOR_CLONE);
+  Log_file_header file_header;
+  file_header.m_format = to_int(Log_format::CURRENT);
+  file_header.m_start_lsn = file_start_lsn;
+  file_header.m_creator_name = LOG_HEADER_CREATOR_CLONE;
+  file_header.m_log_flags = 0;
+  file_header.m_log_uuid = m_current_group->get_uuid();
+  ut_a(file_header.m_log_uuid > 0);
+  log_file_header_serialize(file_header, header);
 
-  ut_ad(checkpoint_lsn >= start_lsn);
+  /* Update checkpoint headers. */
+  Log_checkpoint_header checkpoint_header;
+  checkpoint_header.m_checkpoint_lsn = checkpoint_lsn;
+  log_checkpoint_header_serialize(checkpoint_header, header + LOG_CHECKPOINT_1);
+  log_checkpoint_header_serialize(checkpoint_header, header + LOG_CHECKPOINT_2);
 
-  lsn_offset = checkpoint_lsn - start_lsn;
-  file_size -= LOG_FILE_HDR_SIZE;
-
-  lsn_offset = lsn_offset % file_size;
-  lsn_offset += LOG_FILE_HDR_SIZE;
-
-  /* Update checkpoint lsn and offset. */
-  byte *src = header + LOG_CHECKPOINT_1;
-
-  mach_write_to_8(src + LOG_CHECKPOINT_LSN, checkpoint_lsn);
-  mach_write_to_8(src + LOG_CHECKPOINT_OFFSET, lsn_offset);
-  log_block_set_checksum(src, log_block_calc_checksum_crc32(src));
-
-  /* Copy to second checkpoint location */
-  byte *dest = header + LOG_CHECKPOINT_2;
-
-  memcpy(dest, src, OS_FILE_LOG_BLOCK_SIZE);
+  /* Fill encryption information if needed. */
+  if (log_sys == nullptr || !log_can_encrypt(*log_sys)) {
+    return;
+  }
+  byte *dest = header + LOG_ENCRYPTION;
+  log_file_header_fill_encryption(log_sys->m_encryption_metadata, false, dest);
 }
 
 /** Start redo log archiving.
 If archiving is already in progress, the client
 is attached to current group.
-@param[out]	group		log archive group
-@param[out]	start_lsn	start lsn for client
-@param[out]	header		redo log header
-@param[in]	is_durable	if client needs durable archiving
+@param[out]     group           log archive group
+@param[out]     start_lsn       start lsn for client
+@param[out]     header          redo log header
+@param[in]      is_durable      if client needs durable archiving
 @return error code */
 int Arch_Log_Sys::start(Arch_Group *&group, lsn_t &start_lsn, byte *header,
                         bool is_durable) {
@@ -253,57 +255,74 @@ int Arch_Log_Sys::start(Arch_Group *&group, lsn_t &start_lsn, byte *header,
 
   log_request_checkpoint(*log_sys, true);
 
-  mutex_enter(&m_mutex);
+  arch_mutex_enter();
+
+  if (m_state == ARCH_STATE_READ_ONLY) {
+    arch_mutex_exit();
+    return 0;
+  }
+
+  /* Wait for idle state, if preparing to idle. */
+  if (!wait_idle()) {
+    int err = 0;
+
+    if (srv_shutdown_state.load() >= SRV_SHUTDOWN_CLEANUP) {
+      err = ER_QUERY_INTERRUPTED;
+      my_error(err, MYF(0));
+    } else {
+      err = ER_INTERNAL_ERROR;
+      my_error(err, MYF(0), "Log Archiver wait too long");
+    }
+
+    arch_mutex_exit();
+    return (err);
+  }
+
+  ut_ad(m_state != ARCH_STATE_PREPARE_IDLE);
 
   if (m_state == ARCH_STATE_ABORT) {
-    mutex_exit(&m_mutex);
+    arch_mutex_exit();
     my_error(ER_QUERY_INTERRUPTED, MYF(0));
     return (ER_QUERY_INTERRUPTED);
   }
 
   /* Start archiver task, if needed. */
   if (m_state == ARCH_STATE_INIT) {
-    bool start_archiver = true;
+    auto err = start_log_archiver_background();
 
-    if (arch_page_sys) {
-      arch_page_sys->arch_mutex_enter();
+    if (err != 0) {
+      arch_mutex_exit();
 
-      /* Check if background is started by Page Archiver. */
-      start_archiver = arch_page_sys->is_init();
+      ib::error(ER_IB_MSG_17) << "Could not start Archiver"
+                              << " background task";
 
-      arch_page_sys->arch_mutex_exit();
-    }
-
-    if (start_archiver) {
-      auto err = start_archiver_background();
-
-      if (err != 0) {
-        mutex_exit(&m_mutex);
-
-        ib::error(ER_IB_MSG_17) << "Could not start Archiver"
-                                << " background task";
-
-        return (err);
-      }
+      return (err);
     }
   }
 
   /* Start archiving from checkpoint LSN. */
-  log_checkpointer_mutex_enter(*log_sys);
-  log_writer_mutex_enter(*log_sys);
+  log_writer_mutex_enter(*log_sys); /* protects log_sys->last_checkpoint_lsn */
+  log_files_mutex_enter(*log_sys);  /* protects log_sys->m_files */
 
-  start_lsn = log_sys->last_checkpoint_lsn;
+  start_lsn = log_sys->last_checkpoint_lsn.load();
 
-  auto lsn_offset = log_files_real_offset_for_lsn(*log_sys, start_lsn);
+  const auto file = log_sys->m_files.find(start_lsn);
 
-  auto start_index = static_cast<uint>(lsn_offset / log_sys->file_size);
-  uint64_t start_offset = lsn_offset % log_sys->file_size;
+  ut_ad(file != log_sys->m_files.end());
 
-  /* Copy to first checkpoint location */
-  auto src = static_cast<void *>(log_sys->checkpoint_buf);
-  auto dest = static_cast<void *>(header + LOG_CHECKPOINT_1);
+  if (file == log_sys->m_files.end()) {
+    ib::error(ER_IB_MSG_17) << "Could not start Archiver"
+                            << " background task because of"
+                            << " unexpected internal error";
+    log_files_mutex_exit(*log_sys);
+    log_writer_mutex_exit(*log_sys);
+    return ER_INTERNAL_ERROR;
+  }
 
-  memcpy(dest, src, OS_FILE_LOG_BLOCK_SIZE);
+  const auto start_index = file->m_id;
+
+  const uint64_t start_offset =
+      ut_uint64_align_down(file->offset(start_lsn), OS_FILE_LOG_BLOCK_SIZE);
 
   /* Need to create a new group if archiving is not in progress. */
   if (m_state == ARCH_STATE_IDLE || m_state == ARCH_STATE_INIT) {
@@ -312,43 +331,42 @@ int Arch_Log_Sys::start(Arch_Group *&group, lsn_t &start_lsn, byte *header,
     create_new_group = true;
   }
 
-  log_checkpointer_mutex_exit(*log_sys);
-
   /* Set archiver state to active. */
   if (m_state != ARCH_STATE_ACTIVE) {
-    m_state = ARCH_STATE_ACTIVE;
-    os_event_set(archiver_thread_event);
+    update_state_low(ARCH_STATE_ACTIVE);
+    os_event_set(log_archiver_thread_event);
   }
 
+  log_files_mutex_exit(*log_sys);
   log_writer_mutex_exit(*log_sys);
 
   /* Create a new group. */
   if (create_new_group) {
-    m_current_group = UT_NEW(Arch_Group(start_lsn, LOG_FILE_HDR_SIZE, &m_mutex),
-                             mem_key_archive);
+    m_current_group =
+        ut::new_withkey<Arch_Group>(ut::make_psi_memory_key(mem_key_archive),
+                                    start_lsn, LOG_FILE_HDR_SIZE, &m_mutex);
 
     if (m_current_group == nullptr) {
-      mutex_exit(&m_mutex);
+      arch_mutex_exit();
 
       my_error(ER_OUTOFMEMORY, MYF(0), sizeof(Arch_Group));
       return (ER_OUTOFMEMORY);
     }
 
-    /* Currently use log file size for archived files. */
-    auto db_err = m_current_group->init_file_ctx(
-        ARCH_DIR, ARCH_LOG_DIR, ARCH_LOG_FILE, 0,
-        static_cast<ib_uint64_t>(srv_log_file_size));
+    const Arch_group_uuid uuid{log_generate_uuid()};
+
+    auto db_err =
+        m_current_group->init_file_ctx(ARCH_DIR, ARCH_LOG_DIR, ARCH_LOG_FILE, 0,
+                                       get_recommended_file_size(), uuid);
 
     if (db_err != DB_SUCCESS) {
-      mutex_exit(&m_mutex);
+      arch_mutex_exit();
 
       my_error(ER_OUTOFMEMORY, MYF(0), sizeof(Arch_File_Ctx));
 
       return (ER_OUTOFMEMORY);
     }
 
-    start_offset = ut_uint64_align_down(start_offset, OS_FILE_LOG_BLOCK_SIZE);
-
     m_start_log_index = start_index;
     m_start_log_offset = start_offset;
 
@@ -358,14 +376,19 @@ int Arch_Log_Sys::start(Arch_Group *&group, lsn_t &start_lsn, byte *header,
   }
 
   /* Attach to the current group. */
-  m_current_group->attach(LSN_MAX, 0, is_durable);
+  m_current_group->attach(is_durable);
 
   group = m_current_group;
 
-  mutex_exit(&m_mutex);
+  arch_mutex_exit();
 
-  /* Update header with checkpoint LSN. */
-  update_header(header, start_lsn);
+  /* Update header with checkpoint LSN. Note, that arch mutex is released
+  and m_current_group should no longer be accessed. The group cannot be
+  freed as we have already attached to it. */
+  const lsn_t file_start_lsn =
+      ut_uint64_align_down(group->get_begin_lsn(), OS_FILE_LOG_BLOCK_SIZE);
+
+  update_header(header, file_start_lsn, start_lsn);
 
   return (0);
 }
@@ -379,30 +402,47 @@ void Arch_Group::adjust_end_lsn(lsn_t &stop_lsn, uint32_t &blk_len) {
 
   /* Increase Stop LSN 64 bytes ahead of file end not exceeding
   redo block size. */
-  DBUG_EXECUTE_IF("clone_arch_log_extra_bytes",
-                  blk_len = OS_FILE_LOG_BLOCK_SIZE;
-                  stop_lsn += 64;);
+  DBUG_EXECUTE_IF(
+      "clone_arch_log_extra_bytes", blk_len = OS_FILE_LOG_BLOCK_SIZE;
+      stop_lsn += 64; stop_lsn = std::min(stop_lsn, log_get_lsn(*log_sys)););
 }
+
+void Arch_Group::adjust_copy_length(lsn_t arch_lsn, uint32_t &copy_len) {
+  lsn_t end_lsn = LSN_MAX;
+  uint32_t blk_len = 0;
+  adjust_end_lsn(end_lsn, blk_len);
+
+  if (end_lsn <= arch_lsn) {
+    copy_len = 0;
+    return;
+  }
+
+  /* Adjust if copying beyond end LSN. */
+  auto len_left = end_lsn - arch_lsn;
+  len_left = ut_uint64_align_down(len_left, OS_FILE_LOG_BLOCK_SIZE);
+
+  if (len_left < copy_len) {
+    copy_len = static_cast<uint32_t>(len_left);
+  }
+}
+
 #endif /* UNIV_DEBUG */
 
 /** Stop redo log archiving.
 If other clients are there, the client is detached from
 the current group.
-@param[out]	group		log archive group
-@param[out]	stop_lsn	stop lsn for client
-@param[out]	log_blk		redo log trailer block
-@param[in,out]	blk_len		length in bytes
+@param[out]     group           log archive group
+@param[out]     stop_lsn        stop lsn for client
+@param[out]     log_blk         redo log trailer block
+@param[in,out]  blk_len         length in bytes
 @return error code */
 int Arch_Log_Sys::stop(Arch_Group *group, lsn_t &stop_lsn, byte *log_blk,
                        uint32_t &blk_len) {
   int err = 0;
+  blk_len = 0;
+  stop_lsn = m_archived_lsn.load();
 
-  if (log_blk == nullptr) {
-    mutex_enter(&m_mutex);
-    blk_len = 0;
-    stop_lsn = m_archived_lsn.load();
-
-  } else {
+  if (log_blk != nullptr) {
     /* Get the current LSN and trailer block. */
     log_buffer_get_last_block(*log_sys, stop_lsn, log_blk, blk_len);
 
@@ -412,75 +452,92 @@ int Arch_Log_Sys::stop(Arch_Group *group, lsn_t &stop_lsn, byte *log_blk,
     /* Will throw error, if shutdown. We still continue
     with detach but return the error. */
     err = wait_archive_complete(stop_lsn);
-
-    mutex_enter(&m_mutex);
   }
 
-  auto count_active = group->detach(stop_lsn);
+  arch_mutex_enter();
 
-  if (count_active == 0) {
+  if (m_state == ARCH_STATE_READ_ONLY) {
+    arch_mutex_exit();
+    return 0;
+  }
+
+  auto count_active_client = group->detach(stop_lsn, nullptr);
+  ut_ad(group->is_referenced());
+
+  if (!group->is_active() && err == 0) {
+    /* Archiving for the group has already stopped. */
+    my_error(ER_INTERNAL_ERROR, MYF(0), "Clone: Log Archiver failed");
+    err = ER_INTERNAL_ERROR;
+  }
+
+  if (group->is_active() && count_active_client == 0) {
     /* No other active client. Prepare to get idle. */
-    log_writer_mutex_enter(*log_sys);
-
-    if (m_state != ARCH_STATE_ABORT) {
-      ut_ad(m_state == ARCH_STATE_ACTIVE);
-      m_state = ARCH_STATE_PREPARE_IDLE;
-      os_event_set(archiver_thread_event);
+    if (m_state == ARCH_STATE_ACTIVE) {
+      /* The active group must be the current group. */
+      ut_ad(group == m_current_group);
+      update_state(ARCH_STATE_PREPARE_IDLE);
+      os_event_set(log_archiver_thread_event);
     }
-
-    log_writer_mutex_exit(*log_sys);
   }
 
-  mutex_exit(&m_mutex);
+  arch_mutex_exit();
 
   return (err);
 }
 
-/** Force to abort the archiver (state becomes ARCH_STATE_ABORT). */
 void Arch_Log_Sys::force_abort() {
   lsn_t lsn_max = LSN_MAX; /* unused */
   uint to_archive = 0;     /* unused */
   check_set_state(true, &lsn_max, &to_archive);
+  /* Above line changes state to ARCH_STATE_PREPARE_IDLE or ARCH_STATE_ABORT.
+  Let us notify the background thread to give it chance to notice the change and
+  wait for it to transition to ARCH_STATE_IDLE before returning (in case of
+  ARCH_STATE_ABORT, wait_idle() does nothing).*/
+  arch_mutex_enter();
+  wait_idle();
+  arch_mutex_exit();
 }
 
 /** Release the current group from client.
-@param[in]	group		group the client is attached to
-@param[in]	is_durable	if client needs durable archiving */
+@param[in]      group           group the client is attached to
+@param[in]      is_durable      if client needs durable archiving */
 void Arch_Log_Sys::release(Arch_Group *group, bool is_durable) {
-  mutex_enter(&m_mutex);
-  auto ref_count = group->release(is_durable);
+  arch_mutex_enter();
+
+  group->release(is_durable);
 
   /* Check if there are other references or archiving is still
   in progress. */
-  if (ref_count != 0 || group->is_active()) {
-    mutex_exit(&m_mutex);
+  if (group->is_referenced() || group->is_active()) {
+    arch_mutex_exit();
     return;
   }
 
   /* Cleanup the group. */
   ut_ad(group != m_current_group);
+
   m_group_list.remove(group);
 
-  UT_DELETE(group);
+  ut::delete_(group);
 
-  mutex_exit(&m_mutex);
+  arch_mutex_exit();
 }
 
 /** Check and set log archive system state and output the
 amount of redo log available for archiving.
-@param[in]	is_abort	need to abort
-@param[in,out]	archived_lsn	LSN up to which redo log is archived
-@param[out]	to_archive	amount of redo log to be archived */
+@param[in]      is_abort        need to abort
+@param[in,out]  archived_lsn    LSN up to which redo log is archived
+@param[out]     to_archive      amount of redo log to be archived */
 Arch_State Arch_Log_Sys::check_set_state(bool is_abort, lsn_t *archived_lsn,
                                          uint *to_archive) {
-  auto is_shutdown = (srv_shutdown_state == SRV_SHUTDOWN_LAST_PHASE ||
-                      srv_shutdown_state == SRV_SHUTDOWN_EXIT_THREADS);
+  auto is_shutdown = (srv_shutdown_state.load() == SRV_SHUTDOWN_LAST_PHASE ||
+                      srv_shutdown_state.load() == SRV_SHUTDOWN_EXIT_THREADS);
 
   auto need_to_abort = (is_abort || is_shutdown);
 
   *to_archive = 0;
 
-  mutex_enter(&m_mutex);
+  arch_mutex_enter();
 
   switch (m_state) {
     case ARCH_STATE_ACTIVE:
@@ -514,35 +571,40 @@ Arch_State Arch_Log_Sys::check_set_state(bool is_abort, lsn_t *archived_lsn,
       if (!need_to_abort) {
         break;
       }
-      /* fall through */
+
+      if (!is_shutdown) {
+        ut_ad(is_abort);
+        /* If caller asked to abort, move to prepare idle state. Archiver
+        thread will move to IDLE state eventually. */
+        update_state(ARCH_STATE_PREPARE_IDLE);
+        break;
+      }
+      [[fallthrough]];
 
     case ARCH_STATE_PREPARE_IDLE: {
       /* No active clients. Mark the group inactive and move
       to idle state. */
-      auto ref_count = m_current_group->disable(m_archived_lsn.load(), 0);
+      m_current_group->disable(m_archived_lsn.load());
 
       /* If no client reference, free the group. */
-      if (ref_count == 0) {
+      if (!m_current_group->is_referenced()) {
         m_group_list.remove(m_current_group);
 
-        UT_DELETE(m_current_group);
+        ut::delete_(m_current_group);
       }
 
       m_current_group = nullptr;
 
-      log_writer_mutex_enter(*log_sys);
-      m_state = ARCH_STATE_IDLE;
-      log_writer_mutex_exit(*log_sys);
+      update_state(ARCH_STATE_IDLE);
     }
-      /* fall through */
+      [[fallthrough]];
 
     case ARCH_STATE_IDLE:
     case ARCH_STATE_INIT:
 
-      if (need_to_abort) {
-        log_writer_mutex_enter(*log_sys);
-        m_state = ARCH_STATE_ABORT;
-        log_writer_mutex_exit(*log_sys);
+      /* Abort archiver thread only in case of shutdown. */
+      if (is_shutdown) {
+        update_state(ARCH_STATE_ABORT);
       }
       break;
 
@@ -552,28 +614,25 @@ Arch_State Arch_Log_Sys::check_set_state(bool is_abort, lsn_t *archived_lsn,
       break;
 
     default:
-      ut_ad(false);
+      ut_d(ut_error);
   }
 
   auto ret_state = m_state;
-  mutex_exit(&m_mutex);
+  arch_mutex_exit();
 
   return (ret_state);
 }
 
-/** Copy redo log from file context to archiver files.
-@param[in]	file_ctx	file context for system redo logs
-@param[in]	length		data to copy in bytes
-@return error code */
-dberr_t Arch_Log_Sys::copy_log(Arch_File_Ctx *file_ctx, uint length) {
+dberr_t Arch_Log_Sys::copy_log(Arch_File_Ctx *file_ctx, lsn_t start_lsn,
+                               uint length) {
   dberr_t err = DB_SUCCESS;
 
   if (file_ctx->is_closed()) {
     /* Open system redo log file context */
-    err = file_ctx->open(true, LSN_MAX, m_start_log_index, m_start_log_offset);
-
+    err =
+        file_ctx->open(true, LSN_MAX, m_start_log_index, m_start_log_offset, 0);
     if (err != DB_SUCCESS) {
-      return (err);
+      return err;
     }
   }
 
@@ -584,21 +643,27 @@ dberr_t Arch_Log_Sys::copy_log(Arch_File_Ctx *file_ctx, uint length) {
 
   /* Copy log data into one or more files in archiver group. */
   while (length > 0) {
-    ib_uint64_t len_copy;
-    ib_uint64_t len_left;
+    uint64_t len_copy;
+    uint64_t len_left;
 
-    len_copy = static_cast<ib_uint64_t>(length);
+    len_copy = static_cast<uint64_t>(length);
 
     len_left = file_ctx->bytes_left();
 
     /* Current file is over, switch to next file. */
     if (len_left == 0) {
-      err = file_ctx->open_next(LSN_MAX, LOG_FILE_HDR_SIZE);
+      err = file_ctx->open_next(LSN_MAX, LOG_FILE_HDR_SIZE, 0);
       if (err != DB_SUCCESS) {
         return (err);
       }
 
       len_left = file_ctx->bytes_left();
+
+      ut_ad(len_left > 0);
+    }
+
+    if (len_left == 0) {
+      return DB_ERROR;
     }
 
     /* Write as much as possible from current file. */
@@ -608,7 +673,31 @@ dberr_t Arch_Log_Sys::copy_log(Arch_File_Ctx *file_ctx, uint length) {
       write_size = length;
     }
 
-    err = curr_group->write_to_file(file_ctx, nullptr, write_size);
+    auto get_header_cbk = [start_lsn, this](uint64_t start_offset,
+                                            byte *header) {
+      ut_ad(start_lsn % OS_FILE_LOG_BLOCK_SIZE == 0);
+      ut_ad(start_offset % OS_FILE_LOG_BLOCK_SIZE == 0);
+
+      /* Assertions  above verify that the conditions below shouldn't hold.
+      However, they are only for debug binary. The release binary must not
+      crash in the Archiver. */
+      if (start_lsn % OS_FILE_LOG_BLOCK_SIZE != 0 ||
+          start_offset % OS_FILE_LOG_BLOCK_SIZE != 0) {
+        return DB_ERROR;
+      }
+
+      /* Do not store checkpoint_lsn inside archived log files, because these
+      files become later copied by possibly multiple readers and each of such
+      copies would possibly be started at different checkpoint lsn and after
+      all data >= checkpoint_lsn is copied, the valid checkpoint header would
+      anyway be written to the first file created in the copy. Therefore the
+      checkpoint_lsn is irrelevant for headers of archived log files. */
+      update_header(header, start_lsn + start_offset, 0);
+      return DB_SUCCESS;
+    };
+
+    err = curr_group->write_to_file(file_ctx, nullptr, write_size, false, false,
+                                    get_header_cbk);
 
     if (err != DB_SUCCESS) {
       return (err);
@@ -616,29 +705,92 @@ dberr_t Arch_Log_Sys::copy_log(Arch_File_Ctx *file_ctx, uint length) {
 
     ut_ad(length >= write_size);
     length -= write_size;
+    start_lsn += write_size;
   }
 
   return (DB_SUCCESS);
 }
 
+bool Arch_Log_Sys::wait_idle() {
+  ut_ad(mutex_own(&m_mutex));
+
+  if (m_state == ARCH_STATE_PREPARE_IDLE) {
+    os_event_set(log_archiver_thread_event);
+    bool is_timeout = false;
+    int alert_count = 0;
+
+    auto err = Clone_Sys::wait_default(
+        [&](bool alert, bool &result) {
+          ut_ad(mutex_own(&m_mutex));
+          result = (m_state == ARCH_STATE_PREPARE_IDLE);
+
+          if (srv_shutdown_state.load() >= SRV_SHUTDOWN_CLEANUP) {
+            return (ER_QUERY_INTERRUPTED);
+          }
+
+          if (result) {
+            os_event_set(log_archiver_thread_event);
+
+            /* Print messages every 1 minute - default is 5 seconds. */
+            if (alert && ++alert_count == 12) {
+              alert_count = 0;
+              ib::info(ER_IB_MSG_24) << "Log Archiving start: waiting for "
+                                        "idle state.";
+            }
+          }
+          return (0);
+        },
+        &m_mutex, is_timeout);
+
+    if (err == 0 && is_timeout) {
+      err = ER_INTERNAL_ERROR;
+      ib::info(ER_IB_MSG_25) << "Log Archiving start: wait for idle state "
+                                "timed out";
+      ut_d(ut_error);
+    }
+
+    if (err != 0) {
+      return (false);
+    }
+  }
+  return (true);
+}
+
 /** Wait for redo log archive up to the target LSN.
 We need to wait till current log sys LSN during archive stop.
-@param[in]	target_lsn	target archive LSN to wait for
+@param[in]      target_lsn      target archive LSN to wait for
 @return error code */
 int Arch_Log_Sys::wait_archive_complete(lsn_t target_lsn) {
   target_lsn = ut_uint64_align_down(target_lsn, OS_FILE_LOG_BLOCK_SIZE);
 
   /* Check and wait for archiver thread if needed. */
   if (m_archived_lsn.load() < target_lsn) {
-    os_event_set(archiver_thread_event);
+    os_event_set(log_archiver_thread_event);
 
     bool is_timeout = false;
     int alert_count = 0;
 
     auto err = Clone_Sys::wait_default(
         [&](bool alert, bool &result) {
+          /* Read consistent state. */
+          arch_mutex_enter();
+          auto state = m_state;
+          arch_mutex_exit();
+
+          /* Check if we need to abort. */
+          if (state == ARCH_STATE_ABORT ||
+              srv_shutdown_state.load() >= SRV_SHUTDOWN_CLEANUP) {
+            my_error(ER_QUERY_INTERRUPTED, MYF(0));
+            return (ER_QUERY_INTERRUPTED);
+          }
+
+          if (state == ARCH_STATE_IDLE || state == ARCH_STATE_PREPARE_IDLE) {
+            my_error(ER_INTERNAL_ERROR, MYF(0), "Clone: Log Archiver failed");
+            return (ER_INTERNAL_ERROR);
+          }
+
+          ut_ad(state == ARCH_STATE_ACTIVE);
 
-          int err2 = 0;
           /* Check if archived LSN is behind target. */
           auto archived_lsn = m_archived_lsn.load();
           result = (archived_lsn < target_lsn);
@@ -646,19 +798,9 @@ int Arch_Log_Sys::wait_archive_complete(lsn_t target_lsn) {
           /* Trigger flush if needed */
           auto flush = log_sys->write_lsn.load() < target_lsn;
 
-          /* Check if we need to abort. */
-          if (m_state == ARCH_STATE_ABORT ||
-              srv_shutdown_state != SRV_SHUTDOWN_NONE) {
-            err2 = ER_QUERY_INTERRUPTED;
-          }
-          ut_ad(m_state == ARCH_STATE_ACTIVE);
-
-          if (err2 != 0) {
-            my_error(ER_QUERY_INTERRUPTED, MYF(0));
-
-          } else if (result) {
+          if (result) {
             /* More data needs to be archived. */
-            os_event_set(archiver_thread_event);
+            os_event_set(log_archiver_thread_event);
 
             /* Write system redo log if needed. */
             if (flush) {
@@ -673,18 +815,17 @@ int Arch_Log_Sys::wait_archive_complete(lsn_t target_lsn) {
                   << target_lsn << " Archived LSN: " << archived_lsn;
             }
           }
-          return (err2);
+          return (0);
         },
         nullptr, is_timeout);
 
     if (err == 0 && is_timeout) {
-      ut_ad(false);
-
       ib::info(ER_IB_MSG_19) << "Clone Log archive stop: "
                                 "wait for Archiver timed out";
 
       err = ER_INTERNAL_ERROR;
       my_error(ER_INTERNAL_ERROR, MYF(0), "Clone: Log Archiver wait too long");
+      ut_d(ut_error);
     }
     return (err);
   }
@@ -694,27 +835,29 @@ int Arch_Log_Sys::wait_archive_complete(lsn_t target_lsn) {
 /** Archive accumulated redo log in current group.
 This interface is for archiver background task to archive redo log
 data by calling it repeatedly over time.
-@param[in]	init		true, if called for first time
-@param[in]	curr_ctx	system redo logs to copy data from
-@param[out]	arch_lsn	LSN up to which archiving is completed
-@param[out]	wait		true, if no more redo to archive
+@param[in, out] init            true when called the first time; it will then
+                                be set to false
+@param[in]      curr_ctx        system redo logs to copy data from
+@param[out]     arch_lsn        LSN up to which archiving is completed
+@param[out]     wait            true, if no more redo to archive
 @return true, if archiving is aborted */
 bool Arch_Log_Sys::archive(bool init, Arch_File_Ctx *curr_ctx, lsn_t *arch_lsn,
                            bool *wait) {
   Arch_State curr_state;
-  uint arch_len;
+  uint32_t arch_len;
 
   dberr_t err = DB_SUCCESS;
   bool is_abort = false;
 
   /* Initialize system redo log file context first time. */
   if (init) {
-    uint num_files;
+    /* We will use curr_ctx to read data from existing log files.
+    We set the limit for number of files as the biggest value to
+    avoid any such limitation in practice. */
+    const auto path = log_directory_path(log_sys->m_files_ctx);
 
-    num_files = static_cast<uint>(log_sys->n_files);
-
-    err = curr_ctx->init(srv_log_group_home_dir, nullptr, ib_logfile_basename,
-                         num_files, log_sys->file_size);
+    err = curr_ctx->init(path.c_str(), nullptr, LOG_FILE_BASE_NAME,
+                         std::numeric_limits<uint>::max());
 
     if (err != DB_SUCCESS) {
       is_abort = true;
@@ -724,20 +867,13 @@ bool Arch_Log_Sys::archive(bool init, Arch_File_Ctx *curr_ctx, lsn_t *arch_lsn,
   /* Find archive system state and amount of log data to archive. */
   curr_state = check_set_state(is_abort, arch_lsn, &arch_len);
 
-  if (curr_state == ARCH_STATE_ABORT) {
-    curr_ctx->close();
-    return (true);
+  if (curr_state == ARCH_STATE_ACTIVE) {
+    /* Adjust archiver length to no go beyond file end. */
+    DBUG_EXECUTE_IF("clone_arch_log_stop_file_end",
+                    m_current_group->adjust_copy_length(*arch_lsn, arch_len););
 
-  } else if (curr_state == ARCH_STATE_IDLE || curr_state == ARCH_STATE_INIT) {
-    curr_ctx->close();
-
-    *arch_lsn = LSN_MAX;
-    *wait = true;
-
-    return (false);
-
-  } else {
-    ut_ad(curr_state == ARCH_STATE_ACTIVE);
+    /* Simulate archive error. */
+    DBUG_EXECUTE_IF("clone_redo_no_archive", arch_len = 0;);
 
     if (arch_len == 0) {
       /* Nothing to archive. Need to wait. */
@@ -746,7 +882,10 @@ bool Arch_Log_Sys::archive(bool init, Arch_File_Ctx *curr_ctx, lsn_t *arch_lsn,
     }
 
     /* Copy data from system redo log files to archiver files */
-    err = copy_log(curr_ctx, arch_len);
+    err = copy_log(curr_ctx, *arch_lsn, arch_len);
+
+    /* Simulate archive error. */
+    DBUG_EXECUTE_IF("clone_redo_archive_error", err = DB_ERROR;);
 
     if (err == DB_SUCCESS) {
       *arch_lsn += arch_len;
@@ -756,8 +895,67 @@ bool Arch_Log_Sys::archive(bool init, Arch_File_Ctx *curr_ctx, lsn_t *arch_lsn,
 
     /* Force abort in case of an error archiving data. */
     curr_state = check_set_state(true, arch_lsn, &arch_len);
-    ut_ad(curr_state == ARCH_STATE_ABORT);
+  }
 
+  if (curr_state == ARCH_STATE_ABORT) {
+    curr_ctx->close();
     return (true);
   }
+
+  if (curr_state == ARCH_STATE_IDLE || curr_state == ARCH_STATE_INIT) {
+    curr_ctx->close();
+    *arch_lsn = LSN_MAX;
+    *wait = true;
+    return (false);
+  }
+
+  ut_ad(curr_state == ARCH_STATE_PREPARE_IDLE);
+  *wait = false;
+  return (false);
 }
+
+void Arch_Log_Sys::update_state(Arch_State state) {
+  log_t &log = *log_sys;
+  IB_mutex_guard writer_latch{&(log.writer_mutex), UT_LOCATION_HERE};
+  IB_mutex_guard files_latch{&(log.m_files_mutex), UT_LOCATION_HERE};
+  update_state_low(state);
+}
+
+void Arch_Log_Sys::update_state_low(Arch_State state) {
+  log_t &log = *log_sys;
+  ut_ad(log_writer_mutex_own(log));
+  ut_ad(log_files_mutex_own(log));
+
+  const bool was_active = is_active();
+  m_state = state;
+  const bool is_active_now = is_active();
+
+  if (was_active && !is_active_now) {
+    // De-register - transiting to active state
+    log_consumer_unregister(log, &m_log_consumer);
+  } else if (!was_active && is_active_now) {
+    // Register - transitioning inactive state
+    log_consumer_register(log, &m_log_consumer);
+  }
+}
+
+const std::string &Arch_log_consumer::get_name() const {
+  static std::string name{"log_archiver"};
+  return name;
+}
+
+Log_consumer::consumer_type Arch_log_consumer::get_consumer_type() const {
+  return Log_consumer::consumer_type::SERVER;
+}
+
+lsn_t Arch_log_consumer::get_consumed_lsn() const {
+  ut_a(arch_log_sys != nullptr);
+  ut_a(arch_log_sys->is_active());
+
+  lsn_t archiver_lsn = arch_log_sys->get_archived_lsn();
+  ut_a(archiver_lsn % OS_FILE_LOG_BLOCK_SIZE == 0);
+
+  return archiver_lsn;
+}
+
+void Arch_log_consumer::consumption_requested() { arch_wake_threads(); }
