@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2024, Oracle and/or its affiliates.
+Copyright (c) 1996, 2023, Oracle and/or its affiliates.
 Copyright (c) 2008, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -13,13 +13,12 @@ This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -230,7 +229,7 @@ void btr_search_sys_resize(ulint hash_size) {
     ib::error(ER_IB_MSG_45) << "btr_search_sys_resize failed because"
                                " hash index hash table is not empty.";
     ut_d(ut_error);
-    ut_o(return);
+    ut_o(return );
   }
 
   /* Step-2: Recreate hash tables with new size. */
@@ -869,7 +868,7 @@ bool btr_search_guess_on_hash(const dtuple_t *tuple, ulint mode,
     }
   } else {
     /* If we had a latch, then the guard is not needed. */
-    latch_guard.release();
+    latch_guard.commit();
   }
 
   ut_ad(rw_lock_get_writer(btr_get_search_latch(index)) != RW_LOCK_X);
@@ -906,7 +905,7 @@ bool btr_search_guess_on_hash(const dtuple_t *tuple, ulint mode,
     remove it. Up to this point we have the AHI is S-latched and since we found
     an AHI entry that leads to this block, the entry can't be removed and thus
     the block must be still in the buffer pool. */
-    latch_guard.reset();
+    latch_guard.rollback();
 
     buf_block_dbg_add_level(block, SYNC_TREE_NODE_FROM_HASH);
   }
@@ -1578,7 +1577,7 @@ static void btr_search_build_page_hash_index(dict_index_t *index,
     ha_insert_for_hash(table, hashes[i], block, recs[i]);
   }
 
-  x_latch_guard.reset();
+  x_latch_guard.rollback();
 
   MONITOR_ATOMIC_INC(MONITOR_ADAPTIVE_HASH_PAGE_ADDED);
 }
