@@ -123,8 +123,8 @@ ulint buf_read_page_low(dberr_t *err, bool sync, ulint type, ulint mode,
 
   IORequest request(type | IORequest::READ);
 
-  *err = _fil_io(request, sync, page_id, page_size, 0, page_size.physical(),
-                 dst, bpage, trx, should_buffer);
+  *err = fil_io(request, sync, page_id, page_size, 0, page_size.physical(), dst,
+                bpage, trx, should_buffer);
 
   if (sync) {
     thd_wait_end(nullptr);
