@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -1430,7 +1429,7 @@ void clone_update_gtid_status(std::string &gtids) {
   std::string recovery_file(CLONE_INNODB_RECOVERY_FILE);
   if (!file_exists(recovery_file)) {
     ut_d(ut_error);
-    ut_o(return);
+    ut_o(return );
   }
   /* Open status file to append GTID. */
   std::ofstream status_file;
@@ -2354,7 +2353,7 @@ static int clone_drop_binary_logs(THD *thd) {
     thd->clear_error();
 
     /* 2. Clear all binary logs and GTID. */
-    snprintf(sql_stmt, sizeof(sql_stmt), "RESET MASTER");
+    snprintf(sql_stmt, sizeof(sql_stmt), "RESET BINARY LOGS AND GTIDS");
 
     if (clone_execute_query(thd, &sql_stmt[0], 1, false)) {
       err = ER_INTERNAL_ERROR;
@@ -2443,7 +2442,7 @@ static void clone_init_compression(THD *thd) {
 
   if (dc->fetch_global_component_ids<dd::Table>(&dd_table_ids)) {
     ut_d(ut_error);
-    ut_o(return);
+    ut_o(return );
   }
 
   for (auto dd_table_id : dd_table_ids) {
