@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2020, 2025, Oracle and/or its affiliates.
+Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -179,14 +178,14 @@ struct File_cursor : public Load_cursor {
   @param[in] builder            The index build driver.
   @param[in] file               File to read from.
   @param[in] buffer_size        IO buffer size to use for reads.
-  @param[in] range              Offsets of the chunk to read from the file
+  @param[in] size               Size of the file in bytes.
   @param[in,out] stage          PFS observability. */
   File_cursor(Builder *builder, const Unique_os_file_descriptor &file,
-              size_t buffer_size, const Range &range, Alter_stage *stage,
+              size_t buffer_size, os_offset_t size, Alter_stage *stage,
               const Write_offsets &write_offsets) noexcept;
 
   /** Destructor. */
-  ~File_cursor() override;
+  ~File_cursor() override = default;
 
   /** Open the cursor.
   @return DB_SUCCESS or error code. */
@@ -225,9 +224,6 @@ struct File_cursor : public Load_cursor {
   /** PFS monitoring. */
   Alter_stage *m_stage{};
 
-  /** Number of rows that were fetched but not yet reported to the PFS. */
-  uint64_t m_processed_rows_to_report{};
-
   friend struct Merge_cursor;
 };
 
@@ -237,11 +233,9 @@ bool Load_cursor::duplicates_detected() const noexcept {
 
 dberr_t File_reader::get_tuple(Builder *builder, mem_heap_t *heap,
                                dtuple_t *&dtuple) noexcept {
-  dtuple =
-      row_rec_to_index_entry_low(m_mrec, m_index, &m_field_offsets[0], heap);
+  dtuple = row_rec_to_index_entry_low(m_mrec, m_index, &m_offsets[0], heap);
   if (!builder->is_fts_index()) {
-    return builder->dtuple_copy_blobs(dtuple, &m_field_offsets[0], m_mrec,
-                                      heap);
+    return builder->dtuple_copy_blobs(dtuple, &m_offsets[0], m_mrec, heap);
   } else {
     return DB_SUCCESS;
   }
@@ -249,22 +243,16 @@ dberr_t File_reader::get_tuple(Builder *builder, mem_heap_t *heap,
 
 File_cursor::File_cursor(Builder *builder,
                          const Unique_os_file_descriptor &file,
-                         size_t buffer_size, const Range &range,
+                         size_t buffer_size, os_offset_t size,
                          Alter_stage *stage,
                          const Write_offsets &write_offsets) noexcept
     : Load_cursor(builder, nullptr),
-      m_reader(file, builder->index(), buffer_size, range,
+      m_reader(file, builder->index(), buffer_size, size,
                builder->get_space_id(), write_offsets),
       m_stage(stage) {
   ut_a(m_reader.m_file.is_open());
 }
 
-File_cursor::~File_cursor() {
-  if (m_processed_rows_to_report > 0) {
-    m_stage->inc_progress_if_needed(m_processed_rows_to_report, true);
-  }
-}
-
 dberr_t File_cursor::open() noexcept {
   m_tuple_heap.create(2048, UT_LOCATION_HERE);
 
@@ -275,8 +263,7 @@ dberr_t File_cursor::fetch() noexcept {
   m_tuple_heap.clear();
 
   if (m_stage != nullptr) {
-    m_processed_rows_to_report++;
-    m_stage->inc_progress_if_needed(m_processed_rows_to_report);
+    m_stage->inc(1);
   }
 
   return m_builder->get_error();
@@ -304,7 +291,7 @@ dberr_t File_cursor::fetch(const mrec_t *&mrec, ulint *&offsets) noexcept {
   }
 
   mrec = m_reader.m_mrec;
-  offsets = &m_reader.m_field_offsets[0];
+  offsets = &m_reader.m_offsets[0];
 
   return DB_SUCCESS;
 }
@@ -312,7 +299,7 @@ dberr_t File_cursor::fetch(const mrec_t *&mrec, ulint *&offsets) noexcept {
 dberr_t File_cursor::next() noexcept {
   auto err = m_reader.next();
 
-  if (likely(err != DB_END_OF_INDEX)) {
+  if (unlikely(err != DB_END_OF_INDEX)) {
     m_err = err;
   }
 
@@ -350,24 +337,25 @@ bool Merge_cursor::Compare::operator()(const File_cursor *lhs,
 
   ut_a(l.m_index == r.m_index);
 
-  auto cmp = cmp_rec_rec_simple(r.m_mrec, l.m_mrec, &r.m_field_offsets[0],
-                                &l.m_field_offsets[0], r.m_index,
+  auto cmp = cmp_rec_rec_simple(r.m_mrec, l.m_mrec, &r.m_offsets[0],
+                                &l.m_offsets[0], r.m_index,
                                 m_dup != nullptr ? m_dup->m_table : nullptr);
 
   /* Check for duplicates. */
   if (unlikely(cmp == 0 && m_dup != nullptr)) {
-    m_dup->report(l.m_mrec, &l.m_field_offsets[0]);
+    m_dup->report(l.m_mrec, &l.m_offsets[0]);
   }
 
   return cmp < 0;
 }
 
-dberr_t Merge_cursor::add_file(const ddl::file_t &file, size_t buffer_size,
-                               const Range &range) noexcept {
+dberr_t Merge_cursor::add_file(const ddl::file_t &file,
+                               size_t buffer_size) noexcept {
   ut_a(file.m_file.is_open());
+
   auto cursor = ut::new_withkey<File_cursor>(
       ut::make_psi_memory_key(mem_key_ddl), m_builder, file.m_file, buffer_size,
-      range, m_stage, file.m_write_offsets);
+      file.m_size, m_stage, file.m_write_offsets);
 
   if (cursor == nullptr) {
     m_err = DB_OUT_OF_MEMORY;
@@ -379,9 +367,16 @@ dberr_t Merge_cursor::add_file(const ddl::file_t &file, size_t buffer_size,
   return DB_SUCCESS;
 }
 
-dberr_t Merge_cursor::add_file(const ddl::file_t &file,
-                               size_t buffer_size) noexcept {
-  return add_file(file, buffer_size, Range{0, file.m_size});
+dberr_t Merge_cursor::add_file(const ddl::file_t &file, size_t buffer_size,
+                               os_offset_t offset) noexcept {
+  auto err = add_file(file, buffer_size);
+
+  if (err != DB_SUCCESS) {
+    return err;
+  } else {
+    m_cursors.back()->m_reader.set_offset(offset);
+    return DB_SUCCESS;
+  }
 }
 
 void Merge_cursor::clear_eof() noexcept {
@@ -393,7 +388,7 @@ void Merge_cursor::clear_eof() noexcept {
 
   for (auto cursor : m_cursors) {
     ut_a(cursor->m_err == DB_END_OF_INDEX);
-    if (!cursor->m_reader.end_of_range()) {
+    if (!cursor->m_reader.eof()) {
       cursor->m_err = DB_SUCCESS;
       m_pq.push(cursor);
     }
@@ -808,6 +803,7 @@ dberr_t Builder::get_virtual_column(Copy_ctx &ctx, const dict_field_t *ifield,
                                     size_t &mv_rows_added) noexcept {
   const auto n_added = mv_rows_added;
   auto v_col = reinterpret_cast<const dict_v_col_t *>(col);
+  const auto clust_index = m_ctx.m_new_table->first_index();
   auto key_buffer = m_thread_ctxs[ctx.m_thread_id]->m_key_buffer;
   mem_heap_t **compress_heap = &m_thread_ctxs[ctx.m_thread_id]->m_compress_heap;
 
@@ -822,9 +818,9 @@ dberr_t Builder::get_virtual_column(Copy_ctx &ctx, const dict_field_t *ifield,
       auto p = m_v_heap.get();
 
       src_field = innobase_get_computed_value(
-          ctx.m_row.m_ptr, v_col, m_ctx.m_new_table, &p, key_buffer->heap(),
-          m_ctx.thd(), ctx.m_my_table, compress_heap, ifield,
-          m_ctx.m_old_table);
+          ctx.m_row.m_ptr, v_col, clust_index, &p, key_buffer->heap(), ifield,
+          m_ctx.thd(), ctx.m_my_table, m_ctx.m_old_table, nullptr, nullptr,
+          compress_heap);
 
       m_v_heap.reset(p);
 
@@ -854,8 +850,8 @@ dberr_t Builder::get_virtual_column(Copy_ctx &ctx, const dict_field_t *ifield,
     auto p = m_v_heap.get();
 
     src_field = innobase_get_computed_value(
-        ctx.m_row.m_ptr, v_col, m_ctx.m_new_table, &p, nullptr, m_ctx.thd(),
-        ctx.m_my_table, compress_heap, ifield, m_ctx.m_old_table);
+        ctx.m_row.m_ptr, v_col, clust_index, &p, nullptr, ifield, m_ctx.thd(),
+        ctx.m_my_table, m_ctx.m_old_table, nullptr, nullptr, compress_heap);
 
     m_v_heap.reset(p);
 
@@ -887,6 +883,8 @@ dberr_t Builder::copy_fts_column(Copy_ctx &ctx, dfield_t *field) noexcept {
     doc_id = fts.m_doc_id->current();
   }
 
+  ut_a(doc_id <= 4294967295u);
+
   if (unlikely(!dfield_is_null(field))) {
     auto ptr = ut::malloc_withkey(UT_NEW_THIS_FILE_PSI_KEY,
                                   sizeof(FTS::Doc_item) + field->len);
@@ -912,11 +910,10 @@ dberr_t Builder::copy_columns(Copy_ctx &ctx, size_t &mv_rows_added,
                               doc_id_t &write_doc_id) noexcept {
   auto &fts = m_ctx.m_fts;
   auto key_buffer = m_thread_ctxs[ctx.m_thread_id]->m_key_buffer;
+  auto &fields = key_buffer->m_dtuples[key_buffer->size()];
 
-  dfield_t *fields;
   const dict_field_t *ifield = m_index->get_field(0);
   auto field = fields = key_buffer->alloc(ctx.m_n_fields);
-  key_buffer->m_dtuples.push_back(fields);
   const auto page_size = dict_table_page_size(m_ctx.m_old_table);
 
   for (size_t i = 0; i < ctx.m_n_fields; ++i, ++field, ++ifield) {
@@ -1061,6 +1058,10 @@ dberr_t Builder::copy_row(Copy_ctx &ctx, size_t &mv_rows_added) noexcept {
 
   ut_a(ctx.m_n_rows_added == 0);
 
+  if (unlikely(key_buffer->full())) {
+    return DB_OVERFLOW;
+  }
+
   // clang-format off
   DBUG_EXECUTE_IF(
       "ddl_buf_add_two",
@@ -1075,6 +1076,10 @@ dberr_t Builder::copy_row(Copy_ctx &ctx, size_t &mv_rows_added) noexcept {
   doc_id_t write_doc_id{};
 
   for (;;) {
+    if (unlikely(key_buffer->full())) {
+      return ctx.m_n_rows_added == 0 ? DB_OVERFLOW : DB_SUCCESS;
+    }
+
     // clang-format off
     DBUG_EXECUTE_IF(
         "ddl_add_multi_value",
@@ -1240,23 +1245,6 @@ dberr_t Builder::key_buffer_sort(size_t thread_id) noexcept {
   return DB_SUCCESS;
 }
 
-dberr_t Builder::handle_error(dberr_t err) noexcept {
-  set_error(err);
-
-  if (m_btr_load != nullptr) {
-    /* page_loaders[0] has increased buf_fix_count through release(). This is
-    decremented by calling latch(). Similar release() calls for page_loaders at
-    non-zero levels are handled in finish() */
-    m_btr_load->latch();
-    err = m_btr_load->finish(err);
-
-    ut::delete_(m_btr_load);
-    m_btr_load = nullptr;
-  }
-
-  return get_error();
-}
-
 dberr_t Builder::insert_direct(Cursor &cursor, size_t thread_id) noexcept {
   ut_a(m_id == 0);
   ut_ad(is_skip_file_sort());
@@ -1269,32 +1257,15 @@ dberr_t Builder::insert_direct(Cursor &cursor, size_t thread_id) noexcept {
   {
     auto err = m_ctx.check_state_of_online_build_log();
 
-    DBUG_EXECUTE_IF("builder_insert_direct_trigger_error", {
-      static int count = 0;
-      ++count;
-      if (count > 1) {
-        err = DB_ONLINE_LOG_TOO_BIG;
-        m_ctx.m_trx->error_key_num = SERVER_CLUSTER_INDEX_ID;
-      }
-    });
-
     if (err != DB_SUCCESS) {
-      return handle_error(err);
+      set_error(err);
+      err = m_btr_load->finish(err);
+      ut::delete_(m_btr_load);
+      m_btr_load = nullptr;
+      return get_error();
     }
   }
 
-  DBUG_EXECUTE_IF("builder_insert_direct_no_builder",
-                  { static_cast<void>(handle_error(DB_ERROR)); });
-
-  if (m_btr_load == nullptr) {
-    auto ind = index();
-    ib::error(ER_IB_MSG_DDL_FAIL_NO_BUILDER, static_cast<unsigned>(get_state()),
-              static_cast<unsigned>(get_error()), id(), ind->name(),
-              ind->space_id(), static_cast<unsigned>(ind->page),
-              ctx().old_table()->name.m_name, ctx().new_table()->name.m_name);
-    return DB_ERROR;
-  }
-
   m_btr_load->latch();
 
   auto thread_ctx = m_thread_ctxs[thread_id];
@@ -1430,12 +1401,9 @@ dberr_t Builder::add_to_key_buffer(Copy_ctx &ctx,
     ut_ad(m_id == 0);
     ut_ad(key_buffer->is_clustered());
 
-    /* Detect duplicates by comparing the current record with previous record.
-    The current record will be used to report duplicates. m_prev_fields cannot
-    be used for it, because contrary to current record it contains only unique
-    fields. Which is fine for key comparison, but not enough for reporting. */
+    /* Detect duplicates by comparing the current record with previous record.*/
     if (m_prev_fields != nullptr &&
-        Key_sort_buffer::compare(fields, m_prev_fields, &m_clust_dup) == 0) {
+        Key_sort_buffer::compare(m_prev_fields, fields, &m_clust_dup) == 0) {
       set_error(DB_DUPLICATE_KEY);
       return get_error();
     }
@@ -1487,12 +1455,6 @@ dberr_t Builder::bulk_add_row(Cursor &cursor, Row &row, size_t thread_id,
       ut_a(err == DB_SUCCESS || err == DB_OVERFLOW);
       err = key_buffer_sort(thread_id);
 
-      if (DBUG_EVALUATE_IF("builder_bulk_add_row_trigger_error_1",
-                           (m_btr_load && m_btr_load->get_n_recs() != 0),
-                           false)) {
-        err = DB_DUPLICATE_KEY;
-      }
-
       if (err != DB_SUCCESS) {
         set_error(err);
         return get_error();
@@ -1504,11 +1466,6 @@ dberr_t Builder::bulk_add_row(Cursor &cursor, Row &row, size_t thread_id,
           scan thread. Required for the log_free_check() during mtr.commit(). */
           err = cursor.copy_row(thread_id, row);
 
-          if (DBUG_EVALUATE_IF("builder_bulk_add_row_trigger_error_2", true,
-                               false)) {
-            err = DB_INVALID_NULL;
-          }
-
           if (err != DB_SUCCESS) {
             set_error(err);
             return get_error();
@@ -1527,7 +1484,6 @@ dberr_t Builder::bulk_add_row(Cursor &cursor, Row &row, size_t thread_id,
         key_buffer->clear();
 
         if (err != DB_SUCCESS) {
-          /* @insert_direct cleans up m_btr_load in case of error */
           ut_a(m_btr_load == nullptr);
           set_error(err);
           return get_error();
@@ -1555,34 +1511,33 @@ dberr_t Builder::bulk_add_row(Cursor &cursor, Row &row, size_t thread_id,
 
     IF_ENABLED("ddl_ins_spatial_fail", set_error(DB_FAIL); return get_error();)
 
-    if (DBUG_EVALUATE_IF("builder_bulk_add_row_trigger_error_3", true, false) ||
-        (!thread_ctx->m_file.m_file.is_open() &&
-         !create_file(thread_ctx->m_file))) {
-      set_error(DB_IO_ERROR);
-      return get_error();
+    if (!thread_ctx->m_file.m_file.is_open()) {
+      if (!create_file(thread_ctx->m_file)) {
+        set_error(DB_IO_ERROR);
+        return get_error();
+      }
     }
 
     IF_ENABLED("ddl_write_failure", set_error(DB_TEMP_FILE_WRITE_FAIL);
                return get_error();)
 
-    auto persistor = [&](IO_buffer io_buffer) -> dberr_t {
+    auto persistor = [&](IO_buffer io_buffer, os_offset_t &n) -> dberr_t {
       auto &file = thread_ctx->m_file;
 
       ut_a(!(file.m_size % IO_BLOCK_SIZE));
 
-      os_offset_t n = io_buffer.second;
-      ut_a(n != 0);
-      ut_a(n % IO_BLOCK_SIZE == 0);
+      if (n == 0) {
+        n = ut_uint64_align_down(io_buffer.second, IO_BLOCK_SIZE);
+      } else {
+        ut_a(n == io_buffer.second);
+        n = ut_uint64_align_up(io_buffer.second, IO_BLOCK_SIZE);
+      }
+      ut_a(n >= IO_BLOCK_SIZE);
 
       auto crypt_buffer = thread_ctx->m_io_buffer_crypt;
       auto err = ddl::pwrite(file.m_file.get(), io_buffer.first, n, file.m_size,
                              crypt_buffer.first, get_space_id());
 
-      if (DBUG_EVALUATE_IF("builder_bulk_add_row_trigger_error_4", true,
-                           false)) {
-        err = DB_IO_ERROR;
-      }
-
       if (err != DB_SUCCESS) {
         set_error(DB_TEMP_FILE_WRITE_FAIL);
         return get_error();
@@ -1619,23 +1574,14 @@ dberr_t Builder::add_row(Cursor &cursor, Row &row, size_t thread_id,
                          Latch_release &&latch_release) noexcept {
   auto err = m_ctx.check_state_of_online_build_log();
 
-  DBUG_EXECUTE_IF("builder_add_row_trigger_error", {
-    err = DB_ONLINE_LOG_TOO_BIG;
-    m_ctx.m_trx->error_key_num = SERVER_CLUSTER_INDEX_ID;
-  });
-
   if (err != DB_SUCCESS) {
-    err = handle_error(err);
+    set_error(err);
   } else if (is_spatial_index()) {
     if (!cursor.eof()) {
       err = batch_add_row(row, thread_id);
     }
   } else {
     err = bulk_add_row(cursor, row, thread_id, std::move(latch_release));
-    if (unlikely(err != DB_OVERFLOW && err != DB_SUCCESS &&
-                 err != DB_END_OF_INDEX)) {
-      err = handle_error(err);
-    }
     clear_virtual_heap();
   }
 
