@@ -1,19 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2025, Oracle and/or its affiliates.
+Copyright (c) 1996, 2023, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -573,7 +572,7 @@ void dict_col_t::set_default(const byte *value, size_t length,
   instant_default->len = length;
 }
 
-bool dict_col_default_t::operator==(const dict_col_default_t &other) {
+bool dict_col_default_t::operator==(const dict_col_default_t &other) const {
   /* If the lengths are different, trivially the default values are not
   the same, return false immediately */
   if (len != other.len) {
@@ -595,7 +594,7 @@ bool dict_col_default_t::operator==(const dict_col_default_t &other) {
   return true;
 }
 
-bool dict_col_default_t::operator!=(const dict_col_default_t &other) {
+bool dict_col_default_t::operator!=(const dict_col_default_t &other) const {
   return !(*this == other);
 }
 
@@ -879,28 +878,6 @@ bool dict_foreign_set_validate(const dict_table_t &table) {
   return (dict_foreign_set_validate(table.foreign_set) &&
           dict_foreign_set_validate(table.referenced_set));
 }
-
-bool dict_foreign_t::is_fts_col_affected() const {
-  /* The check is skipped:
-     - if the table has no full text index defined.
-     - if it is a self referential foreign constaint. This is because
-       in the context of cascading DML operation, only the referenced
-       table is relevant for the validation even if the current table
-       has FTS index.*/
-  if (!foreign_table->fts || foreign_table == referenced_table) {
-    return false;
-  }
-
-  for (ulint i = 0; i < n_fields; i++) {
-    const dict_col_t *col = foreign_index->get_col(i);
-    if (dict_table_is_fts_column(foreign_table->fts->indexes,
-                                 dict_col_get_no(col),
-                                 col->is_virtual()) != ULINT_UNDEFINED) {
-      return true;
-    }
-  }
-  return false;
-}
 #endif /* !UNIV_HOTBACKUP */
 
 std::ostream &operator<<(std::ostream &out, const dict_foreign_t &foreign) {
@@ -924,3 +901,10 @@ std::ostream &operator<<(std::ostream &out, const dict_foreign_set &fk_set) {
 page_size_t dict_index_t::get_page_size() const {
   return (dict_table_page_size(table));
 }
+
+bool dict_table_t::has_pk() const {
+  const dict_index_t *first = first_index();
+  const size_t len = strlen(innobase_index_reserve_name);
+  const int cmp = strncmp(innobase_index_reserve_name, first->name(), len);
+  return cmp != 0;
+}
