@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2007, 2025, Oracle and/or its affiliates.
+Copyright (c) 2007, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -54,6 +53,8 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "ut0list.h"
 #include "ut0wqueue.h"
 
+struct CHARSET_INFO;
+
 /** The FTS optimize thread's work queue. */
 static ib_wqueue_t *fts_optimize_wq;
 
@@ -274,12 +275,6 @@ static const char *fts_end_delete_sql =
     "DELETE FROM $being_deleted;\n"
     "DELETE FROM $being_deleted_cache;\n";
 
-/** Set fts_zip_t::word to empty string. */
-static void fts_zip_clear_word(fts_zip_t &zip) {
-  zip.word.f_len = 0;
-  memset(zip.word.f_str, 0, FTS_MAX_WORD_LEN);
-}
-
 /** Initialize fts_zip_t. */
 static void fts_zip_initialize(
     fts_zip_t *zip) /*!< out: zip instance to initialize */
@@ -291,7 +286,8 @@ static void fts_zip_initialize(
 
   zip->last_big_block = 0;
 
-  fts_zip_clear_word(*zip);
+  zip->word.f_len = 0;
+  memset(zip->word.f_str, 0, FTS_MAX_WORD_LEN);
 
   ib_vector_reset(zip->blocks);
 
@@ -469,8 +465,7 @@ dberr_t fts_index_fetch_nodes(
     que_t **graph,            /*!< in: prepared statement */
     fts_table_t *fts_table,   /*!< in: table of the FTS INDEX */
     const fts_string_t *word, /*!< in: the word to fetch */
-    fts_fetch_t *fetch,       /*!< in: fetch callback.*/
-    bool exact_match)         /*!< in: exact match.*/
+    fts_fetch_t *fetch)       /*!< in: fetch callback.*/
 {
   pars_info_t *info;
   dberr_t error;
@@ -500,27 +495,24 @@ dberr_t fts_index_fetch_nodes(
   pars_info_bind_varchar_literal(info, "word", word->f_str, word->f_len);
 
   if (!*graph) {
-    *graph = fts_parse_sql(
-        fts_table, info,
-        mem_heap_printf(info->heap,
-                        "DECLARE FUNCTION my_func;\n"
-                        "DECLARE CURSOR c IS"
-                        " SELECT word, doc_count, first_doc_id, last_doc_id,"
-                        " ilist\n"
-                        " FROM $table_name\n"
-                        " WHERE word %s :word\n"
-                        " ORDER BY first_doc_id;\n"
-                        "BEGIN\n"
-                        "\n"
-                        "OPEN c;\n"
-                        "WHILE 1 = 1 LOOP\n"
-                        "  FETCH c INTO my_func();\n"
-                        "  IF c %% NOTFOUND THEN\n"
-                        "    EXIT;\n"
-                        "  END IF;\n"
-                        "END LOOP;\n"
-                        "CLOSE c;",
-                        (exact_match ? "=" : "LIKE")));
+    *graph = fts_parse_sql(fts_table, info,
+                           "DECLARE FUNCTION my_func;\n"
+                           "DECLARE CURSOR c IS"
+                           " SELECT word, doc_count, first_doc_id, last_doc_id,"
+                           " ilist\n"
+                           " FROM $table_name\n"
+                           " WHERE word LIKE :word\n"
+                           " ORDER BY first_doc_id;\n"
+                           "BEGIN\n"
+                           "\n"
+                           "OPEN c;\n"
+                           "WHILE 1 = 1 LOOP\n"
+                           "  FETCH c INTO my_func();\n"
+                           "  IF c % NOTFOUND THEN\n"
+                           "    EXIT;\n"
+                           "  END IF;\n"
+                           "END LOOP;\n"
+                           "CLOSE c;");
   }
 
   for (;;) {
@@ -550,16 +542,25 @@ dberr_t fts_index_fetch_nodes(
   return (error);
 }
 
-/** Extract a given number of bytes into a buffer */
-static bool fts_zip_extract_bytes(fts_zip_t *zip, /*!< in: Zip state + data */
-                                  byte *buf, /*!< out: buffer to be filled */
-                                  unsigned int size) /*!< in: size of buffer */
+/** Read a word */
+static byte *fts_zip_read_word(
+    fts_zip_t *zip,     /*!< in: Zip state + data */
+    fts_string_t *word) /*!< out: uncompressed word */
 {
-  bool complete = false;
-  zip->zp->next_out = buf;
-  zip->zp->avail_out = size;
+  short len = 0;
   void *null = nullptr;
+  byte *ptr = word->f_str;
   int flush = Z_NO_FLUSH;
+  bool read_something = false;
+
+  /* Either there was an error or we are at the Z_STREAM_END. */
+  if (zip->status != Z_OK) {
+    return (nullptr);
+  }
+
+  zip->zp->next_out = reinterpret_cast<byte *>(&len);
+  zip->zp->avail_out = sizeof(len);
+
   while (zip->status == Z_OK && zip->zp->avail_out > 0) {
     /* Finished decompressing block. */
     if (zip->zp->avail_in == 0) {
@@ -592,26 +593,22 @@ static bool fts_zip_extract_bytes(fts_zip_t *zip, /*!< in: Zip state + data */
 
     switch (zip->status = inflate(zip->zp, flush)) {
       case Z_OK:
-        if (zip->zp->avail_out == 0) {
-          complete = true;
+        if (zip->zp->avail_out == 0 && len > 0) {
+          ut_a(len <= FTS_MAX_WORD_LEN);
+          ptr[len] = 0;
+
+          zip->zp->next_out = ptr;
+          zip->zp->avail_out = len;
+
+          word->f_len = len;
+          len = 0;
+          read_something = true;
         }
         break;
 
-      case Z_STREAM_END:
-        if (zip->zp->avail_out == 0) {
-          complete = true;
-        }
-
-        [[fallthrough]];
       case Z_BUF_ERROR: /* No progress possible. */
+      case Z_STREAM_END:
         inflateEnd(zip->zp);
-        /* All blocks must be freed at end of inflate. */
-        for (ulint i = 0; i < ib_vector_size(zip->blocks); ++i) {
-          if (ib_vector_getp(zip->blocks, i)) {
-            ut::free(ib_vector_getp(zip->blocks, i));
-            ib_vector_set(zip->blocks, i, &null);
-          }
-        }
         break;
 
       case Z_STREAM_ERROR:
@@ -620,37 +617,23 @@ static bool fts_zip_extract_bytes(fts_zip_t *zip, /*!< in: Zip state + data */
     }
   }
 
-  return complete;
-}
-
-/** Read a word */
-static bool fts_zip_read_word(fts_zip_t *zip,     /*!< in: Zip state + data */
-                              fts_string_t *word) /*!< out: uncompressed word */
-{
-  /* Either there was an error or we are at the Z_STREAM_END. */
+  /* All blocks must be freed at end of inflate. */
   if (zip->status != Z_OK) {
-    return false;
+    for (ulint i = 0; i < ib_vector_size(zip->blocks); ++i) {
+      if (ib_vector_getp(zip->blocks, i)) {
+        ut::free(ib_vector_getp(zip->blocks, i));
+        ib_vector_set(zip->blocks, i, &null);
+      }
+    }
   }
 
-  ushort len = 0;
-
-  const bool have_len =
-      fts_zip_extract_bytes(zip, reinterpret_cast<byte *>(&len), sizeof(len));
-
-  if (!have_len) {
-    return false;
+  if (ptr != nullptr) {
+    ut_ad(word->f_len == strlen((char *)ptr));
   }
 
-  ut_a(len <= FTS_MAX_WORD_LEN);
-
-  word->f_len = len;
-  const bool have_word = fts_zip_extract_bytes(zip, word->f_str, len);
-  ut_ad(have_word);
-
-  word->f_str[len] = 0;
-  ut_ad(word->f_len == strlen((char *)word->f_str));
-
-  return have_word;
+  return ((zip->status == Z_OK || zip->status == Z_STREAM_END) && read_something
+              ? ptr
+              : nullptr);
 }
 
 /** Callback function to fetch and compress the word in an FTS
@@ -658,36 +641,19 @@ static bool fts_zip_read_word(fts_zip_t *zip,     /*!< in: Zip state + data */
  @return false on EOF */
 static bool fts_fetch_index_words(
     void *row,      /*!< in: sel_node_t* */
-    void *user_arg) /*!< in: pointer to fts_optimize_t */
+    void *user_arg) /*!< in: pointer to ib_vector_t */
 {
-  const auto *const sel_node = static_cast<sel_node_t *>(row);
-  const auto optim = static_cast<fts_optimize_t *>(user_arg);
-  fts_zip_t *const zip = optim->zip;
-  que_node_t *const exp = sel_node->select_list;
-  const CHARSET_INFO *const charset = optim->fts_index_table.charset;
-  const dfield_t *const dfield = que_node_get_val(exp);
+  sel_node_t *sel_node = static_cast<sel_node_t *>(row);
+  fts_zip_t *zip = static_cast<fts_zip_t *>(user_arg);
+  que_node_t *exp = sel_node->select_list;
+  dfield_t *dfield = que_node_get_val(exp);
   ushort len = static_cast<ushort>(dfield_get_len(dfield));
-  void *const data = dfield_get_data(dfield);
+  void *data = dfield_get_data(dfield);
 
-  /* Ensure each class of collation-equal tokens is compressed only once. */
-  {
-    /* Compare current word with previous using collation. */
-    auto comp = my_strnncoll(charset, static_cast<uint8_t *>(data), len,
-                             zip->word.f_str, zip->word.f_len);
-    /* Duplicate detection is guaranteed by callback invocations being
-    grouped into sequences (corresponding to aux tables), where all
-    collation-equal words are in the same sequence; between these
-    sequences zip->word is reset to an empty string (every non-empty
-    word is assumed to be later in sort order than the empty string).
-    Within each sequence words are sorted (using collation) in ascending
-    order, guaranteeing all collation-equal words are passed to the
-    callback in consecutive calls, which allows us to eliminate
-    duplicates by comparing each word only with the one preceding it. */
-    ut_ad(comp >= 0);
-    /* Skip the duplicate words. */
-    if (comp == 0) {
-      return true;
-    }
+  /* Skip the duplicate words. */
+  if (zip->word.f_len == static_cast<ulint>(len) &&
+      !memcmp(zip->word.f_str, data, len)) {
+    return true;
   }
 
   ut_a(len <= FTS_MAX_WORD_LEN);
@@ -815,30 +781,9 @@ static void fts_zip_deflate_end(
 
     optim->fts_index_table.suffix = fts_get_suffix(selected);
 
-    /* The list of words to be processed may not have any duplicates -
-    meaning here words that are equal in collation order. Elimination
-    of duplicates is performed inside the fts_fetch_index_words callback
-    using optim->zip->word as the previous word processed. It relies on
-    all collation-equal words being passed to the callback in consecutive
-    calls. This property holds because:
-    1. All words equal by collation order are indexed in the same aux
-       table,
-    2. Words from a given aux table are processed by collation
-       (ascending) order.
-    (1) is ensured by the way words are assigned to aux table number -
-    see fts_select_index function. (2) is guaranteed by 'ORDER BY word'
-    clause in the query below.
-
-    optim->zip->word is cleared out when moving from one aux table to
-    another; it is not needed, as there can be no duplicates between
-    tables, but doing so allows the callback to assert ordering within
-    the table. */
-
-    fts_zip_clear_word(*optim->zip);
-
     info = pars_info_create();
 
-    pars_info_bind_function(info, "my_func", fts_fetch_index_words, optim);
+    pars_info_bind_function(info, "my_func", fts_fetch_index_words, optim->zip);
 
     pars_info_bind_varchar_literal(info, "word", word->f_str, word->f_len);
 
@@ -1007,7 +952,7 @@ dberr_t fts_table_fetch_doc_ids(
   if (error == DB_SUCCESS) {
     fts_sql_commit(trx);
 
-    ib_vector_sort(doc_ids->doc_ids, fts_doc_id_field_cmp<fts_update_t>);
+    ib_vector_sort(doc_ids->doc_ids, fts_update_doc_id_cmp);
   } else {
     fts_sql_rollback(trx);
   }
@@ -1118,7 +1063,7 @@ static dberr_t fts_optimize_encode_node(
   /* Calculate the space required to store the ilist. */
   ut_ad(doc_id > node->last_doc_id);
   doc_id_delta = doc_id - node->last_doc_id;
-  enc_len = fts_get_encoded_len(doc_id_delta);
+  enc_len = fts_get_encoded_len(static_cast<ulint>(doc_id_delta));
 
   /* Calculate the size of the encoded pos array. */
   while (*src) {
@@ -1163,8 +1108,9 @@ static dberr_t fts_optimize_encode_node(
   src = enc->src_ilist_ptr;
   dst = node->ilist + node->ilist_size;
 
-  /* Encode the doc id. */
-  dst += fts_encode_int(doc_id_delta, dst);
+  /* Encode the doc id. Cast to ulint, the delta should be small and
+  therefore no loss of precision. */
+  dst += fts_encode_int((ulint)doc_id_delta, dst);
 
   /* Copy the encoded pos array. */
   memcpy(dst, src, pos_enc_len);
@@ -1203,9 +1149,10 @@ static dberr_t fts_optimize_node(
   into in the destination node. */
   while (copied < src_node->ilist_size &&
          dst_node->ilist_size < FTS_ILIST_MAX_SIZE) {
+    doc_id_t delta;
     doc_id_t del_doc_id = FTS_NULL_DOC_ID;
 
-    doc_id_t delta = fts_decode_vlc(&enc->src_ilist_ptr);
+    delta = fts_decode_vlc(&enc->src_ilist_ptr);
 
   test_again:
     /* Check whether the doc id is in the delete list, if
@@ -1696,9 +1643,8 @@ static void fts_optimize_words(
 
     /* Read the index records to optimize. */
     fetch.total_memory = 0;
-
     error = fts_index_fetch_nodes(trx, &graph, &optim->fts_index_table, word,
-                                  &fetch, true);
+                                  &fetch);
     ut_ad(fetch.total_memory < fts_result_cache_limit);
 
     if (error == DB_SUCCESS) {
