@@ -23,10 +23,9 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "sql/auth/sql_acl.h"  // PROCESS_ACL
 
 #include "i_s.h"
-#include "m_ctype.h"
 #include "my_sys.h"
 #include "myisampack.h"
-#include "mysql/innodb_priv.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysys_err.h"
 #include "sql/sql_plugin.h"
 
@@ -38,6 +37,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "read0i_s.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/field.h"
+#include "sql/sql_show.h" /* schema_table_store_record */
 #include "sql/table.h"
 #include "srv0start.h" /* for srv_was_started */
 #include "trx0i_s.h"
@@ -100,7 +100,7 @@ static ST_FIELD_INFO xtradb_read_view_fields_info[] = {
 
     END_OF_ST_FIELD_INFO};
 
-static int xtradb_read_view_fill_table(THD *thd, TABLE_LIST *tables, Item *) {
+static int xtradb_read_view_fill_table(THD *thd, Table_ref *tables, Item *) {
   DBUG_ENTER("xtradb_read_view_fill_table");
 
   /* deny access to non-superusers */
