@@ -127,6 +127,8 @@ constexpr size_t BTR_MODIFY_EXTERNAL = 262144;
 record is in spatial index */
 constexpr size_t BTR_RTREE_DELETE_MARK = 524288;
 
+using Page_range_t = std::pair<page_no_t, page_no_t>;
+
 constexpr ulint BTR_LATCH_MODE_WITHOUT_FLAGS(ulint latch_mode) {
   return latch_mode &
          ~(BTR_INSERT | BTR_DELETE_MARK | BTR_RTREE_UNDO_INS |
@@ -406,6 +408,13 @@ void btr_insert_on_non_leaf_level(uint32_t flags, dict_index_t *index,
 */
 void btr_set_min_rec_mark(rec_t *rec, mtr_t *mtr);
 
+/** Removes a record as the predefined minimum record.
+@param[in]  block  buffer block containing the record.
+@param[in]  rec    the record who info bits will be modified by clearing
+                   the REC_INFO_MIN_REC_FLAG bit.
+@param[in]  mtr    mini transaction context. */
+void btr_unset_min_rec_mark(buf_block_t *block, rec_t *rec, mtr_t *mtr);
+
 /** Deletes on the upper level the node pointer to a page.
 @param[in] index Index tree
 @param[in] block Page whose node pointer is deleted
@@ -499,6 +508,18 @@ returned block is not allocated nor initialized otherwise */
 
 );
 
+/** Allocates all pages of one extent to be used in an index tree.
+@param[in]  index  the index for which pages are allocated.
+@param[in]  is_leaf  true if leaf segment and false if non-leaf segment
+@param[out]  page_range  All pages within this pair of page numbers are
+allocated for this B-tree. The page_range.first is part of the range, while the
+page_range.second is not part of the range.
+@param[in]  mtr  mini transaction context for this operation.
+@return DB_SUCCESS on success, error code on failure. */
+[[nodiscard]] dberr_t btr_extent_alloc(const dict_index_t *const index,
+                                       bool is_leaf, Page_range_t &page_range,
+                                       mtr_t *mtr);
+
 /** Frees a file page used in an index tree. NOTE: cannot free field external
  storage pages because the page must contain info on its level. */
 void btr_page_free(dict_index_t *index, /*!< in: index tree */
@@ -530,14 +551,14 @@ buf_block_t *btr_root_block_get(
                                or RW_X_LATCH */
     mtr_t *mtr);               /*!< in: mtr */
 
-#ifdef UNIV_BTR_PRINT
 /** Prints size info of a B-tree. */
 void btr_print_size(dict_index_t *index); /*!< in: index tree */
-/** Prints directories and other info of all nodes in the index. */
-void btr_print_index(dict_index_t *index, /*!< in: index */
-                     ulint width); /*!< in: print this many entries from start
-                                   and end */
-#endif                             /* UNIV_BTR_PRINT */
+
+/** Prints directories and other info of all nodes in the index.
+@param[in] index  the index to be printed.
+@param[in] width  number of entries to print from start and end. */
+void btr_print_index(dict_index_t *index, ulint width);
+
 /** Checks the size and number of fields in a record based on the definition of
 the index.
  @return true if ok */
@@ -563,6 +584,65 @@ dberr_t btr_sdi_create_index(space_id_t space_id, bool dict_locked);
 constexpr uint32_t BTR_N_LEAF_PAGES = 1;
 constexpr uint32_t BTR_TOTAL_SIZE = 2;
 
+/** Check if the given index is empty.  An index is considered empty if it
+has only the root page with no user records, including del-marked records.
+@param[in]   index   index
+@return true if index is empty, false otherwise. */
+bool btr_is_index_empty(const dict_index_t *index);
+
+#ifdef UNIV_DEBUG
+/** Does a breadth first traversal (BFT) of the B-tree, and invokes the
+callback for each of the B-tree nodes. */
+struct BFT {
+  struct Callback {
+    struct Page_details {
+      page_no_t m_page_no;
+      size_t m_nrows;
+      size_t m_level;
+      std::ostream &print(std::ostream &out) const;
+    };
+    void init(size_t max_level) { m_data.resize(max_level); }
+    void operator()(buf_block_t *block);
+    std::ostream &print(std::ostream &out) const;
+
+   public:
+    BFT *m_bft;
+
+   private:
+    std::vector<std::list<Page_details>> m_data;
+  };
+  BFT(const dict_index_t *index, Callback &cb);
+  void traverse();
+
+  const dict_index_t *index() const { return m_index; }
+
+ private:
+  void children_to_visit(buf_block_t *block);
+  page_no_t visit_next();
+  std::list<page_no_t> m_pages_to_visit;
+  const dict_index_t *m_index;
+  Callback &m_callback;
+};
+
+inline std::ostream &operator<<(std::ostream &out,
+                                const BFT::Callback::Page_details &obj) {
+  return obj.print(out);
+}
+
+inline std::ostream &operator<<(std::ostream &out, const BFT::Callback &obj) {
+  return obj.print(out);
+}
+
+#endif /* UNIV_DEBUG */
+
+/** NOTE - Changing this from the original number of 50 to 45 as
+insert_debug.test was failing in ASAN build because of a stack overflow issue.
+It was found that rtr_info_t was taking up a lot of stack space in the function
+btr_insert_on_non_leaf_level_func which is part of the recursive stack
+trace. */
+/** Maximum B-tree page level (not really a hard limit). Used in debug
+ assertions in btr_page_set_level and btr_page_get_level */
+constexpr uint32_t BTR_MAX_NODE_LEVEL = 45;
 #include "btr0btr.ic"
 
 #endif
