@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2024, Oracle and/or its affiliates.
+Copyright (c) 1995, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -765,7 +764,7 @@ uint32_t fsp_flags_to_dict_tf(uint32_t fsp_flags, bool compact);
 /** Enable encryption for already existing tablespace.
 @param[in,out]	space	tablespace object
 @return true if success, else false */
-[[nodiscard]]
+MY_NODISCARD
 bool fsp_enable_encryption(fil_space_t *space);
 
 /** Calculates the descriptor index within a descriptor page.
@@ -775,6 +774,17 @@ bool fsp_enable_encryption(fil_space_t *space);
 static inline ulint xdes_calc_descriptor_index(const page_size_t &page_size,
                                                ulint offset);
 
+/** Mark all the pages of the extent as used.
+@param[in]  descr   extent descriptor
+@param[in]  mtr  mini transaction context. */
+void xdes_mark_all_used(xdes_t *descr, mtr_t *mtr);
+
+/** Mark all the pages of the extent from given page_no as free.
+@param[in]  descr   extent descriptor
+@param[in]  mtr  mini transaction context.
+@param[in]  from  all pages from this page_no is marked as free. */
+void xdes_mark_pages_free(xdes_t *descr, mtr_t *mtr, const page_no_t from);
+
 /** Gets a descriptor bit of a page.
 @param[in]      descr   descriptor
 @param[in]      bit     XDES_FREE_BIT or XDES_CLEAN_BIT
@@ -1035,4 +1045,52 @@ inline std::ostream &operator<<(std::ostream &out,
 fseg_inode_t *fseg_inode_get(const fseg_header_t *header, space_id_t space,
                              const page_size_t &page_size, mtr_t *mtr,
                              buf_block_t **block = nullptr);
+
+struct Page_alloc_info {
+  Page_alloc_info(space_id_t space_id, const page_size_t &page_size,
+                  fseg_header_t *fseg_hdr)
+      : m_space_id(space_id), m_page_size(page_size), m_fseg_header(fseg_hdr) {}
+
+  /** Tablespace ID. */
+  space_id_t m_space_id;
+
+  /** Page size. */
+  page_size_t m_page_size;
+
+  /** File segment header. */
+  fseg_header_t *m_fseg_header;
+};
+
+using Page_range_t = std::pair<page_no_t, page_no_t>;
+
+/** Allocate pages in bulk. */
+dberr_t fseg_alloc_pages_of_extent(const Page_alloc_info &info, mtr_t *mtr,
+                                   Page_range_t &page_range);
+
+void fsp_init_file_page_low(buf_block_t *block);
+
+/** Allocate a page number.
+@param[in]  space  tablespace object
+@param[in]  page_size  page size in the given tablespace.
+@param[in]  seg_inode  file segment inode where page number is to be allocated.
+@param[in]  hint  hint of which page would be desirable.
+@param[in]  direction  If the new page is needed because of an index page split,
+              and records are inserted there in order, into which direction
+              they go alphabetically: FSP_DOWN, FSP_UP, FSP_NO_DIR
+@param[in]  mtr  mini-transaction context.
+@param[in,out]  has_done_reservation true if caller has done reservation.
+@return the allocated page number.  */
+page_no_t fseg_alloc_page_no(fil_space_t *space, const page_size_t &page_size,
+                             fseg_inode_t *seg_inode, page_no_t hint,
+                             byte direction,
+                             mtr_t *mtr IF_DEBUG(, bool has_done_reservation));
+
+/** Extend space by default extension size.
+@param[in] space_id space ID
+@param[in] make_old add the header to LRU tail to flush at the earliest.
+@param[out] space_size size after extending the space
+@return true iff successful. */
+bool fsp_extend_by_default_size(space_id_t space_id, bool make_old,
+                                size_t &space_size);
+
 #endif
