@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2024, Oracle and/or its affiliates.
+Copyright (c) 1995, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -47,7 +46,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "my_compiler.h"
 #include "ut0dbg.h"
-#include "ut0math.h"
 
 class IB_thread {
  public:
@@ -143,7 +141,7 @@ class Atomic_xor_of_things {
 
  private:
   static constexpr size_t digits_count =
-      ut::div_ceil(sizeof(T_thing), sizeof(T_digit));
+      (sizeof(T_thing) + sizeof(T_digit) - 1) / sizeof(T_digit);
   /* initial value must be all zeros, as opposed to the representation of
   thing{}, because we care about "neutral element of the XOR operation", and not
   "default value of a thing". */
@@ -166,10 +164,10 @@ class Scoped_event : private ut::Non_copyable {
     NULL, for the application to test successful event creation with the
     get_handle() method, but this is currently not supported (and thus not
     tested) by this implementation. */
-    ut_a(m_event != NULL);
+    ut_a(m_event != nullptr);
   }
   ~Scoped_event() {
-    if (m_event != NULL) {
+    if (m_event != nullptr) {
       CloseHandle(m_event);
     }
   }
@@ -195,14 +193,14 @@ using Atomic_xor_of_thread_id =
 thread. On Linux, returns tid. On other systems currently returns
 os_thread_get_curr_id().
 @return current thread identifier */
-[[nodiscard]] os_tid_t os_thread_get_tid() noexcept;
+MY_NODISCARD os_tid_t os_thread_get_tid() noexcept;
 
 /** Set relative scheduling priority for a given thread on
 Linux. Currently a no-op on other systems.
 @param[in]	thread_id	thread id
 @param[in]	relative_priority	system-specific priority value
 @return An actual thread priority after the update  */
-[[nodiscard]]
+MY_NODISCARD
 unsigned long int os_thread_set_priority(
     os_tid_t thread_id, unsigned long int relative_priority) noexcept;
 
