@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2005, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2005, 2023, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -34,17 +34,13 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #ifndef page0zip_h
 #define page0zip_h
 
-#ifdef UNIV_MATERIALIZE
-#undef UNIV_INLINE
-#define UNIV_INLINE
-#endif
-
 #include <sys/types.h>
 #include <zlib.h>
 
 #include "buf0buf.h"
 #include "buf0checksum.h"
 #include "buf0types.h"
+#include "data0type.h"
 #include "dict0types.h"
 #include "mach0data.h"
 #include "mem0mem.h"
@@ -60,79 +56,76 @@ this program; if not, write to the Free Software Foundation, Inc.,
 extern uint page_zip_level;
 
 /* Default compression level. */
-#define DEFAULT_COMPRESSION_LEVEL 6
+constexpr uint32_t DEFAULT_COMPRESSION_LEVEL = 6;
 /** Start offset of the area that will be compressed */
 #define PAGE_ZIP_START PAGE_NEW_SUPREMUM_END
-/** Size of an compressed page directory entry */
-#define PAGE_ZIP_DIR_SLOT_SIZE 2
 /** Predefine the sum of DIR_SLOT, TRX_ID & ROLL_PTR */
-#define PAGE_ZIP_CLUST_LEAF_SLOT_SIZE \
-  (PAGE_ZIP_DIR_SLOT_SIZE + DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN)
+constexpr uint32_t PAGE_ZIP_CLUST_LEAF_SLOT_SIZE =
+    PAGE_ZIP_DIR_SLOT_SIZE + DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN;
 /** Mask of record offsets */
-#define PAGE_ZIP_DIR_SLOT_MASK 0x3fff
+constexpr uint32_t PAGE_ZIP_DIR_SLOT_MASK = 0x3fff;
 /** 'owned' flag */
-#define PAGE_ZIP_DIR_SLOT_OWNED 0x4000
+constexpr uint32_t PAGE_ZIP_DIR_SLOT_OWNED = 0x4000;
 /** 'deleted' flag */
-#define PAGE_ZIP_DIR_SLOT_DEL 0x8000
+constexpr uint32_t PAGE_ZIP_DIR_SLOT_DEL = 0x8000;
 
 /* Whether or not to log compressed page images to avoid possible
 compression algorithm changes in zlib. */
 extern bool page_zip_log_pages;
 
 /** Set the size of a compressed page in bytes.
-@param[in,out]	page_zip	compressed page
-@param[in]	size		size in bytes */
-UNIV_INLINE
-void page_zip_set_size(page_zip_des_t *page_zip, ulint size);
+@param[in,out]  page_zip        compressed page
+@param[in]      size            size in bytes */
+static inline void page_zip_set_size(page_zip_des_t *page_zip, ulint size);
 
+#ifndef UNIV_HOTBACKUP
 /** Determine if a record is so big that it needs to be stored externally.
-@param[in]	rec_size	length of the record in bytes
-@param[in]	comp		nonzero=compact format
-@param[in]	n_fields	number of fields in the record; ignored if
+@param[in]      rec_size        length of the record in bytes
+@param[in]      comp            nonzero=compact format
+@param[in]      n_fields        number of fields in the record; ignored if
 tablespace is not compressed
-@param[in]	page_size	page size
+@param[in]      page_size       page size
 @return false if the entire record can be stored locally on the page */
-UNIV_INLINE
-ibool page_zip_rec_needs_ext(ulint rec_size, ulint comp, ulint n_fields,
-                             const page_size_t &page_size)
-    MY_ATTRIBUTE((warn_unused_result));
+[[nodiscard]] static inline bool page_zip_rec_needs_ext(
+    ulint rec_size, ulint comp, ulint n_fields, const page_size_t &page_size);
+#endif /* !UNIV_HOTBACKUP */
 
 /** Determine the guaranteed free space on an empty page.
- @return minimum payload size on the page */
-ulint page_zip_empty_size(
-    ulint n_fields, /*!< in: number of columns in the index */
-    ulint zip_size) /*!< in: compressed page size in bytes */
-    MY_ATTRIBUTE((const));
+@param[in]  n_fields  number of columns in the index
+@param[in]  zip_size  compressed page size in bytes
+@return minimum payload size on the page */
+ulint page_zip_empty_size(ulint n_fields, ulint zip_size);
 
 #ifndef UNIV_HOTBACKUP
 /** Check whether a tuple is too big for compressed table
-@param[in]	index	dict index object
-@param[in]	entry	entry for the index
-@return	true if it's too big, otherwise false */
+@param[in]      index   dict index object
+@param[in]      entry   entry for the index
+@return true if it's too big, otherwise false */
 bool page_zip_is_too_big(const dict_index_t *index, const dtuple_t *entry);
 #endif /* !UNIV_HOTBACKUP */
 
 /** Initialize a compressed page descriptor. */
-UNIV_INLINE
-void page_zip_des_init(page_zip_des_t *page_zip); /*!< in/out: compressed page
-                                                  descriptor */
+static inline void page_zip_des_init(
+    page_zip_des_t *page_zip); /*!< in/out: compressed page
+                               descriptor */
 
-/** Configure the zlib allocator to use the given memory heap. */
-void page_zip_set_alloc(void *stream,      /*!< in/out: zlib stream */
-                        mem_heap_t *heap); /*!< in: memory heap to use */
+/** Configure the zlib allocator to use the given memory heap.
+@param[in,out] stream zlib stream
+@param[in] heap Memory heap to use */
+void page_zip_set_alloc(void *stream, mem_heap_t *heap);
 
 /** Compress a page.
  @return true on success, false on failure; page_zip will be left
  intact on failure. */
-MY_NODISCARD
-ibool page_zip_compress(page_zip_des_t *page_zip, /*!< in: size; out: data,
-                                                  n_blobs, m_start, m_end,
-                                                  m_nonempty */
-                        const page_t *page,       /*!< in: uncompressed page */
-                        dict_index_t *index,      /*!< in: index tree */
-                        ulint level,              /*!< in: commpression level */
-                        mtr_t *mtr);              /*!< in/out: mini-transaction,
-                                                  or NULL */
+[[nodiscard]] bool page_zip_compress(
+    page_zip_des_t *page_zip, /*!< in: size; out: data,
+                               n_blobs, m_start, m_end,
+                               m_nonempty */
+    const page_t *page,       /*!< in: uncompressed page */
+    dict_index_t *index,      /*!< in: index tree */
+    ulint level,              /*!< in: compression level */
+    mtr_t *mtr);              /*!< in/out: mini-transaction,
+                              or NULL */
 
 /** Write the index information for the compressed page.
  @return used size of buf */
@@ -148,63 +141,62 @@ ulint page_zip_fields_encode(
     byte *buf); /*!< out: buffer of (n + 1) * 2 bytes */
 
 /** Decompress a page.  This function should tolerate errors on the compressed
- page.  Instead of letting assertions fail, it will return FALSE if an
+ page.  Instead of letting assertions fail, it will return false if an
  inconsistency is detected.
  @return true on success, false on failure */
-ibool page_zip_decompress(
+bool page_zip_decompress(
     page_zip_des_t *page_zip, /*!< in: data, ssize;
                              out: m_start, m_end, m_nonempty, n_blobs */
     page_t *page,             /*!< out: uncompressed page, may be trashed */
-    ibool all);               /*!< in: TRUE=decompress the whole page;
-                              FALSE=verify but do not copy some
-                              page header fields that should not change
-                              after page creation */
+    bool all);                /*!< in: true=decompress the whole page;
+                               false=verify but do not copy some
+                               page header fields that should not change
+                               after page creation */
 
 #ifdef UNIV_ZIP_DEBUG
 /** Check that the compressed and decompressed pages match.
  @return true if valid, false if not */
-ibool page_zip_validate_low(
+bool page_zip_validate_low(
     const page_zip_des_t *page_zip, /*!< in: compressed page */
     const page_t *page,             /*!< in: uncompressed page */
     const dict_index_t *index,      /*!< in: index of the page, if known */
-    ibool sloppy);                  /*!< in: FALSE=strict,
-                            TRUE=ignore the MIN_REC_FLAG */
+    bool sloppy);                   /*!< in: false=strict,
+                             true=ignore the MIN_REC_FLAG */
 /** Check that the compressed and decompressed pages match. */
-ibool page_zip_validate(
+bool page_zip_validate(
     const page_zip_des_t *page_zip, /*!< in: compressed page */
     const page_t *page,             /*!< in: uncompressed page */
     const dict_index_t *index);     /*!< in: index of the page, if known */
 #endif                              /* UNIV_ZIP_DEBUG */
 
-/** Determine how big record can be inserted without recompressing the page.
- @return a positive number indicating the maximum size of a record
- whose insertion is guaranteed to succeed, or zero or negative */
-UNIV_INLINE
-lint page_zip_max_ins_size(
-    const page_zip_des_t *page_zip, /*!< in: compressed page */
-    ibool is_clust)                 /*!< in: TRUE if clustered index */
-    MY_ATTRIBUTE((warn_unused_result));
+/** Determine how big record can be inserted without re-compressing the page.
+@param[in] page_zip Compressed page.
+@param[in] is_clust True if clustered index.
+@return a positive number indicating the maximum size of a record
+whose insertion is guaranteed to succeed, or zero or negative */
+[[nodiscard]] static inline lint page_zip_max_ins_size(
+    const page_zip_des_t *page_zip, bool is_clust);
 
 /** Determine if enough space is available in the modification log.
- @return true if page_zip_write_rec() will succeed */
-UNIV_INLINE
-ibool page_zip_available(
-    const page_zip_des_t *page_zip, /*!< in: compressed page */
-    bool is_clust,                  /*!< in: TRUE if clustered index */
-    ulint length,                   /*!< in: combined size of the record */
-    ulint create)                   /*!< in: nonzero=add the record to
-                                    the heap */
-    MY_ATTRIBUTE((warn_unused_result));
+@param[in] page_zip Compressed page.
+@param[in] is_clust True if clustered index.
+@param[in] length Combined size of the record.
+@param[in] create Nonzero=add the record to the heap.
+@return true if page_zip_write_rec() will succeed */
+[[nodiscard]] static inline bool page_zip_available(
+    const page_zip_des_t *page_zip, bool is_clust, ulint length, ulint create);
 
-/** Write data to the uncompressed header portion of a page. The data must
+/** Write data to the uncompressed header portion of a page.  The data must
 already have been written to the uncompressed page.
-@param[in,out]	page_zip	compressed page
-@param[in]	str		address on the uncompressed page
-@param[in]	length		length of the data
-@param[in]	mtr		mini-transaction, or NULL */
-UNIV_INLINE
-void page_zip_write_header(page_zip_des_t *page_zip, const byte *str,
-                           ulint length, mtr_t *mtr);
+However, the data portion of the uncompressed page may differ from the
+compressed page when a record is being inserted in page_cur_insert_rec_low().
+@param[in,out]  page_zip        Compressed page
+@param[in]      str             Address on the uncompressed page
+@param[in]      length          Length of the data
+@param[in]      mtr             Mini-transaction, or NULL */
+static inline void page_zip_write_header(page_zip_des_t *page_zip,
+                                         const byte *str, ulint length,
+                                         mtr_t *mtr);
 
 /** Write an entire record on the compressed page.  The data must already
  have been written to the uncompressed page. */
@@ -243,61 +235,64 @@ byte *page_zip_parse_write_node_ptr(
     page_t *page,              /*!< in/out: uncompressed page */
     page_zip_des_t *page_zip); /*!< in/out: compressed page */
 
-/** Write the node pointer of a record on a non-leaf compressed page. */
-void page_zip_write_node_ptr(
-    page_zip_des_t *page_zip, /*!< in/out: compressed page */
-    byte *rec,                /*!< in/out: record */
-    ulint size,               /*!< in: data size of rec */
-    ulint ptr,                /*!< in: node pointer */
-    mtr_t *mtr);              /*!< in: mini-transaction, or NULL */
+/** Write the node pointer of a record on a non-leaf compressed page.
+@param[in,out] page_zip Compressed page
+@param[in,out] rec Record
+@param[in] size Data size of rec
+@param[in] ptr Node pointer
+@param[in] mtr Mini-transaction, or null */
+void page_zip_write_node_ptr(page_zip_des_t *page_zip, byte *rec, ulint size,
+                             ulint ptr, mtr_t *mtr);
 
-/** Write the trx_id and roll_ptr of a record on a B-tree leaf node page. */
-void page_zip_write_trx_id_and_roll_ptr(
-    page_zip_des_t *page_zip, /*!< in/out: compressed page */
-    byte *rec,                /*!< in/out: record */
-    const ulint *offsets,     /*!< in: rec_get_offsets(rec, index) */
-    ulint trx_id_col,         /*!< in: column number of TRX_ID in rec */
-    trx_id_t trx_id,          /*!< in: transaction identifier */
-    roll_ptr_t roll_ptr);     /*!< in: roll_ptr */
+/** Write the trx_id and roll_ptr of a record on a B-tree leaf node page.
+@param[in,out] page_zip Compressed page
+@param[in,out] rec Record
+@param[in] offsets Rec_get_offsets(rec, index)
+@param[in] trx_id_col Column number of trx_id in rec
+@param[in] trx_id Transaction identifier
+@param[in] roll_ptr Roll_ptr */
+void page_zip_write_trx_id_and_roll_ptr(page_zip_des_t *page_zip, byte *rec,
+                                        const ulint *offsets, ulint trx_id_col,
+                                        trx_id_t trx_id, roll_ptr_t roll_ptr);
 
 /** Write the "deleted" flag of a record on a compressed page.  The flag must
  already have been written on the uncompressed page. */
 void page_zip_rec_set_deleted(
     page_zip_des_t *page_zip, /*!< in/out: compressed page */
     const byte *rec,          /*!< in: record on the uncompressed page */
-    ulint flag);              /*!< in: the deleted flag (nonzero=TRUE) */
+    bool flag);               /*!< in: the deleted flag (nonzero=true) */
 
-/** Write the "owned" flag of a record on a compressed page.  The n_owned field
- must already have been written on the uncompressed page. */
-void page_zip_rec_set_owned(
-    page_zip_des_t *page_zip, /*!< in/out: compressed page */
-    const byte *rec,          /*!< in: record on the uncompressed page */
-    ulint flag);              /*!< in: the owned flag (nonzero=TRUE) */
+/** Write the "owned" flag of a record on a compressed page. The n_owned field
+must already have been written on the uncompressed page.
+@param[in,out]  page_zip  Compressed page
+@param[in]      rec       Record on the uncompressed page
+@param[in]      flag      The owned flag (nonzero=true) */
+void page_zip_rec_set_owned(page_zip_des_t *page_zip, const byte *rec,
+                            ulint flag);
 
-/** Insert a record to the dense page directory. */
-void page_zip_dir_insert(
-    page_zip_des_t *page_zip, /*!< in/out: compressed page */
-    const byte *prev_rec,     /*!< in: record after which to insert */
-    const byte *free_rec,     /*!< in: record from which rec was
-                             allocated, or NULL */
-    byte *rec);               /*!< in: record to insert */
+/** Insert a record to the dense page directory.
+@param[in,out] page_zip Compressed page
+@param[in] prev_rec Record after which to insert
+@param[in] free_rec Record from which rec was allocated, or null
+@param[in] rec Record to insert */
+void page_zip_dir_insert(page_zip_des_t *page_zip, const byte *prev_rec,
+                         const byte *free_rec, byte *rec);
 
 /** Shift the dense page directory and the array of BLOB pointers when a record
 is deleted.
-@param[in,out]	page_zip	compressed page
-@param[in]	rec		deleted record
-@param[in]	index		index of rec
-@param[in]	offsets		rec_get_offsets(rec)
-@param[in]	free		previous start of the free list */
+@param[in,out]  page_zip        compressed page
+@param[in]      rec             deleted record
+@param[in]      index           index of rec
+@param[in]      offsets         rec_get_offsets(rec)
+@param[in]      free            previous start of the free list */
 void page_zip_dir_delete(page_zip_des_t *page_zip, byte *rec,
                          const dict_index_t *index, const ulint *offsets,
                          const byte *free);
 
-/** Add a slot to the dense page directory. */
-void page_zip_dir_add_slot(
-    page_zip_des_t *page_zip, /*!< in/out: compressed page */
-    bool is_clustered);       /*!< in: nonzero for clustered index,
-                              zero for others */
+/** Add a slot to the dense page directory.
+@param[in,out] page_zip Compressed page
+@param[in] is_clustered Nonzero for clustered index, zero for others */
+void page_zip_dir_add_slot(page_zip_des_t *page_zip, bool is_clustered);
 
 /** Parses a log record of writing to the header of a page.
  @return end of log record or NULL */
@@ -307,18 +302,6 @@ byte *page_zip_parse_write_header(
     page_t *page,              /*!< in/out: uncompressed page */
     page_zip_des_t *page_zip); /*!< in/out: compressed page */
 
-/** Write data to the uncompressed header portion of a page.  The data must
-already have been written to the uncompressed page.
-However, the data portion of the uncompressed page may differ from the
-compressed page when a record is being inserted in page_cur_insert_rec_low().
-@param[in,out]  page_zip        compressed page
-@param[in]      str             address on the uncompressed page
-@param[in]      length          length of the data
-@param[in]      mtr             mini-transaction, or NULL */
-UNIV_INLINE
-void page_zip_write_header(page_zip_des_t *page_zip, const byte *str,
-                           ulint length, mtr_t *mtr);
-
 /** Reorganize and compress a page.  This is a low-level operation for
  compressed pages, to be used when page_zip_compress() fails.
  On success, a redo log entry MLOG_ZIP_PAGE_COMPRESS will be written.
@@ -329,7 +312,7 @@ void page_zip_write_header(page_zip_des_t *page_zip, const byte *str,
  will be redo-logged.
  @return true on success, false on failure; page_zip will be left
  intact on failure, but page will be overwritten. */
-ibool page_zip_reorganize(
+bool page_zip_reorganize(
     buf_block_t *block,  /*!< in/out: page with compressed page;
                          on the compressed page, in: size;
                          out: data, n_blobs,
@@ -354,8 +337,7 @@ void page_zip_copy_recs(
 
 /** Parses a log record of compressing an index page.
  @return end of log record or NULL */
-MY_NODISCARD
-byte *page_zip_parse_compress(
+[[nodiscard]] byte *page_zip_parse_compress(
     byte *ptr,                 /*!< in: buffer */
     byte *end_ptr,             /*!< in: buffer end */
     page_t *page,              /*!< out: uncompressed page */
@@ -363,36 +345,32 @@ byte *page_zip_parse_compress(
 
 /** Write a log record of compressing an index page without the data on the
 page.
-@param[in]	level	compression level
-@param[in]	page	page that is compressed
-@param[in]	index	index
-@param[in]	mtr	mtr */
-UNIV_INLINE
-void page_zip_compress_write_log_no_data(ulint level, const page_t *page,
-                                         dict_index_t *index, mtr_t *mtr);
+@param[in]      level   compression level
+@param[in]      page    page that is compressed
+@param[in]      index   index
+@param[in]      mtr     mtr */
+static inline void page_zip_compress_write_log_no_data(ulint level,
+                                                       const page_t *page,
+                                                       dict_index_t *index,
+                                                       mtr_t *mtr);
 
 /** Parses a log record of compressing an index page without the data.
-@param[in]	ptr		buffer
-@param[in]	end_ptr		buffer end
-@param[in]	page		uncompressed page
-@param[out]	page_zip	compressed page
-@param[in]	index		index
+@param[in]      ptr             buffer
+@param[in]      end_ptr         buffer end
+@param[in]      page            uncompressed page
+@param[out]     page_zip        compressed page
+@param[in]      index           index
 @return end of log record or NULL */
-UNIV_INLINE
-byte *page_zip_parse_compress_no_data(byte *ptr, byte *end_ptr, page_t *page,
-                                      page_zip_des_t *page_zip,
-                                      dict_index_t *index);
+static inline byte *page_zip_parse_compress_no_data(byte *ptr, byte *end_ptr,
+                                                    page_t *page,
+                                                    page_zip_des_t *page_zip,
+                                                    dict_index_t *index);
 
 #ifndef UNIV_HOTBACKUP
 /** Reset the counters used for filling
  INFORMATION_SCHEMA.innodb_cmp_per_index. */
-UNIV_INLINE
-void page_zip_reset_stat_per_index();
+static inline void page_zip_reset_stat_per_index();
 
-#ifdef UNIV_MATERIALIZE
-#undef UNIV_INLINE
-#define UNIV_INLINE UNIV_INLINE_ORIGINAL
-#endif
 #endif /* !UNIV_HOTBACKUP */
 
 #include "page0zip.ic"
