@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -212,7 +211,7 @@ ulint row_log_estimate_work(const dict_index_t *index);
 
 /** Find out if temporary log files encrypted.
 @return true if temporary log file should be encrypted, false if not */
-[[nodiscard]]
+MY_NODISCARD
 bool log_tmp_is_encrypted() noexcept;
 
 /** Check the row log encryption is enabled or not.
@@ -226,7 +225,7 @@ void log_tmp_enable_encryption_if_set();
 @param[in]	offs		offset to block
 @param[in]	space_id	tablespace id
 @return whether the operation succeeded */
-[[nodiscard]]
+MY_NODISCARD
 bool log_tmp_block_encrypt(const byte *src_block, ulint size, byte *dst_block,
                            os_offset_t offs, space_id_t space_id);
 
@@ -237,7 +236,7 @@ bool log_tmp_block_encrypt(const byte *src_block, ulint size, byte *dst_block,
 @param[in]	offs		offset to block
 @param[in]	space_id	tablespace id
 @return whether the operation succeeded */
-[[nodiscard]]
+MY_NODISCARD
 bool log_tmp_block_decrypt(const byte *src_block, ulint size, byte *dst_block,
                            os_offset_t offs, space_id_t space_id);
 
