@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -113,7 +112,7 @@ void row_mysql_prebuilt_free_compress_heap(row_prebuilt_t *prebuilt) noexcept;
 @param[in]	dict_data_len	optional dictionary data length
 @param[in]	compress_heap
 @return pointer to the uncompressed data */
-[[nodiscard]]
+MY_NODISCARD
 const byte *row_decompress_column(const byte *data, ulint *len,
                                   const byte *dict_data, ulint dict_data_len,
                                   mem_heap_t **compress_heap);
@@ -126,7 +125,7 @@ const byte *row_decompress_column(const byte *data, ulint *len,
 @param[in]	dict_data_len	optional dictionary data length
 @param[in]	compress_heap
 @return pointer to the compressed data */
-[[nodiscard]]
+MY_NODISCARD
 byte *row_compress_column(const byte *data, ulint *len, ulint lenlen,
                           const byte *dict_data, ulint dict_data_len,
                           mem_heap_t **compress_heap);
@@ -996,28 +995,35 @@ struct SysIndexCallback {
   virtual void operator()(mtr_t *mtr, btr_pcur_t *pcur) noexcept = 0;
 };
 
+/** Get the updated parent field value from the update vector for the
+given col_no.
+@param[in]      foreign         foreign key information
+@param[in]      update          updated parent vector.
+@param[in]      col_no          base column position of the child table to check
+@return updated field from the parent update vector, else NULL */
+dfield_t *innobase_get_field_from_update_vector(dict_foreign_t *foreign,
+                                                upd_t *update, uint32_t col_no);
+
 /** Get the computed value by supplying the base column values.
 @param[in,out]  row             the data row
 @param[in]      col             virtual column
-@param[in]      table           the table on which the virtual column is
-                                defined
+@param[in]      index           index on the virtual column
 @param[in,out]  local_heap      heap memory for processing large data etc.
 @param[in,out]  heap            memory heap that copies the actual index row
+@param[in]      ifield          index field
 @param[in]      thd             MySQL thread handle
 @param[in,out]  mysql_table     mysql table object
-@param[in]      ifield          metadata for secondary index field into which
-                                the computed value is to be materialized,
-                                nullptr if not applicable
 @param[in]      old_table       during ALTER TABLE, this is the old table
-                                or nullptr.
-@param[in]      row_update      update vector for the row, supersedes row
-                                values
-@return the field filled with computed value, or nullptr on failure */
+                                or NULL.
+@param[in]      parent_update   update vector for the parent row
+@param[in]      foreign         foreign key information
+@return the field filled with computed value, or NULL if just want
+to store the value in passed in "my_rec" */
 dfield_t *innobase_get_computed_value(
-    const dtuple_t *row, const dict_v_col_t *col, const dict_table_t *table,
-    mem_heap_t **local_heap, mem_heap_t *heap, THD *thd, TABLE *mysql_table,
-    mem_heap_t **compress_heap, const dict_field_t *ifield = nullptr,
-    const dict_table_t *old_table = nullptr, upd_t *row_update = nullptr);
+    const dtuple_t *row, const dict_v_col_t *col, const dict_index_t *index,
+    mem_heap_t **local_heap, mem_heap_t *heap, const dict_field_t *ifield,
+    THD *thd, TABLE *mysql_table, const dict_table_t *old_table,
+    upd_t *parent_update, dict_foreign_t *foreign, mem_heap_t **compress_heap);
 
 /** Parse out multi-values from a MySQL record
 @param[in]      mysql_table     MySQL table structure
