@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+Copyright (c) 2018, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -41,6 +41,7 @@ Created 2018-01-27 by Sunny Bains. */
 // Forward declarations
 struct trx_t;
 struct mtr_t;
+class PCursor;
 struct btr_pcur_t;
 struct buf_block_t;
 struct dict_table_t;
@@ -62,7 +63,7 @@ If the user has set the maximum number of threads to use at say 4 threads
 and there are 5 sub-trees at the selected level then we will split the 5th
 sub-tree dynamically when it is ready for scan.
 
-We want to allow multiple parallel range scans on diffrent indexes at the
+We want to allow multiple parallel range scans on different indexes at the
 same time. To achieve this split out the scan  context (Scan_ctx) from the
 execution context (Ctx). The Scan_ctx has the index  and transaction
 information and the Ctx keeps track of the cursor for a specific thread
@@ -109,13 +110,28 @@ class Parallel_reader {
   spawned. */
   constexpr static size_t MAX_TOTAL_THREADS{MAX_THREADS + MAX_RESERVED_THREADS};
 
-  using Links = std::vector<page_no_t, ut_allocator<page_no_t>>;
+  using Links = std::vector<page_no_t, ut::allocator<page_no_t>>;
 
   // Forward declaration.
   class Ctx;
   class Scan_ctx;
   struct Thread_ctx;
 
+  /** Scan state. */
+  enum class State : uint8_t {
+    /** Unknown state. */
+    UNKNOWN,
+
+    /** Start/Finish thread state. */
+    THREAD,
+
+    /** Start/Finish Ctx state. */
+    CTX,
+
+    /** Start/Finish page read. */
+    PAGE
+  };
+
   /** Callback to initialise callers state. */
   using Start = std::function<dberr_t(Thread_ctx *thread_ctx)>;
 
@@ -132,8 +148,7 @@ class Parallel_reader {
 
     /** Copy constructor.
     @param[in] scan_range       Instance to copy from. */
-    Scan_range(const Scan_range &scan_range)
-        : m_start(scan_range.m_start), m_end(scan_range.m_end) {}
+    Scan_range(const Scan_range &scan_range) = default;
 
     /** Constructor.
     @param[in] start            Start key
@@ -148,7 +163,7 @@ class Parallel_reader {
     const dtuple_t *m_end{};
 
     /** Convert the instance to a string representation. */
-    std::string to_string() const MY_ATTRIBUTE((warn_unused_result));
+    [[nodiscard]] std::string to_string() const;
   };
 
   /** Scan (Scan_ctx) configuration. */
@@ -157,8 +172,8 @@ class Parallel_reader {
     @param[in] scan_range     Range to scan.
     @param[in] index          Cluster index to scan.
     @param[in] read_level     Btree level from which records need to be read.
-    @param[in] partition_id   Partition id if it the index to be scanned.
-    belongs to a partitioned table. */
+    @param[in] partition_id   Partition id if the index to be scanned.
+                              belongs to a partitioned table. */
     Config(const Scan_range &scan_range, dict_index_t *index,
            size_t read_level = 0,
            size_t partition_id = std::numeric_limits<size_t>::max())
@@ -172,12 +187,8 @@ class Parallel_reader {
     /** Copy constructor.
     @param[in] config           Instance to copy from. */
     Config(const Config &config)
-        : m_scan_range(config.m_scan_range),
-          m_index(config.m_index),
-          m_is_compact(config.m_is_compact),
-          m_page_size(config.m_page_size),
-          m_read_level(config.m_read_level),
-          m_partition_id(config.m_partition_id) {}
+
+        = default;
 
     /** Range to scan. */
     const Scan_range m_scan_range;
@@ -232,15 +243,22 @@ class Parallel_reader {
     /** Create BLOB heap. */
     void create_blob_heap() noexcept {
       ut_a(m_blob_heap == nullptr);
-      m_blob_heap = mem_heap_create(UNIV_PAGE_SIZE);
+      m_blob_heap = mem_heap_create(UNIV_PAGE_SIZE, UT_LOCATION_HERE);
     }
 
+    /** @return the worker thread state. */
+    State get_state() const noexcept { return m_state; }
+
+    /** Save current position, commit any active mtr. */
+    void savepoint() noexcept;
+
+    /** Restore saved position and resume scan.
+    @return DB_SUCCESS or error code. */
+    [[nodiscard]] dberr_t restore_from_savepoint() noexcept;
+
     /** Thread ID. */
     size_t m_thread_id{std::numeric_limits<size_t>::max()};
 
-    /** Partition ID of the index the thread had processed earlier. */
-    size_t m_prev_partition_id{std::numeric_limits<size_t>::max()};
-
     /** Callback information related to the thread.
     @note Needs to be created and destroyed by the callback itself. */
     void *m_callback_ctx{};
@@ -248,6 +266,12 @@ class Parallel_reader {
     /** BLOB heap per thread. */
     mem_heap_t *m_blob_heap{};
 
+    /** Worker thread state. */
+    State m_state{State::UNKNOWN};
+
+    /** Current persistent cursor. */
+    PCursor *m_pcursor{};
+
     Thread_ctx(Thread_ctx &&) = delete;
     Thread_ctx(const Thread_ctx &) = delete;
     Thread_ctx &operator=(Thread_ctx &&) = delete;
@@ -255,9 +279,8 @@ class Parallel_reader {
   };
 
   /** Constructor.
-  @param[in]  max_threads Maximum number of threads to use.
-  @param[in]  sync        true if the read is synchronous */
-  explicit Parallel_reader(size_t max_threads, bool sync = true);
+  @param[in]  max_threads Maximum number of threads to use. */
+  explicit Parallel_reader(size_t max_threads);
 
   /** Destructor. */
   ~Parallel_reader();
@@ -267,32 +290,23 @@ class Parallel_reader {
   @param[in] use_reserved true if reserved threads needs to be considered
   while checking for availability of threads
   @return number of threads available. */
-  static size_t available_threads(size_t n_required, bool use_reserved = false)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] static size_t available_threads(size_t n_required,
+                                                bool use_reserved);
 
   /** Release the parallel read threads. */
   static void release_threads(size_t n_threads) {
-    const auto RELAXED = std::memory_order_relaxed;
-    auto active = s_active_threads.fetch_sub(n_threads, RELAXED);
+    const auto SEQ_CST = std::memory_order_seq_cst;
+    auto active = s_active_threads.fetch_sub(n_threads, SEQ_CST);
     ut_a(active >= n_threads);
   }
 
-  /** Fallback to single threaded mode in case of out of resource
-  issue where extra threads cannot be spawned. */
-  void fallback_to_single_threaded_mode() {
-    m_single_threaded_mode = true;
-    release_unused_threads(m_n_threads);
-    reset_error_state();
-  }
-
   /** Add scan context.
   @param[in,out]  trx         Covering transaction.
   @param[in]      config      Scan condfiguration.
   @param[in]      f           Callback function.
   (default is 0 which is leaf level)
   @return error. */
-  dberr_t add_scan(trx_t *trx, const Config &config, F &&f)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] dberr_t add_scan(trx_t *trx, const Config &config, F &&f);
 
   /** Wait for the join of threads spawned by the parallel reader. */
   void join() {
@@ -303,13 +317,11 @@ class Parallel_reader {
 
   /** Get the error stored in the global error state.
   @return global error state. */
-  dberr_t get_error_state() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_err);
-  }
+  [[nodiscard]] dberr_t get_error_state() const { return m_err; }
 
   /** @return true if the tree is empty, else false. */
-  bool is_tree_empty() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_ctx_id.load(std::memory_order_relaxed) == 0);
+  [[nodiscard]] bool is_tree_empty() const {
+    return m_ctx_id.load(std::memory_order_relaxed) == 0;
   }
 
   /** Set the callback that must be called before any processing.
@@ -320,19 +332,23 @@ class Parallel_reader {
   @param[in] f                  Call after last row is processed.*/
   void set_finish_callback(Finish &&f) { m_finish_callback = std::move(f); }
 
-  /** Start the threads to do the parallel read for the specified range.
-  @param[in]  n_threads number of threads that *needs* to be spawned
+  /** Spawn the threads to do the parallel read for the specified range.
+  Don't wait for the spawned to threads to complete.
+  @param[in]  n_threads number of threads that *need* to be spawned
   @return DB_SUCCESS or error code. */
-  dberr_t run(size_t n_threads = 0) MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] dberr_t spawn(size_t n_threads) noexcept;
+
+  /** Start the threads to do the parallel read for the specified range.
+  @param[in] n_threads          Number of threads to use for the scan.
+  @return DB_SUCCESS or error code. */
+  [[nodiscard]] dberr_t run(size_t n_threads);
 
   /** @return the configured max threads size. */
-  size_t max_threads() const MY_ATTRIBUTE((warn_unused_result)) {
-    return m_max_threads;
-  }
+  [[nodiscard]] size_t max_threads() const { return m_max_threads; }
 
   /** @return true if in error state. */
-  bool is_error_set() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_err.load(std::memory_order_relaxed) != DB_SUCCESS);
+  [[nodiscard]] bool is_error_set() const {
+    return m_err.load(std::memory_order_relaxed) != DB_SUCCESS;
   }
 
   /** Set the error state.
@@ -341,6 +357,13 @@ class Parallel_reader {
     m_err.store(err, std::memory_order_relaxed);
   }
 
+  /** Set the number of threads to be spawned.
+  @param[in]  n_threads number of threads to be spawned. */
+  void set_n_threads(size_t n_threads) {
+    ut_ad(n_threads <= m_max_threads);
+    m_n_threads = n_threads;
+  }
+
   // Disable copying.
   Parallel_reader(const Parallel_reader &) = delete;
   Parallel_reader(const Parallel_reader &&) = delete;
@@ -348,9 +371,6 @@ class Parallel_reader {
   Parallel_reader &operator=(const Parallel_reader &) = delete;
 
  private:
-  /** Reset error state. */
-  void reset_error_state() { m_err = DB_SUCCESS; }
-
   /** Release unused threads back to the pool.
   @param[in] unused_threads     Number of threads to "release". */
   void release_unused_threads(size_t unused_threads) {
@@ -364,10 +384,10 @@ class Parallel_reader {
 
   /** Fetch the next job execute.
   @return job to execute or nullptr. */
-  std::shared_ptr<Ctx> dequeue() MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] std::shared_ptr<Ctx> dequeue();
 
   /** @return true if job queue is empty. */
-  bool is_queue_empty() const MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool is_queue_empty() const;
 
   /** Poll for requests and execute.
   @param[in]  thread_ctx  thread related context information */
@@ -377,25 +397,25 @@ class Parallel_reader {
   void parallel_read();
 
   /** @return true if tasks are still executing. */
-  bool is_active() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_n_completed.load(std::memory_order_relaxed) <
-            m_ctx_id.load(std::memory_order_relaxed));
+  [[nodiscard]] bool is_active() const {
+    return m_n_completed.load(std::memory_order_relaxed) <
+           m_ctx_id.load(std::memory_order_relaxed);
   }
 
  private:
   // clang-format off
   using Ctxs =
       std::list<std::shared_ptr<Ctx>,
-                ut_allocator<std::shared_ptr<Ctx>>>;
+                ut::allocator<std::shared_ptr<Ctx>>>;
 
   using Scan_ctxs =
       std::list<std::shared_ptr<Scan_ctx>,
-                ut_allocator<std::shared_ptr<Scan_ctx>>>;
+                ut::allocator<std::shared_ptr<Scan_ctx>>>;
 
   // clang-format on
 
   /** Maximum number of worker threads to use. */
-  const size_t m_max_threads;
+  size_t m_max_threads{};
 
   /** Number of worker threads that will be spawned. */
   size_t m_n_threads{0};
@@ -409,10 +429,6 @@ class Parallel_reader {
   /** Scan contexts. */
   Scan_ctxs m_scan_ctxs{};
 
-  /** True if we fallback to single threaded mode in case of out of resource
-  issue where extra threads cannot be spawned. */
-  bool m_single_threaded_mode{false};
-
   /** For signalling worker threads about events. */
   os_event_t m_event{};
 
@@ -438,7 +454,7 @@ class Parallel_reader {
   std::atomic<dberr_t> m_err{DB_SUCCESS};
 
   /** List of threads used for paralle_read purpose. */
-  std::vector<IB_thread, ut_allocator<IB_thread>> m_parallel_read_threads;
+  std::vector<IB_thread, ut::allocator<IB_thread>> m_parallel_read_threads;
 
   /** Number of threads currently doing parallel reads. */
   static std::atomic_size_t s_active_threads;
@@ -450,15 +466,12 @@ class Parallel_reader {
   trx_t *m_trx_for_slow_log;
 
   /** Context information related to each parallel reader thread. */
-  std::vector<Thread_ctx *, ut_allocator<Thread_ctx *>> m_thread_ctxs;
-
-  friend class Ctx;
-  friend class Scan_ctx;
+  std::vector<Thread_ctx *, ut::allocator<Thread_ctx *>> m_thread_ctxs;
 };
 
 /** Parallel reader context. */
 class Parallel_reader::Scan_ctx {
- private:
+ public:
   /** Constructor.
   @param[in]  reader          Parallel reader that owns this context.
   @param[in]  id              ID of this scan context.
@@ -468,9 +481,8 @@ class Parallel_reader::Scan_ctx {
   Scan_ctx(Parallel_reader *reader, size_t id, trx_t *trx,
            const Parallel_reader::Config &config, F &&f);
 
- public:
   /** Destructor. */
-  ~Scan_ctx();
+  ~Scan_ctx() = default;
 
  private:
   /** Boundary of the range to scan. */
@@ -499,15 +511,15 @@ class Parallel_reader::Scan_ctx {
   using Savepoint = std::pair<ulint, buf_block_t *>;
 
   /** For releasing the S latches after processing the blocks. */
-  using Savepoints = std::vector<Savepoint, ut_allocator<Savepoint>>;
+  using Savepoints = std::vector<Savepoint, ut::allocator<Savepoint>>;
 
   /** The first cursor should read up to the second cursor [f, s). */
   using Range = std::pair<std::shared_ptr<Iter>, std::shared_ptr<Iter>>;
 
-  using Ranges = std::vector<Range, ut_allocator<Range>>;
+  using Ranges = std::vector<Range, ut::allocator<Range>>;
 
   /** @return the scan context ID. */
-  size_t id() const MY_ATTRIBUTE((warn_unused_result)) { return (m_id); }
+  [[nodiscard]] size_t id() const { return m_id; }
 
   /** Set the error state.
   @param[in] err                Error state to set to. */
@@ -516,8 +528,8 @@ class Parallel_reader::Scan_ctx {
   }
 
   /** @return true if in error state. */
-  bool is_error_set() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_err.load(std::memory_order_relaxed) != DB_SUCCESS);
+  [[nodiscard]] bool is_error_set() const {
+    return m_err.load(std::memory_order_relaxed) != DB_SUCCESS;
   }
 
   /** Fetch a block from the buffer pool and acquire an S latch on it.
@@ -525,9 +537,8 @@ class Parallel_reader::Scan_ctx {
   @param[in,out]  mtr           Mini-transaction covering the fetch.
   @param[in]      line          Line from where called.
   @return the block fetched from the buffer pool. */
-  buf_block_t *block_get_s_latched(const page_id_t &page_id, mtr_t *mtr,
-                                   int line) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] buf_block_t *block_get_s_latched(const page_id_t &page_id,
+                                                 mtr_t *mtr, size_t line) const;
 
   /** Partition the B+Tree for parallel read.
   @param[in] scan_range Range for partitioning.
@@ -542,8 +553,8 @@ class Parallel_reader::Scan_ctx {
   @param[in]  block             Page to look in.
   @param[in] key                Key of the first record in the range.
   @return the left child page number. */
-  page_no_t search(const buf_block_t *block, const dtuple_t *key) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] page_no_t search(const buf_block_t *block,
+                                 const dtuple_t *key) const;
 
   /** Traverse from given sub-tree page number to start of the scan range
   from the given page number.
@@ -552,9 +563,9 @@ class Parallel_reader::Scan_ctx {
   @param[in]      key           Key of the first record in the range.
   @param[in,out]  savepoints    Blocks S latched and accessed.
   @return the leaf node page cursor. */
-  page_cur_t start_range(page_no_t page_no, mtr_t *mtr, const dtuple_t *key,
-                         Savepoints &savepoints) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] page_cur_t start_range(page_no_t page_no, mtr_t *mtr,
+                                       const dtuple_t *key,
+                                       Savepoints &savepoints) const;
 
   /** Create and add the range to the scan ranges.
   @param[in,out]  ranges        Ranges to scan.
@@ -585,9 +596,8 @@ class Parallel_reader::Scan_ctx {
   @param[in]      page_cursor   Current page cursor
   @param[in]      mtr           Mini-transaction covering the read.
   @return Start iterator. */
-  std::shared_ptr<Iter> create_persistent_cursor(const page_cur_t &page_cursor,
-                                                 mtr_t *mtr) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] std::shared_ptr<Iter> create_persistent_cursor(
+      const page_cur_t &page_cursor, mtr_t *mtr) const;
 
   /** Build an old version of the row if required.
   @param[in,out]  rec           Current row read from the index. This can
@@ -598,27 +608,23 @@ class Parallel_reader::Scan_ctx {
                                 built from the undo log.
   @param[in,out]  mtr           Mini-transaction covering the read.
   @return true if row is visible to the transaction. */
-  bool check_visibility(const rec_t *&rec, ulint *&offsets, mem_heap_t *&heap,
-                        mtr_t *mtr) MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool check_visibility(const rec_t *&rec, ulint *&offsets,
+                                      mem_heap_t *&heap, mtr_t *mtr);
 
   /** Create an execution context for a range and add it to
   the Parallel_reader's run queue.
   @param[in] range              Range for which to create the context.
   @param[in] split              true if the sub-tree should be split further.
   @return DB_SUCCESS or error code. */
-  dberr_t create_context(const Range &range, bool split)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] dberr_t create_context(const Range &range, bool split);
 
   /** Create the execution contexts based on the ranges.
   @param[in]  ranges            Ranges for which to create the contexts.
   @return DB_SUCCESS or error code. */
-  dberr_t create_contexts(const Ranges &ranges)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] dberr_t create_contexts(const Ranges &ranges);
 
   /** @return the maximum number of threads configured. */
-  size_t max_threads() const MY_ATTRIBUTE((warn_unused_result)) {
-    return m_reader->m_max_threads;
-  }
+  [[nodiscard]] size_t max_threads() const { return m_reader->max_threads(); }
 
   /** Release unused threads back to the pool.
   @param[in] unused_threads     Number of threads to "release". */
@@ -634,7 +640,7 @@ class Parallel_reader::Scan_ctx {
 
   /** @return true if at least one thread owns the S latch on the index. */
   bool index_s_own() const {
-    return (m_s_locks.load(std::memory_order_acquire) > 0);
+    return m_s_locks.load(std::memory_order_acquire) > 0;
   }
 
  private:
@@ -665,7 +671,6 @@ class Parallel_reader::Scan_ctx {
   std::atomic_size_t m_s_locks{};
 
   friend class Parallel_reader;
-  friend class Parallel_reader::Ctx;
 
   Scan_ctx(Scan_ctx &&) = delete;
   Scan_ctx(const Scan_ctx &) = delete;
@@ -673,11 +678,9 @@ class Parallel_reader::Scan_ctx {
   Scan_ctx &operator=(const Scan_ctx &) = delete;
 };
 
-class PCursor;
-
 /** Parallel reader execution context. */
 class Parallel_reader::Ctx {
- private:
+ public:
   /** Constructor.
   @param[in]    id              Thread ID.
   @param[in]    scan_ctx        Scan context.
@@ -685,44 +688,34 @@ class Parallel_reader::Ctx {
   Ctx(size_t id, Scan_ctx *scan_ctx, const Scan_ctx::Range &range)
       : m_id(id), m_range(range), m_scan_ctx(scan_ctx) {}
 
- public:
   /** Destructor. */
-  ~Ctx();
+  ~Ctx() = default;
 
  public:
   /** @return the context ID. */
-  size_t id() const MY_ATTRIBUTE((warn_unused_result)) { return (m_id); }
+  [[nodiscard]] size_t id() const { return m_id; }
 
   /** The scan ID of the scan context this belongs to. */
-  size_t scan_id() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_scan_ctx->id());
-  }
+  [[nodiscard]] size_t scan_id() const { return m_scan_ctx->id(); }
 
   /** @return the covering transaction. */
-  const trx_t *trx() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_scan_ctx->m_trx);
-  }
+  [[nodiscard]] const trx_t *trx() const { return m_scan_ctx->m_trx; }
 
   /** @return the index being scanned. */
-  const dict_index_t *index() const MY_ATTRIBUTE((warn_unused_result)) {
-    return (m_scan_ctx->m_config.m_index);
+  [[nodiscard]] const dict_index_t *index() const {
+    return m_scan_ctx->m_config.m_index;
   }
 
   /** @return ID of the thread processing this context */
-  size_t thread_id() const MY_ATTRIBUTE((warn_unused_result)) {
-    return m_thread_ctx->m_thread_id;
-  }
+  [[nodiscard]] size_t thread_id() const { return m_thread_ctx->m_thread_id; }
 
-  /** @return context information related to the thread processing this
-  context */
-  Thread_ctx *thread_ctx() const MY_ATTRIBUTE((warn_unused_result)) {
-    return m_thread_ctx;
-  }
+  /** @return the thread context of the reader thread. */
+  [[nodiscard]] Thread_ctx *thread_ctx() const { return m_thread_ctx; }
 
   /** @return the partition id of the index.
   @note this is std::numeric_limits<size_t>::max() if the index does not
   belong to a partition. */
-  size_t partition_id() const MY_ATTRIBUTE((warn_unused_result)) {
+  [[nodiscard]] size_t partition_id() const {
     return m_scan_ctx->m_config.m_partition_id;
   }
 
@@ -737,13 +730,13 @@ class Parallel_reader::Ctx {
   @return true if row is visible to the transaction. */
   bool is_rec_visible(const rec_t *&rec, ulint *&offsets, mem_heap_t *&heap,
                       mtr_t *mtr) {
-    return (m_scan_ctx->check_visibility(rec, offsets, heap, mtr));
+    return m_scan_ctx->check_visibility(rec, offsets, heap, mtr);
   }
 
  private:
   /** Traverse the pages by key order.
   @return DB_SUCCESS or error code. */
-  dberr_t traverse() MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] dberr_t traverse();
 
   /** Traverse the records in a node.
   @param[in]  pcursor persistent b-tree cursor
@@ -753,16 +746,15 @@ class Parallel_reader::Ctx {
 
   /** Move to the next node in the specified level.
   @param[in]  pcursor persistent b-tree cursor
-  @param[in]  mtr mtr
   @return success */
-  bool move_to_next_node(PCursor *pcursor, mtr_t *mtr);
+  bool move_to_next_node(PCursor *pcursor);
 
   /** Split the context into sub-ranges and add them to the execution queue.
   @return DB_SUCCESS or error code. */
-  dberr_t split() MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] dberr_t split();
 
   /** @return true if in error state. */
-  bool is_error_set() const MY_ATTRIBUTE((warn_unused_result)) {
+  [[nodiscard]] bool is_error_set() const {
     return m_scan_ctx->m_reader->is_error_set() || m_scan_ctx->is_error_set();
   }
 
@@ -770,7 +762,7 @@ class Parallel_reader::Ctx {
   /** Context ID. */
   size_t m_id{std::numeric_limits<size_t>::max()};
 
-  /** If true the split the context at the block level. */
+  /** If true then split the context at the block level. */
   bool m_split{};
 
   /** Range to read in this context. */
