@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2025, Oracle and/or its affiliates.
+Copyright (c) 1995, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -215,8 +214,6 @@ enum latch_level_t {
 
   SYNC_ANY_LATCH,
 
-  SYNC_ALTER_STAGE,
-
   SYNC_FIL_SHARD,
 
   SYNC_PAGE_ARCH_OPER,
@@ -253,6 +250,7 @@ enum latch_level_t {
   SYNC_LOG_CHECKPOINTER,
   SYNC_LOG_SN,
   SYNC_LOG_SN_MUTEX,
+  SYNC_LOG_GOVERNOR_MUTEX,
   SYNC_PAGE_ARCH,
   SYNC_PAGE_ARCH_CLIENT,
   SYNC_LOG_ARCH,
@@ -344,7 +342,6 @@ alphabetical ordering seems useful */
 enum latch_id_t {
   LATCH_ID_NONE = 0,
   LATCH_ID_AHI_ENABLED,
-  LATCH_ID_ALTER_STAGE,
   LATCH_ID_AUTOINC,
   LATCH_ID_BUF_BLOCK_MUTEX,
   LATCH_ID_BUF_POOL_CHUNKS,
@@ -388,6 +385,7 @@ enum latch_id_t {
   LATCH_ID_LOG_LIMITS,
   LATCH_ID_LOG_ONLINE,
   LATCH_ID_LOG_FILES,
+  LATCH_ID_LOG_GOVERNOR_MUTEX,
   LATCH_ID_PARSER,
   LATCH_ID_LOG_ARCH,
   LATCH_ID_PAGE_ARCH,
