@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2025, Oracle and/or its affiliates.
+Copyright (c) 1996, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -310,20 +309,26 @@ void lock_wait_suspend_thread(que_thr_t *thr) {
     locksys::Global_exclusive_latch_guard guard{UT_LOCATION_HERE};
     const lock_t *wait_lock = trx->lock.wait_lock;
     if (wait_lock != nullptr) {
-      locksys::find_blockers(*wait_lock, [&](const lock_t &curr_lock) {
-        blocking[blocking_count].trx_id = lock_get_trx_id(&curr_lock);
-        blocking[blocking_count].thread_id =
-            curr_lock.trx->mysql_thd
-                ? thd_get_thread_id(curr_lock.trx->mysql_thd)
-                : 0;
-        blocking[blocking_count].query_id =
-            curr_lock.trx->mysql_thd
-                ? thd_get_query_id(curr_lock.trx->mysql_thd)
-                : 0;
-        /* Only limited number of blocking transaction infos is implemented*/
-        if ((++blocking_count) >= MAX_BLOCKING_TRX_IN_REPORT) return true;
-        return false;
-      });
+      lock_queue_iterator_t iter;
+      const lock_t *curr_lock;
+      lock_queue_iterator_reset(&iter, wait_lock, ULINT_UNDEFINED);
+      for (curr_lock = lock_queue_iterator_get_prev(&iter);
+           curr_lock != nullptr;
+           curr_lock = lock_queue_iterator_get_prev(&iter)) {
+        if (lock_has_to_wait(trx->lock.wait_lock, curr_lock)) {
+          blocking[blocking_count].trx_id = lock_get_trx_id(curr_lock);
+          blocking[blocking_count].thread_id =
+              curr_lock->trx->mysql_thd
+                  ? thd_get_thread_id(curr_lock->trx->mysql_thd)
+                  : 0;
+          blocking[blocking_count].query_id =
+              curr_lock->trx->mysql_thd
+                  ? thd_get_query_id(curr_lock->trx->mysql_thd)
+                  : 0;
+          /* Only limited number of blocking transaction infos is implemented*/
+          if ((++blocking_count) >= MAX_BLOCKING_TRX_IN_REPORT) break;
+        }
+      }
     }
   }
 
