@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+Copyright (c) 2018, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -215,45 +214,12 @@ class PCursor {
   PCursor(btr_pcur_t *pcur, mtr_t *mtr, size_t read_level)
       : m_mtr(mtr), m_pcur(pcur), m_read_level(read_level) {}
 
- private:
   /** Create a savepoint and commit the mini-transaction.*/
   void savepoint() noexcept;
 
   /** Resume from savepoint. */
   void resume() noexcept;
 
- public:
-  /** Store the position of the user record that is added to the key buffer and,
-  commit the mini-transaction. The cursor must not be positioned at infimum
-  or supremum.
-  @note This method must be paired with
-  restore_to_last_processed_user_record() to restore the record position */
-  void save_current_user_record_as_last_processed() noexcept;
-
-  /** Restore the cursor to the saved record. If the saved record was physically
-  removed then restore to the largest record which is smaller than the saved
-  record. If all records were removed then restore the cursor to the infimum;
-  this must be fine if the next step of the caller is to advance the cursor.
-  @note This method must be paired with
-  save_current_user_record_as_last_processed() to save the record position */
-  void restore_to_last_processed_user_record() noexcept;
-
-  /** This method must be called after all records on a page are processed and
-  cursor is positioned at supremum. Under this assumption, it stores the
-  position BTR_PCUR_AFTER the last user record on the page.
-  This method must be paired with restore_to_first_unprocessed() to restore to
-  a record which comes right after the value of the stored last processed
-  record @see restore_to_first_unprocessed for details. */
-  void save_previous_user_record_as_last_processed() noexcept;
-
-  /** Restore the cursor to the record which comes next after the saved
-  position by the paired method save_previous_user_record_as_last_processed().
-  If there is no such record, it will restore to supremum on the last leaf
-  page.
-  @note This method must be paired with
-  save_previous_user_record_as_last_processed() to save the record position */
-  void restore_to_first_unprocessed() noexcept;
-
   /** Move to the next block.
   @param[in]  index             Index being traversed.
   @return DB_SUCCESS or error code. */
@@ -323,7 +289,6 @@ class PCursor {
     return m_pcur->get_page_cur();
   }
 
- private:
   /** Restore from a saved position.
   @return DB_SUCCESS or error code. */
   [[nodiscard]] dberr_t restore_from_savepoint() noexcept;
@@ -331,7 +296,6 @@ class PCursor {
   /** Move to the first user rec on the restored page. */
   [[nodiscard]] dberr_t move_to_user_rec() noexcept;
 
- public:
   /** @return true if cursor is after last on page. */
   [[nodiscard]] bool is_after_last_on_page() const noexcept {
     return m_pcur->is_after_last_on_page();
@@ -367,7 +331,6 @@ buf_block_t *Parallel_reader::Scan_ctx::block_get_s_latched(
 }
 
 void PCursor::savepoint() noexcept {
-  ut_ad(m_read_level != 0);
   /* Store the cursor position on the previous user record on the page. */
   m_pcur->move_to_prev_on_page();
 
@@ -377,7 +340,6 @@ void PCursor::savepoint() noexcept {
 }
 
 void PCursor::resume() noexcept {
-  ut_ad(m_read_level != 0);
   m_mtr->start();
 
   m_mtr->set_log_mode(MTR_LOG_NO_REDO);
@@ -393,62 +355,6 @@ void PCursor::resume() noexcept {
   }
 }
 
-void PCursor::save_current_user_record_as_last_processed() noexcept {
-  ut_a(m_pcur->is_on_user_rec());
-  m_pcur->store_position(m_mtr);
-  m_mtr->commit();
-}
-
-void PCursor::restore_to_last_processed_user_record() noexcept {
-  ut_a(m_pcur->m_rel_pos == BTR_PCUR_ON);
-  m_mtr->start();
-  m_mtr->set_log_mode(MTR_LOG_NO_REDO);
-
-  [[maybe_unused]] const auto same_row =
-      m_pcur->restore_position(BTR_SEARCH_LEAF, m_mtr, UT_LOCATION_HERE);
-
-  /* Since m_rel_pos was BTR_PCUR_ON, the state can't be
-  BTR_PCUR_IS_POSITIONED_OPTIMISTIC */
-  ut_a(BTR_PCUR_IS_POSITIONED_OPTIMISTIC != m_pcur->m_pos_state);
-
-  /* If the cursor was pointing to the user record then 'PAGE_CUR_LE' mode
-  used by the btr_pcur_t::restore_position() restores the cursor to a record
-  lower than the original one if it was removed, and in particular, if all
-  smaller ones were removed, cur could be on infimum now. This would be fine,
-  as our next step will be to advance the cursor, anyway.
-  However, right now the only usage of this method is in the bulk inserter
-  for a B-Tree. which only calls it when the current row was visible to the
-  scan, and thus can't be purged. So, we know it can't happen, unless someone
-  reuses this function in a new context, in which case they should probably
-  repeat the proof of correctness. The purpose of following debug assert is
-  to show the limited the scope of this method, and thus how risky it
-  is to use it in new context: */
-  ut_ad(same_row);
-}
-
-void PCursor::save_previous_user_record_as_last_processed() noexcept {
-  ut_a(m_pcur->is_after_last_on_page());
-  m_pcur->store_position(m_mtr);
-  ut_a(m_pcur->m_rel_pos == BTR_PCUR_AFTER);
-  m_mtr->commit();
-}
-
-void PCursor::restore_to_first_unprocessed() noexcept {
-  ut_a(m_pcur->m_rel_pos == BTR_PCUR_AFTER);
-  m_mtr->start();
-  m_mtr->set_log_mode(MTR_LOG_NO_REDO);
-  m_pcur->restore_position(BTR_SEARCH_LEAF, m_mtr, UT_LOCATION_HERE);
-  if (m_pcur->m_pos_state == BTR_PCUR_IS_POSITIONED_OPTIMISTIC) {
-    /* The BTR_PCUR_IS_POSITIONED_OPTIMISTIC means the implementation of
-    btr_pcur_t::restore_position() successfully re-acquired a block stored in
-    hint and it wasn't modified meanwhile, and in such case it does not advance
-    the cursor to the next position even though BTR_PCUR_AFTER was used, so we
-    have to do it ourselves. */
-    ut_a(!m_pcur->is_after_last_on_page());
-    m_pcur->move_to_next_on_page();
-  }
-}
-
 dberr_t PCursor::move_to_user_rec() noexcept {
   auto cur = m_pcur->get_page_cur();
   const auto next_page_no = btr_page_get_next(page_cur_get_page(cur), m_mtr);
@@ -501,36 +407,25 @@ dberr_t PCursor::move_to_user_rec() noexcept {
 }
 
 dberr_t PCursor::restore_from_savepoint() noexcept {
-  ut_ad(m_read_level != 0);
   resume();
   return m_pcur->is_on_user_rec() ? DB_SUCCESS : move_to_user_rec();
 }
 
-void Parallel_reader::Thread_ctx::
-    save_current_user_record_as_last_processed() noexcept {
-  m_pcursor->save_current_user_record_as_last_processed();
+dberr_t Parallel_reader::Thread_ctx::restore_from_savepoint() noexcept {
+  /* If read_level != 0, might return DB_LOCK_NOWAIT error. */
+  ut_ad(m_pcursor->read_level() == 0);
+  return m_pcursor->restore_from_savepoint();
 }
 
-void Parallel_reader::Thread_ctx::
-    restore_to_last_processed_user_record() noexcept {
-  m_pcursor->restore_to_last_processed_user_record();
-}
-
-void Parallel_reader::Thread_ctx::
-    save_previous_user_record_as_last_processed() noexcept {
-  m_pcursor->save_previous_user_record_as_last_processed();
-}
-
-void Parallel_reader::Thread_ctx::restore_to_first_unprocessed() noexcept {
-  m_pcursor->restore_to_first_unprocessed();
+void Parallel_reader::Thread_ctx::savepoint() noexcept {
+  m_pcursor->savepoint();
 }
 
 dberr_t PCursor::move_to_next_block(dict_index_t *index) {
   ut_ad(m_pcur->is_after_last_on_page());
   dberr_t err = DB_SUCCESS;
 
-  if (DBUG_EVALUATE_IF("pcursor_move_to_next_block_release_latches", true,
-                       rw_lock_get_waiters(dict_index_get_lock(index)))) {
+  if (rw_lock_get_waiters(dict_index_get_lock(index))) {
     /* There are waiters on the index tree lock. Store and restore
     the cursor position, and yield so that scanning a large table
     will not starve other threads. */
@@ -538,27 +433,6 @@ dberr_t PCursor::move_to_next_block(dict_index_t *index) {
     /* We should always yield on a block boundary. */
     ut_ad(m_pcur->is_after_last_on_page());
 
-    if (m_read_level == 0) {
-      save_previous_user_record_as_last_processed();
-      DEBUG_SYNC_C("pcursor_move_to_next_block_latches_released");
-      std::this_thread::yield();
-      restore_to_first_unprocessed();
-      if (m_pcur->is_after_last_on_page()) {
-        /* There's nothing wrong here, as restore_to_first_unprocessed() is not
-        obliged to position the cursor on a *user* record - it can be a
-        supremum. This can happen if
-        a) the block in BP wasn't modified and thus we've restored the cursor
-        exactly where it was, i.e. on supremum
-        b) there is no more records in the table
-        These options aren't mutually exclusive. Both are well handled by: */
-        return move_to_user_rec();
-      } else {
-        /* This happens if we were descending the B-tree, in which case we try
-        to navigate in such a way to arrive at a user record. */
-        ut_a(m_pcur->is_on_user_rec());
-        return DB_SUCCESS;
-      }
-    }
     savepoint();
 
     /* Yield so that another thread can proceed. */
