@@ -1,19 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -94,6 +93,9 @@ description and its property/type and corresponding monitor_id.
 Please note: If you add a monitor here, please add its corresponding
 monitor_id to "enum monitor_id_value" structure in srv0mon.h file. */
 
+// NOTE: please keep the counter descriptions below in sync with the
+// description of the matching metrics in handler/ha_innodb.cc
+
 static monitor_info_t innodb_counter_info[] = {
     /* A dummy item to mark the module start, this is
     to accommodate the default value (0) set for the
@@ -336,7 +338,7 @@ static monitor_info_t innodb_counter_info[] = {
      MONITOR_DEFAULT_START, MONITOR_OVLD_PAGES0_READ},
 
     {"buffer_data_reads", "buffer",
-     "Amount of data read in bytes (innodb_data_reads)",
+     "Amount of data read in bytes (innodb_data_read)",
      static_cast<monitor_type_t>(MONITOR_EXISTING | MONITOR_DEFAULT_ON),
      MONITOR_DEFAULT_START, MONITOR_OVLD_BYTE_READ},
 
@@ -935,7 +937,8 @@ static monitor_info_t innodb_counter_info[] = {
      static_cast<monitor_type_t>(MONITOR_EXISTING | MONITOR_DEFAULT_ON),
      MONITOR_DEFAULT_START, MONITOR_OVLD_LOG_WAITS},
 
-    {"log_write_requests", "log", "Number of log write requests",
+    {"log_write_requests", "log",
+     "Number of log write requests (innodb_log_write_requests)",
      static_cast<monitor_type_t>(MONITOR_EXISTING | MONITOR_DEFAULT_ON),
      MONITOR_DEFAULT_START, MONITOR_OVLD_LOG_WRITE_REQUEST},
 
@@ -1454,7 +1457,7 @@ monitor_value_t innodb_counter_value[NUM_MONITOR];
 
 /* monitor_set_tbl is used to record and determine whether a monitor
 has been turned on/off. */
-std::bitset<NUM_MONITOR> monitor_set_tbl;
+ulint monitor_set_tbl[(NUM_MONITOR + NUM_BITS_ULINT - 1) / NUM_BITS_ULINT];
 
 /** Get a monitor's "monitor_info" by its monitor id (index into the
  innodb_counter_info array.
@@ -1759,7 +1762,8 @@ void srv_mon_process_existing_counter(
       value = srv_stats.page0_read;
       break;
 
-    /* innodb_data_reads, the total number of data reads */
+    /* innodb_data_read, the amount of data read since the server was started
+     * (in bytes) */
     case MONITOR_OVLD_BYTE_READ:
       value = srv_stats.data_read;
       break;
