@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,7 +33,7 @@
 #include <sys/types.h>
 #include <time.h>
 
-#include "m_ctype.h"
+#include "m_string.h"
 #include "my_bit.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
@@ -45,6 +44,9 @@
 #include "my_io.h"
 #include "my_macros.h"
 #include "my_systime.h"  // get_date
+#include "mysql/strings/int2str.h"
+#include "mysql/strings/m_ctype.h"
+#include "nulls.h"
 #include "print_version.h"
 #include "storage/myisam/fulltext.h"
 #include "storage/myisam/myisam_sys.h"
@@ -817,11 +819,11 @@ static int myisamchk(MI_CHECK *param, char *filename) {
   if (!(info = mi_open(
             filename,
             (param->testflag & (T_DESCRIPT | T_READONLY)) ? O_RDONLY : O_RDWR,
-            HA_OPEN_FOR_REPAIR |
-                ((param->testflag & T_WAIT_FOREVER) ? HA_OPEN_WAIT_IF_LOCKED
-                 : (param->testflag & T_DESCRIPT)
-                     ? HA_OPEN_IGNORE_IF_LOCKED
-                     : HA_OPEN_ABORT_IF_LOCKED)))) {
+            HA_OPEN_FOR_REPAIR | ((param->testflag & T_WAIT_FOREVER)
+                                      ? HA_OPEN_WAIT_IF_LOCKED
+                                      : (param->testflag & T_DESCRIPT)
+                                            ? HA_OPEN_IGNORE_IF_LOCKED
+                                            : HA_OPEN_ABORT_IF_LOCKED)))) {
     /* Avoid twice printing of isam file name */
     param->error_printed = 1;
     switch (my_errno()) {
@@ -1397,10 +1399,11 @@ static int mi_sort_records(MI_CHECK *param, MI_INFO *info, char *name,
   MYISAM_SHARE *share = info->s;
   char llbuff[22], llbuff2[22];
   SORT_INFO sort_info;
-  MI_SORT_PARAM sort_param{};
+  MI_SORT_PARAM sort_param;
   DBUG_TRACE;
 
   memset(&sort_info, 0, sizeof(sort_info));
+  memset(&sort_param, 0, sizeof(sort_param));
   sort_param.sort_info = &sort_info;
   sort_info.param = param;
   keyinfo = &share->keyinfo[sort_key];
