@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,7 +31,9 @@
 #define STORAGE_MYISAM_MYISAMDEF_INCLUDED 1
 
 #include "my_alloc.h"
+#include "my_checksum.h"  // ha_checksum
 #include "my_icp.h"
+#include "my_inttypes.h"
 #include "my_pointer_arithmetic.h"
 #include "myisam.h"     /* Structs & some defines */
 #include "myisampack.h" /* packing of keys */
@@ -75,7 +77,7 @@ struct MI_STATE_INFO {
     uchar unique_key_parts[2];  /* Key parts + unique parts */
     uchar keys;                 /* number of keys in file */
     uchar uniques;              /* number of UNIQUE definitions */
-    uchar language;             /* Language for indexes */
+    uchar language;             /* Language (collation) for indexes */
     uchar max_block_size_index; /* max keyblock size */
     uchar fulltext_keys;
     uchar not_used; /* To align to 8 */
@@ -206,7 +208,7 @@ struct MYISAM_SHARE { /* Shared between opens */
   ulong last_process;    /* For table-change-check */
   ulong last_version;    /* Version on start */
   ulong options;         /* Options used */
-  ulong min_pack_length; /* Theese are used by packed data */
+  ulong min_pack_length; /* These are used by packed data */
   ulong max_pack_length;
   ulong state_diff_length;
   uint rec_reflength; /* rec_reflength in use now */
@@ -245,9 +247,7 @@ struct MI_BIT_BUFF { /* Used for packing of record */
   uint error;
 };
 
-C_MODE_START
 typedef ICP_RESULT (*index_cond_func_t)(void *param);
-C_MODE_END
 
 struct MI_INFO {
   MYISAM_SHARE *s; /* Shared between open:s */
@@ -285,7 +285,7 @@ struct MI_INFO {
   my_off_t last_search_keypage; /* Last keypage when searching */
   my_off_t dupp_key_pos;
   ha_checksum checksum; /* Temp storage for row checksum */
-  /* QQ: the folloing two xxx_length fields should be removed,
+  /* QQ: the following two xxx_length fields should be removed,
      as they are not compatible with parallel repair */
   ulong packed_length, blob_length;    /* Length of found, packed record */
   int dfile;                           /* The datafile */
@@ -305,7 +305,7 @@ struct MI_INFO {
   uint save_update;        /* When using KEY_READ */
   int save_lastinx;
   LIST open_list;
-  IO_CACHE rec_cache;        /* When cacheing records */
+  IO_CACHE rec_cache;        /* When caching records */
   uint preload_buff_size;    /* When preloading indexes */
   myf lock_wait;             /* is 0 or MY_DONT_WAIT */
   bool was_locked;           /* Was locked in panic */
@@ -337,10 +337,9 @@ struct BUFFPEK {
 };
 
 struct MI_SORT_PARAM {
-  my_thread_handle thr;
   IO_CACHE read_cache, tempfile, tempfile_for_exceptions;
   DYNAMIC_ARRAY buffpek;
-  MI_BIT_BUFF bit_buff; /* For parallel repair of packrec. */
+  MI_BIT_BUFF bit_buff; /* For sort repair of packrec. */
 
   /*
     The next two are used to collect statistics, see update_key_parts for
@@ -414,7 +413,7 @@ struct MI_SORT_PARAM {
   do {                                             \
     (x)->s->state.changed |= STATE_CRASHED;        \
     DBUG_PRINT("error", ("Marked table crashed")); \
-    mi_report_crashed((x), 0);                     \
+    mi_report_crashed((x), nullptr);               \
   } while (0)
 #define mi_mark_crashed_on_repair(x)                                  \
   do {                                                                \
@@ -461,7 +460,7 @@ struct MI_SORT_PARAM {
 
 #define get_pack_length(length) ((length) >= 255 ? 3 : 1)
 
-#define MI_MIN_BLOCK_LENGTH 20    /* Because of delete-link */
+#define MI_MIN_BLOCK_LENGTH 20U   /* Because of delete-link */
 #define MI_EXTEND_BLOCK_LENGTH 20 /* Don't use to small record-blocks */
 #define MI_SPLIT_LENGTH ((MI_EXTEND_BLOCK_LENGTH + 4) * 2)
 #define MI_MAX_DYN_BLOCK_HEADER 20 /* Max prefix of record-block */
@@ -522,7 +521,8 @@ extern ulong myisam_pid;
 struct MI_KEY_PARAM {
   uint ref_length, key_length, n_ref_length, n_length, totlength,
       part_of_prev_key, prev_length, pack_marker;
-  uchar *key, *prev_key, *next_key_pos;
+  const uchar *key;
+  uchar *prev_key, *next_key_pos;
   bool store_not_null;
 };
 
@@ -558,20 +558,20 @@ extern uchar *_mi_find_half_pos(uint nod_flag, MI_KEYDEF *keyinfo, uchar *page,
                                 uchar *key, uint *return_key_length,
                                 uchar **after_key);
 extern int _mi_calc_static_key_length(MI_KEYDEF *keyinfo, uint nod_flag,
-                                      uchar *key_pos, uchar *org_key,
-                                      uchar *key_buff, uchar *key,
+                                      const uchar *key_pos, uchar *org_key,
+                                      uchar *key_buff, const uchar *key,
                                       MI_KEY_PARAM *s_temp);
 extern int _mi_calc_var_key_length(MI_KEYDEF *keyinfo, uint nod_flag,
-                                   uchar *key_pos, uchar *org_key,
-                                   uchar *key_buff, uchar *key,
+                                   const uchar *key_pos, uchar *org_key,
+                                   uchar *key_buff, const uchar *key,
                                    MI_KEY_PARAM *s_temp);
 extern int _mi_calc_var_pack_key_length(MI_KEYDEF *keyinfo, uint nod_flag,
-                                        uchar *key_pos, uchar *org_key,
-                                        uchar *prev_key, uchar *key,
+                                        const uchar *key_pos, uchar *org_key,
+                                        uchar *prev_key, const uchar *key,
                                         MI_KEY_PARAM *s_temp);
 extern int _mi_calc_bin_pack_key_length(MI_KEYDEF *keyinfo, uint nod_flag,
-                                        uchar *key_pos, uchar *org_key,
-                                        uchar *prev_key, uchar *key,
+                                        const uchar *key_pos, uchar *org_key,
+                                        uchar *prev_key, const uchar *key,
                                         MI_KEY_PARAM *s_temp);
 void _mi_store_static_key(MI_KEYDEF *keyinfo, uchar *key_pos,
                           MI_KEY_PARAM *s_temp);
@@ -601,7 +601,7 @@ extern int _mi_prefix_search(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *page,
                              uchar **ret_pos, uchar *buff, bool *was_last_key);
 extern my_off_t _mi_kpos(uint nod_flag, uchar *after_key);
 extern void _mi_kpointer(MI_INFO *info, uchar *buff, my_off_t pos);
-extern my_off_t _mi_dpos(MI_INFO *info, uint nod_flag, uchar *after_key);
+extern my_off_t _mi_dpos(MI_INFO *info, uint nod_flag, const uchar *after_key);
 extern my_off_t _mi_rec_pos(MYISAM_SHARE *info, uchar *ptr);
 extern void _mi_dpointer(MI_INFO *info, uchar *buff, my_off_t pos);
 extern uint _mi_get_static_key(MI_KEYDEF *keyinfo, uint nod_flag, uchar **page,
@@ -615,9 +615,10 @@ extern uchar *_mi_get_last_key(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *keypos,
                                uint *return_key_length);
 extern uchar *_mi_get_key(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *page,
                           uchar *key, uchar *keypos, uint *return_key_length);
-extern uint _mi_keylength(MI_KEYDEF *keyinfo, uchar *key);
-extern uint _mi_keylength_part(MI_KEYDEF *keyinfo, uchar *key, HA_KEYSEG *end);
-extern uchar *_mi_move_key(MI_KEYDEF *keyinfo, uchar *to, uchar *from);
+extern uint _mi_keylength(MI_KEYDEF *keyinfo, const uchar *key);
+extern uint _mi_keylength_part(MI_KEYDEF *keyinfo, const uchar *key,
+                               HA_KEYSEG *end);
+extern uchar *_mi_move_key(MI_KEYDEF *keyinfo, uchar *to, const uchar *from);
 extern int _mi_search_next(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
                            uint key_length, uint nextflag, my_off_t pos);
 extern int _mi_search_first(MI_INFO *info, MI_KEYDEF *keyinfo, my_off_t pos);
@@ -632,8 +633,8 @@ extern int _mi_dispose(MI_INFO *info, MI_KEYDEF *keyinfo, my_off_t pos,
 extern my_off_t _mi_new(MI_INFO *info, MI_KEYDEF *keyinfo, int level);
 extern uint _mi_make_key(MI_INFO *info, uint keynr, uchar *key,
                          const uchar *record, my_off_t filepos);
-extern uint _mi_pack_key(MI_INFO *info, uint keynr, uchar *key, uchar *old,
-                         key_part_map keypart_map,
+extern uint _mi_pack_key(MI_INFO *info, uint keynr, uchar *key,
+                         const uchar *old, key_part_map keypart_map,
                          HA_KEYSEG **last_used_keyseg);
 extern int _mi_read_key_record(MI_INFO *info, my_off_t filepos, uchar *buf);
 extern int _mi_read_cache(IO_CACHE *info, uchar *buff, my_off_t pos,
@@ -648,8 +649,8 @@ extern uchar *mi_alloc_rec_buff(MI_INFO *, ulong, uchar **);
 #define mi_get_rec_buff_len(info, buf) \
   (*((uint32 *)(mi_get_rec_buff_ptr(info, buf))))
 
-extern ulong _mi_rec_unpack(MI_INFO *info, uchar *to, uchar *from,
-                            ulong reclength);
+extern size_t _mi_rec_unpack(MI_INFO *info, uchar *to, const uchar *from,
+                             ulong reclength);
 extern bool _mi_rec_check(MI_INFO *info, const uchar *record, uchar *packpos,
                           ulong packed_length, bool with_checkum);
 extern int _mi_write_part_record(MI_INFO *info, my_off_t filepos, ulong length,
@@ -693,7 +694,9 @@ struct MI_BLOCK_INFO { /* Parameter to _mi_get_block_info */
 #define BLOCK_SYNC_ERROR 16  /* Right data at wrong place */
 #define BLOCK_FATAL_ERROR 32 /* hardware-error */
 
-#define NEED_MEM ((uint)10 * 4 * (IO_SIZE + 32) + 32) /* Nead for recursion */
+#define NEED_MEM                                               \
+  ((uint)10 * 4 * (IO_SIZE + 32) + 32) /* Needed for recursion \
+                                        */
 #define MAXERR 20
 #define BUFFERS_WHEN_SORTING 16    /* Alloc for sort-key-tree */
 #define MY_HOW_OFTEN_TO_WRITE 1000 /* How often we want info on screen */
@@ -710,7 +713,7 @@ struct MI_BLOCK_INFO { /* Parameter to _mi_get_block_info */
 #define USE_BUFFER_INIT (((1024L * 512L - MALLOC_OVERHEAD) / IO_SIZE) * IO_SIZE)
 #define READ_BUFFER_INIT (1024L * 256L - MALLOC_OVERHEAD)
 #define SORT_BUFFER_INIT (2048L * 1024L - MALLOC_OVERHEAD)
-#define MIN_SORT_BUFFER (4096 - MALLOC_OVERHEAD)
+#define MIN_SORT_BUFFER (4096ULL - MALLOC_OVERHEAD)
 
 enum myisam_log_commands {
   MI_LOG_OPEN,
@@ -826,8 +829,6 @@ void mi_check_print_info(MI_CHECK *param, const char *fmt, ...)
     MY_ATTRIBUTE((format(printf, 2, 3)));
 int flush_pending_blocks(MI_SORT_PARAM *param);
 int sort_ft_buf_flush(MI_SORT_PARAM *sort_param);
-int thr_write_keys(MI_SORT_PARAM *sort_param);
-void *thr_find_all_keys(void *arg);
 int flush_blocks(MI_CHECK *param, KEY_CACHE *key_cache, File file);
 
 int sort_write_record(MI_SORT_PARAM *sort_param);
@@ -842,9 +843,7 @@ extern thread_local st_keycache_thread_var *keycache_tls;
 #endif
 
 #ifdef HAVE_PSI_INTERFACE
-C_MODE_START
-extern PSI_mutex_key mi_key_mutex_MYISAM_SHARE_intern_lock,
-    mi_key_mutex_MI_SORT_INFO_mutex, mi_key_mutex_MI_CHECK_print_msg;
+extern PSI_mutex_key mi_key_mutex_MYISAM_SHARE_intern_lock;
 
 extern PSI_rwlock_key mi_key_rwlock_MYISAM_SHARE_key_root_lock,
     mi_key_rwlock_MYISAM_SHARE_mmap_lock;
@@ -858,11 +857,8 @@ extern PSI_file_key mi_key_file_datatmp, mi_key_file_dfile, mi_key_file_kfile,
 extern PSI_thread_key mi_key_thread_find_all_keys;
 
 void init_myisam_psi_keys();
-C_MODE_END
 #endif /* HAVE_PSI_INTERFACE */
 
-C_MODE_START
-
 extern PSI_memory_key mi_key_memory_MYISAM_SHARE;
 extern PSI_memory_key mi_key_memory_MI_INFO;
 extern PSI_memory_key mi_key_memory_MI_INFO_ft1_to_ft2;
@@ -886,6 +882,4 @@ extern PSI_memory_key mi_key_memory_stPageList_pages;
 extern PSI_memory_key mi_key_memory_keycache_thread_var;
 extern PSI_memory_key key_memory_QUEUE;
 
-C_MODE_END
-
 #endif  // STORAGE_MYISAM_MYISAMDEF_INCLUDED
