@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
    Copyright (c) 2018, Percona and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /*
-  Creates a index for a database by reading keys, sorting them and outputing
+  Creates a index for a database by reading keys, sorting them and outputting
   them in sorted order through SORT_INFO functions.
 */
 
@@ -292,301 +292,6 @@ static ha_rows find_all_keys(MI_SORT_PARAM *info, uint keys, uchar **sort_keys,
   return (*maxbuffer) * (keys - 1) + idx;
 } /* find_all_keys */
 
-/* Search after all keys and place them in a temp. file */
-
-void *thr_find_all_keys(void *arg) {
-  MI_SORT_PARAM *sort_param = (MI_SORT_PARAM *)arg;
-  int error;
-  ulonglong memavl, old_memavl;
-  uint keys = 0, sort_length;
-  uint idx, maxbuffer;
-  uchar **sort_keys = nullptr;
-  st_keycache_thread_var thread_keycache_var;
-
-  error = 1;
-
-  if (my_thread_init()) goto err;
-
-  memset(&thread_keycache_var, 0, sizeof(st_keycache_thread_var));
-  mysql_cond_init(PSI_NOT_INSTRUMENTED, &thread_keycache_var.suspend);
-  keycache_tls = &thread_keycache_var;
-
-  DBUG_PRINT("enter", ("master: %d", sort_param->master));
-  if (sort_param->sort_info->got_error) goto err;
-
-  if (sort_param->keyinfo->flag & HA_VAR_LENGTH_KEY) {
-    sort_param->write_keys = write_keys_varlen;
-    sort_param->read_to_buffer = read_to_buffer_varlen;
-    sort_param->write_key = write_merge_key_varlen;
-  } else {
-    sort_param->write_keys = write_keys;
-    sort_param->read_to_buffer = read_to_buffer;
-    sort_param->write_key = write_merge_key;
-  }
-
-  my_b_clear(&sort_param->tempfile);
-  my_b_clear(&sort_param->tempfile_for_exceptions);
-  memset(&sort_param->buffpek, 0, sizeof(sort_param->buffpek));
-  memset(&sort_param->unique, 0, sizeof(sort_param->unique));
-  sort_keys = (uchar **)nullptr;
-
-  memavl = std::max(sort_param->sortbuff_size, MIN_SORT_BUFFER);
-  idx = (uint)sort_param->sort_info->max_records;
-  sort_length = sort_param->key_length;
-  maxbuffer = 1;
-
-  if ((memavl - sizeof(BUFFPEK)) / (sort_length + sizeof(char *)) > UINT_MAX32)
-    memavl = sizeof(BUFFPEK) + UINT_MAX32 * (sort_length + sizeof(char *));
-
-  while (memavl >= MIN_SORT_BUFFER) {
-    if ((my_off_t)(idx + 1) * (sort_length + sizeof(char *)) <=
-        (my_off_t)memavl)
-      keys = idx + 1;
-    else {
-      uint skr;
-      do {
-        skr = maxbuffer;
-        if (memavl < sizeof(BUFFPEK) * maxbuffer ||
-            (keys = (uint)((memavl - sizeof(BUFFPEK) * maxbuffer) /
-                           (sort_length + sizeof(char *)))) <= 1 ||
-            keys < (uint)maxbuffer) {
-          mi_check_print_error(sort_param->sort_info->param,
-                               "myisam_sort_buffer_size is too small");
-          goto err;
-        }
-      } while ((maxbuffer = (int)(idx / (keys - 1) + 1)) != skr);
-    }
-    if ((sort_keys = (uchar **)my_malloc(
-             PSI_NOT_INSTRUMENTED,
-             keys * (sort_length + sizeof(char *)) +
-                 ((sort_param->keyinfo->flag & HA_FULLTEXT) ? HA_FT_MAXBYTELEN
-                                                            : 0),
-             MYF(0)))) {
-      if (my_init_dynamic_array(&sort_param->buffpek, PSI_NOT_INSTRUMENTED,
-                                sizeof(BUFFPEK), nullptr, maxbuffer,
-                                maxbuffer / 2)) {
-        my_free(sort_keys);
-        sort_keys = (uchar **)nullptr; /* for err: label */
-      } else
-        break;
-    }
-    old_memavl = memavl;
-    if ((memavl = memavl / 4 * 3) < MIN_SORT_BUFFER &&
-        old_memavl > MIN_SORT_BUFFER)
-      memavl = MIN_SORT_BUFFER;
-  }
-  if (memavl < MIN_SORT_BUFFER) {
-    mi_check_print_error(sort_param->sort_info->param,
-                         "MyISAM sort buffer too small");
-    goto err; /* purecov: tested */
-  }
-
-  if (sort_param->sort_info->param->testflag & T_VERBOSE)
-    printf("Key %d - Allocating buffer for %d keys\n", sort_param->key + 1,
-           keys);
-  sort_param->sort_keys = sort_keys;
-
-  idx = error = 0;
-  sort_keys[0] = (uchar *)(sort_keys + keys);
-
-  DBUG_PRINT("info", ("reading keys"));
-  while (!(error = sort_param->sort_info->got_error) &&
-         !(error = (*sort_param->key_read)(sort_param, sort_keys[idx]))) {
-    if (sort_param->real_key_length > sort_param->key_length) {
-      if (write_key(sort_param, sort_keys[idx],
-                    &sort_param->tempfile_for_exceptions))
-        goto err;
-      continue;
-    }
-
-    if (++idx == keys) {
-      if (sort_param->write_keys(sort_param, sort_keys, idx - 1,
-                                 (BUFFPEK *)alloc_dynamic(&sort_param->buffpek),
-                                 &sort_param->tempfile))
-        goto err;
-      sort_keys[0] = (uchar *)(sort_keys + keys);
-      memcpy(sort_keys[0], sort_keys[idx - 1], (size_t)sort_param->key_length);
-      idx = 1;
-    }
-    sort_keys[idx] = sort_keys[idx - 1] + sort_param->key_length;
-  }
-  if (error > 0) goto err;
-  if (sort_param->buffpek.elements) {
-    if (sort_param->write_keys(sort_param, sort_keys, idx,
-                               (BUFFPEK *)alloc_dynamic(&sort_param->buffpek),
-                               &sort_param->tempfile))
-      goto err;
-    sort_param->keys = (sort_param->buffpek.elements - 1) * (keys - 1) + idx;
-  } else
-    sort_param->keys = idx;
-
-  sort_param->sort_keys_length = keys;
-  goto ok;
-
-err:
-  DBUG_PRINT("error", ("got some error"));
-  sort_param->sort_info->got_error = 1; /* no need to protect with a mutex */
-  my_free(sort_keys);
-  sort_param->sort_keys = nullptr;
-  delete_dynamic(&sort_param->buffpek);
-  close_cached_file(&sort_param->tempfile);
-  close_cached_file(&sort_param->tempfile_for_exceptions);
-
-ok:
-  free_root(&sort_param->wordroot, MYF(0));
-  /*
-    Detach from the share if the writer is involved. Avoid others to
-    be blocked. This includes a flush of the write buffer. This will
-    also indicate EOF to the readers.
-    That means that a writer always gets here first and readers -
-    only when they see EOF. But if a reader finishes prematurely
-    because of an error it may reach this earlier - don't allow it
-    to detach the writer thread.
-  */
-  if (sort_param->master && sort_param->sort_info->info->rec_cache.share)
-    remove_io_thread(&sort_param->sort_info->info->rec_cache);
-
-  /* Readers detach from the share if any. Avoid others to be blocked. */
-  if (sort_param->read_cache.share) remove_io_thread(&sort_param->read_cache);
-
-  mysql_mutex_lock(&sort_param->sort_info->mutex);
-  if (!--sort_param->sort_info->threads_running)
-    mysql_cond_signal(&sort_param->sort_info->cond);
-  mysql_mutex_unlock(&sort_param->sort_info->mutex);
-  DBUG_PRINT("exit", ("======== ending thread ========"));
-  mysql_cond_destroy(&thread_keycache_var.suspend);
-  my_thread_end();
-  return nullptr;
-}
-
-int thr_write_keys(MI_SORT_PARAM *sort_param) {
-  SORT_INFO *sort_info = sort_param->sort_info;
-  MI_CHECK *param = sort_info->param;
-  size_t length = 0;
-  ulong keys;
-  ulong *rec_per_key_part = param->rec_per_key_part;
-  int got_error = sort_info->got_error;
-  uint i;
-  MI_INFO *info = sort_info->info;
-  MYISAM_SHARE *share = info->s;
-  MI_SORT_PARAM *sinfo;
-  uchar *mergebuf = nullptr;
-  DBUG_TRACE;
-
-  for (i = 0, sinfo = sort_param; i < sort_info->total_keys; i++, sinfo++) {
-    if (!sinfo->sort_keys) {
-      got_error = 1;
-      my_free(mi_get_rec_buff_ptr(info, sinfo->rec_buff));
-      continue;
-    }
-    if (!got_error) {
-      mi_set_key_active(share->state.key_map, sinfo->key);
-      if (!sinfo->buffpek.elements) {
-        if (param->testflag & T_VERBOSE) {
-          printf("Key %d  - Dumping %u keys\n", sinfo->key + 1, sinfo->keys);
-          fflush(stdout);
-        }
-        if (write_index(sinfo, sinfo->sort_keys, sinfo->keys) ||
-            flush_ft_buf(sinfo) || flush_pending_blocks(sinfo))
-          got_error = 1;
-      }
-    }
-    my_free(sinfo->sort_keys);
-    my_free(mi_get_rec_buff_ptr(info, sinfo->rec_buff));
-    sinfo->sort_keys = nullptr;
-  }
-
-  for (i = 0, sinfo = sort_param; i < sort_info->total_keys; i++,
-      delete_dynamic(&sinfo->buffpek), close_cached_file(&sinfo->tempfile),
-      close_cached_file(&sinfo->tempfile_for_exceptions),
-      rec_per_key_part += sinfo->keyinfo->keysegs, sinfo++) {
-    if (got_error) continue;
-    if (sinfo->keyinfo->flag & HA_VAR_LENGTH_KEY) {
-      sinfo->write_keys = write_keys_varlen;
-      sinfo->read_to_buffer = read_to_buffer_varlen;
-      sinfo->write_key = write_merge_key_varlen;
-    } else {
-      sinfo->write_keys = write_keys;
-      sinfo->read_to_buffer = read_to_buffer;
-      sinfo->write_key = write_merge_key;
-    }
-    if (sinfo->buffpek.elements) {
-      uint maxbuffer = sinfo->buffpek.elements - 1;
-      if (!mergebuf) {
-        length = (size_t)param->sort_buffer_length;
-        while (length >= MIN_SORT_BUFFER) {
-          if ((mergebuf =
-                   (uchar *)my_malloc(PSI_NOT_INSTRUMENTED, length, MYF(0))))
-            break;
-          length = length * 3 / 4;
-        }
-        if (!mergebuf) {
-          got_error = 1;
-          continue;
-        }
-      }
-      keys = length / sinfo->key_length;
-      if (maxbuffer >= MERGEBUFF2) {
-        if (param->testflag & T_VERBOSE)
-          printf("Key %d  - Merging %u keys\n", sinfo->key + 1, sinfo->keys);
-        if (merge_many_buff(sinfo, keys, (uchar **)mergebuf,
-                            dynamic_element(&sinfo->buffpek, 0, BUFFPEK *),
-                            (int *)&maxbuffer, &sinfo->tempfile)) {
-          got_error = 1;
-          continue;
-        }
-      }
-      if (flush_io_cache(&sinfo->tempfile) ||
-          reinit_io_cache(&sinfo->tempfile, READ_CACHE, 0L, false, false)) {
-        got_error = 1;
-        continue;
-      }
-      if (param->testflag & T_VERBOSE)
-        printf("Key %d  - Last merge and dumping keys\n", sinfo->key + 1);
-      if (merge_index(sinfo, keys, (uchar **)mergebuf,
-                      dynamic_element(&sinfo->buffpek, 0, BUFFPEK *), maxbuffer,
-                      &sinfo->tempfile) ||
-          flush_ft_buf(sinfo) || flush_pending_blocks(sinfo)) {
-        got_error = 1;
-        continue;
-      }
-    }
-    if (my_b_inited(&sinfo->tempfile_for_exceptions)) {
-      uint key_length;
-
-      if (param->testflag & T_VERBOSE)
-        printf("Key %d  - Dumping 'long' keys\n", sinfo->key + 1);
-
-      if (flush_io_cache(&sinfo->tempfile_for_exceptions) ||
-          reinit_io_cache(&sinfo->tempfile_for_exceptions, READ_CACHE, 0L,
-                          false, false)) {
-        got_error = 1;
-        continue;
-      }
-
-      while (!got_error &&
-             !my_b_read(&sinfo->tempfile_for_exceptions, (uchar *)&key_length,
-                        sizeof(key_length))) {
-        uchar ft_buf[HA_FT_MAXBYTELEN + HA_FT_WLEN + 10];
-        if (key_length > sizeof(ft_buf) ||
-            my_b_read(&sinfo->tempfile_for_exceptions, (uchar *)ft_buf,
-                      (uint)key_length) ||
-            _mi_ck_write(info, sinfo->key, (uchar *)ft_buf,
-                         key_length - info->s->rec_reflength))
-          got_error = 1;
-      }
-    }
-    if (!got_error && param->testflag & T_STATISTICS)
-      update_key_parts(sinfo->keyinfo, rec_per_key_part, sinfo->unique,
-                       param->stats_method == MI_STATS_METHOD_IGNORE_NULLS
-                           ? sinfo->notnull
-                           : nullptr,
-                       (ulonglong)info->state->records);
-  }
-  my_free(mergebuf);
-  return got_error;
-}
-
 /* Write all keys in memory to file for later merge */
 
 static int write_keys(MI_SORT_PARAM *info, uchar **sort_keys, uint count,
@@ -739,7 +444,7 @@ cleanup:
     buffpek		Where to read from
     sort_length		max length to read
   RESULT
-    > 0	Ammount of bytes read
+    > 0	Number of bytes read
     -1	Error
 */
 
@@ -803,7 +508,7 @@ static int write_merge_key_varlen(MI_SORT_PARAM *info, IO_CACHE *to_file,
   return (0);
 }
 
-static int write_merge_key(MI_SORT_PARAM *info MY_ATTRIBUTE((unused)),
+static int write_merge_key(MI_SORT_PARAM *info [[maybe_unused]],
                            IO_CACHE *to_file, uchar *key, uint sort_length,
                            uint count) {
   return my_b_write(to_file, key, (size_t)sort_length * count);
