@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,6 +25,7 @@
 #ifndef CREATE_EVNT_HPP
 #define CREATE_EVNT_HPP
 
+#include <cstring>
 #include <ndberror.h>
 #include "SignalData.hpp"
 #include <NodeBitmask.hpp>
@@ -43,7 +44,7 @@ class DropEvntReq {
   friend bool printDROP_EVNT_REQ(FILE*, const Uint32*, Uint32, Uint16);
 
 public:
-  STATIC_CONST( SignalLength = 2 );
+  static constexpr Uint32 SignalLength = 2;
   SECTION( EVENT_NAME_SECTION = 0 );
 
   union {             // user block reference
@@ -76,7 +77,7 @@ class DropEvntConf {
   friend bool printDROP_EVNT_CONF(FILE*, const Uint32*, Uint32, Uint16);
 
 public:
-  STATIC_CONST( SignalLength = 2 );
+  static constexpr Uint32 SignalLength = 2;
 
   union {             // user block reference
     Uint32 senderRef;
@@ -117,8 +118,8 @@ public:
     AllocationFailure = 747,
     TableNotFound = 4710
   };
-  STATIC_CONST( SignalLength = 7 );
-  STATIC_CONST( SignalLength2 = SignalLength+1 );
+  static constexpr Uint32 SignalLength = 7;
+  static constexpr Uint32 SignalLength2 = SignalLength+1;
 
   union {             // user block reference
     Uint32 senderRef;
@@ -198,9 +199,9 @@ struct CreateEvntReq {
     EF_NO_REPORT_DDL = 0x4 << 16,
     EF_ALL = 0xFFFF << 16
   };
-  STATIC_CONST( SignalLengthGet = 3 );
-  STATIC_CONST( SignalLengthCreate = 6+MAXNROFATTRIBUTESINWORDS_OLD );
-  STATIC_CONST( SignalLength = 8+MAXNROFATTRIBUTESINWORDS_OLD );
+  static constexpr Uint32 SignalLengthGet = 3;
+  static constexpr Uint32 SignalLengthCreate = 6+MAXNROFATTRIBUTESINWORDS_OLD;
+  static constexpr Uint32 SignalLength = 8+MAXNROFATTRIBUTESINWORDS_OLD;
 
   SECTION( EVENT_NAME_SECTION = 0 );
   SECTION( ATTRIBUTE_MASK = 1 );
@@ -270,7 +271,7 @@ struct CreateEvntReq {
     setAttrListBitmask(val.getSizeInWords(), val.rep.data);
   }
   void setAttrListBitmask(Uint32 sz, const Uint32 data[]){
-    bzero(m_attrListBitmask.data, sizeof(m_attrListBitmask.data));
+    std::memset(m_attrListBitmask.data, 0, sizeof(m_attrListBitmask.data));
     if (sz >= AttributeMask_OLD::Size)
     {
       AttributeMask_OLD::assign(m_attrListBitmask.data, data);
@@ -337,8 +338,9 @@ class CreateEvntConf {
   friend bool printCREATE_EVNT_CONF(FILE*, const Uint32*, Uint32, Uint16);
 
 public:
-  //  STATIC_CONST( InternalLength = 3 );
-  STATIC_CONST( SignalLength = 8+MAXNROFATTRIBUTESINWORDS_OLD );
+  //  static constexpr Uint32 InternalLength = 3;
+  static constexpr Uint32 SignalLength_v8_0_31 = 8+MAXNROFATTRIBUTESINWORDS_OLD;
+  static constexpr Uint32 SignalLength = 13;
 
   union {
     Uint32 m_userRef;             // user block reference
@@ -355,6 +357,7 @@ public:
   Uint32 m_eventType;
   Uint32 m_eventId;
   Uint32 m_eventKey;
+  Uint32 m_reportFlags;          // using CreateEvntReq::EventFlags
 
   Uint32 getUserRef() const {
     return m_userRef;
@@ -410,6 +413,18 @@ public:
   void setEventKey(Uint32 val) {
     m_eventKey = val;
   }
+  void setReportFlags(Uint32 val) {
+    m_reportFlags = val;
+  }
+  Uint32 getReportAll() const {
+    return m_reportFlags & CreateEvntReq::EF_REPORT_ALL;
+  }
+  Uint32 getReportSubscribe() const {
+    return m_reportFlags & CreateEvntReq::EF_REPORT_SUBSCRIBE;
+  }
+  Uint32 getReportDDL() const {
+    return (m_reportFlags & CreateEvntReq::EF_NO_REPORT_DDL) == 0;
+  }
 };
 
 /**
@@ -419,8 +434,8 @@ struct CreateEvntRef {
   friend class SafeCounter;
   friend bool printCREATE_EVNT_REF(FILE*, const Uint32*, Uint32, Uint16);
 
-  STATIC_CONST( SignalLength = 11 );
-  STATIC_CONST( SignalLength2 = SignalLength + 1 );
+  static constexpr Uint32 SignalLength = 11;
+  static constexpr Uint32 SignalLength2 = SignalLength + 1;
   enum ErrorCode {
     NoError = 0,
     Undefined = 1,
