@@ -1,6 +1,5 @@
 /*
- Copyright (c) 2013, 2021, Oracle and/or its affiliates. All rights
- reserved.
+ Copyright (c) 2013, 2023, Oracle and/or its affiliates.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -20,8 +19,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- 02110-1301  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 'use strict';
@@ -86,7 +84,7 @@ var tableHandlers = {
 var implementation = function() {
 };
 
-implementation.prototype.getDefaultProperties = function() {
+implementation.prototype.getConnectionProperties = function() {
   return {
     mysql_host      : 'localhost',
     mysql_port      : 3306,
@@ -129,7 +127,7 @@ implementation.prototype.exec = function(statement, values, callback) {
 
 implementation.prototype.persist = function(parameters, callback) {
   // which object is it
-  var mapping = parameters.object.constructor.prototype.mynode.mapping;
+  var mapping = parameters.object.constructor.prototype.jones.mapping;
   var tableName = mapping.table;
   var object = parameters.object;
   JSCRUND.udebug.log_detail('jscrund_sql implementation.insert object:', object,
@@ -144,7 +142,7 @@ implementation.prototype.persist = function(parameters, callback) {
 
 implementation.prototype.find = function(parameters, callback) {
   // which object is it
-  var mapping = parameters.object.constructor.prototype.mynode.mapping;
+  var mapping = parameters.object.constructor.prototype.jones.mapping;
   var tableName = mapping.table;
   var object = parameters.object;
   JSCRUND.udebug.log_detail('jscrund_sql implementation.find key:', parameters.key, 'table', tableName);
@@ -158,7 +156,7 @@ implementation.prototype.find = function(parameters, callback) {
 
 implementation.prototype.remove = function(parameters, callback) {
   // which object is it
-  var mapping = parameters.object.constructor.prototype.mynode.mapping;
+  var mapping = parameters.object.constructor.prototype.jones.mapping;
   var tableName = mapping.table;
   var object = parameters.object;
   JSCRUND.udebug.log_detail('jscrund_sql implementation.remove key:', parameters.key, 'table', tableName);
