@@ -27,6 +27,7 @@
 
 #include <sstream>
 
+#include "nulls.h"                                    // NullS
 #include "sql/sql_class.h"                            // THD
 #include "storage/ndb/include/ndbapi/Ndb.hpp"         // Ndb
 #include "storage/ndb/plugin/ha_ndbcluster_binlog.h"  // ndbcluster_binlog_setup_table
@@ -912,8 +913,8 @@ bool Ndb_metadata_sync::sync_table(THD *thd, const std::string &schema_name,
                  table_name.c_str());
 
     // Invalidate the table in NdbApi
-    Ndb_table_guard::invalidate_table(ndb, schema_name.c_str(),
-                                      table_name.c_str());
+    Ndb_table_guard ndbtab_guard(ndb, schema_name.c_str(), table_name.c_str());
+    ndbtab_guard.invalidate();
     return true;
   }
 
