@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,11 +32,11 @@
 class DbaccProxy : public LocalProxy {
 public:
   DbaccProxy(Block_context& ctx);
-  virtual ~DbaccProxy();
+  ~DbaccProxy() override;
   BLOCK_DEFINES(DbaccProxy);
 
 protected:
-  virtual SimulatedBlock* newWorker(Uint32 instanceNo);
+  SimulatedBlock* newWorker(Uint32 instanceNo) override;
 };
 
 
