@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,7 +34,9 @@
 #include <DataBuffer.hpp>
 #include <Bitmask.hpp>
 #include <signaldata/DbspjErr.hpp>
+#include <stat_utils.hpp>
 #include "../dbtup/tuppage.hpp"
+#include "../dbtc/Dbtc.hpp"
 
 #define JAM_FILE_ID 481
 
@@ -48,10 +50,12 @@ struct QueryNodeParameters;
 class Dbspj: public SimulatedBlock {
 public:
   Dbspj(Block_context& ctx, Uint32 instanceNumber = 0);
-  virtual ~Dbspj();
+  ~Dbspj() override;
 
   struct Request;
   struct TreeNode;
+  struct ScanFragHandle;
+
 private:
   BLOCK_DEFINES(Dbspj);
 
@@ -101,11 +105,40 @@ private:
 
   void sendSTTORRY(Signal* signal);
 
-protected:
-  //virtual bool getParam(const char* name, Uint32* count);
+  /**
+   * Security layer:
+   *   Provide verification of 'i-pointers' used in the signaling protocol.
+   *   - 'insert' the GuardedPtr to allow it to be referred.
+   *   - 'remove' at end of lifecycle.
+   *   - 'get' will fetch the 'real' pointer to the object.
+   * Crash if ptrI is unknown to us.
+   */
+  void insertGuardedPtr(Ptr<Request>, Ptr<TreeNode>);
+  void removeGuardedPtr(Ptr<TreeNode>);
+  bool getGuardedPtr(Ptr<TreeNode>&, Uint32 ptrI);
+
+  void insertGuardedPtr(Ptr<Request>, Ptr<ScanFragHandle>);
+  void removeGuardedPtr(Ptr<ScanFragHandle>);
+  bool getGuardedPtr(Ptr<ScanFragHandle>&, Uint32 ptrI);
+
+  /**
+   * Calculate a reasonable good hashKey for an i-pointer.
+   * Lower 13 bits in i-pointer is the page offset, with those above
+   * the page no. As the same page_no is reused for multiple object,
+   * and the objects are of same size, there *will* be repeating patterns.
+   * Thus a good hash function is required, based on murmur3 hash.
+   */
+  static Uint32 hashPtrI(Uint32 ptrI)
+  {
+    Uint32 k = (ptrI >> 13) ^ ptrI;  // Fold page_no and pos
+    // Murmur scramble:
+    k *= 0xcc9e2d51;
+    k = (k << 15) | (k >> 17);
+    k *= 0x1b873593;
+    return k;
+  }
 
 public:
-  struct ScanFragHandle;
   typedef DataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Correlation_list;
   typedef LocalDataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Local_correlation_list;
   typedef DataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Dependency_map;
@@ -150,21 +183,10 @@ public:
   };
   typedef Ptr<TableRecord> TableRecordPtr;
 
-  enum Buffer_type {
-    BUFFER_VOID  = 0,
-    BUFFER_STACK = 1,
-    BUFFER_VAR   = 2
-  };
-
   struct RowRef
   {
     Uint32 m_page_id;
     Uint16 m_page_pos;
-    union
-    {
-      Uint16 unused;
-      enum Buffer_type m_alloc_type:16;
-    };
 
     void copyto_link(Uint32 * dst) const {
       dst[0] = m_page_id; dst[1] = m_page_pos;
@@ -201,14 +223,6 @@ public:
    */
   struct RowPtr
   {
-    enum RowType
-    {
-      RT_SECTION = 1,
-      RT_LINEAR = 2,
-      RT_END = 0
-    };
-
-    RowType m_type;
     Uint32  m_src_node_ptrI;
     Uint32  m_src_correlation;
 
@@ -220,32 +234,22 @@ public:
       Uint32 m_offset[1];
     };
 
-    struct Section
+    struct Row
     {
-      const Header * m_header;
-      SegmentedSectionPtrPOD m_dataPtr;
-    };
-
-    struct Linear
-    {
-      RowRef m_row_ref;
       const Header * m_header;
       const Uint32 * m_data;
     };
-    union
-    {
-      struct Section m_section;
-      struct Linear m_linear;
-    } m_row_data;
+
+    struct Row m_row_data;
   };
 
   struct RowBuffer;  // forward decl.
 
   /**
-   * Define overlayed 'base class' for SLFifoRowList and RowMap.
+   * Define overlaid 'base class' for SLFifoRowList and RowMap.
    * As we want these to be POD struct, we does not use 
    * inheritance, but have to take care that first part
-   * of these struct are correctly overlayed.
+   * of these struct are correctly overlaid.
    */
   struct RowCollectionBase
   {
@@ -255,7 +259,7 @@ public:
   struct SLFifoRowList //: public RowCollectionBase
   {
     /**
-     * BEWARE: Overlayed 'struct RowCollectionBase'
+     * BEWARE: Overlaid 'struct RowCollectionBase'
      */
     RowBuffer* m_rowBuffer;
 
@@ -278,7 +282,7 @@ public:
   struct RowMap //: public RowCollectionBase
   {
     /**
-     * BEWARE: Overlayed 'struct RowCollectionBase'
+     * BEWARE: Overlaid 'struct RowCollectionBase'
      */
     RowBuffer* m_rowBuffer;
 
@@ -305,7 +309,7 @@ public:
 
     bool isNull() const { return m_map_ref.isNull(); }
 
-    void assign (RowRef ref) {
+    void assign(RowRef ref) {
       m_map_ref = ref;
     }
 
@@ -336,15 +340,15 @@ public:
       return RowRef::map_is_null(ptr);
     }
 
-    STATIC_CONST( MAP_SIZE_PER_REF_16 = 3 );
+    static constexpr Uint32 MAP_SIZE_PER_REF_16 = 3;
   };
 
   /**
-   * Define overlayed 'base class' for SLFifoRowListIterator
+   * Define overlaid 'base class' for SLFifoRowListIterator
    * and RowMapIterator.
    * As we want these to be POD struct, we does not use 
    * inheritance, but have to take care that first part
-   * of these struct are correctly overlayed.
+   * of these struct are correctly overlaid.
    */
   struct RowIteratorBase
   {
@@ -358,7 +362,7 @@ public:
   struct SLFifoRowListIterator //: public RowIteratorBase
   {
     /**
-     * BEWARE: Overlayed 'struct RowIteratorBase'
+     * BEWARE: Overlaid 'struct RowIteratorBase'
      */
     RowRef m_ref;
     Uint32 * m_row_ptr;
@@ -371,7 +375,7 @@ public:
   struct RowMapIterator //: public RowIteratorBase
   {
     /**
-     * BEWARE: Overlayed 'struct RowIteratorBase'
+     * BEWARE: Overlaid 'struct RowIteratorBase'
      */
     RowRef m_ref;
     Uint32 * m_row_ptr;
@@ -454,7 +458,6 @@ public:
     Uint32 m_cnt;
     Uint32 m_scanPrio;
     Uint32 m_savepointId;
-    Uint32 m_batch_size_rows;
     Uint32 m_resultRef;  // API
     Uint32 m_resultData; // API
     Uint32 m_senderRef;  // TC (used for routing)
@@ -480,51 +483,31 @@ public:
     Uint32 nextList;
     Uint32 prevList;
     Uint32 m_data[GLOBAL_PAGE_SIZE_WORDS - 7];
-    STATIC_CONST( SIZE = GLOBAL_PAGE_SIZE_WORDS - 7 );
+    static constexpr Uint32 SIZE = GLOBAL_PAGE_SIZE_WORDS - 7;
   };
   typedef ArrayPool<RowPage> RowPage_pool;
-  typedef SLList<RowPage_pool> RowPage_list;
-  typedef LocalSLList<RowPage_pool> Local_RowPage_list;
-  typedef DLFifoList<RowPage_pool> RowPage_fifo;
-  typedef LocalDLFifoList<RowPage_pool> Local_RowPage_fifo;
-
-  typedef Tup_varsize_page Var_page;
+  // Use 'counted' list to keep track of GlobalSharedMemory size used.
+  typedef SLCList<RowPage_pool> RowPage_list;
+  typedef LocalSLCList<RowPage_pool> Local_RowPage_list;
+  typedef DLCFifoList<RowPage_pool> RowPage_fifo;
+  typedef LocalDLCFifoList<RowPage_pool> Local_RowPage_fifo;
 
   struct RowBuffer
   {
-    enum Buffer_type m_type;
-
-    RowBuffer() : m_type(BUFFER_VOID) {}
+    RowBuffer() {}
     RowPage_fifo::Head m_page_list;
 
-    void init(enum Buffer_type type)
+    void init()
     {
       new (&m_page_list) RowPage_fifo::Head();
-      m_type = type;
       reset();
     }
     void reset()
     {
-      if (m_type == BUFFER_STACK)
-        m_stack.m_pos = 0xFFFF;
-      else if (m_type == BUFFER_VAR)
-        m_var.m_free = 0;
+      m_stack_pos = 0xFFFF;
     }
 
-    struct Stack
-    {
-      Uint32 m_pos; // position on head-page
-    };
-
-    struct Var
-    {
-      Uint32 m_free; // Free on last page in list
-    };
-
-    union {
-      struct Stack m_stack;
-      struct Var m_var;
-    };
+    Uint32 m_stack_pos;  // Next free position in head-page
   };
 
   /**
@@ -563,9 +546,10 @@ public:
 
     /**
      * This function is called when a waited for signal arrives.
-     * Return 'true' if this completes the wait for this treeNode
+     * Sets Request::m_completed_tree_nodes if this completed the
+     * wait for this treeNode
      */
-    bool (Dbspj::*m_countSignal)(const Signal*,Ptr<Request>,Ptr<TreeNode>);
+    void (Dbspj::*m_countSignal)(Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 cnt);
 
     /**
      * This function is used when getting a LQHKEYREF
@@ -609,7 +593,7 @@ public:
      * This function is called on the *child* by the *parent* when
      *   child should release buffers related to parents current batch
      */
-    void (Dbspj::*m_parent_batch_cleanup)(Ptr<Request>,Ptr<TreeNode>);
+    void (Dbspj::*m_parent_batch_cleanup)(Ptr<Request>,Ptr<TreeNode>,bool);
 
     /**
      * This function is called when getting a SCAN_NEXTREQ
@@ -632,7 +616,7 @@ public:
      * This function is called on node-failure
      */
     Uint32 (Dbspj::*m_execNODE_FAILREP)(Signal*, Ptr<Request>, Ptr<TreeNode>,
-                                        NdbNodeBitmask);
+                                        const NdbNodeBitmask);
     /**
      * This function is called when request/node(s) is/are removed
      *  should only do local cleanup(s)
@@ -675,17 +659,21 @@ public:
       SFH_SCANNING     = 1, // in LQH
       SFH_WAIT_NEXTREQ = 2,
       SFH_COMPLETE     = 3,
-      SFH_WAIT_CLOSE   = 4
+      SFH_WAIT_CLOSE   = 4,
+      SFH_SCANNING_WAIT_CLOSE = 5
     };
 
     void init(Uint32 fid, bool readBackup)
     {
-      m_ref = 0;
       m_fragId = fid;
       m_state = SFH_NOT_STARTED;
-      m_rangePtrI = RNIL;
       m_readBackup = readBackup;
-      reset_ranges();
+      m_ref = 0;
+      m_rangeCnt = 0;
+      m_rangePtrI = RNIL;
+      m_paramPtrI = RNIL;
+      m_keysSent = 0;
+      m_totalRows = 0;
     }
 
     Uint32 m_magic;
@@ -694,17 +682,33 @@ public:
     Uint8 m_state;
     Uint8 m_readBackup;
     Uint32 m_ref;
+    Uint32 m_next_ref;
+    Uint32 m_rangeCnt;
+    Uint32 m_rangePtrI;  // Set of lower/upper bound keys.
+    Uint32 m_paramPtrI;  // Set of interpreter parameters
 
-    void reset_ranges() {
-      // m_rangePtrI is explicitly managed...in code
-      m_range_builder.m_range_cnt = m_range_builder.m_range_size = 0;
+    /**
+     * Number of range/keys sent to this fragment in last SCAN_FRAGREQ.
+     */
+    Uint32 m_keysSent;
+
+    /**
+     * Total number of rows received from this fragment
+     * for the active SCAN_FRAGREQ.
+     */
+    Uint32 m_totalRows;
+
+    // Below are requirements for the hash lists
+    bool equal(const ScanFragHandle &other) const {
+      return key == other.key;
     }
-    struct RangeBuilder
-    {
-      Uint32 m_range_size;
-      Uint16 m_range_cnt; // too set bounds info correctly
-    } m_range_builder;
-    Uint32 m_rangePtrI;
+    Uint32 hashValue() const {
+      return hashPtrI(key);
+    }
+
+    Uint32 key;  // Its own ptrI, used as hash key
+    Uint32 nextHash, prevHash;
+
     union {
       Uint32 nextList;
       Uint32 nextPool;
@@ -714,45 +718,35 @@ public:
   typedef RecordPool<ArenaPool<ScanFragHandle> > ScanFragHandle_pool;
   typedef SLFifoList<ScanFragHandle_pool> ScanFragHandle_list;
   typedef LocalSLFifoList<ScanFragHandle_pool> Local_ScanFragHandle_list;
+  typedef KeyTable<ScanFragHandle_pool> ScanFragHandle_hash;
 
   /**
    * This class computes mean and standard deviation incrementally for a series
-   * of samples.
+   * of samples. It is based on the NdbStatistics class which implement a
+   * 'moving average' where the weight of older samples decrease exponentially.
+   * The statistics may then adapt if different regions of the retrieved data set
+   * has different properties.
    */
-  class IncrementalStatistics
+  class IncrementalStatistics : public NdbStatistics
   {
   public:
-    /**
-     * We cannot have a (non-trivial) constructor, since this class is used in
-     * unions.
-     */
-    void init()
+    IncrementalStatistics() : NdbStatistics(5) {}
+
+    void sample(double observation)
     {
-      m_mean = m_sumSquare = 0.0;
-      m_noOfSamples = 0;
-    }
-
-    // Add another sample.
-    void sample(double observation);
-
-    double getMean() const {
-      return m_mean;
-    }
-
-    double getStdDev() const { 
-      return m_noOfSamples < 2 ? 0.0 : sqrt(m_sumSquare/(m_noOfSamples - 1));
+      update(observation);
     }
 
     bool isValid() const {
       return (m_noOfSamples > 0);
     }
 
-  private:
-    // Mean of all samples
-    double m_mean;
-    //Sum of square of differences from the current mean.
-    double m_sumSquare;
-    Uint32 m_noOfSamples;
+    /* Upper 95% percentile of estimated rows returned pr key range */
+    double getUpperEstimate() const {
+      return (m_noOfSamples >= 2)
+              ? getMean() + (2 * getStdDev())
+              : getMean() * 1.20;
+    }
   }; // IncrementalStatistics
 
   struct ScanFragData
@@ -771,62 +765,110 @@ public:
     Uint32 m_fragCount;
     // The number of fragments that we scan in parallel.
     Uint32 m_parallelism;
-    // True if we are still receiving the first batch for this operation.
-    bool   m_firstBatch;
+
     /**
-     * Mean and standard deviation for the optimal parallelism for earlier
-     * executions of this operation.
+     * The next correlation id known to be available if starting
+     * more fragment scans.
+     */
+    Uint32 m_corrIdStart;
+
+    /**
+     * Mean and standard deviation statistic for the 'record pr key'
+     * (fanout) returned for each key/bound sent *pr fragment*
+     */
+    IncrementalStatistics m_recsPrKeyStat;
+
+    /**
+     * Statistics for the 'BatchByteSize' consumed for each record.
+     */
+    IncrementalStatistics m_recSizeStat;
+
+    /**
+     * Total number of key/bounds in the process of being sent to
+     * the fragments. (Not yet sent)
+     */
+    Uint32 m_keysToSend;
+
+    /**
+     * Total number of key/bounds sent and where the frag scans
+     * has been reported as completed.
+     */
+    Uint32 m_completedKeys;
+
+    /**
+     * Number of rows returned from fragment scans where the scans
+     * has been reported as completed.
+     */
+    Uint32 m_completedRows;
+
+    /**
+     * Total number of rows/bytes reported by SCAN_FRAGCONF for the current
+     * execution of this operation.
      */
-    IncrementalStatistics m_parallelismStat;
-    // Total number of rows for the current execution of this operation.
     Uint32 m_totalRows;
-    // Total number of bytes for the current execution of this operation.
     Uint32 m_totalBytes;
 
+    /**
+     * Non-pruned firstMatch may save their original range and param's
+     * before removeMatchedKeys()
+     */
+    Uint32 m_rangeCntSave;
+    Uint32 m_rangePtrISave;  // Set of lower/upper bound keys.
+    Uint32 m_paramPtrISave;  // Set of interpreter parameters
+
     ScanFragHandle_list::HeadPOD m_fragments; // ScanFrag states
     union
     {
       PatternStore::HeadPOD m_prunePattern;
       Uint32 m_constPrunePtrI;
     };
-    /**
-     * Max number of rows seen in a batch. Used for calculating the number of
-     * rows per fragment in the next next batch when using adaptive batch size.
-     */
-    Uint32 m_largestBatchRows;
-    /**
-     * Max number of bytes seen in a batch. Used for calculating the number of
-     * rows per fragment in the next next batch when using adaptive batch size.
-     */
-    Uint32 m_largestBatchBytes;
     Uint32 m_scanFragReq[ScanFragReq::SignalLength + 2];
+
+    ScanFragData()
+    : m_frags_complete(0),
+      m_frags_outstanding(0),
+      m_frags_not_started(0),
+      m_rows_received(0),
+      m_rows_expecting(0),
+      m_batch_chunks(0),
+      m_scanCookie(0),
+      m_fragCount(0),
+      m_parallelism(0),
+      m_corrIdStart(0),
+      m_recsPrKeyStat(),
+      m_recSizeStat(),
+      m_keysToSend(0),
+      m_completedKeys(0),
+      m_completedRows(0),
+      m_totalRows(0),
+      m_totalBytes(0),
+      m_rangeCntSave(0),
+      m_rangePtrISave(RNIL),
+      m_paramPtrISave(RNIL),
+      m_fragments()
+    {
+      m_fragments.init();
+    }
   };
 
   struct DeferredParentOps
   {
     /**
      * m_correlations contains a list of Correlation Values (Uint32)
-     * which identifies parent rows which has been deferred. 
-     * m_pos are index into this array, identifying the next parent row
+     * which identifies parent rows which has been deferred.
+     * m_it iterates this list, identifying the next parent row
      * for which to resume operation.
      */
     Correlation_list::Head m_correlations;
-    Uint16 m_pos; // Next row operation to resume 
-
-    DeferredParentOps() : m_correlations(), m_pos(0) {}
+    Correlation_list::ConstDataBufferIterator m_it;
 
+    DeferredParentOps() : m_correlations() {
+      m_it.setNull();
+    }
     void init()  {
       m_correlations.init();
-      m_pos = 0;
+      m_it.setNull();
     }
-    bool isEmpty() const {
-      return (m_pos == m_correlations.getSize());
-    }
-  };
-
-  struct TreeNode_cursor_ptr
-  {
-    Uint32 nextList;
   };
 
   /**
@@ -834,15 +876,15 @@ public:
    *   (This is an instantiated version of QueryNode in
    *    include/kernel/signal/QueryTree.hpp)
    */
-  struct TreeNode : TreeNode_cursor_ptr
+  struct TreeNode
   {
-    STATIC_CONST ( MAGIC = ~RT_SPJ_TREENODE );
+    static constexpr Uint32 MAGIC = ~RT_SPJ_TREENODE;
 
     TreeNode()
     : m_magic(MAGIC), m_state(TN_END),
       m_parentPtrI(RNIL), m_requestPtrI(RNIL),
       m_ancestors(), m_coverage(), m_predecessors(), m_dependencies(),
-      m_resumeEvents(0), m_resumePtrI(RNIL),
+      m_resumeEvents(0),
       m_scanAncestorPtrI(RNIL)
     {
     }
@@ -852,9 +894,9 @@ public:
       m_info(0), m_bits(T_LEAF), m_state(TN_BUILDING),
       m_parentPtrI(RNIL), m_requestPtrI(request),
       m_ancestors(), m_coverage(), m_predecessors(), m_dependencies(),
-      m_resumeEvents(0), m_resumePtrI(RNIL),
+      m_resumeEvents(0),
       m_scanAncestorPtrI(RNIL),
-      nextList(RNIL), prevList(RNIL)
+      nextList(RNIL), prevList(RNIL), nextCursor(RNIL)
     {
 //    m_send.m_ref = 0;
       m_send.m_correlation = 0;
@@ -942,7 +984,7 @@ public:
        */
       T_UNIQUE_INDEX_LOOKUP = 0x40,
 
-      /*
+      /**
        * Should this node buffers its rows or 'm_matched' bitmask?
        *  We could request the buffer to store either the 'ROW',
        *  as received by TRANSID_AI, and/or a 'MATCH' bitMask.
@@ -1012,21 +1054,44 @@ public:
        */
       T_EXPECT_TRANSID_AI = 0x80000,
 
+      /**
+       * Results from this TreeNode need to be produced in sorted
+       * order, as defined by the index being used.
+       * (Also require that T_SCAN_PARALLEL is set)
+       */
+      T_SORTED_ORDER = 0x100000,
+
+      /**
+       * Allow FirstMatch elimination when multiple rows matching the
+       * same key or range
+       */
+      T_FIRST_MATCH = 0x200000,
+
+      /**
+       * Need congestion control of this TreeNode. Possible suspend it
+       * and later resume the operations on it.
+       */
+      T_CHK_CONGESTION = 0x400000,
+
+      /**
+       * Reduce number of keys/ranges to be requested in remaining
+       * SCAN_FRAGREQ. Is a part of firstMatch optimization, which in
+       * some cases allows us to conclude the 'firstMatch' after the
+       * first matching row was found.
+       */
+      T_REDUCE_KEYS = 0x800000,
+
       // End marker...
       T_END = 0
     };
 
     /**
-     * Describe whether a LQHKEY-REF and/or CONF whould trigger a 
-     * exec resume of another TreeNode having T_EXEC_SEQUENTIAL.
-     * (Used as a bitmask)
+     * Describe whether a node operation should wait for operations
+     * it depends on to complete, and the resume when all result
+     * rows has been sent. (Used as a bitmask)
      */
     enum TreeNodeResumeEvents
     {
-      TN_ENQUEUE_OP   = 0x01,   // Enqueue and wait for RESUME_REF / _CONF
-      TN_RESUME_REF   = 0x02,
-      TN_RESUME_CONF  = 0x04,
-
       TN_EXEC_WAIT    = 0x08,
       TN_RESUME_NODE  = 0x10
     };
@@ -1069,7 +1134,7 @@ public:
     Dependency_map::Head m_next_nodes;
 
     /**
-     * We provide some TreeNodeBitMap's. Usefull to check how
+     * We provide some TreeNodeBitMap's. Useful to check how
      * a specific node relates to other TreeNodes:
      *
      * - 'ancestors' are the set of TreeNodes reachable through
@@ -1097,26 +1162,29 @@ public:
     PatternStore::Head m_keyPattern;
     PatternStore::Head m_attrParamPattern;
 
+    // Memory Arena with lifetime limited to current result batch / node
+    ArenaHead m_batchArena;
+
+    // RowBuffers for this TreeNode only
+    RowBuffer m_rowBuffer;
+
     /**
      * Rows buffered by this node
      */
     RowCollection m_rows;
 
     /**
-     * T_EXEC_SEQUENTIAL cause execution of child operations to
+     * T_CHK_CONGESTION may cause execution of child operations to
      * be deferred.  These operations are queued in the 'struct DeferredParentOps'
-     * Currently only Lookup operation might be deferred.
-     * Could later be extended to also cover index scans.
+     * The congestion check will always happen on a Scan TreeNode having
+     * some Lookup children, which are the operations which might be deferred.
      */
     DeferredParentOps m_deferred;
 
     /**
      * Set of TreeNodeResumeEvents, possibly or'ed.
-     * Specify whether a REF or CONF will cause a resume
-     * of the TreeNode referred by 'm_resumePtrI'.
      */
     Uint32 m_resumeEvents;
-    Uint32 m_resumePtrI;
 
     /**
      * The Scan-TreeNode being the head of the inner-joined-branch
@@ -1141,23 +1209,35 @@ public:
       Uint32 m_attrInfoPtrI;     // attrInfoSection
     } m_send;
 
+
+    // Below are requirements for the hash lists
+    bool equal(const TreeNode &other) const {
+      return key == other.key;
+    }
+    Uint32 hashValue() const {
+      return hashPtrI(key);
+    }
+
+    Uint32 key;  // Its own ptrI, used as hash key
+    Uint32 nextHash, prevHash;
+
     union {
       Uint32 nextList;
       Uint32 nextPool;
     };
     Uint32 prevList;
+    Uint32 nextCursor;
   };  //struct TreeNode
 
   static const Ptr<TreeNode> NullTreeNodePtr;
 
   typedef RecordPool<ArenaPool<TreeNode> > TreeNode_pool;
+  typedef KeyTable<TreeNode_pool> TreeNode_hash;
   typedef DLFifoList<TreeNode_pool> TreeNode_list;
   typedef LocalDLFifoList<TreeNode_pool> Local_TreeNode_list;
 
-  typedef SLList<TreeNode_pool, TreeNode_cursor_ptr>
-  TreeNodeCursor_list;
-  typedef LocalSLList<TreeNode_pool, TreeNode_cursor_ptr>
-  Local_TreeNodeCursor_list;
+  typedef SLList<TreeNode_pool, IA_Cursor> TreeNodeCursor_list;
+  typedef LocalSLList<TreeNode_pool, IA_Cursor> Local_TreeNodeCursor_list;
 
   /**
    * A request (i.e a query + parameters)
@@ -1167,9 +1247,7 @@ public:
     enum RequestBits
     {
       RT_SCAN                = 0x1  // unbounded result set, scan interface
-      ,RT_BUFFERS            = 0x2  // Do any of nodes use row/match-buffering
       ,RT_MULTI_SCAN         = 0x4  // Is there several scans in request
-//    ,RT_VAR_ALLOC          = 0x8  // DEPRECATED
       ,RT_NEED_PREPARE       = 0x10 // Does any node need m_prepare hook
       ,RT_NEED_COMPLETE      = 0x20 // Does any node need m_complete hook
       ,RT_REPEAT_SCAN_RESULT = 0x40 // Repeat bushy scan result when required
@@ -1206,14 +1284,15 @@ public:
     TreeNodeCursor_list::Head m_cursor_nodes;
     Uint32 m_cnt_active;       // No of "running" nodes
     TreeNodeBitMask
-           m_active_nodes;     // Nodes which will return more data in NEXTREQ
+      m_active_tree_nodes;     // Nodes which will return more data in NEXTREQ
     TreeNodeBitMask
-           m_completed_nodes;  // Nodes wo/ any 'outstanding' signals
+      m_completed_tree_nodes;  // Nodes wo/ any 'outstanding' signals
+    TreeNodeBitMask
+      m_suspended_tree_nodes;  // Nodes suspended by SPJ congestion control
     Uint32 m_rows;             // Rows accumulated in current batch
     Uint32 m_outstanding;      // Outstanding signals, when 0, batch is done
     Uint16 m_lookup_node_data[MAX_NDB_NODES];
     ArenaHead m_arena;
-    RowBuffer m_rowBuffer;
 
 #ifdef SPJ_TRACE_TIME
     Uint32 m_cnt_batches;
@@ -1253,7 +1332,7 @@ private:
   enum CounterId
   {
     /**
-     * This is the number of incomming LQHKEYREQ messages (i.e queries with a
+     * This is the number of incoming LQHKEYREQ messages (i.e queries with a
      * lookup as root).
      */
     CI_READS_RECEIVED = 0,
@@ -1273,12 +1352,12 @@ private:
     /**
      * No of lookup operations which did not return a row (LQHKEYREF).
      * (Most likely due to non matching key, or predicate
-     * filter which evalueted  to 'false').
+     * filter which evaluated  to 'false').
      */
     CI_READS_NOT_FOUND = 3,
 
     /**
-     * This is the number of incomming queries where the root operation is a
+     * This is the number of incoming queries where the root operation is a
      * fragment scan and this is a "direct scan" that does not go via an index.
      */
     CI_TABLE_SCANS_RECEIVED = 4,
@@ -1290,7 +1369,7 @@ private:
     CI_LOCAL_TABLE_SCANS_SENT = 5,
 
     /**
-     * This is the number of incomming queries where the root operation is a
+     * This is the number of incoming queries where the root operation is a
      * fragment scan which scans the fragment via an ordered index..
      */
     CI_RANGE_SCANS_RECEIVED = 6,
@@ -1373,7 +1452,9 @@ private:
   Request_hash m_lookup_request_hash;
   ArenaPool<DataBufferSegment<14> > m_dependency_map_pool;
   TreeNode_pool m_treenode_pool;
+  TreeNode_hash m_treenode_hash;
   ScanFragHandle_pool m_scanfraghandle_pool;
+  ScanFragHandle_hash m_scanfraghandle_hash;
 
   TableRecord *m_tableRecord;
   UintR c_tabrecFilesize;
@@ -1418,15 +1499,17 @@ private:
   Uint32 planSequentialExec(Ptr<Request>  requestPtr,
                             const Ptr<TreeNode> branchPtr,
                             Ptr<TreeNode> prevExecPtr,
-                            const Ptr<TreeNode> outerBranchPtr);
+                            TreeNodeBitMask outerJoins = TreeNodeBitMask());
 
   Uint32 appendTreeNode(Ptr<Request>  requestPtr,
                         Ptr<TreeNode> treeNodePtr,
-                        Ptr<TreeNode> prevExecPtr,
-                        const Ptr<TreeNode> outerBranchPtr);
+                        Ptr<TreeNode> prevExecPtr);
 
   void dumpExecPlan(Ptr<Request>, Ptr<TreeNode> node);
 
+  /**
+   * Prepare and execute
+   */
   void prepare(Signal*, Ptr<Request>);
   void checkPrepareComplete(Signal*, Ptr<Request>);
   void checkBatchComplete(Signal*, Ptr<Request>);
@@ -1434,41 +1517,38 @@ private:
   void prepareNextBatch(Signal*, Ptr<Request>);
   void sendConf(Signal*, Ptr<Request>, bool is_complete);
   void complete(Signal*, Ptr<Request>);
-  void cleanup(Ptr<Request>);
-  void cleanupBatch(Ptr<Request>);
+  void cleanup(Ptr<Request>, bool in_hash);
+  void cleanupBatch(Ptr<Request>, bool done);
   void abort(Signal*, Ptr<Request>, Uint32 errCode);
   Uint32 nodeFail(Signal*, Ptr<Request>, NdbNodeBitmask mask);
 
   Uint32 createNode(Build_context&, Ptr<Request>, Ptr<TreeNode> &);
   void handleTreeNodeComplete(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void reportAncestorsComplete(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void releaseScanBuffers(Ptr<Request> requestPtr);
-  void releaseRequestBuffers(Ptr<Request> requestPtr);
-  void releaseNodeRows(Ptr<Request> requestPtr, Ptr<TreeNode>);
   void registerActiveCursor(Ptr<Request>, Ptr<TreeNode>);
   void nodeFail_checkRequests(Signal*);
   void cleanup_common(Ptr<Request>, Ptr<TreeNode>);
 
+  void removeMatchedKeys(Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
+
   /**
    * Row buffering
    */
   Uint32 storeRow(Ptr<TreeNode> treeNodePtr, const RowPtr &row);
-  void releaseRow(Ptr<TreeNode> treeNodePtr, RowRef ref);
   Uint32* stackAlloc(RowBuffer& dst, RowRef&, Uint32 len);
-  Uint32* varAlloc(RowBuffer& dst, RowRef&, Uint32 len);
-  Uint32* rowAlloc(RowBuffer& dst, RowRef&, Uint32 len);
 
   void add_to_list(SLFifoRowList & list, RowRef);
   Uint32 add_to_map(RowMap& map, Uint32, RowRef);
 
   void setupRowPtr(Ptr<TreeNode> treeNodePtr,
-                   RowPtr& dst, RowRef, const Uint32 * src);
+                   RowPtr &dst, const Uint32 *src);
   Uint32 * get_row_ptr(RowRef pos);
 
   void getBufferedRow(const Ptr<TreeNode>, Uint32 rowId,
                       RowPtr *row);
 
   void resumeBufferedNode(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void resumeCongestedNode(Signal*, Ptr<Request>, Ptr<TreeNode>);
 
   /**
    * SLFifoRowListIterator
@@ -1492,43 +1572,27 @@ private:
   /**
    * Misc
    */
-  Uint32 buildRowHeader(RowPtr::Header *, SegmentedSectionPtr);
-  Uint32 buildRowHeader(RowPtr::Header *, const Uint32 *& src, Uint32 len);
-  void getCorrelationData(const RowPtr::Section & row, Uint32 col,
-                          Uint32& correlationNumber);
-  void getCorrelationData(const RowPtr::Linear & row, Uint32 col,
+  Uint32 buildRowHeader(RowPtr::Header *, LinearSectionPtr);
+  Uint32 buildRowHeader(RowPtr::Header *, const Uint32 *& src, Uint32 cnt);
+  void getCorrelationData(const RowPtr::Row &row, Uint32 col,
                           Uint32& correlationNumber);
   Uint32 appendToPattern(Local_pattern_store &, DABuffer & tree, Uint32);
-  Uint32 appendParamToPattern(Local_pattern_store&,const RowPtr::Linear&,
+  Uint32 appendParamToPattern(Local_pattern_store&,const RowPtr::Row&,
                               Uint32);
-  Uint32 appendParamHeadToPattern(Local_pattern_store&,const RowPtr::Linear&,
+  Uint32 appendParamHeadToPattern(Local_pattern_store&,const RowPtr::Row&,
                                   Uint32);
 
-  Uint32 appendTreeToSection(Uint32 & ptrI, SectionReader &, Uint32);
-  Uint32 appendColToSection(Uint32 & ptrI, const RowPtr::Linear&, Uint32 col, bool& hasNull);
-  Uint32 appendColToSection(Uint32 & ptrI, const RowPtr::Section&, Uint32 col, bool& hasNull);
-  Uint32 appendPkColToSection(Uint32 & ptrI, const RowPtr::Section&,Uint32 col);
-  Uint32 appendPkColToSection(Uint32 & ptrI, const RowPtr::Linear&, Uint32 col);
-  Uint32 appendAttrinfoToSection(Uint32 &, const RowPtr::Linear&, Uint32 col, bool& hasNull);
-  Uint32 appendAttrinfoToSection(Uint32 &, const RowPtr::Section&, Uint32 col, bool& hasNull);
+  Uint32 appendReaderToSection(Uint32 & ptrI, SectionReader&, Uint32);
+  Uint32 appendColToSection(Uint32 & ptrI, const RowPtr::Row&, Uint32 col, bool& hasNull);
+  Uint32 appendPkColToSection(Uint32 & ptrI, const RowPtr::Row&, Uint32 col);
+  Uint32 appendAttrinfoToSection(Uint32 &, const RowPtr::Row&, Uint32 col, bool& hasNull);
   Uint32 appendDataToSection(Uint32 & ptrI, Local_pattern_store&,
 			     Local_pattern_store::ConstDataBufferIterator&,
 			     Uint32 len, bool& hasNull);
   Uint32 appendFromParent(Uint32 & ptrI, Local_pattern_store&,
                           Local_pattern_store::ConstDataBufferIterator&,
                           Uint32 level, const RowPtr&, bool& hasNull);
-  Uint32 expand(Uint32 & ptrI, Local_pattern_store& p, const RowPtr& r, bool& hasNull){
-    switch(r.m_type){
-    case RowPtr::RT_SECTION:
-      return expandS(ptrI, p, r, hasNull);
-    case RowPtr::RT_LINEAR:
-      return expandL(ptrI, p, r, hasNull);
-    default:
-      return DbspjErr::InternalError;
-    }
-  }
-  Uint32 expandS(Uint32 & ptrI, Local_pattern_store&, const RowPtr&, bool& hasNull);
-  Uint32 expandL(Uint32 & ptrI, Local_pattern_store&, const RowPtr&, bool& hasNull);
+  Uint32 expand(Uint32 & ptrI, Local_pattern_store& p, const RowPtr& r, bool& hasNull);
   Uint32 expand(Uint32 & ptrI, DABuffer& pattern, Uint32 len,
                 DABuffer & param, Uint32 cnt, bool& hasNull);
   Uint32 expand(Local_pattern_store& dst, Ptr<TreeNode> treeNodePtr,
@@ -1543,8 +1607,7 @@ private:
   Uint32 checkTableError(Ptr<TreeNode> treeNodePtr) const;
   Uint32 getNodes(Signal*, BuildKeyReq&, Uint32 tableId);
 
-  void common_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
-			     const RowPtr&);
+  void startNextNodes(Signal*, Ptr<Request>, Ptr<TreeNode>, const RowPtr&);
 
   void dumpScanFragHandle(const Ptr<ScanFragHandle> fragPtr) const;
   void dumpNodeCommon(const Ptr<TreeNode>) const;
@@ -1558,14 +1621,12 @@ private:
   Uint32 lookup_build(Build_context&,Ptr<Request>,
 		      const QueryNode*, const QueryNodeParameters*);
   void lookup_start(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void lookup_resume(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_send(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  bool lookup_countSignal(const Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void lookup_countSignal(Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 cnt);
   void lookup_execLQHKEYREF(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_execLQHKEYCONF(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_stop_branch(Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 err);
   void lookup_parent_row(Signal*, Ptr<Request>, Ptr<TreeNode>, const RowPtr &);
-  void lookup_row(Signal*, Ptr<Request>, Ptr<TreeNode>, const RowPtr &);
   void lookup_abort(Signal*, Ptr<Request>, Ptr<TreeNode>);
   Uint32 lookup_execNODE_FAILREP(Signal*signal, Ptr<Request>, Ptr<TreeNode>,
                                NdbNodeBitmask);
@@ -1575,7 +1636,7 @@ private:
   void lookup_cleanup(Ptr<Request>, Ptr<TreeNode>);
 
   Uint32 handle_special_hash(Uint32 tableId, Uint32 dstHash[4],
-                             const Uint64* src,
+                             const Uint32* src,
                              Uint32 srcLen,       // Len in #32bit words
                              const struct KeyDescriptor* desc);
 
@@ -1597,21 +1658,27 @@ private:
                        DABuffer param, Uint32 paramBits);
   void scanFrag_start(Signal*, Ptr<Request>,Ptr<TreeNode>);
   void scanFrag_prepare(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  bool scanFrag_countSignal(const Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void scanFrag_countSignal(Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 cnt);
   void scanFrag_execSCAN_FRAGREF(Signal*, Ptr<Request>, Ptr<TreeNode>,
                                  Ptr<ScanFragHandle>);
   void scanFrag_execSCAN_FRAGCONF(Signal*, Ptr<Request>, Ptr<TreeNode>,
                                   Ptr<ScanFragHandle>);
   void scanFrag_parent_row(Signal*,Ptr<Request>,Ptr<TreeNode>, const RowPtr&);
-  void scanFrag_fixupBound(Ptr<ScanFragHandle> fragPtr, Uint32 ptrI, Uint32);
+  void scanFrag_fixupBound(Uint32 ptrI, Uint32);
   void scanFrag_send(Signal*, Ptr<Request>, Ptr<TreeNode>);
   Uint32 scanFrag_send(Signal* signal,
                        Ptr<Request> requestPtr,
                        Ptr<TreeNode> treeNodePtr,
                        Uint32 noOfFrags,
                        Uint32 bs_bytes,
-                       Uint32 bs_rows,
-                       Uint32& batchRange);
+                       Uint32 bs_rows);
+  Uint32 scanFrag_send_NEXTREQ(
+                       Signal* signal,
+                       Ptr<Request> requestPtr,
+                       Ptr<TreeNode> treeNodePtr,
+                       Uint32 noOfFrags,
+                       Uint32 bs_bytes,
+                       Uint32 bs_rows);
   void scanFrag_batchComplete(Signal* signal);
   Uint32 scanFrag_findFrag(Local_ScanFragHandle_list &, Ptr<ScanFragHandle>&,
                            Uint32 fragId);
@@ -1622,7 +1689,7 @@ private:
   void scanFrag_abort(Signal*, Ptr<Request>, Ptr<TreeNode>);
   Uint32 scanFrag_execNODE_FAILREP(Signal*signal, Ptr<Request>, Ptr<TreeNode>,
                                    NdbNodeBitmask);
-  void scanFrag_parent_batch_cleanup(Ptr<Request>, Ptr<TreeNode>);
+  void scanFrag_parent_batch_cleanup(Ptr<Request>, Ptr<TreeNode>, bool);
   void scanFrag_cleanup(Ptr<Request>, Ptr<TreeNode>);
 
   void scanFrag_release_rangekeys(Ptr<Request>, Ptr<TreeNode>);
@@ -1637,24 +1704,45 @@ private:
   void scanFrag_dumpNode(const Ptr<Request> requestPtr,
                          const Ptr<TreeNode> treeNodePtr);
 
+  Uint32 scanFrag_getBatchSize(Ptr<TreeNode> treeNodePtr,
+                               Uint32 &availableBatchBytes,
+                               Uint32 &availableBatchRows);
+
+  Uint32 scanFrag_parallelism(Ptr<Request> requestPtr,
+                              Ptr<TreeNode> treeNodePtr,
+                              Uint32 batchRows);
+
+  double estmMaxKeys(Ptr<Request> requestPtr,
+                     Ptr<TreeNode> treeNodePtr,
+                     double fanout = 1.0);
+
   Uint32 check_own_location_domain(const Uint32 *nodes, Uint32 node_count);
+  void send_close_scan(Signal*, Ptr<ScanFragHandle>, Ptr<Request>);
   /**
    * Page manager
    */
   bool allocPage(Ptr<RowPage> &);
-  void releasePage(Ptr<RowPage>);
-  void releasePages(Uint32 first, Ptr<RowPage> last);
+  void releasePages(RowBuffer &rowBuffer);
   void releaseGlobal(Signal*);
   RowPage_list::Head m_free_page_list;
   RowPage_pool m_page_pool;
 
-  /* Random fault injection */
+  Uint32 m_allocedPages;
+  Uint32 m_maxUsedPages;
+  Uint32 getUsedPages() const {
+    ndbassert(m_allocedPages >= m_free_page_list.getCount());
+    return m_allocedPages - m_free_page_list.getCount();
+  }
+  NdbStatistics m_usedPagesStat;
 
 #ifdef ERROR_INSERT
+  /* Random fault injection */
   bool appendToSection(Uint32& firstSegmentIVal,
                        const Uint32* src, Uint32 len);
 #endif
 
+  Dbtc *c_tc;
+
   Uint32 m_location_domain_id[MAX_NDB_NODES];
   Uint32 m_load_balancer_location;
   /**
@@ -1664,7 +1752,6 @@ private:
   Uint32 m_buffer1[16*1024]; // 64k
 };
 
-
 #undef JAM_FILE_ID
 
 #endif
