@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -79,12 +79,12 @@
 //
 // T& get();
 // Description:
-//   get's an element from the Pool.
+//   gets an element from the Pool.
 // Parameters:
 //   _
 // Return value:
 //   T& the element extracted from the Pool. (element must be cleared to
-//   mimick newly created element)
+//   mimic newly created element)
 // Errors:
 //      -
 // Asserts:
@@ -148,7 +148,7 @@
 //
 // Pool<T>& operator=(const Pool<T>& cp);
 // Description:
-//   Prohibit use of assignement operator.
+//   Prohibit use of assignment operator.
 // Parameters:
 //   cp
 // Return value:
@@ -202,7 +202,8 @@ public:
     theIncSize(anIncSize),
     theTop(0),
     theCurrentSize(0),   
-    theList(0)
+    theList(0),
+    theInUseList(nullptr)
   {
     allocate(anInitSize);
   }
@@ -219,12 +220,12 @@ public:
   void put(T* aT);
 
   // size() : Return number free items in pool
-  unsigned size(){ return theTop; };
+  unsigned size(){ return theTop; }
 
   // inuse() : Return number items taken from pool
-  unsigned inuse() const { return theCurrentSize - theTop; };
+  unsigned inuse() const { return theCurrentSize - theTop; }
 
-  const T* peekInuseItem(unsigned idx) const;
+  const T* getNextInUseItem(const T*) const;
   
 protected:
   void allocate(int aSize)
@@ -252,6 +253,7 @@ private:
   int theCurrentSize;
   
   T** theList;
+  T*  theInUseList;
 };
 
 //******************************************************************************
@@ -264,6 +266,18 @@ template <class T> inline T* Pool<T>::get()
    }
    --theTop;
    tmp = theList[theTop];
+
+   /* Maintain inuse list */
+   /* Add to head */
+   if (theInUseList)
+   {
+     assert(theInUseList->listPrev == nullptr);
+     theInUseList->listPrev = tmp;
+   }
+   tmp->listNext = theInUseList;
+   tmp->listPrev = nullptr;
+   theInUseList = tmp;
+
    tmp->atGet();
    return tmp;
 }
@@ -274,13 +288,33 @@ template <class T> inline void Pool<T>::put(T* aT)
 {
    theList[theTop]= aT;
    ++theTop;
+
+   /* Remove item from InUse list */
+   T* next = aT->listNext;
+   T* prev = aT->listPrev;
+
+   if (next)
+   {
+     next->listPrev = prev;
+   }
+   if (prev)
+   {
+     prev->listNext = next;
+   }
+   else
+   {
+     theInUseList = next;
+   }
+   aT->listNext = aT->listPrev = nullptr;
 }
 
-template <class T> const T* Pool<T>::peekInuseItem(unsigned idx) const
+template <class T> const T* Pool<T>::getNextInUseItem(const T* item) const
 {
-  assert(idx <= inuse());
-  // theTop is index of the first item in use
-  return theList[theTop + idx];
+  if (item)
+  {
+    return item->listNext;
+  }
+  return theInUseList;
 }
 
 
