@@ -1,5 +1,5 @@
 /*
- Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -278,7 +278,7 @@ private:
  * The reason for the difficulties of this mapping stems from an asymmetry:
  * The natural 1-1 mapping of Java 'MyClass[]' is C 'MyClass**' while C
  * 'MyClass*', strictly speaking, has no Java Object array pendant, for
- * lack of notion of (contigous arrays of) embedded Java Objects.
+ * lack of notion of (contiguous arrays of) embedded Java Objects.
  *
  * While assumed to work, this mapping has not been used and tested yet;
  * we're keeping it, for it still meets best Java programmers' expectations
@@ -336,7 +336,6 @@ struct ObjectArrayConvImpl {
         
         // init return value to error
         jobjectArray j = NULL;
-        jobjectArray ja = NULL;
 
         // get a (local or global) class object reference
         jclass cls = ObjectResult< J *, C * >::J_ctor::getClass(env);
@@ -366,17 +365,17 @@ struct ObjectArrayConvImpl {
 private:
     // Returns a new Java Object array with all elements initialized to null;
     // in case of a NULL return, an exception is pending.
-    J *
+    static J *
     newJavaObjectArray(jclass cls, jsize n, JNIEnv * env);
     
     // Copies objects referred by a Java Object array over a C object array;
     // a non-zero return value indicates failure with an exception pending.
-    cstatus
+    static cstatus
     copyToCObjectArray(C * c, jobjectArray j, jsize n, JNIEnv * env);
 
     // Initializes a Java Object array with references from a C object array;
     // a non-zero return value indicates failure with an exception pending.
-    cstatus
+    static cstatus
     copyToJavaObjectArray(jobjectArray j, C * c, jsize n, JNIEnv * env);
 };
 
