@@ -26,19 +26,19 @@
 
 #include <NdbOut.hpp>
 
+#include <NdbApi.hpp>
+#include <NDBT.hpp> 
+#include <NDBT_Thread.hpp>
+#include <NDBT_Stats.hpp>
 #include <NdbSleep.h>
 #include <getarg.h>
-#include <NDBT.hpp>
-#include <NDBT_Stats.hpp>
-#include <NDBT_Thread.hpp>
-#include <NdbApi.hpp>
 
 #include <HugoTransactions.hpp>
 
 static NDBT_ThreadFunc hugoPkUpdate;
 
 struct ThrInput {
-  const NdbDictionary::Table *pTab;
+  const NdbDictionary::Table* pTab;
   int records;
   int batch;
   int stats;
@@ -50,7 +50,7 @@ struct ThrOutput {
 
 static int _refresh = 0;
 
-int main(int argc, const char **argv) {
+int main(int argc, const char** argv){
   ndb_init();
 
   int _records = 0;
@@ -59,33 +59,29 @@ int main(int argc, const char **argv) {
   int _stats = 0;
   int _abort = 0;
   int _batch = 1;
-  const char *_tabname = NULL, *db = 0;
+  const char* _tabname = NULL, *db = 0;
   int _help = 0;
 
   struct getargs args[] = {
-      {"aborts", 'a', arg_integer, &_abort,
-       "percent of transactions that are aborted", "abort%"},
-      {"loops", 'l', arg_integer, &_loops,
-       "number of times to run this program(0=infinite loop)", "loops"},
-      {"threads", 't', arg_integer, &_threads, "number of threads (default 1)",
-       "threads"},
-      {"stats", 's', arg_flag, &_stats, "report latency per batch", "stats"},
-      //    { "batch", 'b', arg_integer, &_batch, "batch value", "batch" },
-      {"records", 'r', arg_integer, &_records, "Number of records", "records"},
-      {"usage", '?', arg_flag, &_help, "Print help", ""},
-      {"database", 'd', arg_string, &db, "Database", ""},
-      {"refresh", 0, arg_flag, &_refresh,
-       "refresh record rather than update them", ""}
+    { "aborts", 'a', arg_integer, &_abort, "percent of transactions that are aborted", "abort%" },
+    { "loops", 'l', arg_integer, &_loops, "number of times to run this program(0=infinite loop)", "loops" },
+    { "threads", 't', arg_integer, &_threads, "number of threads (default 1)", "threads" },
+    { "stats", 's', arg_flag, &_stats, "report latency per batch", "stats" },
+    //    { "batch", 'b', arg_integer, &_batch, "batch value", "batch" },
+    { "records", 'r', arg_integer, &_records, "Number of records", "records" },
+    { "usage", '?', arg_flag, &_help, "Print help", "" },
+    { "database", 'd', arg_string, &db, "Database", "" },
+    { "refresh", 0, arg_flag, &_refresh, "refresh record rather than update them", "" }
 
   };
   int num_args = sizeof(args) / sizeof(args[0]);
   int optind = 0;
-  char desc[] =
-      "tabname\n"
-      "This program will update all records in a table using PK\n";
-
-  if (getarg(args, num_args, argc, argv, &optind) || argv[optind] == NULL ||
-      _records == 0 || _help) {
+  char desc[] = 
+    "tabname\n"\
+    "This program will update all records in a table using PK\n";
+  
+  if(getarg(args, num_args, argc, argv, &optind) ||
+     argv[optind] == NULL || _records == 0 || _help) {
     arg_printusage(args, num_args, argv[0], desc);
     return NDBT_ProgramExit(NDBT_WRONGARGS);
   }
@@ -93,26 +89,27 @@ int main(int argc, const char **argv) {
 
   // Connect to Ndb
   Ndb_cluster_connection con;
-  if (con.connect(12, 5, 1) != 0) {
+  if(con.connect(12, 5, 1) != 0)
+  {
     return NDBT_ProgramExit(NDBT_FAILED);
   }
 
-  if (con.wait_until_ready(30, 0) < 0) {
+  if (con.wait_until_ready(30,0) < 0)
+  {
     ndbout << "Cluster nodes not ready in 30 seconds." << endl;
     return NDBT_ProgramExit(NDBT_FAILED);
   }
+  
+  Ndb MyNdb( &con, db ? db : "TEST_DB" );
 
-  Ndb MyNdb(&con, db ? db : "TEST_DB");
-
-  if (MyNdb.init() != 0) {
+  if(MyNdb.init() != 0){
     NDB_ERR(MyNdb.getNdbError());
     return NDBT_ProgramExit(NDBT_FAILED);
   }
 
   // Check if table exists in db
-  const NdbDictionary::Table *pTab =
-      NDBT_Table::discoverTableFromDb(&MyNdb, _tabname);
-  if (pTab == NULL) {
+  const NdbDictionary::Table * pTab = NDBT_Table::discoverTableFromDb(&MyNdb, _tabname);
+  if(pTab == NULL){
     ndbout << " Table " << _tabname << " does not exist!" << endl;
     return NDBT_ProgramExit(NDBT_WRONGARGS);
   }
@@ -146,7 +143,8 @@ int main(int argc, const char **argv) {
     ths.start();
     ths.stop();
 
-    if (ths.get_err()) {
+    if (ths.get_err())
+    {
       NDBT_ProgramExit(NDBT_FAILED);
     }
 
@@ -156,17 +154,19 @@ int main(int argc, const char **argv) {
       // add stats from each thread
       int n;
       for (n = 0; n < ths.get_count(); n++) {
-        NDBT_Thread &thr = ths.get_thread(n);
-        ThrOutput *output = (ThrOutput *)thr.get_output();
+        NDBT_Thread& thr = ths.get_thread(n);
+        ThrOutput* output = (ThrOutput*)thr.get_output();
         latency += output->latency;
       }
 
-      ndbout << "latency per batch (us): "
-             << " samples=" << latency.getCount()
-             << " min=" << (int)latency.getMin()
-             << " max=" << (int)latency.getMax()
-             << " mean=" << (int)latency.getMean()
-             << " stddev=" << (int)latency.getStddev() << endl;
+      ndbout
+        << "latency per batch (us): "
+        << " samples=" << latency.getCount()
+        << " min=" << (int)latency.getMin()
+        << " max=" << (int)latency.getMax()
+        << " mean=" << (int)latency.getMean()
+        << " stddev=" << (int)latency.getStddev()
+        << endl;
     }
     i++;
   }
@@ -178,24 +178,33 @@ int main(int argc, const char **argv) {
   return NDBT_ProgramExit(NDBT_OK);
 }
 
-static void hugoPkUpdate(NDBT_Thread &thr) {
-  const ThrInput *input = (const ThrInput *)thr.get_input();
-  ThrOutput *output = (ThrOutput *)thr.get_output();
+static void hugoPkUpdate(NDBT_Thread& thr)
+{
+  const ThrInput* input = (const ThrInput*)thr.get_input();
+  ThrOutput* output = (ThrOutput*)thr.get_output();
 
   HugoTransactions hugoTrans(*input->pTab);
   output->latency.reset();
-  if (input->stats) hugoTrans.setStatsLatency(&output->latency);
+  if (input->stats)
+    hugoTrans.setStatsLatency(&output->latency);
 
-  NDBT_ThreadSet &ths = thr.get_thread_set();
+  NDBT_ThreadSet& ths = thr.get_thread_set();
   hugoTrans.setThrInfo(ths.get_count(), thr.get_thread_no());
 
   int ret;
-  if (_refresh == 0) {
-    ret =
-        hugoTrans.pkUpdateRecords(thr.get_ndb(), input->records, input->batch);
-  } else {
-    ret = hugoTrans.pkRefreshRecords(thr.get_ndb(), 0, input->records,
+  if (_refresh == 0)
+  {
+    ret = hugoTrans.pkUpdateRecords(thr.get_ndb(),
+                                    input->records,
+                                    input->batch);
+  }
+  else
+  {
+    ret = hugoTrans.pkRefreshRecords(thr.get_ndb(),
+                                     0,
+                                     input->records,
                                      input->batch);
   }
-  if (ret != 0) thr.set_err(ret);
+  if (ret != 0)
+    thr.set_err(ret);
 }
