@@ -134,6 +134,8 @@
 #include "storage/perfschema/table_setup_actors.h"
 #include "storage/perfschema/table_setup_consumers.h"
 #include "storage/perfschema/table_setup_instruments.h"
+#include "storage/perfschema/table_setup_meters.h"
+#include "storage/perfschema/table_setup_metrics.h"
 #include "storage/perfschema/table_setup_objects.h"
 #include "storage/perfschema/table_setup_threads.h"
 #include "storage/perfschema/table_socket_instances.h"
@@ -503,6 +505,8 @@ static PFS_engine_table_share *all_shares[] = {
     &table_setup_actors::m_share,
     &table_setup_consumers::m_share,
     &table_setup_instruments::m_share,
+    &table_setup_meters::m_share,
+    &table_setup_metrics::m_share,
     &table_setup_objects::m_share,
     &table_setup_threads::m_share,
     &table_tiws_by_index_usage::m_share,
