@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -27,11 +27,12 @@
 
 #include "storage/perfschema/pfs_events_statements.h"
 
+#include <assert.h>
 #include <atomic>
 
 #include "m_string.h"
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "my_sys.h"
 #include "storage/perfschema/pfs_account.h"
 #include "storage/perfschema/pfs_buffer_container.h"
@@ -44,6 +45,8 @@
 #include "template_utils.h"
 
 PFS_ALIGNED size_t events_statements_history_long_size = 0;
+/** Consumer flag for CPU_TIME columns. */
+PFS_ALIGNED bool flag_events_statements_cpu = false;
 /** Consumer flag for table EVENTS_STATEMENTS_CURRENT. */
 PFS_ALIGNED bool flag_events_statements_current = false;
 /** Consumer flag for table EVENTS_STATEMENTS_HISTORY. */
@@ -56,9 +59,10 @@ PFS_ALIGNED bool events_statements_history_long_full = false;
 /** Index in EVENTS_STATEMENTS_HISTORY_LONG circular buffer. */
 PFS_ALIGNED PFS_cacheline_atomic_uint32 events_statements_history_long_index;
 /** EVENTS_STATEMENTS_HISTORY_LONG circular buffer. */
-PFS_ALIGNED PFS_events_statements *events_statements_history_long_array = NULL;
-static unsigned char *h_long_stmts_digest_token_array = NULL;
-static char *h_long_stmts_text_array = NULL;
+PFS_ALIGNED PFS_events_statements *events_statements_history_long_array =
+    nullptr;
+static unsigned char *h_long_stmts_digest_token_array = nullptr;
+static char *h_long_stmts_text_array = nullptr;
 
 /**
   Initialize table EVENTS_STATEMENTS_HISTORY_LONG.
@@ -70,6 +74,10 @@ int init_events_statements_history_long(
   events_statements_history_long_full = false;
   events_statements_history_long_index.m_u32.store(0);
 
+  events_statements_history_long_array = nullptr;
+  h_long_stmts_digest_token_array = nullptr;
+  h_long_stmts_text_array = nullptr;
+
   if (events_statements_history_long_size == 0) {
     return 0;
   }
@@ -79,21 +87,22 @@ int init_events_statements_history_long(
       events_statements_history_long_size, sizeof(PFS_events_statements),
       PFS_events_statements, MYF(MY_ZEROFILL));
 
-  if (events_statements_history_long_array == NULL) {
+  if (events_statements_history_long_array == nullptr) {
     cleanup_events_statements_history_long();
     return 1;
   }
 
   if (pfs_max_digest_length > 0) {
     /* Size of each digest text array. */
-    size_t digest_text_size = pfs_max_digest_length * sizeof(unsigned char);
+    const size_t digest_text_size =
+        pfs_max_digest_length * sizeof(unsigned char);
 
     h_long_stmts_digest_token_array =
         PFS_MALLOC_ARRAY(&builtin_memory_statements_history_long_tokens,
                          events_statements_history_long_size, digest_text_size,
                          unsigned char, MYF(MY_ZEROFILL));
 
-    if (h_long_stmts_digest_token_array == NULL) {
+    if (h_long_stmts_digest_token_array == nullptr) {
       cleanup_events_statements_history_long();
       return 1;
     }
@@ -101,14 +110,14 @@ int init_events_statements_history_long(
 
   if (pfs_max_sqltext > 0) {
     /* Size of each sql text array. */
-    size_t sqltext_size = pfs_max_sqltext * sizeof(char);
+    const size_t sqltext_size = pfs_max_sqltext * sizeof(char);
 
     h_long_stmts_text_array =
         PFS_MALLOC_ARRAY(&builtin_memory_statements_history_long_sqltext,
                          events_statements_history_long_size, sqltext_size,
                          char, MYF(MY_ZEROFILL));
 
-    if (h_long_stmts_text_array == NULL) {
+    if (h_long_stmts_text_array == nullptr) {
       cleanup_events_statements_history_long();
       return 1;
     }
@@ -126,7 +135,7 @@ int init_events_statements_history_long(
 }
 
 /** Cleanup table EVENTS_STATEMENTS_HISTORY_LONG. */
-void cleanup_events_statements_history_long(void) {
+void cleanup_events_statements_history_long() {
   PFS_FREE_ARRAY(&builtin_memory_statements_history_long,
                  events_statements_history_long_size,
                  sizeof(PFS_events_statements),
@@ -141,9 +150,10 @@ void cleanup_events_statements_history_long(void) {
                  events_statements_history_long_size,
                  (pfs_max_sqltext * sizeof(char)), h_long_stmts_text_array);
 
-  events_statements_history_long_array = NULL;
-  h_long_stmts_digest_token_array = NULL;
-  h_long_stmts_text_array = NULL;
+  events_statements_history_long_array = nullptr;
+  h_long_stmts_digest_token_array = nullptr;
+  h_long_stmts_text_array = nullptr;
+  events_statements_history_long_size = 0;
 }
 
 static inline void copy_events_statements(PFS_events_statements *dest,
@@ -155,7 +165,7 @@ static inline void copy_events_statements(PFS_events_statements *dest,
              pointer_cast<const char *>(&source->m_statement_id));
 
   /* Copy SQL TEXT */
-  int sqltext_length = source->m_sqltext_length;
+  const int sqltext_length = source->m_sqltext_length;
 
   if (sqltext_length > 0) {
     memcpy(dest->m_sqltext, source->m_sqltext, sqltext_length);
@@ -179,7 +189,7 @@ void insert_events_statements_history(PFS_thread *thread,
     return;
   }
 
-  DBUG_ASSERT(thread->m_statements_history != NULL);
+  assert(thread->m_statements_history != nullptr);
 
   uint index = thread->m_statements_history_index;
 
@@ -210,7 +220,7 @@ void insert_events_statements_history_long(PFS_events_statements *statement) {
     return;
   }
 
-  DBUG_ASSERT(events_statements_history_long_array != NULL);
+  assert(events_statements_history_long_array != nullptr);
 
   uint index = events_statements_history_long_index.m_u32++;
 
@@ -229,12 +239,12 @@ static void fct_reset_events_statements_current(PFS_thread *pfs_thread) {
   PFS_events_statements *pfs_stmt_last = pfs_stmt + statement_stack_max;
 
   for (; pfs_stmt < pfs_stmt_last; pfs_stmt++) {
-    pfs_stmt->m_class = NULL;
+    pfs_stmt->m_class = nullptr;
   }
 }
 
 /** Reset table EVENTS_STATEMENTS_CURRENT data. */
-void reset_events_statements_current(void) {
+void reset_events_statements_current() {
   global_thread_container.apply_all(fct_reset_events_statements_current);
 }
 
@@ -245,24 +255,24 @@ static void fct_reset_events_statements_history(PFS_thread *pfs_thread) {
   pfs_thread->m_statements_history_index = 0;
   pfs_thread->m_statements_history_full = false;
   for (; pfs < pfs_last; pfs++) {
-    pfs->m_class = NULL;
+    pfs->m_class = nullptr;
   }
 }
 
 /** Reset table EVENTS_STATEMENTS_HISTORY data. */
-void reset_events_statements_history(void) {
+void reset_events_statements_history() {
   global_thread_container.apply_all(fct_reset_events_statements_history);
 }
 
 /** Reset table EVENTS_STATEMENTS_HISTORY_LONG data. */
-void reset_events_statements_history_long(void) {
+void reset_events_statements_history_long() {
   events_statements_history_long_index.m_u32.store(0);
   events_statements_history_long_full = false;
 
   PFS_events_statements *pfs = events_statements_history_long_array;
   PFS_events_statements *pfs_last = pfs + events_statements_history_long_size;
   for (; pfs < pfs_last; pfs++) {
-    pfs->m_class = NULL;
+    pfs->m_class = nullptr;
   }
 }
 
