@@ -58,7 +58,7 @@
 #define PFS_MAX_MUTEX_CLASS 350
 #endif
 #ifndef PFS_MAX_RWLOCK_CLASS
-#define PFS_MAX_RWLOCK_CLASS 70
+#define PFS_MAX_RWLOCK_CLASS 100
 #endif
 #ifndef PFS_MAX_COND_CLASS
 #define PFS_MAX_COND_CLASS 150
@@ -84,6 +84,12 @@
 #ifndef PFS_MAX_MEMORY_CLASS
 #define PFS_MAX_MEMORY_CLASS 530
 #endif
+#ifndef PFS_MAX_METER_CLASS
+#define PFS_MAX_METER_CLASS 30
+#endif
+#ifndef PFS_MAX_METRIC_CLASS
+#define PFS_MAX_METRIC_CLASS 600
+#endif
 
 #ifndef PFS_MAX_GLOBAL_SERVER_ERRORS
 #define PFS_MAX_GLOBAL_SERVER_ERRORS \
@@ -275,6 +281,18 @@ struct PFS_global_param {
   */
   ulong m_memory_class_sizing;
 
+  /**
+    Maximum number of instrumented meter classes.
+    @sa meter_class_lost.
+  */
+  ulong m_meter_class_sizing;
+
+  /**
+    Maximum number of instrumented metric classes.
+    @sa metric_class_lost.
+  */
+  ulong m_metric_class_sizing;
+
   long m_metadata_lock_sizing;
 
   long m_max_digest_length;
