@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,10 +29,10 @@
 
 #include <assert.h>
 #include "lex_string.h"
-#include "m_string.h"
 #include "my_compiler.h"
 
 #include "my_thread.h"
+#include "mysql/strings/int2str.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
 #include "sql/table.h"
@@ -43,6 +42,7 @@
 #include "storage/perfschema/pfs_instr.h"
 #include "storage/perfschema/pfs_instr_class.h"
 #include "storage/perfschema/pfs_timer.h"
+#include "string_with_len.h"
 
 bool PFS_index_events_waits::match(PFS_thread *pfs) {
   if (m_fields >= 1) {
