@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -27,9 +27,12 @@
 
 #include "storage/perfschema/table_threads.h"
 
+#include <assert.h>
+#include <ctime>
+
 #include "lex_string.h"
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -52,7 +55,7 @@ Plugin_table table_threads::m_table_def(
     "  TYPE VARCHAR(10) not null,\n"
     "  PROCESSLIST_ID BIGINT unsigned,\n"
     "  PROCESSLIST_USER VARCHAR(32),\n"
-    "  PROCESSLIST_HOST VARCHAR(60),\n"
+    "  PROCESSLIST_HOST VARCHAR(255) CHARACTER SET ASCII default null,\n"
     "  PROCESSLIST_DB VARCHAR(64),\n"
     "  PROCESSLIST_COMMAND VARCHAR(16),\n"
     "  PROCESSLIST_TIME BIGINT,\n"
@@ -65,6 +68,12 @@ Plugin_table table_threads::m_table_def(
     "  CONNECTION_TYPE VARCHAR(16),\n"
     "  THREAD_OS_ID BIGINT unsigned,\n"
     "  RESOURCE_GROUP VARCHAR(64),\n"
+    "  EXECUTION_ENGINE ENUM ('PRIMARY', 'SECONDARY'),\n"
+    "  CONTROLLED_MEMORY BIGINT unsigned not null,\n"
+    "  MAX_CONTROLLED_MEMORY BIGINT unsigned not null,\n"
+    "  TOTAL_MEMORY BIGINT unsigned not null,\n"
+    "  MAX_TOTAL_MEMORY BIGINT unsigned not null,\n"
+    "  TELEMETRY_ACTIVE ENUM ('YES', 'NO') not null,\n"
     "  PRIMARY KEY (THREAD_ID) USING HASH,\n"
     "  KEY (PROCESSLIST_ID) USING HASH,\n"
     "  KEY (THREAD_OS_ID) USING HASH,\n"
@@ -81,8 +90,8 @@ Plugin_table table_threads::m_table_def(
 PFS_engine_table_share table_threads::m_share = {
     &pfs_updatable_acl,
     table_threads::create,
-    NULL, /* write_row */
-    NULL, /* delete_all_rows */
+    nullptr, /* write_row */
+    nullptr, /* delete_all_rows */
     cursor_by_thread::get_row_count,
     sizeof(PFS_simple_index), /* ref length */
     &m_table_lock,
@@ -176,7 +185,7 @@ bool PFS_index_threads_by_resource_group::match(PFS_thread *pfs) {
 }
 
 int table_threads::index_init(uint idx, bool) {
-  PFS_index_threads *result = NULL;
+  PFS_index_threads *result = nullptr;
 
   switch (idx) {
     case 0:
@@ -201,7 +210,7 @@ int table_threads::index_init(uint idx, bool) {
       result = PFS_NEW(PFS_index_threads_by_resource_group);
       break;
     default:
-      DBUG_ASSERT(false);
+      assert(false);
   }
 
   m_opened_index = result;
@@ -220,7 +229,11 @@ int table_threads::make_row(PFS_thread *pfs) {
   pfs->m_lock.begin_optimistic_lock(&lock);
 
   safe_class = sanitize_thread_class(pfs->m_class);
-  if (unlikely(safe_class == NULL)) {
+  if (unlikely(safe_class == nullptr)) {
+    return HA_ERR_RECORD_DELETED;
+  }
+
+  if (pfs->m_disable_instrumentation) {
     return HA_ERR_RECORD_DELETED;
   }
 
@@ -228,29 +241,14 @@ int table_threads::make_row(PFS_thread *pfs) {
   m_row.m_parent_thread_internal_id = pfs->m_parent_thread_internal_id;
   m_row.m_processlist_id = pfs->m_processlist_id;
   m_row.m_thread_os_id = pfs->m_thread_os_id;
-  m_row.m_name = safe_class->m_name;
-  m_row.m_name_length = safe_class->m_name_length;
+  m_row.m_name = safe_class->m_name.str();
+  m_row.m_name_length = safe_class->m_name.length();
 
   /* Protect this reader against session attribute changes */
   pfs->m_session_lock.begin_optimistic_lock(&session_lock);
 
-  m_row.m_username_length = pfs->m_username_length;
-  if (unlikely(m_row.m_username_length > sizeof(m_row.m_username))) {
-    return HA_ERR_RECORD_DELETED;
-  }
-
-  if (m_row.m_username_length != 0) {
-    memcpy(m_row.m_username, pfs->m_username, m_row.m_username_length);
-  }
-
-  m_row.m_hostname_length = pfs->m_hostname_length;
-  if (unlikely(m_row.m_hostname_length > sizeof(m_row.m_hostname))) {
-    return HA_ERR_RECORD_DELETED;
-  }
-
-  if (m_row.m_hostname_length != 0) {
-    memcpy(m_row.m_hostname, pfs->m_hostname, m_row.m_hostname_length);
-  }
+  m_row.m_user_name = pfs->m_user_name;
+  m_row.m_host_name = pfs->m_host_name;
 
   m_row.m_groupname_length = pfs->m_groupname_length;
   if (unlikely(m_row.m_groupname_length > sizeof(m_row.m_groupname))) {
@@ -271,21 +269,14 @@ int table_threads::make_row(PFS_thread *pfs) {
       Do not loop waiting for a stable value.
       Just return NULL values.
     */
-    m_row.m_username_length = 0;
-    m_row.m_hostname_length = 0;
+    m_row.m_user_name.reset();
+    m_row.m_host_name.reset();
   }
 
   /* Protect this reader against statement attributes changes */
   pfs->m_stmt_lock.begin_optimistic_lock(&stmt_lock);
 
-  m_row.m_dbname_length = pfs->m_dbname_length;
-  if (unlikely(m_row.m_dbname_length > sizeof(m_row.m_dbname))) {
-    return HA_ERR_RECORD_DELETED;
-  }
-
-  if (m_row.m_dbname_length != 0) {
-    memcpy(m_row.m_dbname, pfs->m_dbname, m_row.m_dbname_length);
-  }
+  m_row.m_db_name = pfs->m_db_name;
 
   m_row.m_processlist_info_ptr = &pfs->m_processlist_info[0];
   m_row.m_processlist_info_length = pfs->m_processlist_info_length;
@@ -300,7 +291,7 @@ int table_threads::make_row(PFS_thread *pfs) {
       Do not loop waiting for a stable value.
       Just return NULL values.
     */
-    m_row.m_dbname_length = 0;
+    m_row.m_db_name.reset();
     m_row.m_processlist_info_length = 0;
   }
 
@@ -313,11 +304,11 @@ int table_threads::make_row(PFS_thread *pfs) {
   m_row.m_start_time = pfs->m_start_time;
 
   stage_class = find_stage_class(pfs->m_stage);
-  if (stage_class != NULL) {
+  if (stage_class != nullptr) {
     m_row.m_processlist_state_ptr =
-        stage_class->m_name + stage_class->m_prefix_length;
+        stage_class->m_name.str() + stage_class->m_prefix_length;
     m_row.m_processlist_state_length =
-        stage_class->m_name_length - stage_class->m_prefix_length;
+        stage_class->m_name.length() - stage_class->m_prefix_length;
     if (m_row.m_processlist_state_length > 64) {
       /*
         Column PROCESSLIST_STATE is VARCHAR(64)
@@ -334,10 +325,16 @@ int table_threads::make_row(PFS_thread *pfs) {
   }
   m_row.m_connection_type = pfs->m_connection_type;
 
-  m_row.m_enabled = pfs->m_enabled;
+  m_row.m_enabled = !pfs->m_disable_instrumentation && pfs->m_enabled;
   m_row.m_history = pfs->m_history;
   m_row.m_psi = pfs;
 
+  m_row.m_secondary = pfs->m_secondary;
+
+  m_row.m_session_all_memory_row.set(&pfs->m_session_all_memory_stat);
+
+  m_row.m_telemetry_active = (pfs->m_telemetry_session != nullptr);
+
   if (!pfs->m_lock.end_optimistic_lock(&lock)) {
     return HA_ERR_RECORD_DELETED;
   }
@@ -348,28 +345,28 @@ int table_threads::make_row(PFS_thread *pfs) {
 int table_threads::read_row_values(TABLE *table, unsigned char *buf,
                                    Field **fields, bool read_all) {
   Field *f;
-  const char *str = NULL;
+  const char *str = nullptr;
   int len = 0;
 
   /* Set the null bits */
-  DBUG_ASSERT(table->s->null_bytes == 2);
+  assert(table->s->null_bytes == 2);
   buf[0] = 0;
   buf[1] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
-      switch (f->field_index) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
+      switch (f->field_index()) {
         case 0: /* THREAD_ID */
           set_field_ulonglong(f, m_row.m_thread_internal_id);
           break;
         case 1: /* NAME */
-          set_field_varchar_utf8(f, m_row.m_name, m_row.m_name_length);
+          set_field_varchar_utf8mb4(f, m_row.m_name, m_row.m_name_length);
           break;
         case 2: /* TYPE */
           if (m_row.m_processlist_id != 0) {
-            set_field_varchar_utf8(f, "FOREGROUND", 10);
+            set_field_varchar_utf8mb4(f, "FOREGROUND", 10);
           } else {
-            set_field_varchar_utf8(f, "BACKGROUND", 10);
+            set_field_varchar_utf8mb4(f, "BACKGROUND", 10);
           }
           break;
         case 3: /* PROCESSLIST_ID */
@@ -380,40 +377,41 @@ int table_threads::read_row_values(TABLE *table, unsigned char *buf,
           }
           break;
         case 4: /* PROCESSLIST_USER */
-          if (m_row.m_username_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_username,
-                                   m_row.m_username_length);
+          if (m_row.m_user_name.length() > 0) {
+            set_field_varchar_utf8mb4(f, m_row.m_user_name.ptr(),
+                                      m_row.m_user_name.length());
           } else {
             f->set_null();
           }
           break;
         case 5: /* PROCESSLIST_HOST */
-          if (m_row.m_hostname_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_hostname,
-                                   m_row.m_hostname_length);
+          if (m_row.m_host_name.length() > 0) {
+            set_field_varchar_utf8mb4(f, m_row.m_host_name.ptr(),
+                                      m_row.m_host_name.length());
           } else {
             f->set_null();
           }
           break;
         case 6: /* PROCESSLIST_DB */
-          if (m_row.m_dbname_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_dbname, m_row.m_dbname_length);
+          if (m_row.m_db_name.length() > 0) {
+            set_field_varchar_utf8mb4(f, m_row.m_db_name.ptr(),
+                                      m_row.m_db_name.length());
           } else {
             f->set_null();
           }
           break;
         case 7: /* PROCESSLIST_COMMAND */
-          if (m_row.m_processlist_id != 0)
-            set_field_varchar_utf8(f, command_name[m_row.m_command].str,
-                                   (uint)command_name[m_row.m_command].length);
-          else {
+          if (m_row.m_processlist_id != 0) {
+            const std::string &cn = Command_names::str_session(m_row.m_command);
+            set_field_varchar_utf8mb4(f, cn.c_str(), cn.length());
+          } else {
             f->set_null();
           }
           break;
         case 8: /* PROCESSLIST_TIME */
           if (m_row.m_start_time) {
-            time_t now = my_time(0);
-            ulonglong elapsed =
+            const time_t now = time(nullptr);
+            const ulonglong elapsed =
                 (now > m_row.m_start_time ? now - m_row.m_start_time : 0);
             set_field_ulonglong(f, elapsed);
           } else {
@@ -422,8 +420,8 @@ int table_threads::read_row_values(TABLE *table, unsigned char *buf,
           break;
         case 9: /* PROCESSLIST_STATE */
           if (m_row.m_processlist_state_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_processlist_state_ptr,
-                                   m_row.m_processlist_state_length);
+            set_field_varchar_utf8mb4(f, m_row.m_processlist_state_ptr,
+                                      m_row.m_processlist_state_length);
           } else {
             f->set_null();
           }
@@ -455,7 +453,7 @@ int table_threads::read_row_values(TABLE *table, unsigned char *buf,
         case 15: /* CONNECTION_TYPE */
           get_vio_type_name(m_row.m_connection_type, &str, &len);
           if (len > 0) {
-            set_field_varchar_utf8(f, str, len);
+            set_field_varchar_utf8mb4(f, str, len);
           } else {
             f->set_null();
           }
@@ -469,14 +467,26 @@ int table_threads::read_row_values(TABLE *table, unsigned char *buf,
           break;
         case 17: /* RESOURCE_GROUP */
           if (m_row.m_groupname_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_groupname,
-                                   m_row.m_groupname_length);
+            set_field_varchar_utf8mb4(f, m_row.m_groupname,
+                                      m_row.m_groupname_length);
           } else {
             f->set_null();
           }
           break;
+        case 18: /* EXECUTION_ENGINE */
+          set_field_enum(f, m_row.m_secondary ? ENUM_SECONDARY : ENUM_PRIMARY);
+          break;
+        case 19: /* CONTROLLED_MEMORY */
+        case 20: /* MAX_CONTROLLED_MEMORY */
+        case 21: /* TOTAL_MEMORY */
+        case 22: /* MAX_TOTAL_MEMORY */
+          m_row.m_session_all_memory_row.set_field(f->field_index() - 19, f);
+          break;
+        case 23: /* TELEMETRY_ACTIVE */
+          set_field_enum(f, m_row.m_telemetry_active ? ENUM_YES : ENUM_NO);
+          break;
         default:
-          DBUG_ASSERT(false);
+          assert(false);
       }
     }
   }
@@ -489,37 +499,18 @@ int table_threads::update_row_values(TABLE *table, const unsigned char *,
   enum_yes_no value;
 
   for (; (f = *fields); fields++) {
-    if (bitmap_is_set(table->write_set, f->field_index)) {
-      switch (f->field_index) {
-        case 0:  /* THREAD_ID */
-        case 1:  /* NAME */
-        case 2:  /* TYPE */
-        case 3:  /* PROCESSLIST_ID */
-        case 4:  /* PROCESSLIST_USER */
-        case 5:  /* PROCESSLIST_HOST */
-        case 6:  /* PROCESSLIST_DB */
-        case 7:  /* PROCESSLIST_COMMAND */
-        case 8:  /* PROCESSLIST_TIME */
-        case 9:  /* PROCESSLIST_STATE */
-        case 10: /* PROCESSLIST_INFO */
-        case 11: /* PARENT_THREAD_ID */
-        case 12: /* ROLE */
-          return HA_ERR_WRONG_COMMAND;
+    if (bitmap_is_set(table->write_set, f->field_index())) {
+      switch (f->field_index()) {
         case 13: /* INSTRUMENTED */
           value = (enum_yes_no)get_field_enum(f);
-          m_row.m_psi->set_enabled((value == ENUM_YES) ? true : false);
+          m_row.m_psi->set_enabled(value == ENUM_YES);
           break;
         case 14: /* HISTORY */
           value = (enum_yes_no)get_field_enum(f);
-          m_row.m_psi->set_history((value == ENUM_YES) ? true : false);
+          m_row.m_psi->set_history(value == ENUM_YES);
           break;
-        case 15: /* CONNECTION_TYPE */
-        case 16: /* THREAD_OS_ID */
-          return HA_ERR_WRONG_COMMAND;
-        case 17: /* RESOURCE_GROUP */
-          return HA_ERR_WRONG_COMMAND;
         default:
-          DBUG_ASSERT(false);
+          return HA_ERR_WRONG_COMMAND;
       }
     }
   }
