@@ -163,9 +163,10 @@ int Rdb_index_merge::add(const rocksdb::Slice &key, const rocksdb::Slice &val) {
       add is too large for the buffer.
     */
     if (m_offset_tree.empty()) {
-      LogPluginErrMsg(ERROR_LEVEL, 0,
-                      "Sort buffer size is too small to process merge. Please "
-                      "set merge buffer size to a higher value.");
+      LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
+                      "Current value of rocksdb_merge_buf_size=%" PRIu64
+                      " is too small. At least %u bytes required.",
+                      m_rec_buf_unsorted->m_total_size, total_offset);
       return HA_ERR_ROCKSDB_MERGE_FILE_ERR;
     }
 
@@ -173,6 +174,19 @@ int Rdb_index_merge::add(const rocksdb::Slice &key, const rocksdb::Slice &val) {
       LogPluginErrMsg(ERROR_LEVEL, 0, "Error writing sort buffer to disk.");
       return HA_ERR_ROCKSDB_MERGE_FILE_ERR;
     }
+
+    /*
+      The unsorted buffer may be too small for the key-value pair.
+    */
+    const uint data_size = RDB_MERGE_CHUNK_LEN + RDB_MERGE_KEY_DELIMITER +
+                           RDB_MERGE_VAL_DELIMITER + key.size() + val.size();
+    if (data_size > m_rec_buf_unsorted->m_total_size) {
+      LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
+                      "Current value of rocksdb_merge_buf_size=%" PRIu64
+                      " is too small. At least %u bytes required.",
+                      m_rec_buf_unsorted->m_total_size, data_size);
+      return HA_ERR_ROCKSDB_MERGE_FILE_ERR;
+    }
   }
 
   const ulonglong rec_offset = m_rec_buf_unsorted->m_curr_offset;
