@@ -32,8 +32,6 @@ void thd_enter_cond(MYSQL_THD thd, mysql_cond_t *cond, mysql_mutex_t *mutex,
 void thd_exit_cond(MYSQL_THD thd, const PSI_stage_info *stage);
 }
 
-using namespace rocksdb;
-
 namespace myrocks {
 
 static PSI_stage_info stage_waiting_on_row_lock2 = {0, "Waiting for row lock",
@@ -55,7 +53,8 @@ Rdb_cond_var::Rdb_cond_var() { mysql_cond_init(PSI_NOT_INSTRUMENTED, &m_cond); }
 
 Rdb_cond_var::~Rdb_cond_var() { mysql_cond_destroy(&m_cond); }
 
-Status Rdb_cond_var::Wait(const std::shared_ptr<TransactionDBMutex> mutex_arg) {
+rocksdb::Status Rdb_cond_var::Wait(
+    const std::shared_ptr<rocksdb::TransactionDBMutex> mutex_arg) {
   return WaitFor(mutex_arg, ONE_YEAR_IN_MICROSECS);
 }
 
@@ -68,24 +67,23 @@ Status Rdb_cond_var::Wait(const std::shared_ptr<TransactionDBMutex> mutex_arg) {
      timeout_micros  Timeout in microseconds. Negative value means no timeout.
 
   @return
-    Status::OK()       - Wait successfull
-    Status::TimedOut() - Timed out or wait killed (the caller can check
+    rocksdb::Status::OK()       - Wait successfull
+    rocksdb::Status::TimedOut() - Timed out or wait killed (the caller can check
                          thd_killed() to determine which occurred)
 */
 
-Status
-Rdb_cond_var::WaitFor(const std::shared_ptr<TransactionDBMutex> mutex_arg,
-                      int64_t timeout_micros) {
+rocksdb::Status Rdb_cond_var::WaitFor(
+    const std::shared_ptr<rocksdb::TransactionDBMutex> mutex_arg,
+    int64_t timeout_micros) {
   auto *mutex_obj = reinterpret_cast<Rdb_mutex *>(mutex_arg.get());
-  DBUG_ASSERT(mutex_obj != nullptr);
+  assert(mutex_obj != nullptr);
 
   mysql_mutex_t *const mutex_ptr = &mutex_obj->m_mutex;
 
   int res = 0;
   struct timespec wait_timeout;
 
-  if (timeout_micros < 0)
-    timeout_micros = ONE_YEAR_IN_MICROSECS;
+  if (timeout_micros < 0) timeout_micros = ONE_YEAR_IN_MICROSECS;
   set_timespec_nsec(&wait_timeout, timeout_micros * 1000);
 
 #ifndef STANDALONE_UNITTEST
@@ -114,15 +112,15 @@ Rdb_cond_var::WaitFor(const std::shared_ptr<TransactionDBMutex> mutex_arg,
     res = mysql_cond_timedwait(&m_cond, mutex_ptr, &wait_timeout);
 
 #ifndef STANDALONE_UNITTEST
-    if (current_thd)
-      killed = my_core::thd_killed(current_thd);
+    if (current_thd) killed = my_core::thd_killed(current_thd);
 #endif
   } while (!killed && res == EINTR);
 
-  if (res || killed)
-    return Status::TimedOut();
-  else
-    return Status::OK();
+  if (res || killed) {
+    return rocksdb::Status::TimedOut();
+  } else {
+    return rocksdb::Status::OK();
+  }
 }
 
 /*
@@ -171,32 +169,33 @@ Rdb_mutex::Rdb_mutex() {
 
 Rdb_mutex::~Rdb_mutex() { mysql_mutex_destroy(&m_mutex); }
 
-Status Rdb_mutex::Lock() {
+rocksdb::Status Rdb_mutex::Lock() {
   RDB_MUTEX_LOCK_CHECK(m_mutex);
-  DBUG_ASSERT(m_old_stage_info.count(current_thd) == 0);
-  return Status::OK();
+  assert(m_old_stage_info.count(current_thd) == 0);
+  return rocksdb::Status::OK();
 }
 
 // Attempt to acquire lock.  If timeout is non-negative, operation may be
 // failed after this many milliseconds.
 // If implementing a custom version of this class, the implementation may
 // choose to ignore the timeout.
-// Return OK on success, or other Status on failure.
-Status Rdb_mutex::TryLockFor(int64_t timeout_time MY_ATTRIBUTE((__unused__))) {
+// Return OK on success, or other rocksdb::Status on failure.
+rocksdb::Status Rdb_mutex::TryLockFor(
+    int64_t timeout_time MY_ATTRIBUTE((__unused__))) {
   /*
     Note: PThreads API has pthread_mutex_timedlock(), but mysql's
     mysql_mutex_* wrappers do not wrap that function.
   */
   RDB_MUTEX_LOCK_CHECK(m_mutex);
-  return Status::OK();
+  return rocksdb::Status::OK();
 }
 
 #ifndef STANDALONE_UNITTEST
 void Rdb_mutex::set_unlock_action(const PSI_stage_info *const old_stage_arg) {
-  DBUG_ASSERT(old_stage_arg != nullptr);
+  assert(old_stage_arg != nullptr);
 
   mysql_mutex_assert_owner(&m_mutex);
-  DBUG_ASSERT(m_old_stage_info.count(current_thd) == 0);
+  assert(m_old_stage_info.count(current_thd) == 0);
 
   m_old_stage_info[current_thd] =
       std::make_shared<PSI_stage_info>(*old_stage_arg);
