@@ -29,6 +29,8 @@
 
 /* RocksDB header files */
 #include "rocksdb/db.h"
+#include "rocksdb/file_system.h"
+#include "rocksdb/io_status.h"
 #include "rocksdb/options.h"
 
 /* MyRocks header files */
@@ -139,7 +141,8 @@ rocksdb::Status Rdb_sst_file_ordered::Rdb_sst_file::commit() {
     if (m_tracing) {
       LogPluginErrMsg(INFORMATION_LEVEL, 0,
                       "SST Tracing: Adding file %s, smallest key: %s, "
-                      "largest key: %s, file size: %lu, num_entries: %lu",
+                      "largest key: %s, file size: %" PRIu64
+                      ", num_entries: %" PRIu64,
                       fileinfo.file_path.c_str(),
                       generateKey(fileinfo.smallest_key).c_str(),
                       generateKey(fileinfo.largest_key).c_str(),
@@ -309,7 +312,7 @@ Rdb_sst_info::Rdb_sst_info(rocksdb::DB *const db, const std::string &tablename,
       m_sst_file(nullptr),
       m_tracing(tracing),
       m_print_client_error(true) {
-  m_prefix = db->GetName() + "/";
+  m_prefix = db->GetName() + '/';
 
   std::string normalized_table;
   if (rdb_normalize_tablename(tablename.c_str(), &normalized_table)) {
@@ -518,34 +521,28 @@ void Rdb_sst_info::report_error_msg(const rocksdb::Status &s,
 }
 
 void Rdb_sst_info::init(const rocksdb::DB *const db) {
-  const std::string path = db->GetName() + FN_DIRSEP;
+  const std::string dir = db->GetName();
+  const auto &fs = db->GetEnv()->GetFileSystem();
+  std::vector<std::string> files_in_dir;
 
-  // Skip temp SST cleanup in WSEnv for now
-  if (!rdb_has_wsenv()) {
-    struct MY_DIR *const dir_info = my_dir(path.c_str(), MYF(MY_DONT_SORT));
+  // Get the files in the specified directory
+  rocksdb::IOStatus s =
+      fs->GetChildren(dir, rocksdb::IOOptions(), &files_in_dir, nullptr);
+  if (!s.ok()) {
+    LogPluginErrMsg(WARNING_LEVEL, 0, "Could not access database directory: %s",
+                    dir.c_str());
+    return;
+  }
 
-    // Access the directory
-    if (dir_info == nullptr) {
-      LogPluginErrMsg(WARNING_LEVEL, 0,
-                      "Could not access database directory: %s", path.c_str());
-      return;
+  // Scan through the files in the directory
+  for (const auto &file : files_in_dir) {
+    // Find any files ending with m_suffix ...
+    const size_t pos = file.find(m_suffix);
+    if (pos != std::string::npos && file.size() - pos == m_suffix.size()) {
+      // Remove
+      const auto fullname = rdb_concat_paths(dir, file);
+      fs->DeleteFile(fullname, rocksdb::IOOptions(), nullptr);
     }
-
-    // Scan through the files in the directory
-    const struct fileinfo *file_info = dir_info->dir_entry;
-    for (uint ii = 0; ii < dir_info->number_off_files; ii++, file_info++) {
-      // find any files ending with m_suffix ...
-      const std::string name = file_info->name;
-      const size_t pos = name.find(m_suffix);
-      if (pos != std::string::npos && name.size() - pos == m_suffix.size()) {
-        // ... and remove them
-        const std::string fullname = path + name;
-        my_delete(fullname.c_str(), MYF(0));
-      }
-    }
-
-    // Release the directory entry
-    my_dirend(dir_info);
   }
 }
 
