@@ -16,7 +16,7 @@
 #include "db/db_impl/db_impl.h"
 #include "monitoring/histogram.h"
 #include "port/port.h"
-#include "rocksdb/advanced_cache.h"
+#include "rocksdb/cache.h"
 #include "rocksdb/convenience.h"
 #include "rocksdb/db.h"
 #include "rocksdb/env.h"
@@ -77,13 +77,11 @@ DEFINE_bool(lean, false,
             "If true, no additional computation is performed besides cache "
             "operations.");
 
-DEFINE_bool(early_exit, false,
-            "Exit before deallocating most memory. Good for malloc stats, e.g."
-            "MALLOC_CONF=\"stats_print:true\"");
-
+#ifndef ROCKSDB_LITE
 DEFINE_string(secondary_cache_uri, "",
               "Full URI for creating a custom secondary cache object");
 static class std::shared_ptr<ROCKSDB_NAMESPACE::SecondaryCache> secondary_cache;
+#endif  // ROCKSDB_LITE
 
 DEFINE_string(cache_type, "lru_cache", "Type of block cache.");
 
@@ -259,15 +257,12 @@ void DeleteFn(Cache::ObjectPtr value, MemoryAllocator* /*alloc*/) {
   delete[] static_cast<char*>(value);
 }
 
-Cache::CacheItemHelper helper1_wos(CacheEntryRole::kDataBlock, DeleteFn);
 Cache::CacheItemHelper helper1(CacheEntryRole::kDataBlock, DeleteFn, SizeFn,
-                               SaveToFn, CreateFn, &helper1_wos);
-Cache::CacheItemHelper helper2_wos(CacheEntryRole::kIndexBlock, DeleteFn);
+                               SaveToFn, CreateFn);
 Cache::CacheItemHelper helper2(CacheEntryRole::kIndexBlock, DeleteFn, SizeFn,
-                               SaveToFn, CreateFn, &helper2_wos);
-Cache::CacheItemHelper helper3_wos(CacheEntryRole::kFilterBlock, DeleteFn);
+                               SaveToFn, CreateFn);
 Cache::CacheItemHelper helper3(CacheEntryRole::kFilterBlock, DeleteFn, SizeFn,
-                               SaveToFn, CreateFn, &helper3_wos);
+                               SaveToFn, CreateFn);
 }  // namespace
 
 class CacheBench {
@@ -310,6 +305,7 @@ class CacheBench {
       LRUCacheOptions opts(FLAGS_cache_size, FLAGS_num_shard_bits,
                            false /* strict_capacity_limit */,
                            0.5 /* high_pri_pool_ratio */);
+#ifndef ROCKSDB_LITE
       if (!FLAGS_secondary_cache_uri.empty()) {
         Status s = SecondaryCache::CreateFromString(
             ConfigOptions(), FLAGS_secondary_cache_uri, &secondary_cache);
@@ -322,6 +318,7 @@ class CacheBench {
         }
         opts.secondary_cache = secondary_cache;
       }
+#endif  // ROCKSDB_LITE
 
       cache_ = NewLRUCache(opts);
     } else {
@@ -551,7 +548,7 @@ class CacheBench {
         }
         // do lookup
         handle = cache_->Lookup(key, &helper2, /*context*/ nullptr,
-                                Cache::Priority::LOW);
+                                Cache::Priority::LOW, true);
         if (handle) {
           if (!FLAGS_lean) {
             // do something with the data
@@ -580,7 +577,7 @@ class CacheBench {
         }
         // do lookup
         handle = cache_->Lookup(key, &helper2, /*context*/ nullptr,
-                                Cache::Priority::LOW);
+                                Cache::Priority::LOW, true);
         if (handle) {
           if (!FLAGS_lean) {
             // do something with the data
@@ -597,10 +594,6 @@ class CacheBench {
       }
       thread->latency_ns_hist.Add(timer.ElapsedNanos());
     }
-    if (FLAGS_early_exit) {
-      MutexLock l(thread->shared->GetMutex());
-      exit(0);
-    }
     if (handle) {
       cache_->Release(handle);
       handle = nullptr;
