@@ -23,13 +23,14 @@ namespace ROCKSDB_NAMESPACE {
 const uint64_t kRangeTombstoneSentinel =
     PackSequenceAndType(kMaxSequenceNumber, kTypeRangeDeletion);
 
-int sstableKeyCompare(const Comparator* uc, const Slice& a, const Slice& b) {
-  auto c = uc->CompareWithoutTimestamp(ExtractUserKey(a), ExtractUserKey(b));
+int sstableKeyCompare(const Comparator* user_cmp, const InternalKey& a,
+                      const InternalKey& b) {
+  auto c = user_cmp->CompareWithoutTimestamp(a.user_key(), b.user_key());
   if (c != 0) {
     return c;
   }
-  auto a_footer = ExtractInternalKeyFooter(a);
-  auto b_footer = ExtractInternalKeyFooter(b);
+  auto a_footer = ExtractInternalKeyFooter(a.Encode());
+  auto b_footer = ExtractInternalKeyFooter(b.Encode());
   if (a_footer == kRangeTombstoneSentinel) {
     if (b_footer != kRangeTombstoneSentinel) {
       return -1;
@@ -464,11 +465,6 @@ bool Compaction::IsTrivialMove() const {
     return false;
   }
 
-  if (compaction_reason_ == CompactionReason::kChangeTemperature) {
-    // Changing temperature usually requires rewriting the file.
-    return false;
-  }
-
   // Used in universal compaction, where trivial move can be done if the
   // input files are non overlapping
   if ((mutable_cf_options_.compaction_options_universal.allow_trivial_move) &&
@@ -485,24 +481,25 @@ bool Compaction::IsTrivialMove() const {
 
   // assert inputs_.size() == 1
 
-  if (output_level_ + 1 < number_levels_) {
-    std::unique_ptr<SstPartitioner> partitioner = CreateSstPartitioner();
-    for (const auto& file : inputs_.front().files) {
-      std::vector<FileMetaData*> file_grand_parents;
-      input_vstorage_->GetOverlappingInputs(output_level_ + 1, &file->smallest,
-                                            &file->largest,
-                                            &file_grand_parents);
-      const auto compaction_size =
-          file->fd.GetFileSize() + TotalFileSize(file_grand_parents);
-      if (compaction_size > max_compaction_bytes_) {
-        return false;
-      }
+  std::unique_ptr<SstPartitioner> partitioner = CreateSstPartitioner();
 
-      if (partitioner.get() != nullptr) {
-        if (!partitioner->CanDoTrivialMove(file->smallest.user_key(),
-                                           file->largest.user_key())) {
-          return false;
-        }
+  for (const auto& file : inputs_.front().files) {
+    std::vector<FileMetaData*> file_grand_parents;
+    if (output_level_ + 1 >= number_levels_) {
+      continue;
+    }
+    input_vstorage_->GetOverlappingInputs(output_level_ + 1, &file->smallest,
+                                          &file->largest, &file_grand_parents);
+    const auto compaction_size =
+        file->fd.GetFileSize() + TotalFileSize(file_grand_parents);
+    if (compaction_size > max_compaction_bytes_) {
+      return false;
+    }
+
+    if (partitioner.get() != nullptr) {
+      if (!partitioner->CanDoTrivialMove(file->smallest.user_key(),
+                                         file->largest.user_key())) {
+        return false;
       }
     }
   }
@@ -561,49 +558,6 @@ bool Compaction::KeyNotExistsBeyondOutputLevel(
   return false;
 }
 
-bool Compaction::KeyRangeNotExistsBeyondOutputLevel(
-    const Slice& begin_key, const Slice& end_key,
-    std::vector<size_t>* level_ptrs) const {
-  assert(input_version_ != nullptr);
-  assert(level_ptrs != nullptr);
-  assert(level_ptrs->size() == static_cast<size_t>(number_levels_));
-  assert(cfd_->user_comparator()->CompareWithoutTimestamp(begin_key, end_key) <
-         0);
-  if (bottommost_level_) {
-    return true /* does not overlap */;
-  } else if (output_level_ != 0 &&
-             cfd_->ioptions()->compaction_style == kCompactionStyleLevel) {
-    const Comparator* user_cmp = cfd_->user_comparator();
-    for (int lvl = output_level_ + 1; lvl < number_levels_; lvl++) {
-      const std::vector<FileMetaData*>& files =
-          input_vstorage_->LevelFiles(lvl);
-      for (; level_ptrs->at(lvl) < files.size(); level_ptrs->at(lvl)++) {
-        auto* f = files[level_ptrs->at(lvl)];
-        // Advance until the first file with begin_key <= f->largest.user_key()
-        if (user_cmp->CompareWithoutTimestamp(begin_key,
-                                              f->largest.user_key()) > 0) {
-          continue;
-        }
-        // We know that the previous file prev_f, if exists, has
-        // prev_f->largest.user_key() < begin_key.
-        if (user_cmp->CompareWithoutTimestamp(end_key,
-                                              f->smallest.user_key()) <= 0) {
-          // not overlapping with this level
-          break;
-        } else {
-          // We have:
-          // - begin_key < end_key,
-          // - begin_key <= f->largest.user_key(), and
-          // - end_key > f->smallest.user_key()
-          return false /* overlap */;
-        }
-      }
-    }
-    return true /* does not overlap */;
-  }
-  return false /* overlaps */;
-};
-
 // Mark (or clear) each file that is being compacted
 void Compaction::MarkFilesBeingCompacted(bool mark_as_compacted) {
   for (size_t i = 0; i < num_input_levels(); i++) {
