@@ -18,7 +18,7 @@
 #include "port/port.h"
 #include "test_util/sync_point.h"
 #include "util/random.h"
-#include "util/rate_limiter_impl.h"
+#include "util/rate_limiter.h"
 
 namespace ROCKSDB_NAMESPACE {
 
@@ -162,9 +162,6 @@ Status FilePrefetchBuffer::Prefetch(const IOOptions& opts,
 
   Status s = Read(opts, reader, rate_limiter_priority, read_len, chunk_len,
                   rounddown_offset, curr_);
-  if (usage_ == FilePrefetchBufferUsage::kTableOpenPrefetchTail && s.ok()) {
-    RecordInHistogram(stats_, TABLE_OPEN_PREFETCH_TAIL_READ_BYTES, read_len);
-  }
   return s;
 }
 
@@ -572,11 +569,10 @@ Status FilePrefetchBuffer::PrefetchAsyncInternal(
     bufs_[second].offset_ = rounddown_start2;
     assert(roundup_len2 >= chunk_len2);
     uint64_t read_len2 = static_cast<size_t>(roundup_len2 - chunk_len2);
-    s = ReadAsync(opts, reader, read_len2, rounddown_start2, second);
-    if (!s.ok()) {
+    Status tmp_s = ReadAsync(opts, reader, read_len2, rounddown_start2, second);
+    if (!tmp_s.ok()) {
       DestroyAndClearIOHandle(second);
       bufs_[second].buffer_.Clear();
-      return s;
     }
   }
 
@@ -612,22 +608,6 @@ bool FilePrefetchBuffer::TryReadFromCache(const IOOptions& opts,
                                           Slice* result, Status* status,
                                           Env::IOPriority rate_limiter_priority,
                                           bool for_compaction /* = false */) {
-  bool ret = TryReadFromCacheUntracked(opts, reader, offset, n, result, status,
-                                       rate_limiter_priority, for_compaction);
-  if (usage_ == FilePrefetchBufferUsage::kTableOpenPrefetchTail && enable_) {
-    if (ret) {
-      RecordTick(stats_, TABLE_OPEN_PREFETCH_TAIL_HIT);
-    } else {
-      RecordTick(stats_, TABLE_OPEN_PREFETCH_TAIL_MISS);
-    }
-  }
-  return ret;
-}
-
-bool FilePrefetchBuffer::TryReadFromCacheUntracked(
-    const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
-    size_t n, Slice* result, Status* status,
-    Env::IOPriority rate_limiter_priority, bool for_compaction /* = false */) {
   if (track_min_offset_ && offset < min_offset_read_) {
     min_offset_read_ = static_cast<size_t>(offset);
   }
@@ -685,22 +665,6 @@ bool FilePrefetchBuffer::TryReadFromCacheAsync(
     const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
     size_t n, Slice* result, Status* status,
     Env::IOPriority rate_limiter_priority) {
-  bool ret = TryReadFromCacheAsyncUntracked(opts, reader, offset, n, result,
-                                            status, rate_limiter_priority);
-  if (usage_ == FilePrefetchBufferUsage::kTableOpenPrefetchTail && enable_) {
-    if (ret) {
-      RecordTick(stats_, TABLE_OPEN_PREFETCH_TAIL_HIT);
-    } else {
-      RecordTick(stats_, TABLE_OPEN_PREFETCH_TAIL_MISS);
-    }
-  }
-  return ret;
-}
-
-bool FilePrefetchBuffer::TryReadFromCacheAsyncUntracked(
-    const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
-    size_t n, Slice* result, Status* status,
-    Env::IOPriority rate_limiter_priority) {
   if (track_min_offset_ && offset < min_offset_read_) {
     min_offset_read_ = static_cast<size_t>(offset);
   }
@@ -831,7 +795,7 @@ Status FilePrefetchBuffer::PrefetchAsync(const IOOptions& opts,
   bool is_eligible_for_prefetching = false;
   if (readahead_size_ > 0 &&
       (!implicit_auto_readahead_ ||
-       num_file_reads_ >= num_file_reads_for_auto_readahead_)) {
+       num_file_reads_ + 1 >= num_file_reads_for_auto_readahead_)) {
     is_eligible_for_prefetching = true;
   }
 
@@ -941,7 +905,6 @@ Status FilePrefetchBuffer::PrefetchAsync(const IOOptions& opts,
     prev_len_ = 0;
   }
   if (read_len2) {
-    TEST_SYNC_POINT("FilePrefetchBuffer::PrefetchAsync:ExtraPrefetching");
     s = ReadAsync(opts, reader, read_len2, rounddown_start2, second);
     if (!s.ok()) {
       DestroyAndClearIOHandle(second);
