@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,33 +26,54 @@
 #include <stddef.h>
 #include <sys/types.h>
 
+#include <array>
+#include <memory>
+#include <new>
+#include <utility>
+
+#include "decimal.h"
+#include "field_types.h"
 #include "lex_string.h"
+#include "my_alloc.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_table_map.h"
+#include "mysql/strings/dtoa.h"
+#include "mysql/strings/m_ctype.h"
+#include "mysql_time.h"
 #include "mysys_err.h"
+#include "sql-common/json_dom.h"
+#include "sql-common/my_decimal.h"
 #include "sql/item.h"
 #include "sql/item_cmpfunc.h"
 #include "sql/item_create.h"
 #include "sql/item_strfunc.h"
+#include "sql/item_sum.h"
 #include "sql/item_timefunc.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
 #include "sql/tztime.h"
+#include "sql_string.h"
+#include "string_with_len.h"
 #include "unittest/gunit/fake_table.h"
+#include "unittest/gunit/mock_field_long.h"
 #include "unittest/gunit/mock_field_timestamp.h"
+#include "unittest/gunit/mysys_util.h"
 #include "unittest/gunit/test_utils.h"
 
 namespace item_unittest {
 
-using ::testing::Return;
 using my_testing::Mock_error_handler;
 using my_testing::Server_initializer;
+using std::array;
+using std::make_pair;
+using std::pair;
+using ::testing::Return;
 
 class ItemTest : public ::testing::Test {
  protected:
-  virtual void SetUp() { initializer.SetUp(); }
-  virtual void TearDown() { initializer.TearDown(); }
+  void SetUp() override { initializer.SetUp(); }
+  void TearDown() override { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -66,15 +87,26 @@ class ItemTest : public ::testing::Test {
 class Mock_field_long : public Field_long {
  public:
   Mock_field_long(uint32 length)
-      : Field_long(0,            // ptr_arg
+      : Field_long(nullptr,      // ptr_arg
                    length,       // len_arg
-                   NULL,         // null_ptr_arg
+                   nullptr,      // null_ptr_arg
                    0,            // null_bit_arg
                    Field::NONE,  // auto_flags_arg
-                   0,            // field_name_arg
+                   nullptr,      // field_name_arg
                    false,        // zero_arg
                    false)        // unsigned_arg
-  {}
+  {
+    table = new Fake_TABLE(this);
+    ptr = table->record[0];
+
+    // Make it possible to write into this field
+    bitmap_set_bit(table->write_set, 0);
+  }
+
+  ~Mock_field_long() override {
+    delete static_cast<Fake_TABLE *>(table);
+    table = nullptr;
+  }
 
   // Avoid warning about hiding other overloaded versions of store().
   using Field_long::store;
@@ -96,18 +128,18 @@ class Mock_field_string : public Field_string {
 
  public:
   Mock_field_string(uint32 length, const CHARSET_INFO *cs = &my_charset_latin1)
-      : Field_string(0,            // ptr_arg
+      : Field_string(nullptr,      // ptr_arg
                      length,       // len_arg
-                     NULL,         // null_ptr_arg
+                     nullptr,      // null_ptr_arg
                      0,            // null_bit_arg
                      Field::NONE,  // auto_flags_arg
-                     NULL,         // field_name_arg
+                     nullptr,      // field_name_arg
                      cs)           // char set
   {
     m_fake_tbl = new Fake_TABLE(this);
 
     // Allocate place for storing the field value
-    ptr = new uchar[length];
+    ptr = m_fake_tbl->record[0];
 
     // Make it possible to write into this field
     bitmap_set_bit(m_fake_tbl->write_set, 0);
@@ -120,10 +152,8 @@ class Mock_field_string : public Field_string {
   }
 
   ~Mock_field_string() {
-    delete[] ptr;
-    ptr = NULL;
     delete m_fake_tbl;
-    m_fake_tbl = NULL;
+    m_fake_tbl = nullptr;
   }
 };
 
@@ -138,16 +168,16 @@ class Mock_field_varstring : public Field_varstring {
  public:
   Mock_field_varstring(uint32 length, TABLE_SHARE *share,
                        const CHARSET_INFO *cs = &my_charset_latin1)
-      : Field_varstring(length,  // len_arg
-                        false,   // maybe_null_arg
-                        NULL,    // field_name_arg
-                        share,   // share
-                        cs)      // char set
+      : Field_varstring(length,   // len_arg
+                        false,    // maybe_null_arg
+                        nullptr,  // field_name_arg
+                        share,    // share
+                        cs)       // char set
   {
     m_fake_tbl = new Fake_TABLE(this);
 
     // Allocate place for storing the field value
-    ptr = new uchar[length + 1];
+    ptr = m_fake_tbl->record[0];
 
     // Make it possible to write into this field
     bitmap_set_bit(m_fake_tbl->write_set, 0);
@@ -160,10 +190,8 @@ class Mock_field_varstring : public Field_varstring {
   }
 
   ~Mock_field_varstring() {
-    delete[] ptr;
-    ptr = NULL;
     delete m_fake_tbl;
-    m_fake_tbl = NULL;
+    m_fake_tbl = nullptr;
   }
 };
 
@@ -202,7 +230,7 @@ TEST_F(ItemTest, ItemInt) {
   EXPECT_TRUE(item_int->eq(item_int, true));
 
   String print_val;
-  item_int->print(&print_val, QT_ORDINARY);
+  item_int->print(thd(), &print_val, QT_ORDINARY);
   EXPECT_STREQ(stringbuf, print_val.c_ptr_safe());
 
   const uint precision = item_int->decimal_precision();
@@ -210,7 +238,7 @@ TEST_F(ItemTest, ItemInt) {
 
   item_int->neg();
   EXPECT_EQ(-val, item_int->val_int());
-  EXPECT_EQ(precision - 1, item_int->decimal_precision());
+  EXPECT_EQ(precision, item_int->decimal_precision());
 
   // Functions inherited from parent class(es).
   const table_map tmap = 0;
@@ -250,7 +278,7 @@ TEST_F(ItemTest, ItemString) {
   EXPECT_EQ(MYSQL_TYPE_STRING, field_string.type());
 
   // CHAR field for testing strings with illegal values
-  Mock_field_string field_string_utf8(20, &my_charset_utf8_general_ci);
+  Mock_field_string field_string_utf8(20, &my_charset_utf8mb3_general_ci);
   EXPECT_EQ(MYSQL_TYPE_STRING, field_string_utf8.type());
 
   /*
@@ -297,7 +325,7 @@ TEST_F(ItemTest, ItemString) {
 
   // VARCHAR field for testing strings with illegal values
   Mock_field_varstring field_varstring_utf8(20, &table_share,
-                                            &my_charset_utf8_general_ci);
+                                            &my_charset_utf8mb3_general_ci);
   EXPECT_EQ(MYSQL_TYPE_VARCHAR, field_varstring_utf8.type());
 
   /*
@@ -351,10 +379,107 @@ TEST_F(ItemTest, ItemEqual) {
       new Item_equal(new Item_string(STRING_WITH_LEN(foo), &my_charset_bin),
                      new Item_field(&mft));
 
-  EXPECT_FALSE(item_equal->fix_fields(thd(), NULL));
+  EXPECT_FALSE(item_equal->fix_fields(thd(), nullptr));
   EXPECT_EQ(1, item_equal->val_int());
 }
 
+TEST_F(ItemTest, ItemViewRef) {
+  Mock_field_long f_field1(true);
+  Mock_field_long f_field2(true);
+  Item_field *field1 = new Item_field(&f_field1);
+  Item_field *field2 = new Item_field(&f_field2);
+  Item *field3 = new Item_int(123);
+
+  // Create a view reference over a constant expression from an inner
+  // table of an outer join.
+  Table_ref table;
+  table.outer_join = true;
+  table.set_tableno(2);
+  Item_view_ref *view_ref =
+      new Item_view_ref(&thd()->lex->current_query_block()->context, &field3,
+                        nullptr, nullptr, "t1", "f1", &table);
+
+  Item_func_eq *eq1 = new Item_func_eq(field1, field2);
+  Item_func_eq *eq2 = new Item_func_eq(field1, field3);
+  Item_func_eq *eq3 = new Item_func_eq(field1, view_ref);
+
+  // True because both arguments are fields.
+  EXPECT_TRUE(eq1->contains_only_equi_join_condition());
+  // False because the right side argument is a constant and
+  // therefore concluded as a filter.
+  EXPECT_FALSE(eq2->contains_only_equi_join_condition());
+  // Even though the used table information for the right side
+  // argument says it is not a constant expression, it examines
+  // the underlying expression in the view reference and concludes
+  // it as constant expression there by making it a filter and
+  // not a equi-join condition.
+  EXPECT_FALSE(eq3->contains_only_equi_join_condition());
+}
+
+TEST_F(ItemTest, ItemRollupSwitcher) {
+  Mock_field_long f_field1(true);
+  Item_field *field1 = new Item_field(&f_field1);
+  POS p;
+  Item_sum_sum *agg1 = new Item_sum_sum(p, field1, false, nullptr);
+  Item_sum_sum *agg2 = new Item_sum_sum(p, field1, false, nullptr);
+
+  List<Item> li1;
+  li1.push_back(agg1);
+  Item_rollup_sum_switcher *sw1 = new Item_rollup_sum_switcher(&li1);
+  List<Item> li2;
+  li2.push_back(agg2);
+  Item_rollup_sum_switcher *sw2 = new Item_rollup_sum_switcher(&li2);
+  EXPECT_TRUE(agg1->eq(agg2, false));
+  EXPECT_TRUE(agg1->eq(sw2, false));
+  EXPECT_TRUE(sw1->eq(sw2, false));
+  EXPECT_TRUE(sw1->eq(agg2, false));
+
+  EXPECT_FALSE(agg1->is_rollup_sum_wrapper());
+  EXPECT_TRUE(sw1->is_rollup_sum_wrapper());
+}
+
+TEST_F(ItemTest, ItemEqualEq) {
+  Mock_field_timestamp field1;
+  field1.field_name = "field1";
+  Mock_field_timestamp field2;
+  field2.field_name = "field2";
+
+  Item_equal *item_equal1 =
+      new Item_equal(new Item_field(&field1), new Item_field(&field2));
+  Item_equal *item_equal2 =
+      new Item_equal(new Item_field(&field2), new Item_field(&field1));
+  Item_equal *item_equal3 =
+      new Item_equal(new Item_int(123), new Item_field(&field1));
+  Item_equal *item_equal4 =
+      new Item_equal(new Item_int(123), new Item_field(&field1));
+  item_equal4->add(new Item_field(&field2));
+
+  Item_func_equal *spaceship =
+      new Item_func_equal(new Item_field(&field1), new Item_field(&field2));
+
+  // Create an array of <int, Item *> pairs. If the int members of two pairs are
+  // the same, the Item * members should be considered equal by Item::eq(). (The
+  // first two objects are equivalent. The rest of them are different.)
+  array<pair<int, Item *>, 5> items{
+      make_pair(1, item_equal1), make_pair(1, item_equal2),
+      make_pair(2, item_equal3), make_pair(3, item_equal4),
+      make_pair(4, spaceship),
+  };
+
+  // Now compare all items with each other.
+  for (bool binary_cmp : {true, false}) {
+    for (pair<int, Item *> it1 : items) {
+      for (pair<int, Item *> it2 : items) {
+        EXPECT_EQ(it1.first == it2.first,
+                  it1.second->eq(it2.second, binary_cmp))
+            << "Item 1: " << ItemToString(it1.second)
+            << ", Item 2: " << ItemToString(it2.second)
+            << ", binary_cmp = " << binary_cmp;
+      }
+    }
+  }
+}
+
 TEST_F(ItemTest, ItemFuncExportSet) {
   String str;
   Item *on_string = new Item_string(STRING_WITH_LEN("on"), &my_charset_bin);
@@ -365,9 +490,9 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set =
         new Item_func_export_set(POS(), new Item_int(2), on_string, off_string,
                                  sep_string, new Item_int(4));
-    Parse_context pc(thd(), thd()->lex->current_select());
+    Parse_context pc(thd(), thd()->lex->current_query_block());
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
-    EXPECT_FALSE(export_set->fix_fields(thd(), NULL));
+    EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(&str, export_set->val_str(&str));
     EXPECT_STREQ("off,on,off,off", str.c_ptr_safe());
   }
@@ -376,9 +501,9 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set =
         new Item_func_export_set(POS(), new Item_int(2), on_string, off_string,
                                  sep_string, new Item_int(0));
-    Parse_context pc(thd(), thd()->lex->current_select());
+    Parse_context pc(thd(), thd()->lex->current_query_block());
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
-    EXPECT_FALSE(export_set->fix_fields(thd(), NULL));
+    EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(&str, export_set->val_str(&str));
     EXPECT_STREQ("", str.c_ptr_safe());
   }
@@ -387,12 +512,12 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Bug#11765562 58545:
     EXPORT_SET() CAN BE USED TO MAKE ENTIRE SERVER COMPLETELY UNRESPONSIVE
    */
-  const ulong max_size = 1024;
-  const ulonglong repeat = max_size / 2;
+  const ulong max_packet_size = 1024;
+  const ulonglong repeat = max_packet_size / 2;
   Item *item_int_repeat = new Item_int(repeat);
   Item *string_x = new Item_string(STRING_WITH_LEN("x"), &my_charset_bin);
-  String *const null_string = NULL;
-  thd()->variables.max_allowed_packet = max_size;
+  String *const null_string = nullptr;
+  thd()->variables.max_allowed_packet = max_packet_size;
   {
     // Testing overflow caused by 'on-string'.
     Mock_error_handler error_handler(thd(), ER_WARN_ALLOWED_PACKET_OVERFLOWED);
@@ -400,10 +525,10 @@ TEST_F(ItemTest, ItemFuncExportSet) {
         POS(), new Item_int(0xff),
         new Item_func_repeat(POS(), string_x, item_int_repeat), string_x,
         sep_string);
-    Parse_context pc(thd(), thd()->lex->current_select());
+    Parse_context pc(thd(), thd()->lex->current_query_block());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
-    EXPECT_FALSE(export_set->fix_fields(thd(), NULL));
+    EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(null_string, export_set->val_str(&str));
     EXPECT_STREQ("", str.c_ptr_safe());
     EXPECT_EQ(1, error_handler.handle_called());
@@ -414,10 +539,10 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set = new Item_func_export_set(
         POS(), new Item_int(0xff), string_x,
         new Item_func_repeat(POS(), string_x, item_int_repeat), sep_string);
-    Parse_context pc(thd(), thd()->lex->current_select());
+    Parse_context pc(thd(), thd()->lex->current_query_block());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
-    EXPECT_FALSE(export_set->fix_fields(thd(), NULL));
+    EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(null_string, export_set->val_str(&str));
     EXPECT_STREQ("", str.c_ptr_safe());
     EXPECT_EQ(1, error_handler.handle_called());
@@ -428,10 +553,10 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set = new Item_func_export_set(
         POS(), new Item_int(0xff), string_x, string_x,
         new Item_func_repeat(POS(), string_x, item_int_repeat));
-    Parse_context pc(thd(), thd()->lex->current_select());
+    Parse_context pc(thd(), thd()->lex->current_query_block());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
-    EXPECT_FALSE(export_set->fix_fields(thd(), NULL));
+    EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(null_string, export_set->val_str(&str));
     EXPECT_STREQ("", str.c_ptr_safe());
     EXPECT_EQ(1, error_handler.handle_called());
@@ -450,10 +575,10 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set = new Item_func_export_set(
         POS(), new Item_string(STRING_WITH_LEN("1111111"), &my_charset_bin),
         lpad, new Item_int(1));
-    Parse_context pc(thd(), thd()->lex->current_select());
+    Parse_context pc(thd(), thd()->lex->current_query_block());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
-    EXPECT_FALSE(export_set->fix_fields(thd(), NULL));
+    EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(null_string, export_set->val_str(&str));
     EXPECT_STREQ("", str.c_ptr_safe());
     EXPECT_EQ(1, error_handler.handle_called());
@@ -467,10 +592,10 @@ TEST_F(ItemTest, ItemFuncIntDivOverflow) {
   const char divisor_str[] = "0.5";
   Item_float *dividend = new Item_float(dividend_str, sizeof(dividend_str));
   Item_float *divisor = new Item_float(divisor_str, sizeof(divisor_str));
-  Item_func_int_div *quotient = new Item_func_int_div(dividend, divisor);
+  Item_func_div_int *quotient = new Item_func_div_int(dividend, divisor);
 
   Mock_error_handler error_handler(thd(), ER_TRUNCATED_WRONG_VALUE);
-  EXPECT_FALSE(quotient->fix_fields(thd(), NULL));
+  EXPECT_FALSE(quotient->fix_fields(thd(), nullptr));
   initializer.set_expected_error(ER_DATA_OUT_OF_RANGE);
   quotient->val_int();
 }
@@ -481,10 +606,10 @@ TEST_F(ItemTest, ItemFuncIntDivUnderflow) {
   const char divisor_str[] = "1.7976931348623157E+308";
   Item_float *dividend = new Item_float(dividend_str, sizeof(dividend_str));
   Item_float *divisor = new Item_float(divisor_str, sizeof(divisor_str));
-  Item_func_int_div *quotient = new Item_func_int_div(dividend, divisor);
+  Item_func_div_int *quotient = new Item_func_div_int(dividend, divisor);
 
   Mock_error_handler error_handler(thd(), ER_TRUNCATED_WRONG_VALUE);
-  EXPECT_FALSE(quotient->fix_fields(thd(), NULL));
+  EXPECT_FALSE(quotient->fix_fields(thd(), nullptr));
   EXPECT_EQ(0, quotient->val_int());
 }
 
@@ -493,7 +618,7 @@ TEST_F(ItemTest, ItemFuncNegLongLongMin) {
   const longlong longlong_min = LLONG_MIN;
   Item_func_neg *item_neg = new Item_func_neg(new Item_int(longlong_min));
 
-  EXPECT_FALSE(item_neg->fix_fields(thd(), NULL));
+  EXPECT_FALSE(item_neg->fix_fields(thd(), nullptr));
   initializer.set_expected_error(ER_DATA_OUT_OF_RANGE);
   EXPECT_EQ(0, item_neg->int_op());
 }
@@ -507,11 +632,11 @@ TEST_F(ItemTest, ItemFuncSetUserVar) {
   Item_decimal *item_dec = new Item_decimal(val1, false);
   Item_string *item_str = new Item_string("1", 1, &my_charset_latin1);
 
-  LEX_STRING var_name = {C_STRING_WITH_LEN("a")};
+  LEX_CSTRING var_name = {STRING_WITH_LEN("a")};
   Item_func_set_user_var *user_var =
-      new Item_func_set_user_var(var_name, item_str, false);
+      new Item_func_set_user_var(var_name, item_str);
   EXPECT_FALSE(user_var->set_entry(thd(), true));
-  EXPECT_FALSE(user_var->fix_fields(thd(), NULL));
+  EXPECT_FALSE(user_var->fix_fields(thd(), nullptr));
   EXPECT_EQ(val1, user_var->val_int());
 
   my_decimal decimal;
@@ -529,7 +654,7 @@ TEST_F(ItemTest, OutOfMemory) {
   Item_int *item = new Item_int(42);
   EXPECT_NE(nullptr, item);
 
-#if !defined(DBUG_OFF)
+#if !defined(NDEBUG)
   // Setting debug flags triggers enter/exit trace, so redirect to /dev/null.
   DBUG_SET("o," IF_WIN("NUL", "/dev/null"));
 
@@ -547,7 +672,7 @@ TEST_F(ItemTest, OutOfMemory) {
 // We never use dynamic_cast, but we expect it to work.
 TEST_F(ItemTest, DynamicCast) {
   Item *item = new Item_int(42);
-  const Item_int *null_item = NULL;
+  const Item_int *null_item = nullptr;
   EXPECT_NE(null_item, dynamic_cast<Item_int *>(item));
 }
 
@@ -558,7 +683,7 @@ TEST_F(ItemTest, ItemFuncXor) {
 
   Item_func_xor *item_xor = new Item_func_xor(item_zero, item_one_a);
 
-  EXPECT_FALSE(item_xor->fix_fields(thd(), NULL));
+  EXPECT_FALSE(item_xor->fix_fields(thd(), nullptr));
   EXPECT_EQ(1, item_xor->val_int());
   EXPECT_EQ(1U, item_xor->decimal_precision());
 
@@ -566,28 +691,28 @@ TEST_F(ItemTest, ItemFuncXor) {
 
   Item_func_xor *item_xor_same = new Item_func_xor(item_one_a, item_one_b);
 
-  EXPECT_FALSE(item_xor_same->fix_fields(thd(), NULL));
+  EXPECT_FALSE(item_xor_same->fix_fields(thd(), nullptr));
   EXPECT_EQ(0, item_xor_same->val_int());
   EXPECT_FALSE(item_xor_same->val_bool());
   EXPECT_FALSE(item_xor_same->is_null());
 
   String print_buffer;
-  item_xor->print(&print_buffer, QT_ORDINARY);
+  item_xor->print(thd(), &print_buffer, QT_ORDINARY);
   EXPECT_STREQ("(0 xor 1)", print_buffer.c_ptr_safe());
 
-  Item *neg_xor = item_xor->neg_transformer(thd());
-  EXPECT_FALSE(neg_xor->fix_fields(thd(), NULL));
+  Item *neg_xor = item_xor->truth_transformer(thd(), Item::BOOL_NEGATED);
+  EXPECT_FALSE(neg_xor->fix_fields(thd(), nullptr));
   EXPECT_EQ(0, neg_xor->val_int());
   EXPECT_DOUBLE_EQ(0.0, neg_xor->val_real());
   EXPECT_FALSE(neg_xor->val_bool());
   EXPECT_FALSE(neg_xor->is_null());
 
   print_buffer = String();
-  neg_xor->print(&print_buffer, QT_ORDINARY);
+  neg_xor->print(thd(), &print_buffer, QT_ORDINARY);
   EXPECT_STREQ("((not(0)) xor 1)", print_buffer.c_ptr_safe());
 
   Item_func_xor *item_xor_null = new Item_func_xor(item_zero, new Item_null());
-  EXPECT_FALSE(item_xor_null->fix_fields(thd(), NULL));
+  EXPECT_FALSE(item_xor_null->fix_fields(thd(), nullptr));
 
   EXPECT_EQ(0, item_xor_null->val_int());
   EXPECT_TRUE(item_xor_null->is_null());
@@ -598,10 +723,10 @@ TEST_F(ItemTest, ItemFuncXor) {
 */
 TEST_F(ItemTest, MysqlTimeCache) {
   String str_buff, *str;
-  MYSQL_TIME datetime6 = {
-      2011, 11, 7, 10, 20, 30, 123456, 0, MYSQL_TIMESTAMP_DATETIME};
-  MYSQL_TIME time6 = {0, 0, 0, 10, 20, 30, 123456, 0, MYSQL_TIMESTAMP_TIME};
-  struct timeval tv6 = {1320661230, 123456};
+  MysqlTime datetime6(2011, 11, 7, 10, 20, 30, 123456, false,
+                      MYSQL_TIMESTAMP_DATETIME);
+  MysqlTime time6(0, 0, 0, 10, 20, 30, 123456, false, MYSQL_TIMESTAMP_TIME);
+  my_timeval tv6 = {1320661230, 123456};
   const MYSQL_TIME *ltime;
   MYSQL_TIME_cache cache;
 
@@ -680,8 +805,8 @@ TEST_F(ItemTest, MysqlTimeCache) {
   /*
     Testing DATETIME(5)
   */
-  MYSQL_TIME datetime5 = {
-      2011, 11, 7, 10, 20, 30, 123450, 0, MYSQL_TIMESTAMP_DATETIME};
+  MysqlTime datetime5 = {
+      2011, 11, 7, 10, 20, 30, 123450, false, MYSQL_TIMESTAMP_DATETIME};
   cache.set_datetime(&datetime5, 5);
   EXPECT_EQ(1840440237558456890LL, cache.val_packed());
   EXPECT_EQ(5, cache.decimals());
@@ -698,7 +823,7 @@ TEST_F(ItemTest, MysqlTimeCache) {
     Testing DATE.
     Initializing from MYSQL_TIME.
   */
-  MYSQL_TIME date = {2011, 11, 7, 0, 0, 0, 0, 0, MYSQL_TIMESTAMP_DATE};
+  MysqlTime date(2011, 11, 7, 0, 0, 0, 0, false, MYSQL_TIMESTAMP_DATE);
   cache.set_date(&date);
   EXPECT_EQ(1840439528385413120LL, cache.val_packed());
   EXPECT_EQ(0, cache.decimals());
@@ -720,8 +845,8 @@ TEST_F(ItemTest, MysqlTimeCache) {
 
 extern "C" {
 // Verifies that Item_func_conv::val_str does not call my_strntoll()
-longlong fail_strntoll(const CHARSET_INFO *, const char *, size_t, int, char **,
-                       int *) {
+longlong fail_strntoll(const CHARSET_INFO *, const char *, size_t, int,
+                       const char **, int *) {
   ADD_FAILURE() << "Unexpected call";
   return 0;
 }
@@ -749,10 +874,10 @@ TEST_F(ItemTest, ItemFuncConvIntMin) {
   Item *item_conv = new Item_func_conv(POS(), new Item_string("5", 1, &charset),
                                        new Item_int(INT_MIN),   // from_base
                                        new Item_int(INT_MIN));  // to_base
-  Parse_context pc(thd(), thd()->lex->current_select());
+  Parse_context pc(thd(), thd()->lex->current_query_block());
   EXPECT_FALSE(item_conv->itemize(&pc, &item_conv));
-  EXPECT_FALSE(item_conv->fix_fields(thd(), NULL));
-  const String *null_string = NULL;
+  EXPECT_FALSE(item_conv->fix_fields(thd(), nullptr));
+  const String *null_string = nullptr;
   String str;
   EXPECT_EQ(null_string, item_conv->val_str(&str));
 }
@@ -761,35 +886,31 @@ TEST_F(ItemTest, ItemDecimalTypecast) {
   const char msg[] = "";
   POS pos;
   pos.cpp.start = pos.cpp.end = pos.raw.start = pos.raw.end = msg;
-  // Sun Studio needs this null_item,
-  // it fails to compile EXPECT_EQ(NULL, create_func_cast());
-  const Item *null_item = NULL;
 
   Cast_type type;
   type.target = ITEM_CAST_DECIMAL;
-
   type.length = "123456789012345678901234567890";
-  type.dec = NULL;
+  type.dec = nullptr;
 
   {
     initializer.set_expected_error(ER_TOO_BIG_PRECISION);
-    EXPECT_EQ(null_item, create_func_cast(thd(), pos, NULL, &type));
+    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
   }
 
   {
     char buff[20];
     snprintf(buff, sizeof(buff) - 1, "%d", DECIMAL_MAX_PRECISION + 1);
     type.length = buff;
-    type.dec = NULL;
+    type.dec = nullptr;
     initializer.set_expected_error(ER_TOO_BIG_PRECISION);
-    EXPECT_EQ(null_item, create_func_cast(thd(), pos, NULL, &type));
+    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
   }
 
   {
-    type.length = NULL;
+    type.length = nullptr;
     type.dec = "123456789012345678901234567890";
     initializer.set_expected_error(ER_TOO_BIG_SCALE);
-    EXPECT_EQ(null_item, create_func_cast(thd(), pos, NULL, &type));
+    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
   }
 
   {
@@ -798,7 +919,7 @@ TEST_F(ItemTest, ItemDecimalTypecast) {
     type.length = buff;
     type.dec = buff;
     initializer.set_expected_error(ER_TOO_BIG_SCALE);
-    EXPECT_EQ(null_item, create_func_cast(thd(), pos, NULL, &type));
+    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
   }
 }
 
@@ -806,12 +927,12 @@ TEST_F(ItemTest, NormalizedPrint) {
   Item_null *item_null = new Item_null;
   {
     String s;
-    item_null->print(&s, QT_ORDINARY);
+    item_null->print(thd(), &s, QT_ORDINARY);
     EXPECT_STREQ("NULL", s.c_ptr());
   }
   {
     String s;
-    item_null->print(&s, QT_NORMALIZED_FORMAT);
+    item_null->print(thd(), &s, QT_NORMALIZED_FORMAT);
     EXPECT_STREQ("?", s.c_ptr());
   }
 }
@@ -829,4 +950,68 @@ TEST_F(ItemTest, CompareEmptyStrings) {
   EXPECT_EQ(0, comparator.compare_binary_string());
 }
 
+TEST_F(ItemTest, ItemJson) {
+  MEM_ROOT *const mem_root = initializer.thd()->mem_root;
+
+  const Item_name_string name(Name_string(STRING_WITH_LEN("json")));
+
+  Json_string jstr("123");
+  Item_json *item = new Item_json(
+      make_unique_destroy_only<Json_wrapper>(mem_root, &jstr, true), name);
+
+  Json_wrapper wr;
+  EXPECT_FALSE(item->val_json(&wr));
+  EXPECT_EQ(&jstr, wr.get_dom());
+
+  String string_buffer;
+  const String *str = item->val_str(&string_buffer);
+  EXPECT_EQ("\"123\"", to_string(*str));
+  EXPECT_EQ(item->collation.collation, str->charset());
+
+  EXPECT_EQ(123.0, item->val_real());
+  EXPECT_EQ(123, item->val_int());
+
+  my_decimal decimal_buffer;
+  const my_decimal *decimal = item->val_decimal(&decimal_buffer);
+  double dbl = 0;
+  EXPECT_EQ(E_DEC_OK, decimal2double(decimal, &dbl));
+  EXPECT_EQ(123.0, dbl);
+
+  Item *clone = item->clone_item();
+  EXPECT_NE(item, clone);
+  EXPECT_TRUE(item->eq(clone, true));
+  EXPECT_FALSE(clone->val_json(&wr));
+  EXPECT_NE(&jstr, wr.get_dom());
+  EXPECT_EQ(0, wr.compare(Json_wrapper(&jstr, true)));
+  EXPECT_EQ(123, clone->val_int());
+
+  const MysqlTime date(2020, 1, 2);
+  EXPECT_EQ(MYSQL_TIMESTAMP_DATE, date.time_type);
+  item = new Item_json(
+      make_unique_destroy_only<Json_wrapper>(
+          mem_root, std::unique_ptr<Json_dom>(new (std::nothrow) Json_datetime(
+                        date, MYSQL_TYPE_DATE))),
+      name);
+  MYSQL_TIME time_result;
+  EXPECT_FALSE(item->get_date(&time_result, 0));
+  EXPECT_EQ(date.time_type, time_result.time_type);
+  EXPECT_EQ(date.year, time_result.year);
+  EXPECT_EQ(date.month, time_result.month);
+  EXPECT_EQ(date.day, time_result.day);
+
+  const MysqlTime time(0, 0, 0, 10, 20, 30, 40, false, MYSQL_TIMESTAMP_TIME);
+  item = new Item_json(
+      make_unique_destroy_only<Json_wrapper>(
+          mem_root, std::unique_ptr<Json_dom>(new (std::nothrow) Json_datetime(
+                        time, MYSQL_TYPE_TIME))),
+      name);
+  EXPECT_FALSE(item->get_time(&time_result));
+  EXPECT_EQ(time.time_type, time_result.time_type);
+  EXPECT_EQ(time.hour, time_result.hour);
+  EXPECT_EQ(time.minute, time_result.minute);
+  EXPECT_EQ(time.second, time_result.second);
+  EXPECT_EQ(time.second_part, time_result.second_part);
+  EXPECT_EQ(time.neg, time_result.neg);
+}
+
 }  // namespace item_unittest
