@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,11 +27,11 @@
 #include "base64.h"
 #include "my_byteorder.h"
 #include "my_inttypes.h"
-#include "sql/json_binary.h"
+#include "sql-common/json_binary.h"
+#include "sql-common/json_dom.h"
+#include "sql-common/json_path.h"
+#include "sql-common/my_decimal.h"
 #include "sql/json_diff.h"
-#include "sql/json_dom.h"
-#include "sql/json_path.h"
-#include "sql/my_decimal.h"
 #include "sql/sql_class.h"
 #include "sql/sql_time.h"
 #include "sql_string.h"
@@ -39,6 +39,7 @@
 #include "unittest/gunit/base_mock_field.h"
 #include "unittest/gunit/benchmark.h"
 #include "unittest/gunit/fake_table.h"
+#include "unittest/gunit/mysys_util.h"
 #include "unittest/gunit/test_utils.h"
 
 /**
@@ -49,15 +50,16 @@ namespace json_dom_unittest {
 class JsonDomTest : public ::testing::Test {
  protected:
   Base_mock_field_json m_field{};
-  Fake_TABLE m_table{&m_field};
-  virtual void SetUp() {
+  Fake_TABLE *m_table;
+  void SetUp() override {
     initializer.SetUp();
+    m_table = new Fake_TABLE(&m_field);
     m_field.make_writable();
-    m_table.in_use = thd();
-    init_alloc_root(PSI_NOT_INSTRUMENTED, &m_table.mem_root, 256, 0);
+    m_table->in_use = thd();
   }
-  virtual void TearDown() {
-    m_table.cleanup_partial_update();
+  void TearDown() override {
+    m_table->cleanup_partial_update();
+    delete m_table;
     initializer.TearDown();
   }
   my_testing::Server_initializer initializer;
@@ -73,7 +75,8 @@ class JsonDomTest : public ::testing::Test {
 static std::string format(const Json_dom &d) {
   String buffer;
   Json_wrapper w(d.clone());
-  EXPECT_FALSE(w.to_string(&buffer, true, "format"));
+  EXPECT_FALSE(
+      w.to_string(&buffer, true, "format", [] { ASSERT_TRUE(false); }));
 
   return std::string(buffer.ptr(), buffer.length());
 }
@@ -88,8 +91,9 @@ static std::string format(const Json_dom_ptr &ptr) { return format(*ptr); }
   @return a DOM representing the JSON document
 */
 static Json_dom_ptr parse_json(const char *json_text) {
-  auto dom =
-      Json_dom::parse(json_text, std::strlen(json_text), nullptr, nullptr);
+  auto dom = Json_dom::parse(
+      json_text, std::strlen(json_text), [](const char *, size_t) {},
+      [] { ASSERT_TRUE(false); });
   EXPECT_FALSE(dom == nullptr);
   return dom;
 }
@@ -100,9 +104,10 @@ static Json_dom_ptr parse_json(const char *json_text) {
   @return the parsed path
 */
 static Json_path parse_path(const char *json_path) {
-  Json_path path;
+  Json_path path(key_memory_JSON);
   size_t bad_index;
-  EXPECT_FALSE(parse_path(std::strlen(json_path), json_path, &path, &bad_index))
+  EXPECT_FALSE(parse_path(std::strlen(json_path), json_path, &path, &bad_index,
+                          [] { ASSERT_TRUE(false); }))
       << "bad index: " << bad_index;
   return path;
 }
@@ -242,7 +247,7 @@ TEST_F(JsonDomTest, BasicTest) {
   /* Object access: key look-up */
   EXPECT_EQ(enum_json_type::J_OBJECT, elt->json_type());
   Json_object *const object_elt = down_cast<Json_object *>(elt);
-  EXPECT_TRUE(object_elt != NULL);
+  EXPECT_TRUE(object_elt != nullptr);
   const Json_dom *const elt2 = object_elt->get(std::string("key1"));
   EXPECT_EQ(std::string("null"), format(elt2));
 
@@ -336,7 +341,7 @@ TEST_F(JsonDomTest, BasicTest) {
   const char *encoded = "vrr+yg==";
   char *buff = new char[static_cast<size_t>(
       base64_needed_decoded_length(static_cast<int>(std::strlen(encoded))))];
-  EXPECT_EQ(4, base64_decode(encoded, std::strlen(encoded), buff, NULL, 0));
+  EXPECT_EQ(4, base64_decode(encoded, std::strlen(encoded), buff, nullptr, 0));
   EXPECT_EQ(0xCAFEBABE, uint4korr(buff));
   delete[] buff;
 
@@ -396,13 +401,15 @@ TEST_F(JsonDomTest, BasicTest) {
      Included so we test error recovery
   */
   const char *half_object_item = "{\"label\": ";
-  dom = Json_dom::parse(half_object_item, std::strlen(half_object_item),
-                        nullptr, nullptr);
+  dom = Json_dom::parse(
+      half_object_item, std::strlen(half_object_item),
+      [](const char *, size_t) {}, [] { ASSERT_TRUE(false); });
   EXPECT_EQ(nullptr, dom);
 
   const char *half_array_item = "[1,";
-  dom = Json_dom::parse(half_array_item, std::strlen(half_array_item), nullptr,
-                        nullptr);
+  dom = Json_dom::parse(
+      half_array_item, std::strlen(half_array_item),
+      [](const char *, size_t) {}, [] { ASSERT_TRUE(false); });
   EXPECT_EQ(nullptr, dom);
 }
 
@@ -428,16 +435,16 @@ TEST_F(JsonDomTest, EscapeSpecialChars) {
   EXPECT_EQ(str.value(), str2->value());
 }
 
-void vet_wrapper_length(const THD *thd, const char *text,
-                        size_t expected_length) {
+void vet_wrapper_length(const char *text, size_t expected_length) {
   Json_wrapper dom_wrapper(parse_json(text));
 
   EXPECT_EQ(expected_length, dom_wrapper.length())
       << "Wrapped DOM: " << text << "\n";
 
   String serialized_form;
-  EXPECT_FALSE(
-      json_binary::serialize(thd, dom_wrapper.to_dom(thd), &serialized_form));
+  EXPECT_FALSE(json_binary::serialize(
+      dom_wrapper.to_dom(), &serialized_form, &JsonDepthErrorHandler,
+      &JsonKeyTooBigErrorHandler, &JsonValueTooBigErrorHandler));
   json_binary::Value binary = json_binary::parse_binary(
       serialized_form.ptr(), serialized_form.length());
   Json_wrapper binary_wrapper(binary);
@@ -457,31 +464,30 @@ TEST_F(JsonDomTest, WrapperTest) {
   // Constructors, assignment, copy constructors, aliasing
   Json_dom *d = new (std::nothrow) Json_null();
   Json_wrapper w(d);
-  const THD *thd = this->thd();
-  EXPECT_EQ(w.to_dom(thd), d);
+  EXPECT_EQ(w.to_dom(), d);
   Json_wrapper w_2(w);
-  EXPECT_NE(w.to_dom(thd), w_2.to_dom(thd));  // deep copy
+  EXPECT_NE(w.to_dom(), w_2.to_dom());  // deep copy
 
   Json_wrapper w_2b;
   EXPECT_TRUE(w_2b.empty());
   w_2b = w;
-  EXPECT_NE(w.to_dom(thd), w_2b.to_dom(thd));  // deep copy
+  EXPECT_NE(w.to_dom(), w_2b.to_dom());  // deep copy
 
   w.set_alias();  // d is now "free" again
   Json_wrapper w_3(w);
-  EXPECT_EQ(w.to_dom(thd), w_3.to_dom(thd));  // alias copy
+  EXPECT_EQ(w.to_dom(), w_3.to_dom());  // alias copy
   w_3 = w;
-  EXPECT_EQ(w.to_dom(thd), w_3.to_dom(thd));  // alias copy
+  EXPECT_EQ(w.to_dom(), w_3.to_dom());  // alias copy
 
   Json_wrapper w_4(d);  // give d a new owner
   Json_wrapper w_5;
   w_5 = std::move(w_4);  // takes over d
-  EXPECT_EQ(w_4.to_dom(thd), w_5.to_dom(thd));
+  EXPECT_EQ(w_4.to_dom(), w_5.to_dom());
 
   Json_wrapper w_6;
   EXPECT_EQ(enum_json_type::J_ERROR, w_6.type());
-  EXPECT_EQ(nullptr, w_6.to_dom(thd));
-  EXPECT_EQ(nullptr, w_6.clone_dom(thd));
+  EXPECT_EQ(nullptr, w_6.to_dom());
+  EXPECT_EQ(nullptr, w_6.clone_dom());
   EXPECT_EQ(0U, w_6.length());
 
   Json_dom *i = new (std::nothrow) Json_int(1);
@@ -489,30 +495,31 @@ TEST_F(JsonDomTest, WrapperTest) {
   w_5 = std::move(w_7);  // should deallocate w_5's original
 
   // scalars
-  vet_wrapper_length(thd, "false", 1);
-  vet_wrapper_length(thd, "true", 1);
-  vet_wrapper_length(thd, "null", 1);
-  vet_wrapper_length(thd, "1.1", 1);
-  vet_wrapper_length(thd, "\"hello world\"", 1);
+  vet_wrapper_length("false", 1);
+  vet_wrapper_length("true", 1);
+  vet_wrapper_length("null", 1);
+  vet_wrapper_length("1.1", 1);
+  vet_wrapper_length("\"hello world\"", 1);
 
   // objects
-  vet_wrapper_length(thd, "{}", 0);
-  vet_wrapper_length(thd, "{ \"a\" : 100 }", 1);
-  vet_wrapper_length(thd, "{ \"a\" : 100, \"b\" : 200 }", 2);
+  vet_wrapper_length("{}", 0);
+  vet_wrapper_length("{ \"a\" : 100 }", 1);
+  vet_wrapper_length("{ \"a\" : 100, \"b\" : 200 }", 2);
 
   // arrays
-  vet_wrapper_length(thd, "[]", 0);
-  vet_wrapper_length(thd, "[ 100 ]", 1);
-  vet_wrapper_length(thd, "[ 100, 200 ]", 2);
+  vet_wrapper_length("[]", 0);
+  vet_wrapper_length("[ 100 ]", 1);
+  vet_wrapper_length("[ 100, 200 ]", 2);
 
   // nested objects
-  vet_wrapper_length(thd, "{ \"a\" : 100, \"b\" : { \"c\" : 300 } }", 2);
+  vet_wrapper_length("{ \"a\" : 100, \"b\" : { \"c\" : 300 } }", 2);
 
   // nested arrays
-  vet_wrapper_length(thd, "[ 100, [ 200, 300 ] ]", 2);
+  vet_wrapper_length("[ 100, [ 200, 300 ] ]", 2);
 }
 
-void vet_merge(char *left_text, char *right_text, std::string expected) {
+void vet_merge(const char *left_text, const char *right_text,
+               std::string expected) {
   Json_dom_ptr result_dom =
       merge_doms(parse_json(left_text), parse_json(right_text));
   EXPECT_EQ(expected, format(*result_dom));
@@ -522,128 +529,125 @@ TEST_F(JsonDomTest, MergeTest) {
   // merge 2 scalars
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"1", (char *)"true", "[1, true]");
+    vet_merge("1", "true", "[1, true]");
   }
 
   // merge a scalar with an array
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"1", (char *)"[true, false]", "[1, true, false]");
+    vet_merge("1", "[true, false]", "[1, true, false]");
   }
 
   // merge an array with a scalar
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"[true, false]", (char *)"1", "[true, false, 1]");
+    vet_merge("[true, false]", "1", "[true, false, 1]");
   }
 
   // merge a scalar with an object
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"1", (char *)"{\"a\": 2}", "[1, {\"a\": 2}]");
+    vet_merge("1", "{\"a\": 2}", "[1, {\"a\": 2}]");
   }
 
   // merge an object with a scalar
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 2}", (char *)"1", "[{\"a\": 2}, 1]");
+    vet_merge("{\"a\": 2}", "1", "[{\"a\": 2}, 1]");
   }
 
   // merge 2 arrays
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"[1, 2]", (char *)"[3, 4]", "[1, 2, 3, 4]");
+    vet_merge("[1, 2]", "[3, 4]", "[1, 2, 3, 4]");
   }
 
   // merge 2 objects
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 1, \"b\": 2 }", (char *)"{\"c\": 3, \"d\": 4 }",
+    vet_merge("{\"a\": 1, \"b\": 2 }", "{\"c\": 3, \"d\": 4 }",
               "{\"a\": 1, \"b\": 2, \"c\": 3, \"d\": 4}");
   }
 
   // merge an array with an object
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"[1, 2]", (char *)"{\"c\": 3, \"d\": 4 }",
+    vet_merge("[1, 2]", "{\"c\": 3, \"d\": 4 }",
               "[1, 2, {\"c\": 3, \"d\": 4}]");
   }
 
   // merge an object with an array
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"c\": 3, \"d\": 4 }", (char *)"[1, 2]",
+    vet_merge("{\"c\": 3, \"d\": 4 }", "[1, 2]",
               "[{\"c\": 3, \"d\": 4}, 1, 2]");
   }
 
   // merge two objects which share a key. scalar + scalar
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 1, \"b\": 2 }", (char *)"{\"b\": 3, \"d\": 4 }",
+    vet_merge("{\"a\": 1, \"b\": 2 }", "{\"b\": 3, \"d\": 4 }",
               "{\"a\": 1, \"b\": [2, 3], \"d\": 4}");
   }
 
   // merge two objects which share a key. scalar + array
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 1, \"b\": 2 }",
-              (char *)"{\"b\": [3, 4], \"d\": 4 }",
+    vet_merge("{\"a\": 1, \"b\": 2 }", "{\"b\": [3, 4], \"d\": 4 }",
               "{\"a\": 1, \"b\": [2, 3, 4], \"d\": 4}");
   }
 
   // merge two objects which share a key. array + scalar
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 1, \"b\": [2, 3] }",
-              (char *)"{\"b\": 4, \"d\": 4 }",
+    vet_merge("{\"a\": 1, \"b\": [2, 3] }", "{\"b\": 4, \"d\": 4 }",
               "{\"a\": 1, \"b\": [2, 3, 4], \"d\": 4}");
   }
 
   // merge two objects which share a key. scalar + object
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 1, \"b\": 2 }",
-              (char *)"{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
+    vet_merge("{\"a\": 1, \"b\": 2 }",
+              "{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
               "{\"a\": 1, \"b\": [2, {\"e\": 7, \"f\": 8}], \"d\": 4}");
   }
 
   // merge two objects which share a key. object + scalar
   {
     SCOPED_TRACE("");
-    vet_merge((char *)(char *)"{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
-              (char *)"{\"a\": 1, \"b\": 2 }",
+    vet_merge("{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
+              "{\"a\": 1, \"b\": 2 }",
               "{\"a\": 1, \"b\": [{\"e\": 7, \"f\": 8}, 2], \"d\": 4}");
   }
 
   // merge two objects which share a key. array + array
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 1, \"b\": [2, 9] }",
-              (char *)"{\"b\": [10, 11], \"d\": 4 }",
+    vet_merge("{\"a\": 1, \"b\": [2, 9] }", "{\"b\": [10, 11], \"d\": 4 }",
               "{\"a\": 1, \"b\": [2, 9, 10, 11], \"d\": 4}");
   }
 
   // merge two objects which share a key. array + object
   {
     SCOPED_TRACE("");
-    vet_merge((char *)"{\"a\": 1, \"b\": [2, 9] }",
-              (char *)"{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
+    vet_merge("{\"a\": 1, \"b\": [2, 9] }",
+              "{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
               "{\"a\": 1, \"b\": [2, 9, {\"e\": 7, \"f\": 8}], \"d\": 4}");
   }
 
   // merge two objects which share a key. object + array
   {
     SCOPED_TRACE("");
-    vet_merge((char *)(char *)"{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
-              (char *)"{\"a\": 1, \"b\": [2, 9] }",
+    vet_merge("{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
+              "{\"a\": 1, \"b\": [2, 9] }",
               "{\"a\": 1, \"b\": [{\"e\": 7, \"f\": 8}, 2, 9], \"d\": 4}");
   }
 
   // merge two objects which share a key. object + object
   {
     SCOPED_TRACE("");
-    vet_merge((char *)(char *)"{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
-              (char *)"{\"a\": 1, \"b\": {\"e\": 20, \"g\": 21 } }",
+    vet_merge("{\"b\": {\"e\": 7, \"f\": 8}, \"d\": 4 }",
+              "{\"a\": 1, \"b\": {\"e\": 20, \"g\": 21 } }",
               "{\"a\": 1, \"b\": {\"e\": [7, 20], \"f\": 8, \"g\": 21}, "
               "\"d\": 4}");
   }
@@ -651,7 +655,7 @@ TEST_F(JsonDomTest, MergeTest) {
 
 /// Create a JSON path for accessing an array element at the given position.
 static Json_path array_accessor(size_t idx) {
-  Json_path path;
+  Json_path path(key_memory_JSON);
   path.append(Json_path_leg(idx));
   return path;
 }
@@ -697,31 +701,33 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
   auto dom = parse_json("[\"abc\", 123, \"def\", -70000]");
 
   String buffer;
-  EXPECT_FALSE(json_binary::serialize(thd(), dom.get(), &buffer));
+  EXPECT_FALSE(json_binary::serialize(
+      dom.get(), &buffer, &JsonDepthErrorHandler, &JsonKeyTooBigErrorHandler,
+      &JsonValueTooBigErrorHandler));
 
   json_binary::Value binary =
       json_binary::parse_binary(buffer.ptr(), buffer.length());
 
-  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table.setup_partial_update(true));
+  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table->setup_partial_update(true));
 
   // Verify that the table interface for partial update works.
-  EXPECT_TRUE(m_table.has_binary_diff_columns());
-  EXPECT_TRUE(m_table.is_marked_for_partial_update(&m_field));
-  EXPECT_TRUE(m_table.is_binary_diff_enabled(&m_field));
-  EXPECT_NE(nullptr, m_table.get_binary_diffs(&m_field));
-  m_table.disable_binary_diffs_for_current_row(&m_field);
-  EXPECT_FALSE(m_table.has_binary_diff_columns());
-  EXPECT_TRUE(m_table.is_marked_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table.is_binary_diff_enabled(&m_field));
-  EXPECT_EQ(nullptr, m_table.get_binary_diffs(&m_field));
-  m_table.clear_partial_update_diffs();
-  EXPECT_TRUE(m_table.has_binary_diff_columns());
-  EXPECT_TRUE(m_table.is_marked_for_partial_update(&m_field));
-  EXPECT_TRUE(m_table.is_binary_diff_enabled(&m_field));
-  EXPECT_NE(nullptr, m_table.get_binary_diffs(&m_field));
+  EXPECT_TRUE(m_table->has_binary_diff_columns());
+  EXPECT_TRUE(m_table->is_marked_for_partial_update(&m_field));
+  EXPECT_TRUE(m_table->is_binary_diff_enabled(&m_field));
+  EXPECT_NE(nullptr, m_table->get_binary_diffs(&m_field));
+  m_table->disable_binary_diffs_for_current_row(&m_field);
+  EXPECT_FALSE(m_table->has_binary_diff_columns());
+  EXPECT_TRUE(m_table->is_marked_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table->is_binary_diff_enabled(&m_field));
+  EXPECT_EQ(nullptr, m_table->get_binary_diffs(&m_field));
+  m_table->clear_partial_update_diffs();
+  EXPECT_TRUE(m_table->has_binary_diff_columns());
+  EXPECT_TRUE(m_table->is_marked_for_partial_update(&m_field));
+  EXPECT_TRUE(m_table->is_binary_diff_enabled(&m_field));
+  EXPECT_NE(nullptr, m_table->get_binary_diffs(&m_field));
 
-  const Binary_diff_vector *diffs = m_table.get_binary_diffs(&m_field);
+  const Binary_diff_vector *diffs = m_table->get_binary_diffs(&m_field);
   EXPECT_TRUE(diffs != nullptr);
   EXPECT_EQ(0U, diffs->size());
 
@@ -731,7 +737,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     Json_wrapper doc(binary);
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     Json_wrapper jstr(new (std::nothrow) Json_string("abcd"));
-    m_table.clear_partial_update_diffs();
+    m_table->clear_partial_update_diffs();
     auto path = array_accessor(i);
     String shadow;
     bool success = true;
@@ -742,7 +748,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_FALSE(replaced);
     EXPECT_EQ(0U, diffs->size());
     EXPECT_EQ(0, doc.compare(Json_wrapper(binary)));
-    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
   }
 
   // Enough space for an inlinable value anywhere in the array.
@@ -752,7 +758,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
     Json_wrapper jint(new (std::nothrow) Json_int(456));
-    m_table.clear_partial_update_diffs();
+    m_table->clear_partial_update_diffs();
     auto path = array_accessor(i);
     String shadow;
     bool success = false;
@@ -765,7 +771,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
 
     Json_array_ptr array(down_cast<Json_array *>(dom->clone().release()));
     array->remove(i);
-    array->insert_clone(i, jint.to_dom(thd()));
+    array->insert_clone(i, jint.to_dom());
     EXPECT_EQ(0, doc.compare(Json_wrapper(std::move(array))));
 
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
@@ -779,7 +785,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
     Json_wrapper jint(new (std::nothrow) Json_uint(80000));
-    m_table.clear_partial_update_diffs();
+    m_table->clear_partial_update_diffs();
     auto path = array_accessor(i);
     String shadow;
     bool success = false;
@@ -790,7 +796,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     if (!success) {
       EXPECT_FALSE(replaced);
       EXPECT_EQ(0U, diffs->size());
-      EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
+      EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
       continue;
     }
     EXPECT_TRUE(replaced);
@@ -798,7 +804,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
 
     Json_array_ptr array(down_cast<Json_array *>(dom->clone().release()));
     array->remove(i);
-    array->insert_clone(i, jint.to_dom(thd()));
+    array->insert_clone(i, jint.to_dom());
     EXPECT_EQ(0, doc.compare(Json_wrapper(std::move(array))));
 
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
@@ -811,7 +817,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
     Json_wrapper jint(new (std::nothrow) Json_int(456));
-    m_table.clear_partial_update_diffs();
+    m_table->clear_partial_update_diffs();
     String shadow;
     auto path = array_accessor(100);
 
@@ -826,7 +832,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_FALSE(success);
     EXPECT_FALSE(replaced);
     EXPECT_EQ(0U, diffs->size());
-    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
 
     /*
       JSON_REPLACE is a no-op if a non-existing path is given, so expect
@@ -838,21 +844,22 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_FALSE(replaced);
     EXPECT_EQ(0U, diffs->size());
     EXPECT_EQ(0, doc.compare(Json_wrapper(binary)));
-    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
 
     /*
       If we replace the top-level document (empty path), we do a full update.
       Expect the attempt to do partial update to fail.
     */
-    EXPECT_FALSE(doc.attempt_binary_update(&m_field, Json_path(), &jint, false,
-                                           &shadow, &success, &replaced));
+    EXPECT_FALSE(
+        doc.attempt_binary_update(&m_field, Json_path(PSI_NOT_INSTRUMENTED),
+                                  &jint, false, &shadow, &success, &replaced));
     EXPECT_FALSE(success);
     EXPECT_FALSE(replaced);
   }
 
   {
     SCOPED_TRACE("");
-    m_table.clear_partial_update_diffs();
+    m_table->clear_partial_update_diffs();
     Json_wrapper doc(binary);
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
@@ -870,7 +877,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
       EXPECT_TRUE(success);
       EXPECT_TRUE(replaced);
       array->remove(i);
-      array->insert_clone(i, jstr.to_dom(thd()));
+      array->insert_clone(i, jstr.to_dom());
       EXPECT_EQ(0, doc.compare(array_wrapper));
 
       EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
@@ -895,8 +902,8 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
 */
 TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
   // Make the table ready for partial update.
-  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table.setup_partial_update(true));
+  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table->setup_partial_update(true));
 
   my_decimal decimal;
   EXPECT_FALSE(double2my_decimal(0, 3.14, &decimal));
@@ -940,7 +947,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
   };
 
   for (auto dom : doms) {
-    m_table.clear_partial_update_diffs();
+    m_table->clear_partial_update_diffs();
 
     Json_array_ptr original_dom(new (std::nothrow) Json_array);
     original_dom->append_alias(new (std::nothrow) Json_string(20, 'x'));
@@ -955,7 +962,9 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
     EXPECT_FALSE(m_field.val_json(&doc));
 
     StringBuffer<STRING_BUFFER_USUAL_SIZE> original;
-    EXPECT_FALSE(doc.to_binary(thd(), &original));
+    EXPECT_FALSE(doc.to_binary(
+        &original, &JsonDepthErrorHandler, &JsonKeyTooBigErrorHandler,
+        &JsonValueTooBigErrorHandler, &InvalidJsonErrorHandler));
 
     Json_wrapper new_value(dom->clone());
 
@@ -969,9 +978,9 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
                                            &replaced));
     EXPECT_TRUE(success);
     EXPECT_FALSE(replaced);
-    EXPECT_EQ(0U, m_table.get_binary_diffs(&m_field)->size());
-    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
-    m_table.clear_partial_update_diffs();
+    EXPECT_EQ(0U, m_table->get_binary_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
+    m_table->clear_partial_update_diffs();
 
     // Try with non-existing path and JSON_SET logic. Requires full update.
     EXPECT_FALSE(doc.attempt_binary_update(&m_field, array_accessor(100),
@@ -979,8 +988,8 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
                                            &replaced));
     EXPECT_FALSE(success);
     EXPECT_FALSE(replaced);
-    EXPECT_EQ(0U, m_table.get_binary_diffs(&m_field)->size());
-    m_table.clear_partial_update_diffs();
+    EXPECT_EQ(0U, m_table->get_binary_diffs(&m_field)->size());
+    m_table->clear_partial_update_diffs();
 
     // Try with a valid path. Expect success.
     EXPECT_FALSE(doc.attempt_binary_update(&m_field, array_accessor(0),
@@ -988,10 +997,10 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
                                            &replaced));
     EXPECT_TRUE(success);
     EXPECT_TRUE(replaced);
-    EXPECT_NE(0U, m_table.get_binary_diffs(&m_field)->size());
+    EXPECT_NE(0U, m_table->get_binary_diffs(&m_field)->size());
 
     String str;
-    new_value.to_string(&str, true, "test");
+    new_value.to_string(&str, true, "test", [] { ASSERT_TRUE(false); });
     // Verify the updated document.
     {
       Json_array_ptr a(new (std::nothrow) Json_array);
@@ -1001,7 +1010,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
 
     // Verify the binary diffs.
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
-    verify_binary_diffs(&m_field, m_table.get_binary_diffs(&m_field), original,
+    verify_binary_diffs(&m_field, m_table->get_binary_diffs(&m_field), original,
                         buffer);
 
     delete dom;
@@ -1012,8 +1021,8 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
   Test attempt_binary_update() with invalid input.
 */
 TEST_F(JsonDomTest, AttemptBinaryUpdate_Error) {
-  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table.setup_partial_update(true));
+  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table->setup_partial_update(true));
   Json_wrapper doc(parse_json("[1,2,3,4]"));
   EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
   EXPECT_FALSE(m_field.val_json(&doc));
@@ -1026,7 +1035,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_Error) {
   EXPECT_EQ(enum_json_type::J_ERROR, error.type());
 
   // Expect the calls to attempt_binary_update() to fail.
-  const auto old_error_handler_hook = error_handler_hook;
+  const ErrorHandlerFunctionPointer old_error_handler_hook = error_handler_hook;
   error_handler_hook = my_message_sql;
   String buffer;
   bool success;
@@ -1083,9 +1092,12 @@ void test_apply_json_diffs(Field_json *field, const Json_diff_vector &diffs,
   if (table->is_binary_diff_enabled(field)) {
     StringBuffer<STRING_BUFFER_USUAL_SIZE> original;
     EXPECT_FALSE(json_binary::serialize(
-        table->in_use, parse_json(orig_json).get(), &original));
+        parse_json(orig_json).get(), &original, &JsonDepthErrorHandler,
+        &JsonKeyTooBigErrorHandler, &JsonValueTooBigErrorHandler));
     StringBuffer<STRING_BUFFER_USUAL_SIZE> updated;
-    EXPECT_FALSE(doc.to_binary(table->in_use, &updated));
+    EXPECT_FALSE(doc.to_binary(
+        &updated, &JsonDepthErrorHandler, &JsonKeyTooBigErrorHandler,
+        &JsonValueTooBigErrorHandler, &InvalidJsonErrorHandler));
     verify_binary_diffs(field, table->get_binary_diffs(field), original,
                         updated);
   }
@@ -1237,8 +1249,8 @@ static void do_apply_json_diffs_tests(Field_json *field) {
 TEST_F(JsonDomTest, ApplyJsonDiffs) { do_apply_json_diffs_tests(&m_field); }
 
 TEST_F(JsonDomTest, ApplyJsonDiffs_CollectBinaryDiffs) {
-  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table.setup_partial_update(true));
+  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table->setup_partial_update(true));
   do_apply_json_diffs_tests(&m_field);
 }
 
@@ -1246,10 +1258,10 @@ TEST_F(JsonDomTest, ApplyJsonDiffs_CollectBinaryDiffs) {
   Run a microbenchmarks that tests how fast Json_wrapper::seek() is on
   a wrapper that wraps a Json_dom.
 
-  @param num_iterations  the number of iterations to run
-  @param path            the JSON path to search for
-  @param need_only_one   true if the search should stop after the first match
-  @param expected_hits   the number of expected matches
+  @param num_iterations   the number of iterations to run
+  @param path             the JSON path to search for
+  @param need_only_one    true if the search should stop after the first match
+  @param expected_matches the number of expected matches
 */
 static void benchmark_dom_seek(size_t num_iterations, const Json_path &path,
                                bool need_only_one, size_t expected_matches) {
@@ -1300,14 +1312,71 @@ static void BM_JsonDomSearchKey(size_t num_iterations) {
 }
 BENCHMARK(BM_JsonDomSearchKey)
 
+/**
+  Microbenchmark for Json_dom::parse(const json_binary::Value &v).
+
+  @param num_iterations the number of iterations to run
+  @param json_text      the string to be parsed(after being converted to binary)
+*/
+static void benchmark_dom_parse(size_t num_iterations, const char *json_text) {
+  StopBenchmarkTiming();
+  my_testing::Server_initializer initializer;
+  initializer.SetUp();
+
+  auto dom = parse_json(json_text);
+
+  String buffer;
+  EXPECT_FALSE(json_binary::serialize(
+      dom.get(), &buffer, &JsonDepthErrorHandler, &JsonKeyTooBigErrorHandler,
+      &JsonValueTooBigErrorHandler));
+
+  json_binary::Value binary =
+      json_binary::parse_binary(buffer.ptr(), buffer.length());
+
+  StartBenchmarkTiming();
+  for (size_t i = 0; i < num_iterations; ++i) {
+    Json_dom_ptr ptr = Json_dom::parse(binary);
+    assert(ptr);
+  }
+
+  StopBenchmarkTiming();
+}
+
+/**
+  Microbenchmark that tests Json_dom::parse(const json_binary::Value &v)
+  with a deep JSON value
+
+  @param num_iterations the number of iterations to run
+*/
+static void BM_JsonDomParseDeep(size_t num_iterations) {
+  benchmark_dom_parse(num_iterations,
+                      "[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[["
+                      "[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[["
+                      R"({"key":123})"
+                      "]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]"
+                      "]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]");
+}
+BENCHMARK(BM_JsonDomParseDeep)
+
+/**
+  Microbenchmark that tests Json_dom::parse(const json_binary::Value &v)
+  with a shallow JSON value
+
+  @param num_iterations the number of iterations to run
+*/
+static void BM_JsonDomParseShallow(size_t num_iterations) {
+  benchmark_dom_parse(num_iterations, R"([{"key":123}])");
+}
+BENCHMARK(BM_JsonDomParseShallow)
+
 /**
   Run a microbenchmarks that tests how fast Json_wrapper::seek() is on
   a wrapper that wraps a binary JSON value.
 
-  @param num_iterations  the number of iterations to run
-  @param path            the JSON path to search for
-  @param need_only_one   true if the search should stop after the first match
-  @param expected_hits   the number of expected matches
+  @param num_iterations   the number of iterations to run
+  @param path             the JSON path to search for
+  @param need_only_one    true if the search should stop after the first match
+  @param expected_matches the number of expected matches
 */
 static void benchmark_binary_seek(size_t num_iterations, const Json_path &path,
                                   bool need_only_one, size_t expected_matches) {
@@ -1321,7 +1390,9 @@ static void benchmark_binary_seek(size_t num_iterations, const Json_path &path,
   initializer.SetUp();
 
   String buffer;
-  EXPECT_FALSE(json_binary::serialize(initializer.thd(), &o, &buffer));
+  EXPECT_FALSE(json_binary::serialize(&o, &buffer, &JsonDepthErrorHandler,
+                                      &JsonKeyTooBigErrorHandler,
+                                      &JsonValueTooBigErrorHandler));
   json_binary::Value val =
       json_binary::parse_binary(buffer.ptr(), buffer.length());
 
@@ -1386,7 +1457,7 @@ static void BM_JsonStringToString_Plain(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wr.to_string(&buf, true, "test");
+    wr.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
     EXPECT_EQ(quoted_length, buf.length());
   }
 
@@ -1414,7 +1485,7 @@ static void BM_JsonStringToString_SpecialChars(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wr.to_string(&buf, true, "test");
+    wr.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
     EXPECT_LT(str.size(), buf.length());
   }
 
@@ -1444,7 +1515,7 @@ static void BM_JsonObjectToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wr.to_string(&buf, true, "test");
+    wr.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1474,7 +1545,7 @@ static void BM_JsonBooleanArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test");
+    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1504,7 +1575,7 @@ static void BM_JsonDoubleArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test");
+    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1536,7 +1607,7 @@ static void BM_JsonDecimalArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test");
+    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1557,7 +1628,7 @@ static void BM_JsonDateArrayToString(size_t num_iterations) {
   initializer.SetUp();
 
   Json_array_ptr array = create_dom_ptr<Json_array>();
-  MYSQL_TIME date = {2018, 11, 20, 0, 0, 0, 0, false, MYSQL_TIMESTAMP_DATE};
+  MysqlTime date(2018, 11, 20, 0, 0, 0, 0, false, MYSQL_TIMESTAMP_DATE);
   for (size_t i = 0; i < 1000; ++i) {
     array->append_alias(create_dom_ptr<Json_datetime>(date, MYSQL_TYPE_DATE));
   }
@@ -1567,7 +1638,7 @@ static void BM_JsonDateArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test");
+    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1599,9 +1670,9 @@ static void BM_JsonWrapperObjectIteratorDOM(size_t num_iterations) {
   StartBenchmarkTiming();
 
   for (size_t i = 0; i < num_iterations; ++i) {
-    for (Json_wrapper_object_iterator it(wrapper); !it.empty(); it.next()) {
-      EXPECT_NE(0U, it.key().length);
-      EXPECT_EQ(enum_json_type::J_NULL, it.value().type());
+    for (const auto &it : Json_object_wrapper(wrapper)) {
+      EXPECT_NE(0U, it.first.length);
+      EXPECT_EQ(enum_json_type::J_NULL, it.second.type());
     }
   }
 
@@ -1628,8 +1699,9 @@ static void BM_JsonWrapperObjectIteratorBinary(size_t num_iterations) {
   }
 
   String serialized_object;
-  EXPECT_FALSE(json_binary::serialize(initializer.thd(), &dom_object,
-                                      &serialized_object));
+  EXPECT_FALSE(json_binary::serialize(
+      &dom_object, &serialized_object, &JsonDepthErrorHandler,
+      &JsonKeyTooBigErrorHandler, &JsonValueTooBigErrorHandler));
   Json_wrapper wrapper(json_binary::parse_binary(serialized_object.ptr(),
                                                  serialized_object.length()));
   EXPECT_EQ(enum_json_type::J_OBJECT, wrapper.type());
@@ -1637,9 +1709,9 @@ static void BM_JsonWrapperObjectIteratorBinary(size_t num_iterations) {
   StartBenchmarkTiming();
 
   for (size_t i = 0; i < num_iterations; ++i) {
-    for (Json_wrapper_object_iterator it(wrapper); !it.empty(); it.next()) {
-      EXPECT_NE(0U, it.key().length);
-      EXPECT_EQ(enum_json_type::J_NULL, it.value().type());
+    for (const auto &it : Json_object_wrapper(wrapper)) {
+      EXPECT_NE(0U, it.first.length);
+      EXPECT_EQ(enum_json_type::J_NULL, it.second.type());
     }
   }
 
