@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -633,7 +632,8 @@ static void print_ssl_session_id(SSL_SESSION *sess, const char *action) {
 
 static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
                   SSL_SESSION *ssl_session, ssl_handshake_func_t func,
-                  unsigned long *ssl_errno_holder, SSL **sslptr) {
+                  unsigned long *ssl_errno_holder, SSL **sslptr,
+                  const char *sni_servername) {
   SSL *ssl = nullptr;
   my_socket sd = mysql_socket_getfd(vio->mysql_socket);
 
@@ -671,6 +671,20 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
       }
     }
 
+    if (sni_servername) {
+// Check if the openssl supports SNI, and only then set the tlsext_host_name
+#if !defined(OPENSSL_NO_TLSEXT)
+      if (!SSL_set_tlsext_host_name(ssl, const_cast<char *>(sni_servername))) {
+        DBUG_PRINT("error", ("SSL_set_tlsext_host_name failure"));
+        *ssl_errno_holder = ERR_get_error();
+        return 1;
+      }
+#else
+      *ssl_errno_holder = 0;
+      return 1;
+#endif /* !def(OPENSSL_NO_TLSEXT) */
+    }
+
     DBUG_PRINT("info", ("ssl: %p timeout: %ld", ssl, timeout));
     SSL_clear(ssl);
     SSL_SESSION_set_timeout(SSL_get_session(ssl), timeout);
@@ -770,17 +784,17 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
 int sslaccept(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
               unsigned long *ssl_errno_holder) {
   DBUG_TRACE;
-  int ret =
-      ssl_do(ptr, vio, timeout, nullptr, SSL_accept, ssl_errno_holder, nullptr);
+  int ret = ssl_do(ptr, vio, timeout, nullptr, SSL_accept, ssl_errno_holder,
+                   nullptr, nullptr);
   return ret;
 }
 
 int sslconnect(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
-               SSL_SESSION *session, unsigned long *ssl_errno_holder,
-               SSL **ssl) {
+               SSL_SESSION *session, unsigned long *ssl_errno_holder, SSL **ssl,
+               const char *sni_servername) {
   DBUG_TRACE;
-  int ret =
-      ssl_do(ptr, vio, timeout, session, SSL_connect, ssl_errno_holder, ssl);
+  int ret = ssl_do(ptr, vio, timeout, session, SSL_connect, ssl_errno_holder,
+                   ssl, sni_servername);
   return ret;
 }
 
